Package: ExtremeRisks
Version: 0.0.6
Date: 2026-05-08
Title: Extreme Risk Measures
Authors@R: c(person("Simone","Padoan", role = c("cre", "aut"), email="simone.padoan@unibocconi.it"),
	    person("Gilles", "Stupfler", role = "aut", email="gilles.stupfler@univ-angers.fr"),
	    person("Carlotta", "Pacifici", role = c("aut"), email="carlotta.pacifici@unibocconi.it"))
Author: Simone Padoan [cre, aut],
  Gilles Stupfler [aut],
  Carlotta Pacifici [aut]
Maintainer: Simone Padoan <simone.padoan@unibocconi.it>
Imports: evd, copula, mvtnorm, plot3D, tmvtnorm, pracma
Depends: R (>= 3.5.0)
Description: A set of procedures for estimating risks related to extreme events via risk measures such as Expectile, Value-at-Risk, etc. is provided. Estimation methods for univariate independent observations and temporal dependent observations are available. The methodology is extended to the case of independent multidimensional observations.  The statistical inference is performed through parametric and non-parametric estimators. Inferential procedures such as confidence intervals, confidence regions and hypothesis testing are obtained by exploiting the asymptotic theory. Adapts the methodologies derived in Padoan and Stupfler (2022) <doi:10.3150/21-BEJ1375>, Davison et al. (2023) <doi:10.1080/07350015.2022.2078332>, Daouia et al. (2018) <doi:10.1111/rssb.12254>, Drees (2000) <doi:10.1214/aoap/1019487617>, Drees (2003) <doi:10.3150/bj/1066223272>, de Haan and Ferreira (2006) <doi:10.1007/0-387-34471-3>, de Haan et al. (2016) <doi:10.1007/s00780-015-0287-6>, Padoan and Rizzelli (2024) <doi:10.3150/23-BEJ1668>, Daouia et al. (2024) <doi:10.3150/23-BEJ1632>.
License: GPL (>= 2)
URL: https://www.unibocconi.it/en/faculty/simone-padoan
NeedsCompilation: yes
Repository: CRAN
Repository/R-Forge/Project: extremerisks
Encoding: UTF-8
Packaged: 2026-05-11 15:01:16 UTC; padoan
Date/Publication: 2026-05-12 05:10:27 UTC
Built: R 4.6.0; ; 2026-05-12 05:45:37 UTC; unix

Package: ExtremeRisks
Version: 0.0.6
Date: 2026-05-08
Title: Extreme Risk Measures
Authors@R: c(person("Simone","Padoan", role = c("cre", "aut"), email="simone.padoan@unibocconi.it"),
	    person("Gilles", "Stupfler", role = "aut", email="gilles.stupfler@univ-angers.fr"),
	    person("Carlotta", "Pacifici", role = c("aut"), email="carlotta.pacifici@unibocconi.it"))
Author: Simone Padoan [cre, aut],
  Gilles Stupfler [aut],
  Carlotta Pacifici [aut]
Maintainer: Simone Padoan <simone.padoan@unibocconi.it>
Imports: evd, copula, mvtnorm, plot3D, tmvtnorm, pracma
Depends: R (>= 3.5.0)
Description: A set of procedures for estimating risks related to extreme events via risk measures such as Expectile, Value-at-Risk, etc. is provided. Estimation methods for univariate independent observations and temporal dependent observations are available. The methodology is extended to the case of independent multidimensional observations.  The statistical inference is performed through parametric and non-parametric estimators. Inferential procedures such as confidence intervals, confidence regions and hypothesis testing are obtained by exploiting the asymptotic theory. Adapts the methodologies derived in Padoan and Stupfler (2022) <doi:10.3150/21-BEJ1375>, Davison et al. (2023) <doi:10.1080/07350015.2022.2078332>, Daouia et al. (2018) <doi:10.1111/rssb.12254>, Drees (2000) <doi:10.1214/aoap/1019487617>, Drees (2003) <doi:10.3150/bj/1066223272>, de Haan and Ferreira (2006) <doi:10.1007/0-387-34471-3>, de Haan et al. (2016) <doi:10.1007/s00780-015-0287-6>, Padoan and Rizzelli (2024) <doi:10.3150/23-BEJ1668>, Daouia et al. (2024) <doi:10.3150/23-BEJ1632>.
License: GPL (>= 2)
URL: https://www.unibocconi.it/en/faculty/simone-padoan
NeedsCompilation: yes
Repository: CRAN
Repository/R-Forge/Project: extremerisks
Encoding: UTF-8
Packaged: 2026-05-11 15:01:16 UTC; padoan
Date/Publication: 2026-05-12 05:10:27 UTC
Built: R 4.5.2; ; 2026-05-12 06:23:12 UTC; unix

export(MomTailIndex, MLTailIndex, HTailIndex, EBTailIndex, extQuantile, estExpectiles, predExpectiles, expectiles, rtimeseries, estMultiExpectiles, predMultiExpectiles, estExtLevel, QuantMES, ExpectMES, rbtimeseries, HypoTesting, MultiHTailIndex, extMultiQuantile, rmdata,
plotBayes, fitdGPD, estPOT, extBQuant, predDens, predDensx, predQuant, extBQuantx, scedastic.test, schedastic.test, cpost_stat, Bqgpd_c, Bqgpd_d, testTailHomo, quantF)

importFrom("stats", "df", "qf", "dnorm", "dt", "ecdf", "integrate", "optim",
           "pnorm", "pt", "quantile", "rnorm", "runif", "dunif", "uniroot",
           "var", "rpois", "dpois", "rbeta", "dbeta", "dnbinom", "pexp", "qt", 
	   "qchisq", "pbeta", "qnorm", "cov", "rnbinom", "rt", "rbinom", "acf",
	   "dgamma", "rexp", "rgamma", "sd")

importFrom("graphics", "par", "text", "legend", "lines", "points", "curve", "hist")

importFrom("utils", "combn", "setTxtProgressBar", "txtProgressBar")

importFrom("evd", "rbvevd", "abvnonpar", "amvnonpar", "pfrechet", "pgumbel", 
	   "pgev", "abvevd", "hbvevd", "rgpd", "rfrechet", "pgpd", "qgpd", "fpot")

importFrom("mvtnorm", "rmvt", "rmvnorm")

importFrom("tmvtnorm", "rtmvt", "rtmvnorm")

importFrom("plot3D", "scatter3D", "points3D")

importFrom("copula", "rCopula", "claytonCopula", "gumbelCopula", "frankCopula", "normalCopula")

importFrom("pracma", "sqrtm")

importFrom("stats", "median", "pchisq")

library(ExtremeRisks)

#### fitdGPD----
rm(list=ls())
fitdGPD(rpois(1000,2))

#### estPOT----
rm(list=ls())
# generate data
set.seed(1234)
n <- 500
samp <- evd::rfrechet(n,0,3,4)
# set effective sample size and threshold
k <- 50
threshold <- sort(samp,decreasing = TRUE)[k+1]
# preliminary mle estimates of scale and shape parameters
mlest <- evd::fpot(samp, threshold)
# empirical bayes procedure
proc <- estPOT(
  samp,
  k = k,
  pn = c(0.01, 0.005),
  type = "continuous",
  method = "bayesian",
  prior = "empirical",
  start = as.list(mlest$estimate),
  sig0 = 0.1)

#### extBQuant----
rm(list=ls())
# generate data
set.seed(1234)
n <- 500
samp <- evd::rfrechet(500,0,3,4)
# set effective sample size and threshold
k <- 50
threshold <- sort(samp,decreasing = TRUE)[k+1]
# preliminary mle estimates of scale and shape parameters
mlest <- evd::fpot(samp, threshold)
# empirical bayes procedure
proc <- estPOT(
  samp,
  k = k,
  pn = c(0.01, 0.005),
  type = "continuous",
  method = "bayesian",
  prior = "empirical",
  start = as.list(mlest$estimate),
  sig0 = 0.1)
# extreme quantile corresponding to a return period of 100
extBQuant(
  proc$t,proc$post_sample,
  k,
  n,
  100,
  0.05,
  type = "continuous")


#### predDens----
rm(list=ls())
# generate data
set.seed(1234)
n <- 500
samp <- evd::rfrechet(n,0,3,4)
# set effective sample size and threshold
k <- 50
threshold <- sort(samp,decreasing = TRUE)[k+1]
# preliminary mle estimates of scale and shape parameters
mlest <- evd::fpot(samp, threshold)
# empirical bayes procedure
proc <- estPOT(
  samp,
  k = k,
  pn = c(0.01, 0.005),
  type = "continuous",
  method = "bayesian",
  prior = "empirical",
  start = as.list(mlest$estimate),
  sig0 = 0.1)
# predictive density estimation
yg <- seq(0, 50, by = 2)
nyg <- length(yg)
predDens_int <- predDens(
  yg,
  proc$post_sample,
  proc$t,
  "continuous",
  extrapolation = FALSE)
predDens_ext <- predDens(
  yg,
  proc$post_sample,
  proc$t,
  "continuous",
  extrapolation = TRUE,
  p = 0.001,
  k = k,
  n = n)
# plot
plot(
 x = yg,
 y = predDens_int,
 type = "l",
 lwd = 2,
 col = "dodgerblue",
 ylab = "",
 main = "Predictive posterior density")
lines(
 x = yg,
 y = predDens_ext,
 lwd = 2,
 col = "violet")
legend(
  "topright",
  legend = c("Intermediate threshold", "Extreme threshold"),
  lwd = 2,
  col = c("dodgerblue", "violet"))



#### predQuant----
rm(list=ls())
# generate data
set.seed(1234)
n <- 500
samp <- evd::rfrechet(n,0,3,4)
# set effective sample size and threshold
k <- 50
threshold <- sort(samp,decreasing = TRUE)[k+1]
# preliminary mle estimates of scale and shape parameters
mlest <- evd::fpot(samp, threshold)
# empirical bayes procedure
proc <- estPOT(
  samp,
  k = k,
  pn = c(0.01, 0.005),
  type = "continuous",
  method = "bayesian",
  prior = "empirical",
  start = as.list(mlest$estimate),
  sig0 = 0.1)
# predictive density estimation
yg <- seq(0, 50, by = 2)
nyg <- length(yg)
predDens_int <- predDens(
  yg,
  proc$post_sample,proc$t,
  "continuous",
  extrapolation=FALSE)
predQuant_int <- predQuant(
  0.5,
  proc$post_sample,
  proc$t,
  proc$t + 0.01,
  50,
  "continuous",
  extrapolation = FALSE)
predDens_ext <- predDens(
  yg,
  proc$post_sample,
  proc$t,
  "continuous",
  extrapolation = TRUE,
  p = 0.001,
  k = k,
  n = n)
predQuant_ext <- predQuant(
  0.5,
  proc$post_sample,
  proc$t,
  proc$t + 0.01,
  100,
  "continuous",
  extrapolation = TRUE,
  p = 0.005,
  k = k,
  n = n)


#### scedastic.test----
# generate data
rm(list=ls())
# generate data
set.seed(1234)
n <- 500
samp <- evd::rfrechet(n,0,1:n,4)
# set effective sample size and threshold
k <- 50
threshold <- sort(samp,decreasing = TRUE)[k+1]
# preliminary mle estimates of scale and shape parameters
mlest <- evd::fpot(samp,
 threshold,
 control=list(maxit = 500))
# empirical bayes procedure
proc <- estPOT(
  samp,
  k = k,
  pn = c(0.01, 0.005),
  type = "continuous",
  method = "bayesian",
  prior = "empirical",
  start = as.list(mlest$estimate),
  sig0 = 0.1)
# conditional predictive density estimation
yg <- seq(0, 50, by = 2)
nyg <- length(yg)
# estimation of scedasis function
# setting
M <- 1e3
C <- 5
alpha <- 0.05
bw <- .5
nsim <- 5000
burn <- 1000
# create covariate
# in sample obs
n_in = n
# number of years ahead
nY = 1
n_out = 365 * nY
# total obs
n_tot = n_in + n_out
# total covariate (in+out sample period)
x <- seq(0, 1, length = n_tot)
# in sample grid dimension for covariate
ng_in <- 150
xg <- seq(0, x[n_in], length = ng_in)
# in+out of sample grid
xg <- c(xg,
 seq(x[n_in + 1],
     x[(n_tot)],
     length = ng_in))
# in+out sample grid dimension
nxg <- length(xg)
xg <- array(xg, c(nxg, 1))
# in sample observations
samp_in <- samp[1:n_in]
ssamp_in <- sort(samp_in, decreasing = TRUE, index = TRUE)
x_in <- x[1:n_in] # in sample covariate
xs <- x_in[ssamp_in$ix[1:k]] # in sample concomitant covariate
# test on covariate effect
test <- scedastic.test(
  cbind(samp, x[1:n]),
  k,
  M,
  array(xg[1:ng_in], c(ng_in, 1)),
  ng_in,
  TRUE,
  C,
  0.05
)


#### cpost_stat----
rm(list=ls())
# generate data
set.seed(1234)
n <- 500
samp <- evd::rfrechet(n, 0, 1:n, 4)
# set effective sample size and threshold
k <- 50
threshold <- sort(samp, decreasing = TRUE)[k+1]
# preliminary mle estimates of scale and shape parameters
mlest <- evd::fpot(
  samp,
  threshold,
  control = list(maxit = 500))
# empirical bayes procedure
proc <- estPOT(
  samp,
  k = k,
  pn = c(0.01, 0.005),
  type = "continuous",
  method = "bayesian",
  prior = "empirical",
  start = as.list(mlest$estimate),
  sig0 = 0.1)
# conditional predictive density estimation
yg <- seq(0, 50, by = 2)
nyg <- length(yg)
# estimation of scedasis function
# setting
M <- 1e3
C <- 5
alpha <- 0.05
bw <- .5
nsim <- 5000
burn <- 1000
# create covariate
# in sample obs
n_in = n
# number of years ahead
nY = 1
n_out = 365 * nY
# total obs
n_tot = n_in + n_out
# total covariate (in+out sample period)
x <- seq(0, 1, length = n_tot)
# in sample grid dimension for covariate
ng_in <- 150
xg <- seq(0, x[n_in], length = ng_in)
# in+out of sample grid
xg <- c(xg, seq(x[n_in + 1], x[(n_tot)], length = ng_in))
# in+out sample grid dimension
nxg <- length(xg)
xg <- array(xg, c(nxg, 1))
# in sample observations
samp_in <- samp[1:n_in]
ssamp_in <- sort(samp_in, decreasing = TRUE, index = TRUE)
x_in <- x[1:n_in] # in sample covariate
xs <- x_in[ssamp_in$ix[1:k]]
# in sample concomitant covariate
# estimate scedasis function over the in and out of sample period
res_stat <- apply(
  xg,
  1,
  cpost_stat,
  N = nsim - burn,
  x = x_in,
  xs = xs,
  bw = bw,
  k = k,
  C = C
)




#### extBQuantx----
rm(list=ls())
# generate data# generate data
set.seed(1234)
n <- 500
samp <- evd::rfrechet(n,0,1:n,4)
# set effective sample size and threshold
k <- 50
threshold <- sort(samp,decreasing = TRUE)[k+1]
# preliminary mle estimates of scale and shape parameters
mlest <- evd::fpot(
  samp,
  threshold,
  control = list(maxit = 500))
# empirical bayes procedure
proc <- estPOT(
  samp,
  k = k,
  pn = c(0.01, 0.005),
  type = "continuous",
  method = "bayesian",
  prior = "empirical",
  start = as.list(mlest$estimate),
  sig0 = 0.1)
# conditional predictive density estimation
yg <- seq(0, 50, by = 2)
nyg <- length(yg)
# estimation of scedasis function
# setting
M <- 1e3
C <- 5
alpha <- 0.05
bw <- .5
nsim <- 5000
burn <- 1000
# create covariate
# in sample obs
n_in = n
# number of years ahead
nY = 1
n_out = 365 * nY
# total obs
n_tot = n_in + n_out
# total covariate (in+out sample period)
x <- seq(0, 1, length = n_tot)
# in sample grid dimension for covariate
ng_in <- 150
xg <- seq(0, x[n_in], length = ng_in)
# in+out of sample grid
xg <- c(xg, seq(x[n_in + 1], x[(n_tot)], length = ng_in))
# in+out sample grid dimension
nxg <- length(xg)
xg <- array(xg, c(nxg, 1))
# in sample observations
samp_in <- samp[1:n_in]
ssamp_in <- sort(samp_in, decreasing = TRUE, index = TRUE)
x_in <- x[1:n_in] # in sample covariate
xs <- x_in[ssamp_in$ix[1:k]] # in sample concomitant covariate
# estimate scedasis function over the in and out of sample period
res_stat <- apply(
  xg,
  1,
  cpost_stat,
  N = nsim - burn,
  x = x_in,
  xs = xs,
  bw = bw,
  k = k,
  C = C
)
# conditional predictive posterior density
probq = 1 - 0.99
res_AQ <- extBQuantx(
 cx = res_stat,
 postsamp = proc$post_sample,
 threshold = proc$t,
 n = n,
 qlev = probq,
 k = k,
 type = "continuous",
 confint = "asymmetric")




#### predDensx----
rm(list=ls())
# generate data
set.seed(1234)
n <- 500
samp <- evd::rfrechet(n,0,1:n,4)
# set effective sample size and threshold
k <- 50
threshold <- sort(samp,decreasing = TRUE)[k+1]
# preliminary mle estimates of scale and shape parameters
mlest <- evd::fpot(samp, threshold, control=list(maxit = 500))
# empirical bayes procedure
proc <- estPOT(
  samp,
  k = k,
  pn = c(0.01, 0.005),
  type = "continuous",
  method = "bayesian",
  prior = "empirical",
  start = as.list(mlest$estimate),
  sig0 = 0.1)
# conditional predictive density estimation
yg <- seq(0, 50, by = 2)
nyg <- length(yg)
# estimation of scedasis function
# setting
M <- 1e3
C <- 5
alpha <- 0.05
bw <- .5
nsim <- 5000
burn <- 1000
# create covariate
# in sample obs
n_in = n
# number of years ahead
nY = 1
n_out = 365 * nY
# total obs
n_tot = n_in + n_out
# total covariate (in+out sample period)
x <- seq(0, 1, length = n_tot)
# in sample grid dimension for covariate
ng_in <- 150
xg <- seq(0, x[n_in], length = ng_in)
# in+out of sample grid
xg <- c(xg, seq(x[n_in + 1], x[(n_tot)], length = ng_in))
# in+out sample grid dimension
nxg <- length(xg)
xg <- array(xg, c(nxg, 1))
# in sample observations
samp_in <- samp[1:n_in]
ssamp_in <- sort(samp_in, decreasing = TRUE, index = TRUE)
x_in <- x[1:n_in] # in sample covariate
xs <- x_in[ssamp_in$ix[1:k]] # in sample concomitant covariate
# estimate scedasis function over the in and out of sample period
res_stat <- apply(
  xg,
  1,
  cpost_stat,
  N = nsim - burn,
  x = x_in,
  xs = xs,
  bw = bw,
  k = k,
  C = C
)
# conditional predictive posterior density
yg <- seq(500, 5000, by = 100)
nyg = length(yg)
# intermediate threshold
predDens_intx <- predDensx(
  x = yg,
  postsamp = proc$post_sample,
  scedasis = res_stat,
  threshold = proc$t,
  "continuous",
  extrapolation = FALSE)
# extreme threshold
predDens_extx <- predDensx(
  x = yg,
  postsamp = proc$post_sample,
  scedasis = res_stat,
  threshold = proc$t,
  "continuous",
  extrapolation = TRUE,
  p = 0.001,
  k = k,
  n = n)
# plot intermediate and extreme density conditional on a specific value of scedasis function
# disclaimer: to speed up the procedure, we specify a coarse grid
plot(
 x = predDens_intx$x,
 y = predDens_intx$preddens[,20],
 type = "l",
 lwd = 2,
 col="dodgerblue",
 ylab = "",
 xlab = "yg",
 main = "Conditional predictive posterior density")
lines(
  x = predDens_extx$x,
  y = predDens_extx$preddens[,20],
  lwd = 2,
  col = "violet")
legend("topright",
  legend = c("Intermediate threshold","Extreme threshold"),
  lwd = 2,
  col = c("dodgerblue", "violet"))






#### quantF----
rm(list=ls())
# generate data
set.seed(1234)
n <- 500
samp <- evd::rfrechet(n,0,1:n,4)
# set effective sample size and threshold
k <- 50
threshold <- sort(samp,decreasing = TRUE)[k+1]
# preliminary mle estimates of scale and shape parameters
mlest <- evd::fpot(samp, threshold, control=list(maxit = 500))
# empirical bayes procedure
proc <- estPOT(
  samp,
  k = k,
  pn = c(0.01, 0.005),
  type = "continuous",
  method = "bayesian",
  prior = "empirical",
  start = as.list(mlest$estimate),
  sig0 = 0.1)
# conditional predictive density estimation
yg <- seq(0, 50, by = 2)
nyg <- length(yg)
# estimation of scedasis function
# setting
M <- 1e3
C <- 5
alpha <- 0.05
bw <- .5
nsim <- 5000
burn <- 1000
# create covariate
# in sample obs
n_in = n
# number of years ahead
nY = 1
n_out = 365 * nY
# total obs
n_tot = n_in + n_out
# total covariate (in+out sample period)
x <- seq(0, 1, length = n_tot)
# in sample grid dimension for covariate
ng_in <- 150
xg <- seq(0, x[n_in], length = ng_in)
# in+out of sample grid
xg <- c(xg, seq(x[n_in + 1], x[(n_tot)], length = ng_in))
# in+out sample grid dimension
nxg <- length(xg)
xg <- array(xg, c(nxg, 1))
# in sample observations
samp_in <- samp[1:n_in]
ssamp_in <- sort(samp_in, decreasing = TRUE, index = TRUE)
x_in <- x[1:n_in] # in sample covariate
xs <- x_in[ssamp_in$ix[1:k]] # in sample concomitant covariate
# estimate scedasis function over the in and out of sample period
res_stat <- apply(
  xg,
  1,
  cpost_stat,
  N = nsim - burn,
  x = x_in,
  xs = xs,
  bw = bw,
  k = k,
  C = C
)
# conditional predictive posterior density
yg <- seq(500, 5000, by = 100)
nyg = length(yg)
# intermediate threshold
predDens_intx <- predDensx(
  x = yg,
  postsamp = proc$post_sample,
  scedasis = res_stat,
  threshold = proc$t,
  "continuous",
  extrapolation = FALSE)
# extreme threshold
predDens_extx <- predDensx(
  x = yg,
  postsamp = proc$post_sample,
  scedasis = res_stat,
  threshold = proc$t,
  "continuous",
  extrapolation = TRUE,
  p = 0.001,
  k = k,
  n = n)
  # predictive conditional quantiles
predQuant_intxL <- predQuant_intxU <- predQuant_extxL <- predQuant_extxU <- numeric()
for (i in 1:nxg){
  predQuant_intxU[i] <- apply(
    array(predDens_intx$preddens[,i], c(1, nyg)),
    1,
    quantF,
    x = yg,
    p = c(0.975)
  )
  predQuant_intxL[i] <- apply(
    array(predDens_intx$preddens[,i], c(1, nyg)),
    1,
    quantF,
    x = yg,
    p = c(0.025)
  )
  predQuant_extxU[i] <- apply(
    array(predDens_extx$preddens[,i], c(1, nyg)),
    1,
    quantF,
    x = yg,
    p = c(0.975)
  )
  predQuant_extxL[i] <- apply(
    array(predDens_extx$preddens[,i], c(1, nyg)),
    1,
    quantF,
    x = yg,
    p = c(0.025)
  )
}





#### testTailHomo----
rm(list=ls())
# generate two samples of data
set.seed(1234)
y1 <- evd::rgpd(500, 0, 1, 0.2)
y2 <- evd::rgpd(700, 0, 2, 0.7)
y <- list(y1 = y1, y2 = y2)
# set effective sample size
k <- 50
# perform test
test <- testTailHomo(y,k)





#### plotBayes----
rm(list=ls())
# generate data
set.seed(1234)
n <- 500
samp <- evd::rfrechet(n, 0, 3, 4)
# set effective sample size and threshold
k <- 50
threshold <- sort(samp, decreasing = TRUE)[k+1]
# preliminary mle estimates of scale and shape parameters
mlest <- evd::fpot(samp, threshold)
# empirical bayes procedure
proc <- estPOT(
 samp,
 k = k,
 pn = c(0.01, 0.005),
 type = "continuous",
 method = "bayesian",
 prior = "empirical",
 start = as.list(mlest$estimate),
 sig0 = 0.1)
plotBayes(
  proc$post_sample[,1],
  mlest$estimate,
  param = "scale")
plotBayes(
  proc$post_sample[,2],
  param = "shape")

#########################################################
### Authors: Simone Padoan and Gilles Stuplfer        ###
### Emails: simone.padoan@unibocconi.it               ###
### gilles.stupfler@ensai.fr                          ###
### Institution: Department of Decision Sciences,     ###
### University Bocconi of Milan, Italy,               ###
### Ecole Nationale de la Statistique et de           ###
### l'Analyse de l'Information, France                ###
### File name: Expectiles.r                           ###
### Description:                                      ###
### This file contains a set of routines              ###
### to compute the theoretical expectiles             ###
### for different families of parametric models       ###
### for i.i.d. and serial dependent observations      ###
### Last change: 2020-02-28                           ###
#########################################################

#########################################################
### START
### Sub-routines (HIDDEN FUNCTIONS)
###
#########################################################

# Compute the cost function for the GPD distribution
gpdcostfun <- function(x, scale, shape, tau){
  mu <- scale/(1-shape)
  pmx <- pmean(x,scale,shape)
  pgx <- pgpd(x,scale=scale,shape=shape)

  ppx <- pmx - x*pgx
  res <- ppx/(2*ppx + (x-mu))

  return(res-tau)
}
# Compute the wighted mean
pmean <- function(x, scale, shape){
  y <- scale * pgpd(x,scale=scale,shape=shape)/(1-shape) - x*(1-pgpd(x,scale=scale,shape=shape))/(1-shape)
  return(y)
}

# Approximate the true expectile via Monte Carlo approximation

# 1) Case: IID data from parametric families
# Frechet
expect_frec_iid <- function(ndata, tau, scale, shape, estMethod){
  data <- rfrechet(ndata, scale=scale, shape=shape)
  res <- estExpectiles(data, tau, estMethod)$ExpctHat
  return(res)
}
# Pareto
expect_par_iid <- function(ndata, tau, shape, estMethod){
  data <- (rgpd(ndata, scale=1, loc=1, shape=1))^(1/shape)
  res <- estExpectiles(data, tau, estMethod)$ExpctHat
  return(res)
}
# GPD
expect_gpd_iid <- function(ndata, tau, scale, shape, estMethod){
  data <- rgpd(ndata, scale=scale, shape=shape)
  res <- estExpectiles(data, tau, estMethod)$ExpctHat
  return(res)
}
# student-t
expect_std_iid <- function(ndata, tau, df, estMethod){
  data <- rt(ndata, df)
  res <- estExpectiles(data, tau, estMethod)$ExpctHat
  return(res)
}

# 2) Case: time serie from parametric families
expect_std_ts <- function(ndata, burnin, corr, df, smooth, nsim, tau, estMethod){
  data <- rep(0, nsim)
  z <- sqrt(1-corr^2) * rt(nsim, df)
  for(i in 1:(nsim-1)) data[i+1] <- corr*data[i] + smooth * z[i] + z[i+1]
  res <- estExpectiles(data[(burnin+1):ndata], tau, estMethod)$ExpctHat
 return(res)
}

# double Frechet innovations
expect_dfrec_ts <- function(ndata, burnin, corr, scale, shape, smooth, nsim, tau, estMethod){
  data <- rep(0, nsim)
  z <- sqrt(1-corr^2) * rdoublefrechet(nsim, scale=scale, shape=shape)
  for(i in 1:(nsim-1)) data[i+1] <- corr*data[i] + smooth * z[i] + z[i+1]
  res <- estExpectiles(data[(burnin+1):ndata], tau, estMethod)$ExpctHat
  return(res)
}

# double Pareto innovations
expect_dpareto_ts <- function(ndata, burnin, corr, scale, shape, smooth, nsim, tau, estMethod){
  data <- rep(0, nsim)
  z <- sqrt(1-corr^2) * rdoublepareto(nsim, scale=scale, shape=1/shape)
  for(i in 1:(nsim-1)) data[i+1] <- corr*data[i] + smooth * z[i] + z[i+1]
  res <- estExpectiles(data[(burnin+1):ndata], tau, estMethod)$ExpctHat
  return(res)
}

# maxima auto-regressive with Frechet innovations
expect_armax_ts <- function(ndata, burnin, corr, scale, shape, nsim, tau, estMethod){
  data <- rep(0, nsim)
  z <- (1-corr)^(1/shape) * rfrechet(nsim, scale=scale, shape=shape)
  for(i in 1:(nsim-1)) data[i+1] <- max((corr)^(1/shape) * data[i],  z[i])
  res <- estExpectiles(data[(burnin+1):ndata], tau, estMethod)$ExpctHat
  return(res)
}

# GARCH model
expect_garch_ts <- function(ndata, burnin, alpha0, alpha, beta, tau, estMethod){
  nsim <- ndata+burnin
  data <- rep(0, nsim)
  sigma <- rep(0, nsim)
  z <- rnorm(nsim)
  for(i in 1:(nsim-1)){
    sigma[i+1] <- sqrt(alpha[1] + alpha[2] * data[i]^2 + beta * sigma[i]^2)
    data[i+1] <- sigma[i+1] * z[i]
  }
  data <- data[(burnin+1):nsim]
  res <- estExpectiles(data, tau, estMethod)$ExpctHat
  return(res)
}

#########################################################
### END
### Sub-routines (HIDDEN FUNCTIONS)
###
#########################################################


#########################################################
### START
### Main-routines (VISIBLE FUNCTIONS)
###
#########################################################

expectiles <- function(par, tau, tsDist="gPareto", tsType="IID", trueMethod="true",
                       estMethod="LAWS", nrep=1e+05, ndata=1e+06, burnin=1e+03){
  # main body
  # Check whether the distribution in input is available
  if(!(tsDist %in% c("studentT", "Frechet", "Pareto", "gPareto", "double-Frechet",
                     "double-Pareto", "Gaussian"))) stop("insert the correct DISTRIBUTION!")
  # Set output
  res <- NULL
  if(trueMethod=="true"){
    # STANDARD PARETO DISTRIBUTION
    if (tsDist == "Pareto"){
      shape <- 1/par[1]
      if (tau < 0.5){
        low <- 0
        up <- 1/(1 - shape)
      }
      else{
        low <- 1/(1 - shape)
        up <- 10*(1/shape-1)^(-shape) * ((1-tau)^(-shape)-1)/shape
      }
        res <- 1 + shape * uniroot(gpdcostfun, c(low, up), scale = 1,
                                   shape = shape, tau = tau)$root
    }
    # GENERALISED PARETO DISTRIBUTION
    if(tsDist=="gPareto"){
      # THEORETICAL EXPECTILES FOR PARETO MARGINAL DISTIBUTION
      # SEE JONES'S PAPER FORMULA 1.1
      scale <- par[1]
      shape <- par[2]
      if(tau<0.5){
        low <- 0
        up <- scale/(1-shape)
      }
      else {
        low <- scale/(1-shape)
        up <- 10*(1/shape-1)^(-shape) * scale*((1-tau)^(-shape)-1)/shape
      }
      res <- uniroot(gpdcostfun, c(low,up), scale=scale, shape=shape, tau=tau)$root
    }
    # NON-CENTERED STUDENT-T DISTRIBUTION
    if(tsDist=="studentT"){
      df <- par[1]
      res <- (df-1)^(-1/df) * qt(tau,df=df)
    }
  }
  if(trueMethod=="approx"){
    if(tsDist=="studentT"){
      # Stationary auto-regressive time series with iid student-t innovations
      if(tsType=="AR"){
        corr <- par[1]
        df <- par[2]
        smooth <- 0
        nsim <- ndata+burnin
        res <- replicate(nrep,expect_std_ts(ndata, burnin, corr, df, smooth, nsim, tau, estMethod))
        res <- mean(res)
      }
      if(tsType=="ARMA"){
        corr <- par[1]
        df <- par[2]
        smooth <- par[3]
        nsim <- ndata+burnin
        res <- replicate(nrep,expect_std_ts(ndata, burnin, corr, df, smooth, nsim, tau, estMethod))
        res <- mean(res)
      }
      # Stationary time series with iid student-t innovations
      if(tsType=="IID"){
        df <- par[1]
        res <- replicate(nrep, expect_std_iid(ndata, tau, df, estMethod))
        res <- mean(res)
      }
    }
    if(tsDist=="double-Frechet"){
      # Stationary auto-regressive time series with double-frechet iid innovations
      if(tsType=="AR"){
        corr <- par[1]
        scale <- par[2]
        shape <- par[3]
        smooth <-0
        nsim <- ndata+burnin
        res <- replicate(nrep,expect_dfrec_ts(ndata, burnin, corr, scale, shape, smooth, nsim, tau, estMethod))
        res <- mean(res)
      }
      if(tsType=="ARMA"){
        corr <- par[1]
        scale <- par[2]
        shape <- par[3]
        smooth <-par[4]
        nsim <- ndata+burnin
        res <- replicate(nrep,expect_dfrec_ts(ndata, burnin, corr, scale, shape, smooth, nsim, tau, estMethod))
        res <- mean(res)
      }
    }
    if(tsDist=="double-Pareto"){
      # Stationary auto-regressive time series with double-frechet iid innovations
      if(tsType=="AR"){
        corr <- par[1]
        scale <- par[2]
        shape <- par[3]
        smooth <- 0
        nsim <- ndata+burnin
        res <- replicate(nrep,expect_dpareto_ts(ndata, burnin, corr, scale, shape, smooth, nsim, tau, estMethod))
        res <- mean(res)
      }
      if(tsType=="ARMA"){
        corr <- par[1]
        scale <- par[2]
        shape <- par[3]
        smooth <- par[4]
        nsim <- ndata+burnin
        res <- replicate(nrep,expect_dpareto_ts(ndata, burnin, corr, scale, shape, smooth, nsim, tau, estMethod))
        res <- mean(res)
      }
    }
    if(tsDist=="Frechet"){
      # Stationary auto-regressione time series with maxima innovations Frechet distributed
      if(tsType=="ARMAX"){
        corr <- par[1]
        scale <- par[2]
        shape <- par[3]
        nsim <- ndata+burnin
        res <- replicate(nrep,expect_armax_ts(ndata, burnin, corr, scale, shape, nsim, tau, estMethod))
        res <- mean(res)
      }
      # Stationary time series with iid components Frechet distributed
      if(tsType=="IID"){
        scale <- par[1]
        shape <- par[2]
        res <- replicate(nrep, expect_frec_iid(ndata, tau, scale, shape, estMethod))
        res <- mean(res)
      }
    }
    # Stationary time series with iid components Generalised Pareto distributed
    if(tsDist=="gPareto"){
      if(tsType=="IID") {
        scale <- par[1]
        shape <- par[2]
        res <- replicate(nrep, expect_gpd_iid(ndata, tau, scale, shape, estMethod))
        res <- mean(res)
    }}
    if(tsDist=="Pareto"){
      if(tsType=="IID"){
        shape <- par[1]
        res <- replicate(nrep, expect_par_iid(ndata, tau, shape, estMethod))
        res <- mean(res)
    }}
    if(tsDist=="Gaussian"){
      if(tsType=="GARCH"){
        alpha <- par[1:2]
        beta <- par[3]
        res <- replicate(nrep, expect_garch_ts(ndata, burnin, alpha, beta, tau, estMethod))
        res <- mean(res)
      }
    }
  }
  return(res)
}

#########################################################
### END
### Main-routines (VISIBLE FUNCTIONS)
###
#########################################################

#########################################################
### Authors: Simone Padoan and Gilles Stuplfer        ###
### Emails: simone.padoan@unibocconi.it               ###
### gilles.stupfler@ensai.fr                          ###
### Institution: Department of Decision Sciences,     ###
### University Bocconi of Milan, Italy,               ###
### Ecole Nationale de la Statistique et de           ###
### l'Analyse de l'Information, France                ###
### File name: MultiEstimation.r                      ###
### Description:                                      ###
### This file contains a set of procedures            ###
### for estimating (point and interval estimation)    ###
### risk measures as Value-At-Risk and Expectile      ###
### for i.i.d. and dependent data for the             ###
### multivariate case                                 ###
### Last change: 2020-07-13                           ###
#########################################################


#########################################################
### START
### Sub-routines (HIDDEN FUNCTIONS)
###
#########################################################

################################################################################
###
### Computation of the ellipse corresponding to a bivariate normal
### distribution, provided the mean vector and the 2x2
### variance-covariance matrix
###
################################################################################
ellipse <- function(mean=c(0,0), varcov=matrix(c(1,0,0,1),2,2), alpha=0.05, ngrid=250)
{
  ##############################################################################
  # 1) alpha in (0,1) such that (1-alpha) defines the confidence level
  # 2) mean defines the center of the multivariate normal
  # 3) varcov defines the variance-covariance of the multivariate normal
  # 4) ngrid gives the number of points to use for the graphical of the ellipse
  ##############################################################################
  # defines the Mahalanobis distance
  d <- sqrt(qchisq(1-alpha, 2))
  # defines the angles
  theta <- seq(0, 2*pi, length.out=ngrid)
  # computes a circle of radius d
  x <- d * cbind(cos(theta), sin(theta))
  # Spectral decomposition of a variance-covariance matrix
  eig <- eigen(varcov)
  # combines the magnitudes (scaling) and the rotation associated to varcov
  sp <- eig$vec %*% diag(sqrt(eig$values))
  # computes coordinates of the ellipse
  ell <- t(mean - (sp %*% t(x)))
  return(ell)
}

################################################################################
###
### Computation of the ellipsoid corresponding to a trivariate normal
### distribution, provided the mean vector and the 3x3
### variance-covariance matrix
###
################################################################################
ellipsoid <- function(mean=c(0,0,0), varcov=matrix(c(1,0,0,0,1,0,0,0,1),3,3),
                      alpha=0.05, ngrid=60)
{
  ##############################################################################
  # 1) alpha in (0,1) such that (1-alpha) defines the confidence level
  # 2) mean defines the center of the multivariate normal
  # 3) varcov defines the variance-covariance of the multivariate normal
  # 4) ngrid gives the number of points to use for the graphical of the ellipsoid
  ##############################################################################
  # defines subroutine
  EucCoord <- function(x) return(c(cos(x[1])*sin(x[2]), sin(x[1])*sin(x[2]), cos(x[2])))
  # defines the Mahalanobis distance
  d <- sqrt(qchisq(1-alpha, 2))
  # defines the angles
  theta <- seq(0, 2*pi, length.out=ngrid)
  # computes a sphere of radius d
  x <- d * apply(expand.grid(theta,theta), 1, EucCoord)
  # Eigendecomposition of a variance-covariance matrix
  eig <- eigen(varcov)
  # combines the magnitude (scaling) and the rotation associated to varcov
  sp <- eig$vec %*% diag(sqrt(eig$values))
  # computes coordinates of the ellipse
  ell <- t(mean - (sp %*% x))
  return(ell)
}

################################################################################
###
### Inspection whether a values falls inside the ellipsoid (ellipse)
### for a given the mean vector and the 2x2
### variance-covariance matrix
###
################################################################################
isInsideEllipse  <- function(dim=2, varcov=diag(1, dim), alpha=0.05, est)
{
  res <- FALSE
  # inverse square root of covariance matrix
  invSqrtVarCov <- sqrtm(varcov, kmax = 20, tol = .Machine$double.eps^(1/2))$Binv
  # transformation to standard normal
  # est is the log of the estimate divided by true value or the relative estimate
  Z <- invSqrtVarCov %*% est
  S <- sum(Z^2, na.rm=TRUE)
  if(is.na(S)) return(NA)
  # checks if this is within the relevant ball
  if (S <= qchisq(1 - alpha, dim)) res <- TRUE
  return(res)
}

# empirical joint exceeding probability
empTailProb <- function(data, x, ndata){
  return(sum(data[,1] > x[1] & data[,2] > x[2])/ndata)
}

# pairwise empirical joint exceeding probability
empPairCov <- function(indx, data, x, stDev, ndata){
  return(empTailProb(data[, indx], x, ndata) * stDev[indx[1]]* stDev[indx[2]])
}

estCovExpInt <- function(data, ndata, gamma, c, k){
  rdata <- ndata * data / k
  I <- rdata[,1] <= c[1] & rdata[,2] <= c[2]
  res <- sum(((rdata[,1] / c[1])^(-gamma[1]) - 1) *
            ((rdata[,2] / c[2])^(-gamma[2]) - 1) * I) / k

  return(res)
}

estPairCovExpInt <- function(indx, data, ndata, gamma, c, k){
  return(estCovExpInt(data[,indx], ndata, gamma[indx], c[indx], k))
}

intLeftEmpTailCopula <- function(data, ndata , k){
  rdata <- ndata * data / k
  I <- rdata[,1] <= 1 & rdata[,2] <= 1
  res <- sum(log(1/rdata[,1]) * I) / k
  return(res)
}

estPairLeftETC <- function(indx, data, ndata, k){
  return(intLeftEmpTailCopula(data[,indx], ndata, k))
}

intRightEmpTailCopula <- function(data, ndata , k){
  rdata <- ndata * data / k
  I <- rdata[,1] <= 1 & rdata[,2] <= 1
  res <- sum(log(1/rdata[,2]) * I) / k
  return(res)
}

estPairRightETC <- function(indx, data, ndata, k){
  return(intRightEmpTailCopula(data[,indx], ndata, k))
}

intEmpTailCopula <- function(data, ndata , k){
  rdata <- ndata * data / k
  I <- rdata[,1] <= 1 & rdata[,2] <= 1
  res <- sum(I) / k
  return(res)
}

estPairETC <- function(indx, data, ndata, k){
  return(intEmpTailCopula(data[,indx], ndata, k))
}

estAsymCov <- function(data, k, stDev){
  # derives dimensions
  dimData <- dim(data)
  # computes marginal empirical distribution functions
  mecdf <- apply(data,2,ecdf)
  # defines ranks
  rdata <- NULL
  for(i in 1:dimData[2]) rdata <- cbind(rdata, mecdf[[i]](data[,i]))
  # defines the pairwise combinations
  indx <- t(combn(dimData[2],2))
  # defines threhold
  x <- c(1-k/dimData[1], 1-k/dimData[1])
  # computes approximate tail copula
  res <- apply(indx, 1, empPairCov, data=rdata, x=x, stDev=stDev, ndata=dimData[1])

  return(dimData[1] * res / k)
}

################################################################################
###
### Estimation of the multivariate expectile estimator variance-covariance
### matrix @ the intermediate level
### given a sample of iid high-dimnensional observations
###
################################################################################
estMultiExpectVar <- function(data, dimData, ExpctHat, gammaHat, method, k, tau,
                              VarEHat, varType){
  # main body:
  # initialization of important quantities
  CovEHat <- NULL
  VarCovEHat <- NULL
  # A) computes the variance diagonal terms for the variance-covariance matrix
  # for the expectile estimator
  VarCovEHat <- diag(VarEHat)
  # a.1) computes the number of distinct pairs
  indx <- t(combn(dimData[2],2))
  # B) computes the off-diagonal covariance terms of the variance-covariance
  #    matrix for the expectile estimator
  # B.1) consider the independence case
  if(varType %in% c("asym-Ind", "asym-Ind-Rel")) CovEHat <- rep(0, nrow(indx))
  # B.2) computes the off-diagonal covariance of the LAWS estimator
  #      using the proposed adjustment based on dependence observations
  if(varType %in% c("asym-Ind-Adj", "asym-Ind-Adj-Rel") & method=="LAWS"){
    # B.2.1) computes the empirical marginals asymmetric cost functions
    cdata <- data - apply(array(ExpctHat, c(1,dimData[2])), 2, rep, times=dimData[1])
    mij <- apply(cdata, 2, acostfun, tau=tau, u=0)
    # B.2.2) computes the pairwise empirical asymmetric marginals cost functions
    mijkl <- apply(indx, 1, pacostfun, data=cdata, tau=tau, u=0)
    # B.2.3) computes the covariance terms
    CovEHat <- gammaHat[indx[,1]] * gammaHat[indx[,2]] * (mijkl - mij[indx[,1]] * mij[indx[,2]]) /
    ((1-tau) * ExpctHat[indx[,1]] * ExpctHat[indx[,2]])
  }
  # B.3) computes the off-diagonal covariance of the QB estimator
  #      using the proposed adjustment based on dependence observations
  if(varType %in% c("asym-Ind-Adj", "asym-Ind-Adj-Rel") & method=="QB"){
    rdata <- NULL
    # B.3.1) computes marginal emprical distribution functions
    mecdf <- apply(data, 2, ecdf)
    # B.3.2) computes marginal empirical survival functions
    for(i in 1:dimData[2])
      rdata <- cbind(rdata, 1 - dimData[1] * mecdf[[i]](data[, i]) / (dimData[1] + 1))
    mg <- 1 / (1 - gammaHat) - log(1 / gammaHat - 1)
    # B.3.3) computes the covariance terms
    R11 <- apply(indx, 1, estPairETC, data=rdata, ndata=dimData[1],k=k)
    Rj1 <- apply(indx, 1, estPairLeftETC, data=rdata, ndata=dimData[1],k=k)
    R1l <- apply(indx, 1, estPairRightETC, data=rdata, ndata=dimData[1],k=k)
    # B.3.4) covariance of the normalised estimator
    CovEHat <- gammaHat[indx[,1]] * gammaHat[indx[,2]] * (R11 * (1 - mg[indx[,1]]) *
               (1 - mg[indx[,2]]) + mg[indx[,1]] * Rj1 + mg[indx[,2]] * R1l)
  }
  # C) fills in the variance-covariance matrix
  VarCovEHat[lower.tri(VarCovEHat)] <- CovEHat
  VarCovEHat <- t(VarCovEHat)
  VarCovEHat[lower.tri(VarCovEHat)] <- CovEHat
  # C.1) with the relative estimator the variance-covariance matrix needs
  # to be rescaled by the expectile estimator
  if(regexpr("Rel", varType, fixed=TRUE)[1]==-1){
    diag(VarCovEHat) <- ExpctHat^2 * VarEHat
    CovEHat <- ExpctHat[indx[,1]] * ExpctHat[indx[,2]] * CovEHat
    VarCovEHat[lower.tri(VarCovEHat)] <- CovEHat
    VarCovEHat <- t(VarCovEHat)
    VarCovEHat[lower.tri(VarCovEHat)] <- CovEHat
  }
  # C.2) Normalised variance-covariance matrix
  VarCovEHat <- VarCovEHat / k
  return(VarCovEHat)
}

################################################################################
###
### Estimation of the multivariate expectile estimator variance-covariance
### matrix @ the extreme level
### given a sample of iid high-dimnensional observations
###
################################################################################
predMultiExpectVar <- function(data, dimData, ExpctHat, gammaHat, method, k,
                                tau, tau1, VarEHat, varType){
  # main body:
  # initialization of important quantities
  CovEHat <- NULL
  VarCovEHat <- NULL
  # scaling factor
  rn <- log((1-tau)/(1-tau1))
  # A) computes diagonal variance terms of the variance-covariance matrix
  # of the expectile estimator
  VarCovEHat <- diag(VarEHat)
  # A.1) computes the number of distinct pairs
  indx <- t(combn(dimData[2],2))
  # B) computes the off-diagonal covariance terms of the variance-covariance
  #    matrix for the expectile estimator
  # B.1) consider the independence case
  if(varType %in% c("asym-Ind", "asym-Ind-Log")) CovEHat <- rep(0, nrow(indx))
  # B.2) computes the off-diagonal covariance of the LAWS estimator
  #      using the proposed adjustment based on dependence observations
  if(varType %in% c("asym-Ind-Adj", "asym-Ind-Adj-Log") & method=="LAWS"){
    rdata <- NULL
    I <- NULL
    # B.2.1) computes the marginal empirical distribution functions
    mecdf <- apply(data, 2, ecdf)
    # B.2.2) computes the marginal empirical quantile functions
    qHat <- apply(data, 2, quantile, probs=tau)
    for(i in 1:dimData[2]){
      # B.2.3) computes the ranks
      rdata <- cbind(rdata, 1 - dimData[1] * mecdf[[i]](data[, i]) / (dimData[1] + 1))
      I <- cbind(I, data[, i] > qHat[i])
    }
    # B.2.4) computes the empirical asymmetric marginals cost functions
    cdata <- data - apply(array(ExpctHat, c(1,dimData[2])), 2, rep, times=dimData[1])
    mij <- apply(cdata, 2, acostfun, tau=tau, u=0)
    # B.2.5) computes the pairwise asymmetric marginals cost functions
    mijkl <- apply(indx, 1, pacostfun, data=cdata, tau=tau, u=0)
    # B.2.6) computes the off diagonal covariance terms @ the intermediate case
    CovEHatInt <- gammaHat[indx[,1]] * gammaHat[indx[,2]] * (mijkl - mij[indx[,1]] * mij[indx[,2]]) /
    ((1-tau) * ExpctHat[indx[,1]] * ExpctHat[indx[,2]])
    # B.2.7) computes the tail dependence copula
    R11 <- apply(indx, 1, estPairETC, data=rdata, ndata=dimData[1], k=k)
    # B.2.8) computes the Sigma matrices
    varphi <- apply(cdata, 2, varfun, tau=tau, u=0)
    S12 <- S21 <- NULL
    for(i in 1:nrow(indx)){
      S12 <- c(S12, (gammaHat[indx[i,2]] * mean((log(data[I[,indx[i,1]],indx[i,1]]) -
               log(qHat[indx[i,1]])) * varphi[I[,indx[i,1]],indx[i,2]]) -
               gammaHat[indx[i,1]] * gammaHat[indx[i,2]] * mean(varphi[I[,indx[i,1]],indx[i,2]]))/
               ((1-tau)*ExpctHat[indx[i,2]]))
      S21 <- c(S21, (gammaHat[indx[i,1]] * mean((log(data[I[,indx[i,2]],indx[i,2]]) -
               log(qHat[indx[i,2]])) * varphi[I[,indx[i,2]],indx[i,1]]) -
               gammaHat[indx[i,2]] * gammaHat[indx[i,1]] * mean(varphi[I[,indx[i,2]],indx[i,1]]))/
               ((1-tau)*ExpctHat[indx[i,1]]))
    }
    # B.2.9) computes the asymptotic covariance terms
    CovEHat <- gammaHat[indx[,1]] * gammaHat[indx[,2]] * R11 + CovEHatInt / rn^2 +
     (S12 + S21) / rn
  }
  # B.3) computes the off-diagonal covariance of the QB estimator
  #      using the proposed adjustment based on dependence observations
  if(varType %in% c("asym-Ind-Adj", "asym-Ind-Adj-Log") & method=="QB"){
    mg <- 1 / (1 - gammaHat) - log(1 / gammaHat - 1)
    rdata <- NULL
    # B.3.1) computes the marginal empirical distribution functions
    mecdf <- apply(data, 2, ecdf)
    # B.3.2) computes the marginal empirical quantile functions
    for(i in 1:dimData[2]) rdata <- cbind(rdata, 1 - dimData[1] * mecdf[[i]](data[, i]) / (dimData[1] + 1))
    # computes the tail dependence functions
    R11 <- apply(indx, 1, estPairETC, data=rdata, ndata=dimData[1], k=k)
    Rj1 <- apply(indx, 1, estPairLeftETC, data=rdata, ndata=dimData[1],k=k)
    R1l <- apply(indx, 1, estPairRightETC, data=rdata, ndata=dimData[1],k=k)
    # computes the off diagonal covariance terms
    CovEHat <- gammaHat[indx[,1]] * gammaHat[indx[,2]] * (R11 * (mg[indx[,1]] +
               rn - 1) * (mg[indx[,2]] + rn - 1) + (mg[indx[,1]] + rn) *
               Rj1 + (mg[indx[,2]] + rn) * R1l) / rn^2
  }
  # C) fills in the variance-covariance matrix
  VarCovEHat[lower.tri(VarCovEHat)] <- CovEHat
  VarCovEHat <- t(VarCovEHat)
  VarCovEHat[lower.tri(VarCovEHat)] <- CovEHat
  # C.1) with the relative estimator the variance-covariance matrix needs
  # to be rescaled by the expectile estimator
  if(regexpr("Log", varType, fixed=TRUE)[1]==-1){
    diag(VarCovEHat) <- ExpctHat^2 * VarEHat
    CovEHat <- ExpctHat[indx[,1]] * ExpctHat[indx[,2]] * CovEHat
    VarCovEHat[lower.tri(VarCovEHat)] <- CovEHat
    VarCovEHat <- t(VarCovEHat)
    VarCovEHat[lower.tri(VarCovEHat)] <- CovEHat
  }
  # C.2) Normalised variance-covariance matrix
  VarCovEHat <- rn^2 * VarCovEHat / k
  return(VarCovEHat)
}

# computes the Quantile-Based marginal Expected Shortfall
# @ the intermediate level
QBmarExpShortF <- function(data, tau, n, q){
  res <- sum(data[,1] * as.numeric(data[,1]>0 & data[,2]>q))
  return(res / (n*(1-tau)))
}

# computes the Expectile-Based marginal Expected Shortfall
# @ the intermediate level
EBmarExpShortF <- function(data, tau, n, xi){
  res <- sum(data[,1] * as.numeric(data[,1]>0 & data[,2]>xi))
  return(res / sum(data[,2]>xi))
}

################################################################################
### END
### Sub-routines (HIDDEN FUNCTIONS)
###
################################################################################


################################################################################
### START
### Main-routines (VISIBLE FUNCTIONS)
###
################################################################################

################################################################################
###
### START
###
################## MULTIIVARIATE CASE ##########################################

################################################################################
###
### Estimation of multiple tail indices
### using the Hill estimator
### variance-covariance matrix of the estimator
### and (1-alpha)100% confidence regions are computed
###
################################################################################
MultiHTailIndex <- function(data, k, var=FALSE, varType="asym-Dep", bias=FALSE,
                            alpha=0.05, plot=FALSE){

  # initialization
  gammaHat <- NULL
  VarGamHat <- NULL
  VarCovGHat <- NULL
  EstConReg <- NULL
  biasTerm <- NULL
  dimData <- dim(data)

  # computes multiple tail indices estimates
  res <- apply(data, 2, HTailIndex, k=k, var=var, bias=bias, varType="asym-Ind",
               bigBlock=NULL, smallBlock=NULL, alpha=alpha)

  # computes the estimator variance-covariance matrix
  if(var){
    # computes the number of distinct pairs
    for(i in 1:dimData[2]) {
      gammaHat <- c(gammaHat, as.numeric(res[[i]][1]))
      VarGamHat <- c(VarGamHat, as.numeric(res[[i]][2]))
      if(bias)  biasTerm <- c(biasTerm, as.numeric(res[[i]][4]))
    }
    # sets the off doagonal variance terms of the normalised estimator
    VarCovGHat <- diag(VarGamHat)
    # derives the covariance structure
    if(varType=="asym-Dep"){
      # compute the covariance of the normalised tail index estimator
      estAC <- estAsymCov(data, k, sqrt(VarGamHat))
      # fill in the covariance terms of the variance-covariance matrix
      VarCovGHat[lower.tri(VarCovGHat)] <- estAC
      VarCovGHat <- t(VarCovGHat)
      VarCovGHat[lower.tri(VarCovGHat)] <- estAC
      VarCovGHat <- VarCovGHat / k
    }
    # checks whether the variance-covariance matrix is well defined
    if(any(diag(VarCovGHat)<=0)){
      warning("Variance-covariance matrix is not well defined")
      VarCovGHat <- NA
      EstConReg <- NA
      return(list(gammaHat=gammaHat, VarCovGHat=VarCovGHat, EstConReg=EstConReg))
    }
    ev <- eigen(VarCovGHat, symmetric = TRUE)
    if(!all(ev$values >= 0)){
      warning("Variance-covariance matrix is numerically not positive semidefinite")
      VarCovGHat <- NA
      EstConReg <- NA
      return(list(gammaHat=gammaHat, VarCovGHat=VarCovGHat, EstConReg=EstConReg))
    }
    # 2D graphical representation of the symmetric confidence regions
    if(dimData[2]==2){
      EstConReg <- ellipse(gammaHat, VarCovGHat, alpha)
      if(plot){
        xrange <- range(EstConReg[,1])
        yrange <- range(EstConReg[,2])
        par(mai=c(.6,.6,.2,.1), mgp=c(1.5,.5,0))
        ylab <-eval(bquote(expression(widehat(gamma)[list(.(k), 2)])))
        xlab <-eval(bquote(expression(widehat(gamma)[list(.(k), 1)])))
        plot(gammaHat[1], gammaHat[2], pch=20, xlim=c(xrange[1]-.1, xrange[2]+.1),
             ylim=c(yrange[1]-.1, yrange[2]+.1), xlab=xlab, ylab=ylab,
             main="Bivariate Tail Index Estimation")
        lines(EstConReg, lwd=2, col="red")
        legend("topleft", col=c(1,2), lwd=c(2,2), cex=1.2, bty="n", c("Point estimate",
               paste((1-alpha)*100, "% Asymptotic Confidence Region", sep="")))
      }
    }
    # 3D graphical reppresentation of the symmetric confidence regions
    if(dimData[2]==3){
      EstConReg <- ellipsoid(gammaHat, VarCovGHat, alpha)
      if(plot){
        zlab <-eval(bquote(expression(widehat(gamma)[list(.(k), 3)])))
        ylab <-eval(bquote(expression(widehat(gamma)[list(.(k), 2)])))
        xlab <-eval(bquote(expression(widehat(gamma)[list(.(k), 1)])))
        scatter3D(EstConReg[,1], EstConReg[,2], EstConReg[,3], type="l", xlab="",
                  ylab="", zlab="", ticktype="detailed", main="Trivariate Tail Index Estimation")
        text(-0.2675761, -0.3988516, xlab)
        text(0.3005687, -0.3688111, ylab)
        text(-0.4494528, -0.03598391, zlab)
        points3D(gammaHat[1], gammaHat[2], gammaHat[3], pch=21, col=2, add=TRUE)
      }
    }
  }
  else for(i in 1:dimData[2]) {
    gammaHat <- c(gammaHat, as.numeric(res[[i]][1]))
    if(bias) biasTerm <- c(biasTerm, as.numeric(res[[i]][4]))
  }
  return(list(gammaHat=gammaHat,
              VarCovGHat=VarCovGHat,
              biasTerm=biasTerm,
              EstConReg=EstConReg))
}


################################################################################
###
### Estimation of multiple Expectiles @ the intermediate level
### using the LAWS and QB estimators
### variance-covariance matrix of the estimator
### and (1-alpha)100% confidence regions are computed
###
################################################################################
estMultiExpectiles <- function(data, tau, method="LAWS", tailest="Hill", var=FALSE,
                               varType="asym-Ind-Adj", k=NULL, alpha=0.05, plot=FALSE){
  # initialization
  ExpctHat <- NULL
  VarEHat <- NULL
  VarCovEHat <- NULL
  EstConReg <- NULL
  gammaHat <- NULL
  biasTerm <- NULL
  dimData <- dim(data)

  # checks whether the essential routine parameters have been inserted
  isNk <- is.null(k)
  isNtau <- is.null(tau)
  if(isNk & isNtau) stop("Enter a value for at least one of the two parameters 'tau' and 'k' ")
  if(isNk) k <- floor(dimData[1]*(1-tau))
  if(isNtau) tau <- 1-k/dimData[1]
  # computes multiple expectiles
  res <- apply(data, 2, estExpectiles, tau=tau, method=method, tailest=tailest,
               var=var, varType=varType, bigBlock=NULL, smallBlock=NULL, k=k,
               alpha=alpha)
  # computes the variance-covariance matrix of the expctile estimator
  if(var){
    # computes expectiles, bias terms, variance and tail indices
    for(i in 1:dimData[2]) {
      # 1) computes expectile estimates at the intermediate level for all the margins
      ExpctHat <- c(ExpctHat, as.numeric(res[[i]][1]))
      # 2) computes the bias term for all the margins
      biasTerm <- c(biasTerm, as.numeric(res[[i]][2]))
      # 3) computes estimates of the variances of expectile estimates
      VarEHat <- c(VarEHat, as.numeric(res[[i]][3]))
      # 4) computes tail index estimates for all the margins
      if(tailest=="Hill") gammaHat <- c(gammaHat, HTailIndex(data[,i], k=k)$gammaHat)
    }
    # computes the covariance terms
    VarCovEHat <- estMultiExpectVar(data, dimData, ExpctHat, gammaHat, method,
                                    k, tau, VarEHat, varType)
    #  checks whether the variance-covariance matrix is well defined
    if(any(is.na(VarCovEHat))){
      warning("Variance-covariance matrix is not well defined")
      VarCovEHat <- NA
      EstConReg <- NA
      return(list(ExpctHat=ExpctHat, biasTerm=biasTerm, VarCovEHat=VarCovEHat, EstConReg=EstConReg))
    }
    if(any(diag(VarCovEHat)<=0)){
      warning("Variance-covariance matrix is not well defined")
      VarCovEHat <- NA
      EstConReg <- NA
      return(list(ExpctHat=ExpctHat, biasTerm=biasTerm, VarCovEHat=VarCovEHat, EstConReg=EstConReg))
    }
    ev <- eigen(VarCovEHat, symmetric = TRUE)
    if(!all(ev$values >= 0)){
      warning("Variance-covariance matrix is numerically not positive semidefinite")
      VarCovEHat <- NA
      EstConReg <- NA
      return(list(ExpctHat=ExpctHat, biasTerm=biasTerm, VarCovEHat=VarCovEHat, EstConReg=EstConReg))
    }
    # computes a rescaling of the mean and variance components
    b <- 1 / (1 + biasTerm / sqrt(k))
    mu <- ExpctHat * b
    B <- diag(b)
    Sigma <- B %*% VarCovEHat %*% B
    # 2D graphical reppresentation
    if(dimData[2]==2){
      EstConReg <- ellipse(mu, Sigma, alpha)
      if(plot){
        xrange <- range(EstConReg[,1])
        yrange <- range(EstConReg[,2])
        par(mai=c(.6,.6,.2,.1), mgp=c(1.5,.5,0))
        ylab <-eval(bquote(expression(widehat(xi)[list(.(tau), 2)])))
        xlab <-eval(bquote(expression(widehat(xi)[list(.(tau), 1)])))
        plot(mu[1], mu[2], pch=20, xlim=c(xrange[1]-1, xrange[2]+1),
             ylim=c(yrange[1]-1, yrange[2]+1), xlab=xlab, ylab=ylab,
             main=eval(bquote(expression("Bivariate Expectile Estimation" ~ - ~ tau[n]==~.(tau)))))
        points(data, col="gray", pch=20)
        lines(EstConReg, lwd=2, col="red")
        legend("topleft", col=c(1,2), lwd=c(2,2), cex=1.2, bty="n", c("Point estimate",
               paste((1-alpha)*100, "% Asymptotic Confidence Region", sep="")))
      }
    }
    # 3D graphical reppresentation
    if(dimData[2]==3){
      EstConReg <- ellipsoid(mu, Sigma, alpha)
      if(plot){
        zlab <-eval(bquote(expression(widehat(xi)[list(.(tau), 3)])))
        ylab <-eval(bquote(expression(widehat(xi)[list(.(tau), 2)])))
        xlab <-eval(bquote(expression(widehat(xi)[list(.(tau), 1)])))
        scatter3D(EstConReg[,1],EstConReg[,2],EstConReg[,3], type="l", xlab="",
                  ylab="", zlab="", ticktype="detailed", main="Trivariate Expectile Estimation")
        text(-0.2675761, -0.3988516, xlab)
        text(0.3005687, -0.3688111, ylab)
        text(-0.4494528, -0.03598391, zlab)
        points3D(mu[1],mu[2],mu[3], pch=21, col=2, add=TRUE)
      }
    }
  }
  # independence case
  else for(i in 1:dimData[2]){
    # 1) computes expectile estimates at the intermediate level for all the margins
    ExpctHat <- c(ExpctHat, as.numeric(res[[i]][1]))
    # 2) computes the bias term for all the margins
    biasTerm <- c(biasTerm, as.numeric(res[[i]][2]))
  }
  return(list(ExpctHat=ExpctHat,
              biasTerm=biasTerm,
              VarCovEHat=VarCovEHat,
              EstConReg=EstConReg))
}


################################################################################
###
### Estimation of multiple Expectiles @ the extreme level
### using the LAWS and QB estimators
### variance-covariance matrix of the estimator
### and (1-alpha)100% confidence regions are computed
###
################################################################################
predMultiExpectiles <- function(data, tau, tau1, method="LAWS", tailest="Hill",
                                var=FALSE, varType="asym-Ind-Adj-Log", bias=FALSE,
                                k=NULL, alpha=0.05, plot=FALSE){
  # initialization
  ExpctHat <- NULL
  VarEHat <- NULL
  VarCovEHat <- NULL
  EstConReg <- NULL
  biasTerm <- NULL
  gammaHat <- NULL
  dimData <- dim(data)

  # checks whether the essential routine oarameters have been inserted
  isNk <- is.null(k)
  isNtau <- is.null(tau)
  if(isNk & isNtau) stop("Enter a value for at least one of the two parameters 'tau' and 'k' ")
  if(isNk) k <- floor(dimData[1]*(1-tau))
  if(isNtau) tau <- 1-k/dimData[1]
  # computes multiple expctiles
  res <- apply(data, 2, predExpectiles, tau=tau, tau1=tau1, method=method,
               tailest=tailest, var=var, varType=varType, bias=FALSE,
               bigBlock=NULL, smallBlock=NULL, k=k, alpha_n=NULL, alpha=alpha)
  # compute the variance-covariance of the expectile estimator
  if(var){
    # computes the number of distinct pairs
    indx <- t(combn(dimData[2],2))
    # computes expectiles, bias terms, variance and tail indices
    for(i in 1:dimData[2]) {
      # 1) computes expectile estimates at the intermediate level for all the margins
      ExpctHat <- c(ExpctHat, as.numeric(res[[i]][1]))
      # 2) computes the bias term for all the margins
      biasTerm <- c(biasTerm, as.numeric(res[[i]][2]))
      # 3) computes estimates of the variances of expectile estimates
      VarEHat <- c(VarEHat, as.numeric(res[[i]][3]))
      # 4) computes tail index estimates for all the margins
      if(tailest=="Hill") gammaHat <- c(gammaHat, HTailIndex(data[,i], k=k)$gammaHat)
    }
    # computes the covariance terms of the variance-covariance matrix
    VarCovEHat <- predMultiExpectVar(data, dimData, ExpctHat, gammaHat, method,
                                     k, tau, tau1, VarEHat, varType)
    # checks the validity of the variance-covariance matrix
    if(any(is.na(VarCovEHat))){
      warning("Variance-covariance matrix is not well defined")
      VarCovEHat <- NA
      EstConReg <- NA
      return(list(ExpctHat=ExpctHat, biasTerm=biasTerm, VarCovEHat=VarCovEHat, EstConReg=EstConReg))
    }
    if(any(diag(VarCovEHat)<=0)){
      warning("Variance-covariance matrix is not well defined")
      VarCovEHat <- NA
      EstConReg <- NA
      return(list(ExpctHat=ExpctHat, biasTerm=biasTerm, VarCovEHat=VarCovEHat, EstConReg=EstConReg))
    }
    ev <- eigen(VarCovEHat, symmetric = TRUE)
    if(!all(ev$values >= 0)){
      warning("Variance-covariance matrix is numerically not positive semidefinite")
      VarCovEHat <- NA
      EstConReg <- NA
      return(list(ExpctHat=ExpctHat, biasTerm=biasTerm, VarCovEHat=VarCovEHat, EstConReg=EstConReg))
    }
    # graphical reppresentation in the log-scale
    if(regexpr("Log", varType, fixed=TRUE)[1]==-1){
      cn <- sqrt(k) / log((1-tau)/(1-tau1))
      b <- 1 / (1 + biasTerm / cn)
      mu <- ExpctHat * b
      B <- diag(b)
      Sigma <- B %*% VarCovEHat %*% B
      # 2D graphical representation of the confidence regions
      if(dimData[2]==2){
        EstConReg <- ellipse(mu, Sigma, alpha)
        if(plot){
          xrange <- range(EstConReg[,1])
          yrange <- range(EstConReg[,2])
          par(mai=c(.6,.6,.2,.1), mgp=c(1.5,.5,0))
          ylab <-eval(bquote(expression(widehat(xi)[list(.(tau1), 2)])))
          xlab <-eval(bquote(expression(widehat(xi)[list(.(tau1), 1)])))
          plot(mu[1], mu[2], pch=20, xlim=c(xrange[1]-1, xrange[2]+1),
               ylim=c(yrange[1]-1, yrange[2]+1), xlab=xlab, ylab=ylab,
               main="Bivariate Expectile Estimation")
          lines(EstConReg, lwd=2, col="red")
          legend("topleft", col=c(1,2), lwd=c(2,2), cex=1.2, bty="n", c("Point estimate",
                 paste((1-alpha)*100, "% Asymptotic Confidence Region", sep="")))
        }
      }
      # 3D graphical representation of the confidence regions
      if(dimData[2]==3){
        EstConReg <- ellipsoid(mu, Sigma, alpha)
        if(plot){
          zlab <-eval(bquote(expression(widehat(xi)[list(.(tau1), 3)])))
          ylab <-eval(bquote(expression(widehat(xi)[list(.(tau1), 2)])))
          xlab <-eval(bquote(expression(widehat(xi)[list(.(tau1), 1)])))
          scatter3D(EstConReg[,1],EstConReg[,2],EstConReg[,3], type="l", xlab="",
                    ylab="", zlab="", ticktype="detailed", main="Trivariate Expectile Estimation")
          text(-0.2675761, -0.3988516, xlab)
          text(0.3005687, -0.3688111, ylab)
          text(-0.4494528, -0.03598391, zlab)
          points3D(mu[1],mu[2],mu[3], pch=21, col=2, add=TRUE)
        }
      }
    }
    # graphical representation in relative scale
    else{
      cn <- sqrt(k) / log((1-tau)/(1-tau1))
      mu <- biasTerm / cn
      Sigma <- VarCovEHat
      # 2D graphical representation of the confidence regions
      if(dimData[2]==2){
        EH <- apply(array(ExpctHat, c(1,dimData[2])), 2, rep, times=250)
        EstConReg <-  EH * exp(ellipse(mu, Sigma, alpha))
        mu <- ExpctHat * exp(mu)
        if(plot){
          xrange <- range(EstConReg[,1])
          yrange <- range(EstConReg[,2])
          par(mai=c(.6,.6,.2,.1), mgp=c(1.5,.5,0))
          ylab <-eval(bquote(expression(widehat(xi)[list(.(tau1), 2)])))
          xlab <-eval(bquote(expression(widehat(xi)[list(.(tau1), 1)])))
          plot(mu[1], mu[2], pch=20, xlim=c(xrange[1]-1, xrange[2]+1),
               ylim=c(yrange[1]-1, yrange[2]+1), xlab=xlab, ylab=ylab,
               main="Bivariate Expectile Estimation")
          lines(EstConReg, lwd=2, col="red")
          legend("topleft", col=c(1,2), lwd=c(2,2), cex=1.2, bty="n", c("Point estimate",
                 paste((1-alpha)*100, "% Asymptotic Confidence Region", sep="")))
        }
      }
      # 3D graphical representation of the confidence regions
      if(dimData[2]==3){
        EH <- apply(array(ExpctHat, c(1,dimData[2])), 2, rep, times=3600)
        EstConReg <- EH * exp(ellipsoid(mu, Sigma, alpha))
        mu <- ExpctHat * exp(mu)
        if(plot){
          zlab <-eval(bquote(expression(widehat(xi)[list(.(tau1), 3)])))
          ylab <-eval(bquote(expression(widehat(xi)[list(.(tau1), 2)])))
          xlab <-eval(bquote(expression(widehat(xi)[list(.(tau1), 1)])))
          scatter3D(EstConReg[,1],EstConReg[,2],EstConReg[,3], type="l", xlab="",
                    ylab="", zlab="", ticktype="detailed", main="Trivariate Expectile Estimation")
          text(-0.2675761, -0.3988516, xlab)
          text(0.3005687, -0.3688111, ylab)
          text(-0.4494528, -0.03598391, zlab)
          points3D(mu[1],mu[2],mu[3], pch=21, col=2, add=TRUE)
        }
      }
    }
  }
  else for(i in 1:dimData[2]){
    # 1) computes expectile estimates at the intermediate level for all the margins
    ExpctHat <- c(ExpctHat, as.numeric(res[[i]][1]))
    # 2) computes the bias term for all the margins
    biasTerm <- c(biasTerm, as.numeric(res[[i]][2]))
  }
  return(list(ExpctHat=ExpctHat,
              biasTerm=biasTerm,
              VarCovEHat=VarCovEHat,
              EstConReg=EstConReg))
}

################################################################################
###
### Estimation of multiple quantiles @ the extreme level
### using the LAWS and QB estimators
### variance-covariance matrix of the estimator
### and (1-alpha)100% confidence regions are computed
###
################################################################################
extMultiQuantile <- function(data, tau, tau1, var=FALSE, varType="asym-Ind-Log",
                             bias=FALSE, k=NULL, alpha=0.05, plot=FALSE){
  # initialization
  ExtQHat <- NULL
  VarExQHat <- NULL
  VarCovExQHat <- NULL
  EstConReg <- NULL
  dimData <- dim(data)

  # checks whether the essential routine parameters have been inserted
  isNk <- is.null(k)
  isNtau <- is.null(tau)
  if(isNk & isNtau) stop("Enter a value for at least one of the two parameters 'tau' and 'k' ")
  if(isNk) k <- floor(dimData[1]*(1-tau))
  if(isNtau) tau <- 1-k/dimData[1]
  cn <- sqrt(k) / log((1-tau)/(1-tau1))
  # computes multiple extreme quantiles
  res <- apply(data, 2, extQuantile, tau=tau, tau1=tau1, var=var, varType=varType,
               bias=FALSE, bigBlock=NULL, smallBlock=NULL, k=k, alpha=alpha)
  # computes the variance-covariance matrix of estimator
  if(var){
    # computes the number of distinct pairs
    indx <- t(combn(dimData[2],2))
    # computes quantiles, bias terms, variance and tail indices
    for(i in 1:dimData[2]) {
      # 1) computes quantile estimates at the extreme level for all the margins
      ExtQHat <- c(ExtQHat, as.numeric(res[[i]][1]))
      # 3) computes estimates of the variances of expectile estimates
      VarExQHat <- c(VarExQHat, as.numeric(res[[i]][2]))
    }
    # sets the variances of the normalised estimator
    VarCovExQHat <- diag(VarExQHat)
    # derives the covariance structure
    # compute the covariance of the relative quantile estimator
    estAC <- estAsymCov(data, k, sqrt(VarExQHat))
    # fills in the covariance terms of the variance-covariance matrix
    VarCovExQHat[lower.tri(VarCovExQHat)] <- estAC
    VarCovExQHat <- t(VarCovExQHat)
    VarCovExQHat[lower.tri(VarCovExQHat)] <- estAC
    VarCovExQHat <- VarCovExQHat / cn

    # checks the validity of the variance-covariance matrix
    if(any(is.na(VarCovExQHat))){
      warning("Variance-covariance matrix is not well defined")
      VarCovExQHat <- NA
      EstConReg <- NA
      return(list(ExtQHat=ExtQHat, VarCovExQHat=VarCovExQHat, EstConReg=EstConReg))
    }
    if(any(diag(VarCovExQHat)<=0)){
      warning("Variance-covariance matrix is not well defined")
      VarCovExQHat <- NA
      EstConReg <- NA
      return(list(ExtQHat=ExtQHat, VarCovExQHat=VarCovExQHat, EstConReg=EstConReg))
    }
    ev <- eigen(VarCovExQHat, symmetric = TRUE)
    if(!all(ev$values >= 0)){
      warning("Variance-covariance matrix is numerically not positive semidefinite")
      VarCovExQHat <- NA
      EstConReg <- NA
      return(list(ExtQHat=ExtQHat, VarCovExQHat=VarCovExQHat, EstConReg=EstConReg))
    }
    # graphical reppresentation of the estimation results in the log-scale
    if(regexpr("Log", varType, fixed=TRUE)[1]==-1){
      mu <- ExtQHat
      B <- diag(mu)
      VarCovExQHat <- B %*% VarCovExQHat %*% B
      Sigma <- VarCovExQHat
      # 2D graphical reppresentation of the confidence regions
      if(dimData[2]==2){
        EstConReg <- ellipse(mu, Sigma, alpha)
        if(plot){
          xrange <- range(EstConReg[,1])
          yrange <- range(EstConReg[,2])
          par(mai=c(.6,.6,.2,.1), mgp=c(1.5,.5,0))
          ylab <-eval(bquote(expression(widehat(q)[list(.(tau1), 2)])))
          xlab <-eval(bquote(expression(widehat(q)[list(.(tau1), 1)])))
          plot(mu[1], mu[2], pch=20, xlim=c(xrange[1]-1, xrange[2]+1),
               ylim=c(yrange[1]-1, yrange[2]+1), xlab=xlab, ylab=ylab,
               main="Bivariate Quantile Estimation")
          lines(EstConReg, lwd=2, col="red")
          legend("topleft", col=c(1,2), lwd=c(2,2), cex=1.2, bty="n", c("Point estimate",
                 paste((1-alpha)*100, "% Asymptotic Confidence Region", sep="")))
        }
      }
      # 3D graphical reppresentation of the confidence regions
      if(dimData[2]==3){
        EstConReg <- ellipsoid(mu, Sigma, alpha)
        if(plot){
          zlab <-eval(bquote(expression(widehat(q)[list(.(tau1), 3)])))
          ylab <-eval(bquote(expression(widehat(q)[list(.(tau1), 2)])))
          xlab <-eval(bquote(expression(widehat(q)[list(.(tau1), 1)])))
          scatter3D(EstConReg[,1],EstConReg[,2],EstConReg[,3], type="l", xlab="",
                    ylab="", zlab="", ticktype="detailed", main="Trivariate Quantile Estimation")
          text(-0.2675761, -0.3988516, xlab)
          text(0.3005687, -0.3688111, ylab)
          text(-0.4494528, -0.03598391, zlab)
          points3D(mu[1],mu[2],mu[3], pch=21, col=2, add=TRUE)
        }
      }
    }
    # graphical reppresentation of the estimation results in the relative scale
    else{
      mu <- rep(0, dimData[2])
      Sigma <- VarCovExQHat
      # 2D graphical reppresentation of the confidence regions
      if(dimData[2]==2){
        EQH <- apply(array(ExtQHat, c(1,dimData[2])), 2, rep, times=250)
        EstConReg <-  EQH * exp(ellipse(mu, Sigma, alpha))
        if(plot){
          xrange <- range(EstConReg[,1])
          yrange <- range(EstConReg[,2])
          par(mai=c(.6,.6,.2,.1), mgp=c(1.5,.5,0))
          ylab <-eval(bquote(expression(widehat(q)[list(.(tau1), 2)])))
          xlab <-eval(bquote(expression(widehat(q)[list(.(tau1), 1)])))
          plot(mu[1], mu[2], pch=20, xlim=c(xrange[1]-1, xrange[2]+1),
               ylim=c(yrange[1]-1, yrange[2]+1), xlab=xlab, ylab=ylab,
               main="Bivariate Quantile Estimation")
          lines(EstConReg, lwd=2, col="red")
          legend("topleft", col=c(1,2), lwd=c(2,2), cex=1.2, bty="n", c("Point estimate",
                 paste((1-alpha)*100, "% Asymptotic Confidence Region", sep="")))
        }
      }
      # 3D graphical reppresentation of the confidence regions
      if(dimData[2]==3){
        EQH <- apply(array(ExtQHat, c(1,dimData[2])), 2, rep, times=3600)
        EstConReg <- EQH * exp(ellipsoid(mu, Sigma, alpha))
        mu <- ExtQHat
        if(plot){
          zlab <-eval(bquote(expression(widehat(q)[list(.(tau1), 3)])))
          ylab <-eval(bquote(expression(widehat(q)[list(.(tau1), 2)])))
          xlab <-eval(bquote(expression(widehat(q)[list(.(tau1), 1)])))
          scatter3D(EstConReg[,1],EstConReg[,2],EstConReg[,3], type="l", xlab="",
                    ylab="", zlab="", ticktype="detailed", main="Trivariate Quantile Estimation")
          text(-0.2675761, -0.3988516, xlab)
          text(0.3005687, -0.3688111, ylab)
          text(-0.4494528, -0.03598391, zlab)
          points3D(mu[1],mu[2],mu[3], pch=21, col=2, add=TRUE)
        }
      }
    }
  }
  else for(i in 1:dimData[2]){
    # 1) computes quantile estimates at the intermediate level for all the margins
    ExtQHat <- c(ExtQHat, as.numeric(res[[i]][1]))
  }
  return(list(ExtQHat=ExtQHat,
              VarCovExQHat=VarCovExQHat,
              EstConReg=EstConReg))
}

################################################################################
###
### Wald-type hypothesis testing with test statistics based on the
### LAWS, QB expectile estimators, quantile estimator and tail index estimator
### the observed statistics tests together with the chi-square critical value
### are computed
###
################################################################################
HypoTesting <- function(data, tau, tau1=NULL, type="ExpectRisks", level="extreme",
                        method="LAWS", bias=FALSE, k=NULL, alpha=0.05){
  # initialization
  ExpctHat <- NULL
  VarCovEHat <- NULL

  dimData <- dim(data)
  ones <- array(1, c(dimData[2],1))
  biasTerm <- rep(0, dimData[2])
  critVal <- qchisq(1-alpha, dimData[2]-1)

  # checks whether the essential routine parameters have been inserted
  isNk <- is.null(k)
  isNtau <- is.null(tau)
  if(isNk & isNtau) stop("Enter a value for at least one of the two parameters 'tau' and 'k' ")
  if(isNk) k <- floor(dimData[1]*(1-tau))
  if(isNtau) tau <- 1-k/dimData[1]

  # Wald-type statistics based on the expectile estimator
  # @ the intermediate level
  if(type=="ExpectRisks"){
    # estimation expectiles point estimates and their variance-covariance matrix
    if(level=="inter"){
      res <- estMultiExpectiles(data, tau,  method, var=TRUE, k=k)
      # checks if the variance-covariance is well defined
      if(any(is.na(res$VarCovEHat))) return(list(logLikR=NA, critVal=critVal))
      # correct the estimator and its variance-covariance matrix for the bias term
      b <- 1 / (1 + res$biasTerm / sqrt(k))
      ExpctHat <- array(res$ExpctHat * b, c(dimData[2], 1))
      B <- diag(b)
      VarCovEHat <- B %*% res$VarCovEHat %*% B
    }
    # Wald-type statistics based on the expectile estimator
    # @ the extreme level
    if(level=="extreme"){
      res <- predMultiExpectiles(data, tau, tau1, method, var=TRUE,
                                 varType="asym-Ind-Adj-Log", k=k)
      # checks if the variance-covariance is well defined
      if(any(is.na(res$VarCovEHat))) return(list(logLikR=NA, critVal=critVal))
      cn <- sqrt(k) / log((1-tau)/(1-tau1))
      ExpctHat <- array(log(res$ExpctHat) - res$biasTerm / cn, c(dimData[2], 1))
      VarCovEHat <- res$VarCovEHat
    }
  }
  # Wald-type statistics based on the extreme quantile estimator
  if(type=="QuantRisks"){
    res <- extMultiQuantile(data, tau, tau1, var=TRUE, varType="asym-Ind-Log",
                              k=k)
    # checks if the variance-covariance is well defined
    if(any(is.na(res$VarCovExQHat))) return(list(logLikR=NA, critVal=critVal))
    ExpctHat <- array(log(res$ExtQHat), c(dimData[2], 1))
    VarCovEHat <- res$VarCovExQHat
  }
  # Wald-type statistics based on the tail index estimator
  if(type=="tails"){
    res <- MultiHTailIndex(data, k, var=TRUE, varType="asym-Dep", bias=bias,
                           alpha=alpha)
    ExpctHat <- array(res$gammaHat, c(dimData[2], 1))
    if(any(is.na(res$VarCovGHat))) return(list(logLikR=NA, critVal=critVal))
    VarCovEHat <- res$VarCovGHat
  }
  # computes the inverse of the estimator variance-covariance matrix
  InvVC <- try(solve(VarCovEHat), silent=TRUE)
  if(!is.matrix(InvVC)) return(list(logLikR=NA, critVal=critVal))
  # compute the mean under the null hypothesis
  ms <- ((t(ExpctHat) %*% InvVC) %*% ones) / ((t(ones) %*% InvVC) %*% ones)
  # centered estimator
  SExpctHat <- ExpctHat - array(ms, c(dimData[2],1))
  # computes the log-likelihood ratio-test statistic
  logLikR <- as.numeric(t(SExpctHat) %*% InvVC %*% SExpctHat)

  return(list(logLikR=logLikR, critVal=critVal))
}


################################################################################
###
### Estimation of the Quantile-Based marginal
### Expected Shortfall
### @ the extreme level
###
################################################################################
QuantMES <- function(data, tau, tau1, var=FALSE, varType="asym-Dep", bias=FALSE,
                     bigBlock=NULL, smallBlock=NULL, k=NULL, alpha=0.05){
  # initialization
  VarHatQMES <- NULL
  CIHatQMES <- NULL
  CIHatQMES <- NULL

  # main body:
  isNk <- is.null(k)
  isNtau <- is.null(tau)
  if(isNk & isNtau) stop("Enter a value for at least one of the two parameters 'tau' and 'k' ")
  # computes the sample size
  ndata <- nrow(data)
  if(isNk) k <- floor(ndata*(1-tau))
  if(isNtau) tau <- 1-k/ndata

  # estimation of the quantile @ the intermediate level based on the empirical quantile
  QHat <- as.numeric(quantile(data[,2], tau))
  # estimation of the tail index and related quantities using the Hill estimator
  if(varType=="asym-Alt-Dep") estHill <- HTailIndex(data[,1], k)
  else estHill <- HTailIndex(data[,1], k, var, varType, bias, bigBlock, smallBlock, alpha)
  # computes an estimate of the tail index with a bias-correction
  gammaHat <- estHill$gammaHat - estHill$BiasGamHat
  # estimation of the QB marginal Expected Shortfall @ the intermediate level
  QMESt <- QBmarExpShortF(data, tau, ndata, QHat)
  # estimation of the QB marginal Expected Shortfall @ the intermediate level
  QMESt1 <- ((1-tau1)/(1-tau))^(-gammaHat) * QMESt
  # estimation of the asymptotic variance of the Hill estimator
  if(var){
    # The default approach for computing the asymptotic variance with serial dependent observations
    # is the method proposed by Drees (2003). In this case it is assumed that there is no bias term
    # If it is assumed that the Hill estimator requires a bias correction then the the asymptotic
    # variance is estimated after correcting the tail index estimate by the bias correction
    VarHatQMES <- estHill$VarGamHat

    if((varType=="asym-Alt-Dep") & (!bias)){
      # Drees (2003) approach
      j <- max(2, ceiling(ndata*(1-tau1)))
      VarHatQMES <- DHVar(data, tau, tau1, j, k, ndata)
    }
    # compute lower and upper bounds of the (1-alpha)% CI
    K <- sqrt(VarHatQMES)*log((1-tau)/(1-tau1))/sqrt(k)
    lbound <- QMESt1 * exp(qnorm(alpha/2)*K)
    ubound <- QMESt1 * exp(qnorm(1-alpha/2)*K)
    CIHatQMES <- c(lbound, ubound)
  }
  return(list(HatQMES=QMESt1,
              VarHatQMES=VarHatQMES,
              CIHatQMES=CIHatQMES))
}
################################################################################
###
### Estimation of the Expectile-Based marginal
### Expected Shortfall
### @ the extreme level
###
################################################################################
ExpectMES <- function(data, tau, tau1, method="LAWS", var=FALSE, varType="asym-Dep",
                      bias=FALSE, bigBlock=NULL, smallBlock=NULL, k=NULL,
                      alpha_n=NULL, alpha=0.05){
  # initialization
  VarHatXMES <- NULL
  CIHatXMES <- NULL
  CIHatXMES <- NULL
  # main body:
  isNk <- is.null(k)
  isNtau <- is.null(tau)
  if(isNk & isNtau) stop("Enter a value for at least one of the two parameters 'tau' and 'k' ")
  # compute the sample size
  ndata <- nrow(data)
  if(isNk) k <- floor(ndata*(1-tau))
  if(isNtau) tau <- 1-k/ndata

  # estimation of the tail index for X using the Hill estimator
  estHill <- HTailIndex(data[,1], k, var, varType, bias, bigBlock, smallBlock, alpha)
  gammaHatX <- estHill$gammaHat - estHill$BiasGamHat
  # estimation of the tail index for Y using the Hill estimator
  gammaHatY <- HTailIndex(data[,2], k)$gammaHat
  # estimation of the extreme level if requested
  if(is.null(tau1)) tau1 <- estExtLevel(alpha_n, gammaHat=gammaHatY)$tauHat
  # expectile-Based Esitmation
  if(method=="LAWS"){
    ExpctHat <- estExpectiles(data[,2], tau)$ExpctHat
    # estimation of the EB marginal Expected Shortfall @ the intermediate level
    XMESt <- EBmarExpShortF(data, tau, ndata, ExpctHat)
    # estimation of the Expectile-EB marginal Expected Shortfall @ the extreme level
    XMESt1 <- ((1-tau1)/(1-tau))^(-gammaHatX) * XMESt
  }
  # Quantile-Based Estimation
  if(method=="QB"){
    # estimation of the QB marginal Expected Shortfall @ the intermediate level
    QMESt <- QuantMES(data, NULL, tau1, k=k)$HatQMES
    # estimation of the Expectile-QB marginal Expected Shortfall @ the extreme level
    XMESt1 <- (1/gammaHatY - 1)^(-gammaHatX) * QMESt
  }
  # compute an estimate of the asymptotic variance of the expecile estimator at the extreme level
  if(var){
    # computes the asymptotic variance for independent and serial dependent data
    VarHatXMES <- estHill$VarGamHat
    # computes the "margin error"
    K <- sqrt(VarHatXMES)*log((1-tau)/(1-tau1))/sqrt(k)
    # compute lower and upper bounds of the (1-alpha)% CI
    lbound <- XMESt1 * exp(qnorm(alpha/2)*K)
    ubound <- XMESt1 * exp(qnorm(1-alpha/2)*K)
    # defines the (1-alpha)% CI
    CIHatXMES <- c(lbound, ubound)
  }
  return(list(HatXMES=XMESt1,
              VarHatXMES=VarHatXMES,
              CIHatXMES=CIHatXMES))
}


################################################################################
### END
###
################## MULTIIVARIATE CASE ##########################################


################################################################################
### END
### Main-routines (VISIBLE FUNCTIONS)
###
################################################################################

#########################################################
### Authors: Carlotta Pacifici and Simone Padoan      ###
### Emails: simone.padoan@unibocconi.it               ###
### carlotta.pacifici@unibocconi.it                   ###
### Institution: Department of Decision Sciences,     ###
### University Bocconi of Milan, Italy,               ###
### File name: Prediction.r                           ###
### Description:                                      ###
### This file contains the functions to produce       ###
### the same type of statistical analysis that        ###
### is introduced by the article:                     ###
### "Predicting hazards of climate extremes:          ###
### a statistical perspective"                        ###
### Last change: 08/10/2025                           ###
### by Carlotta Pacifici                              ###
#########################################################

#########################################################
### START
### Sub-routines (HIDDEN FUNCTIONS)
###
#########################################################

###########################################
######################## STATIONARY SETTING
###########################################

# Density of truncated Student-t (location-scale version)
dtt <- function(
  x,
  location = 0,
  scale = 1,
  df = 1,
  left = -Inf,
  right = Inf,
  log = FALSE
) {
  stopifnot(length(left) == 1L, length(right) == 1L, left < right)
  xl <- is.finite(x) & x >= left & x <= right
  dens <- rep(0, length.out = length(x))
  dens[xl] <- dt((x[xl] - location) / scale, df = df, log = FALSE) /
    (pt(q = (right - location) / scale, df = df) -
      pt(q = (left - location) / scale, df = df)) /
    scale
  dens[is.na(x)] <- NA
  if (isTRUE(log)) {
    return(log(dens))
  } else {
    dens
  }
}

# Negative log likelihood for discrete generalized Pareto distribution
nll_dgpd <- function(par, excess, constraint = TRUE) {
  scale <- ifelse(isTRUE(constraint), exp(par[1]), par[1])
  shape <- par[2]
  out <- evd::pgpd(q = excess + 1, scale = scale, shape = shape) -
    evd::pgpd(q = excess, scale = scale, shape = shape)
  return(-sum(log(out)))
}


# Internal function to adjust the value of sigma (=log(theta)) in each iteration
updateSig <- function(sig, acc, d = d, p = p, alpha = alpha, i) {
  c <- ((1 - 1 / d) *
    sqrt(2 * pi) *
    exp(alpha^2 / 2) /
    (2 * alpha) +
    1 / (d * p * (1 - p))) # Eq(7) of Garthwaite, Fan & Sisson (2016)
  Theta <- log(sig)
  # Theta = Theta + c * (acc-p) / max(200, i/d)
  Theta <- Theta + c * (acc - p) / i
  return(exp(Theta))
}
# Internal function to update the covariance matrix
updateCov <- function(sigMat, i, thetaM, theta, d) {
  epsilon <- 1 / i
  thetaM2 <- ((thetaM * i) + theta) / (i + 1)
  sigMat <- (i - 1) /
    i *
    sigMat +
    thetaM %*% t(thetaM) -
    (i + 1) / i * thetaM2 %*% t(thetaM2) +
    1 / i * theta %*% t(theta) +
    epsilon * diag(d)
  return(list(sigMat = sigMat, thetaM = thetaM2))
}


# Random Walk Metropolis Hastings algorithm for the Bayesian estimation of continuous GP parameters
cRWMH.POT <- function(data, start, p, sig0, nsim, mle, prior, verbose = TRUE) {
  # A) INITIALIZATION
  alpha <- -qnorm(p / 2)
  d <- length(start) # Dimension of the vector of parameters
  if (!any(d == 2)) {
    stop("Wrong length of parameter vector")
  }
  sig <- sig0 # Initial value of sigma
  sig.vec <- sig
  sigMat <- diag(d) # Initial proposal covarince matrix
  acc.vec <- rep(NaN, nsim) # Vector of acceptances
  accepted <- rep(0, nsim)
  straight.reject <- rep(0, nsim) # Monitor the number of proposed parameters that don't respect the constraints
  sig.start <- sig
  sig.restart <- sig
  n0 <- round(5 / (p * (1 - p)))
  iMax <- 100 # iMax is the max number of iterations before the last restart
  Numbig <- 0
  Numsmall <- 0
  param <- start
  msg <- "none"
  # sample size
  k <- length(data)
  # B) DEFINITION OF SUBROUTINES
  # B.1) GPD-LOG-LIKELIHOOD CONCERNING POT
  gpd_llik_fun <- function(param) {
    # likelihood function
    res <- sum(evd::dgpd(data, 0, param[1], param[2], log = TRUE))
    if (is.infinite(res)) {
      res <- -1e300
    }
    return(res)
  }
  # C) LIKELIHOOD DEFNITION, SET PARAMETERS CONSTRAINTS AND ACCEPTANCE PROBABILITY DEFINITION
  llikfun <- function(para) return(gpd_llik_fun(para))
  parcheck <- function(para) {
    res <- any(para[1] <= 0, (para[2] < 0 & !(max(data) <= -para[1] / para[2])))
    return(res)
  }
  if (prior == "uniform") {
    accept_prob <- function(par, par_new) {
      res <- min(
        exp(llikfun(par_new) - llikfun(par) + log(par[1]) - log(par_new[1])),
        1
      )
      return(res)
    }
  }
  if (prior == "empirical") {
    accept_prob <- function(par, par_new) {
      res <- min(
        exp(
          llikfun(par_new) -
            llikfun(par) +
            dgamma(par_new[1], 1, scale = mle[1], log = TRUE) -
            dgamma(par[1], 1, scale = mle[1], log = TRUE) +
            dtt(par_new[2], df = 1, left = -1, right = Inf, log = TRUE) -
            dtt(par[2], df = 1, left = -1, right = Inf, log = TRUE)
        ),
        1
      )
      return(res)
    }
  }

  # D) START ALGORITHM
  # set the chains
  sparam <- param
  # create a progress bar
  if (isTRUE(verbose)) {
    pb <- txtProgressBar(
      min = 0,
      max = nsim,
      initial = 0,
      style = 3
    )
  }
  # main algorithm loop
  for (i in 1:nsim) {
    # simulation from the proposal (MVN):
    s <- try(eigen(sig^2 * sigMat, symmetric = TRUE), silent = TRUE)
    if (inherits(s, "try-error")) {
      msg <- "The algorithm has failed to converge."
      return(list(
        param_post = sparam,
        accepted = accepted,
        straight.reject = straight.reject,
        nsim = nsim,
        sig.vec = sig.vec,
        sig.restart = sig.restart,
        acc.vec = acc.vec,
        msg = msg
      ))
    }
    param_new <- as.vector(mvtnorm::rmvnorm(
      1,
      mean = param,
      sigma = sig^2 * sigMat
    ))
    if (any(is.na(param_new))) {
      straight.reject[i] <- 1
      acc.vec[i] <- 0
    } else {
      # check basic condition
      if (parcheck(param_new)) {
        straight.reject[i] <- 1
        acc.vec[i] <- 0
      } else {
        # compute the acceptance probability
        ratio <- accept_prob(param, param_new)
        if (is.na(ratio)) {
          straight.reject[i] <- 1
          acc.vec[i] <- 0
        } else {
          acc.vec[i] <- ratio
          u <- runif(1)
          if (u < ratio) {
            param <- param_new
            accepted[i] <- 1
          }
        }
      }
    }
    sparam <- rbind(sparam, param)
    # update covariance matrix with adaptive MCMC
    if (i > 100) {
      if (i == 101) {
        sigMat <- cov(sparam)
        thetaM <- apply(sparam, 2, mean)
      } else {
        tmp <- updateCov(
          sigMat = sigMat,
          i = i,
          thetaM = thetaM,
          theta = param,
          d = d
        )
        sigMat <- tmp$sigMat
        thetaM <- tmp$thetaM
      }
    }
    # Update sigma
    if (i > n0) {
      sig <- updateSig(
        sig = sig,
        acc = ratio,
        d = d,
        p = p,
        alpha = alpha,
        i = i
      )
      sig.vec <- c(sig.vec, sig)
      if ((i <= (iMax + n0)) && (Numbig < 5 || Numsmall < 5)) {
        Toobig <- (sig > (3 * sig.start))
        Toosmall <- (sig < (sig.start / 3)) #
        if (Toobig || Toosmall) {
          #restart the algorithm
          message("restart the program at", i, "th iteration", "\n")
          sig.restart <- c(sig.restart, sig)
          Numbig <- Numbig + Toobig
          Numsmall <- Numsmall + Toosmall
          i <- n0
          sig.start <- sig
        }
      } #end iMax
    }
    if (isTRUE(verbose)) {
      print.i <- seq(0, nsim, by = 100)
      if (i %in% print.i) setTxtProgressBar(pb, i)
    }
  }
  return(list(
    param_post = sparam,
    accepted = accepted,
    straight.reject = straight.reject,
    nsim = nsim,
    sig.vec = sig.vec,
    sig.restart = sig.restart,
    acc.vec = acc.vec,
    msg = msg
  ))
}

# Bayesian or frequentist estimation of the continuous GP distribution parameters
cPOT <- function(
  data,
  k,
  pn,
  method,
  prior,
  start,
  sig0,
  nsim,
  burn,
  p,
  optim.method,
  control,
  ...
) {
  # A) CHECK INPUT
  if (missing(data)) {
    stop("Missing data input")
  }
  if (method != "bayesian" && method != "frequentist") {
    stop("Method should be `bayesian` or `frequentist` ")
  }
  #if(any(evt==evt.method)) stop("Evt should be `GEV-CENS-ML` or `GP-CENS-ML` or `POT-ML` or `BM-AT-ML`")
  if (!is.vector(data)) {
    stop("Data must be a vector")
  }
  if (is.null(pn)) {
    stop(
      "Must specify the probability associated with the quantile(s) to extrapolate"
    )
  }
  if (is.null(start)) {
    stop("Need to specify start parameter")
  } # start is the starting value for both methods
  nparam <- length(start) # number of inputed parameters
  if (nparam != 2) {
    stop("Wrong length of initial parameter vector")
  }
  #
  # B) INITIALIZATION
  # B.1) IMPORTANT VARIABLES
  n <- length(data) # set sample size
  nQuant <- length(pn) # set the number of quantiles to extrapolate
  I <- VarCov <- NaN # set the expected information matrix

  msg <- "none" # output messagge
  conv <- 0 # convergence parameter for the maximazing optimization routine
  #
  # B.2) DEFINE THRESHOLD EXCEEDANCES
  sdata <- sort(data, decreasing = TRUE)

  # BAYESIAN INFERENCE
  if (method == "bayesian") {
    # check the minimal required parameters
    if (is.null(sig0) || is.null(nsim) || is.null(p)) {
      stop("Missing initial values")
    }
    if (is.null(nsim)) {
      stop("Missing number of replications in algorithm")
    }
    #
    # ML estimates of GPD's parameters
    #
    mle <- evd::fpot(
      data,
      sdata[k + 1],
      "gpd",
      start,
      std.err = FALSE
    )$estimate
    # MCMC ALGORITHM
    mcmc <- cRWMH.POT(
      data = (sdata[1:k] - sdata[k + 1]),
      start = unlist(start),
      p = p,
      sig0 = sig0,
      nsim = nsim,
      mle = mle,
      prior = prior
    )

    meanacc <- rep(NA, length(mcmc$acc.vec))
    for (j in c(1:length(mcmc$acc.vec))) {
      meanacc[j] <- mean(mcmc$acc.vec[round(j / 2):j])
    }

    index <- c(1:2000, seq(2001, length(mcmc$sig.vec), by = 100))

    post_sample <- mcmc$param_post[(burn + 1):nsim, ]
    sig.ps <- post_sample[, 1]
    gam.ps <- post_sample[, 2]
    # derive approximate samples from the quantile posterior distribution
    Q.est <- matrix(NaN, nrow = nrow(post_sample), ncol = nQuant)
    R.est <- matrix(NaN, nrow = nrow(post_sample), ncol = nQuant)
    for (j in 1:nQuant) {
      Q.est[, j] <- sdata[k + 1] +
        sig.ps * ((k / (n * pn[j]))^gam.ps - 1) / gam.ps
    }
    return(list(
      Q.est = Q.est,
      post_sample = post_sample,
      burn = burn,
      straight.reject = mcmc$straight.reject[-c(1:(burn - 1))],
      sig.vec = mcmc$sig.vec,
      accept.prob = cbind(index, meanacc[index]),
      msg = mcmc$msg,
      mle = mle,
      t = sdata[k + 1]
    ))
    # FREQUENTIST INFERENCE
  } else if (method == "frequentist") {
    if (is.null(optim.method)) {
      stop("Need to specify an optimisation method in frequentist setting")
    }

    if (!("control" %in% names(sys.call()))) {
      control <- list(fnscale = -1, maxit = 8e+5)
    } else {
      if (!("fnscale" %in% names(control))) {
        control[["fnscale"]] <- -1
        control[["maxit"]] <- 8e+5
      }
    }
    # COMPUTE MAXIMUM LIKELIHOOD ESTIMATES
    est.para <- evd::fpot(data, sdata[k + 1], "gpd", start, std.err = FALSE)
    # compute the expected information matrix at the ML estimates
    sigma <- est.para$estimate[1]
    gam <- est.para$estimate[2]
    # COMPUTE VARIANCE-COVARIANCE MATRIX
    VarCov <- diag(2)
    VarCov[1, 1] <- (1 + gam)^2
    VarCov[1, 2] <- VarCov[2, 1] <- -(1 + gam)
    VarCov[2, 2] <- 1 + (1 + gam)^2
    # VarCov <- diag(c(sigma, sigma, 1)) %*% VarCov %*% diag(c(sigma, sigma, 1))
    VarCov <- diag(c(1, sigma)) %*% VarCov %*% diag(c(1, sigma))

    # compute extreme quantile
    Q.est <- array(NaN, c(1, nQuant))
    for (j in 1:nQuant) {
      Q.est[j] <- sdata[k + 1] + sigma * ((k / (n * pn[j]))^gam - 1) / gam
    }
    Q.gra <- c(
      ((-(n / k) * log(1 - pn))^(-gam) - 1) / gam,
      -sigma *
        ((-(n / k) * log(1 - pn))^(-gam) - 1) /
        gam^2 -
        (sigma / gam) *
        ((-(n / k) * log(1 - pn))^(-gam)) *
        log(-(n / k) * log(1 - pn))
    )
    Q.VC <- (array(Q.gra, c(1, 2)) %*% VarCov %*% array(Q.gra, c(2, 1))) / k
    VarCov <- VarCov / k
    return(list(
      est = est.para$estimate,
      t = sdata[k + 1],
      Q.est = Q.est,
      VarCov = VarCov,
      Q.VC = Q.VC,
      msg = est.para$convergence
    ))
  }
}


# Random Walk Metropolis Hastings algorithm for the Bayesian estimation of d-GP parameters
dRWMH.POT <- function(
  data,
  start,
  p,
  sig0,
  nsim,
  mle,
  prior,
  verbose = TRUE
) {
  # A) INITIALIZATION
  alpha <- -qnorm(p / 2)
  d <- length(start) # Dimension of the vector of parameters
  if (!any(d == 2)) {
    stop("Wrong length of parameter vector")
  }
  sig <- sig0 # Initial value of sigma
  sig.vec <- sig
  sigMat <- diag(d) # Initial proposal covarince matrix
  acc.vec <- rep(NaN, nsim) # Vector of acceptances
  accepted <- rep(0, nsim)
  straight.reject <- rep(0, nsim) # Monitor the number of proposed parameters that don't respect the constraints
  sig.start <- sig
  sig.restart <- sig
  n0 <- round(5 / (p * (1 - p)))
  iMax <- 100 # iMax is the max number of iterations before the last restart
  Numbig <- 0
  Numsmall <- 0
  param <- start
  msg <- "none"
  # sample size
  k <- length(data)
  # B) DEFINITION OF SUBROUTINES
  # discrete GP likelihood
  dgpd_llik_fun <- function(par) {
    scale <- par[1]
    shape <- par[2]
    out <- (1 + shape / scale * data)^(-1 / shape) -
      (1 + shape / scale * (data + 1))^(-1 / shape)
    res <- sum(log(out))
    if (is.infinite(res)) {
      res <- -1e300
    }
    return(res)
  }
  # C) LIKELIHOOD DEFNITION, SET PARAMETERS CONSTRAINTS AND ACCEPTANCE PROBABILITY DEFINITION
  llikfun <- function(para) return(dgpd_llik_fun(para))
  parcheck <- function(para) {
    res <- any(para[1] <= 0, (para[2] < 0 & !(max(data) <= -para[1] / para[2])))
    return(res)
  }
  if (prior == "uniform") {
    accept_prob <- function(par, par_new) {
      res <- min(
        exp(llikfun(par_new) - llikfun(par) + log(par[1]) - log(par_new[1])),
        1
      )
      return(res)
    }
  }
  if (prior == "empirical") {
    accept_prob <- function(par, par_new) {
      res <- min(
        exp(
          llikfun(as.numeric(par_new)) -
            llikfun(as.numeric(par)) +
            dgamma(par_new[1], 1, scale = mle[1], log = TRUE) -
            dgamma(par[1], 1, scale = mle[1], log = TRUE) +
            dtt(par_new[2], df = 1, left = -1, right = Inf, log = TRUE) -
            dtt(par[2], df = 1, left = -1, right = Inf, log = TRUE)
        ),
        1
      )
      return(res)
    }
  }

  # D) START ALGORITHM
  # set the chains
  sparam <- as.numeric(param)
  # create a progress bar
  if (isTRUE(verbose)) {
    pb <- txtProgressBar(
      min = 0,
      max = nsim,
      initial = 0,
      style = 3
    )
  }
  # main algorithm loop
  for (i in 1:nsim) {
    # simulation from the proposal (MVN):
    s <- try(eigen(sig^2 * sigMat, symmetric = TRUE), silent = TRUE)
    if (inherits(s, "try-error")) {
      msg <- "The algorithm has failed."
      return(list(
        param_post = sparam,
        accepted = accepted,
        straight.reject = straight.reject,
        nsim = nsim,
        sig.vec = sig.vec,
        sig.restart = sig.restart,
        acc.vec = acc.vec,
        msg = msg
      ))
    }
    param_new <- as.vector(mvtnorm::rmvnorm(
      1,
      mean = as.numeric(param),
      sigma = sig^2 * sigMat
    ))
    if (any(is.na(param_new))) {
      straight.reject[i] <- 1
      acc.vec[i] <- 0
    } else {
      # check basic condition
      if (parcheck(param_new)) {
        straight.reject[i] <- 1
        acc.vec[i] <- 0
      } else {
        # compute the acceptance probability
        ratio <- accept_prob(as.numeric(param), param_new)
        if (is.na(ratio)) {
          straight.reject[i] <- 1
          acc.vec[i] <- 0
        } else {
          acc.vec[i] <- ratio
          u <- runif(1)
          if (u < ratio) {
            param <- param_new
            accepted[i] <- 1
          }
        }
      }
    }
    sparam <- rbind(sparam, param)
    # update covariance matrix with adaptive MCMC
    if (i > 100) {
      if (i == 101) {
        sigMat <- cov(sparam)
        thetaM <- apply(sparam, 2, mean)
      } else {
        tmp <- updateCov(
          sigMat = sigMat,
          i = i,
          thetaM = thetaM,
          theta = param,
          d = d
        )
        sigMat <- tmp$sigMat
        thetaM <- tmp$thetaM
      }
    }
    # Update sigma
    if (i > n0) {
      sig <- updateSig(
        sig = sig,
        acc = ratio,
        d = d,
        p = p,
        alpha = alpha,
        i = i
      )
      sig.vec <- c(sig.vec, sig)
      if ((i <= (iMax + n0)) && (Numbig < 5 || Numsmall < 5)) {
        Toobig <- (sig > (3 * sig.start))
        Toosmall <- (sig < (sig.start / 3)) #
        if (Toobig || Toosmall) {
          #restart the algorithm
          message("restart the program at", i, "th iteration", "\n")
          sig.restart <- c(sig.restart, sig)
          Numbig <- Numbig + Toobig
          Numsmall <- Numsmall + Toosmall
          i <- n0
          sig.start <- sig
        }
      } #end iMax
    }
    if (isTRUE(verbose)) {
      print.i <- seq(0, nsim, by = 100)
      if (i %in% print.i) setTxtProgressBar(pb, i)
    }
  }
  # results
  ret <- list(
    param_post = sparam,
    accepted = accepted,
    straight.reject = straight.reject,
    nsim = nsim,
    sig.vec = sig.vec,
    sig.restart = sig.restart,
    acc.vec = acc.vec,
    msg = msg
  )
  class(ret) <- "RWMH"
  return(invisible(ret))
}

# Bayesian or frequentist estimation of the discrete GP distribution parameters
dPOT <- function(
  data,
  k,
  pn,
  method = c("bayesian", "frequentist"),
  prior,
  start,
  sig0,
  nsim,
  burn,
  p,
  optim.method,
  control,
  verbose = FALSE,
  ...
) {
  # A) CHECK INPUT
  if (missing(data)) {
    stop("Missing data input")
  }
  method <- match.arg(method)
  if (method != "bayesian" && method != "frequentist") {
    stop("Method should be `bayesian` or `frequentist` ")
  }
  if (!is.vector(data)) {
    stop("Data must be a vector")
  }
  if (is.null(pn)) {
    stop(
      "Must specify the probability associated with the quantile(s) to extrapolate"
    )
  }
  # Define exceedances
  n <- length(data) # set sample size
  sdata <- sort(data, decreasing = TRUE)
  excess <- sdata[1:k] - sdata[k]

  if (is.null(start)) {
    start <- evd::fpot(
      x = excess,
      threshold = 0,
      model = "gpd",
      std.err = FALSE
    )
    start <- list(
      scale = as.numeric(start$estimate[1]),
      shape = as.numeric(start$estimate[2])
    )
  } else {
    # start is the starting value for both methods
    if (length(start) != 2) {
      stop("Wrong length of initial parameter vector")
    }
    if (!isTRUE(all(c("scale", "shape") %in% names(start)))) {
      stop(
        "Invalid list for \"start\": must contain arguments \"scale\" and \"shape\"."
      )
    }
    if (
      !is.finite(nll_dgpd(
        par = c(start$scale, start$shape),
        excess = excess,
        constraint = FALSE
      ))
    ) {
      stop("Invalid arguments")
    }
  }
  #
  # B) INITIALIZATION
  # B.1) IMPORTANT VARIABLES

  nQuant <- length(pn) # set the number of quantiles to extrapolate
  I <- VarCov <- NaN # set the expected information matrix

  msg <- "none" # output message
  conv <- 0 # convergence parameter for the maximazing optimization routine

  # B.2) DEFINE THRESHOLD EXCEEDANCES

  # BAYESIAN INFERENCE
  if (method == "bayesian") {
    # check the minimal required parameters
    if (is.null(sig0) || is.null(nsim) || is.null(p)) {
      stop("Missing initial values")
    }
    if (is.null(nsim)) {
      stop("Missing number of simulations in algorithm")
    }
    #
    # ML estimates of GPD's parameters
    mle <- unlist(start)
    # MCMC ALGORITHM
    mcmc <- dRWMH.POT(
      data = excess,
      start = c(start$scale, start$shape),
      p = p,
      sig0 = sig0,
      nsim = nsim,
      mle = mle,
      prior = prior,
      verbose = verbose
    )

    meanacc <- rep(NA, length(mcmc$acc.vec))
    for (j in c(1:length(mcmc$acc.vec))) {
      meanacc[j] <- mean(mcmc$acc.vec[round(j / 2):j])
    }

    index <- c(1:2000, seq(2001, length(mcmc$sig.vec), by = 100))

    post_sample <- mcmc$param_post[(burn + 1):nsim, ]
    sig.ps <- post_sample[, 1]
    gam.ps <- post_sample[, 2]
    # derive approximate samples from the quantile posterior distribution
    Q.est <- matrix(NaN, nrow = nrow(post_sample), ncol = nQuant)
    for (j in 1:nQuant) {
      Q.est[, j] <- floor(
        sdata[k] + sig.ps * ((k / (n * pn[j]))^gam.ps - 1) / gam.ps
      ) - 1
    }
    return(list(
      Q.est = Q.est,
      post_sample = post_sample,
      burn = burn,
      straight.reject = mcmc$straight.reject[-c(1:(burn - 1))],
      sig.vec = mcmc$sig.vec,
      accept.prob = cbind(index, meanacc[index]),
      msg = mcmc$msg,
      mle = mle,
      t = sdata[k]
    ))
    # FREQUENTIST INFERENCE
  } else if (method == "frequentist") {
    if (is.null(optim.method)) {
      stop("Need to specify an optimisation method in frequentist setting")
    }

    if (!("control" %in% names(sys.call()))) {
      control <- list(fnscale = -1, maxit = 8e+5)
    } else {
      if (!("fnscale" %in% names(control))) {
        control[["fnscale"]] <- -1
        control[["maxit"]] <- 8e+5
      }
    }
    # COMPUTE MAXIMUM LIKELIHOOD ESTIMATES
    est.para <- fitdGPD(excess)
    # compute the expected information matrix at the ML estimates
    sigma <- est.para$mle[1]
    gam <- est.para$mle[2]
    # COMPUTE VARIANCE-COVARIANCE MATRIX
    VarCov <- diag(2)
    VarCov[1, 1] <- (1 + gam)^2
    VarCov[1, 2] <- VarCov[2, 1] <- -(1 + gam)
    VarCov[2, 2] <- 1 + (1 + gam)^2
    # VarCov <- diag(c(sigma, sigma, 1)) %*% VarCov %*% diag(c(sigma, sigma, 1))
    VarCov <- diag(c(1, sigma)) %*% VarCov %*% diag(c(1, sigma))

    # compute extreme quantile
    Q.est <- array(NaN, c(1, nQuant))
    for (j in 1:nQuant) {
      Q.est[j] <- sdata[k + 1] + sigma * ((k / (n * pn[j]))^gam - 1) / gam
    }
    Q.gra <- c(
      ((-(n / k) * log(1 - pn))^(-gam) - 1) / gam,
      -sigma *
        ((-(n / k) * log(1 - pn))^(-gam) - 1) /
        gam^2 -
        (sigma / gam) *
        ((-(n / k) * log(1 - pn))^(-gam)) *
        log(-(n / k) * log(1 - pn))
    )
    Q.VC <- (array(Q.gra, c(1, 2)) %*% VarCov %*% array(Q.gra, c(2, 1))) / k
    VarCov <- VarCov / k
    return(list(
      est = est.para$estimate,
      t = sdata[k],
      Q.est = Q.est,
      VarCov = VarCov,
      Q.VC = Q.VC,
      msg = est.para$convergence
    ))
  }
}


########## prediction
# Density for continuous GP
Fdgpd_c <- function(par, x) {
  res <- evd::dgpd(x, par[1], par[2], par[3])
  if (is.na(res)) {
    res <- 0
  }
  return(res)
}

# Density for discrete GP
Fdgpd_d <- function(par, x) {
  loc <- par[1]
  scale <- par[2]
  shape <- par[3]
  z1 <- pmax(0, x - loc) / scale
  z2 <- pmax(0, x + 1 - loc) / scale
  out <- (1 + shape * z1)^(-1 / shape) - (1 + shape * z2)^(-1 / shape)
  if (is.na(out)) {
    out <- 0
  }
  return(out)
}

# Frequentist continuous GP distribution
Fpgpd_c <- function(par, x) {
  res <- evd::pgpd(x, par[1], par[2], par[3])
  if (is.na(res)) {
    res <- 0
  }
  return(res)
}

# Bayesian predictive distribution - continuous GP
Bpgpd_c <- function(x, post) {
  res <- apply(post, 1, Fpgpd_c, x = x)
  return(mean(res, na.rm = TRUE))
}

#' Bayesian predictive quantile for generalized Pareto distribution
#'
#' @description WARNING: this function will not be exported in a future version of the package.
#'
#' @export
#' @keywords internal
#' @param p probability levels
#' @param post matrix of posterior samples
#' @param lower lower bound for the search for the quantile
#' @param upper upper bound for the region over which to search for the quantile
#' @return a vector of quantiles of the same length as \code{p}
Bqgpd_c <- function(p, post, lower, upper) {
  myf <- function(x, p, post) return(p - Bpgpd_c(x, post))

  res <- uniroot(
    myf,
    lower = lower,
    upper = upper,
    p = p,
    post = post,
    extendInt = "yes"
  )$root
  return(res)
}

# Frequentist discrete GP distribution
Fpgpd_d <- function(par, x) {
  z <- pmax(0, x + 1 - par[1]) / par[2]
  res <- 1 - (1 + par[3] * z)^(-1 / par[3])
  if (is.na(res)) {
    res <- 0
  }
  return(res)
}

# Bayesian predictive distribution - discrete GP
Bpgpd_d <- function(x, post) {
  res <- apply(post, 1, Fpgpd_d, x = x)
  return(mean(res, na.rm = TRUE))
}

#' Bayesian predictive quantile for discrete generalized Pareto distribution
#'
#' @description WARNING: this function will not be exported in a future version of the package.
#'
#' @export
#' @keywords internal
#' @param p probability levels
#' @param post matrix of posterior samples
#' @param lower lower bound for the search for the quantile
#' @param upper upper bound for the region over which to search for the quantile
#' @return a vector of quantiles of the same length as \code{p}
Bqgpd_d <- function(p, post, lower, upper) {
  myf <- function(x, p, post) return(p - Bpgpd_d(x, post))

  res <- uniroot(
    myf,
    lower = lower,
    upper = upper,
    p = p,
    post = post,
    extendInt = "yes"
  )$root
  return(floor(res))
}


# Internal functions for tail homogeneity test from the evt0 package
mop.RBMOP <- function (osx, k, p){
  n = length(osx)
  losx = log(osx)
  dk = length(k)
  dp = length(p)
  est = matrix(NA, dk, dp)
  H = mop.MOP(osx, k, p)
  H = H$EVI
  rhoest = mop.rho(osx)
  betaest = mop.beta(losx, rhoest)
  for (j in 1:dp) est[, j] = H[, j] * (1 - ((betaest * (1 -
                                                          p[j] * H[, j]))/(1 - rhoest - p[j] * H[, j])) * (n/k)^(rhoest))
  return(list(EVI = est, rho = rhoest, beta = betaest))
}

mop.MOP <- function (osx, k, p){
  n = length(osx)
  dk = length(k)
  dp = length(p)
  km = max(k)
  nk = km + 1
  est = matrix(NA, dk, dp)
  for (j in 1:dp) if (p[j] == 0) {
    losx = rev(log(osx))
    losx = losx[1:nk]
    tmp = cumsum(losx)/(1:nk)
    estc = tmp[-nk] - losx[-1]
    est[, j] = estc[k]
  }
  else {
    tosx = rev(osx)
    tosx = (tosx[1:nk])^p[j]
    tmp = cumsum(tosx)/(1:nk)
    estc = tmp[-nk]/tosx[-1]
    estc = (1 - estc^-1)/p[j]
    est[, j] = estc[k]
  }
  return(list(EVI = est))
}

mop.rho <- function (osx)
{
  losx = log(osx)
  n = length(losx)
  krho = floor(n^(0.995)):floor(n^(0.999))
  krhom = max(krho)
  nkrho = krhom + 1
  M = matrix(NA, length(krho), 3)
  tau0 = numeric(length(krho))
  tau1 = numeric(length(krho))
  W0 = numeric(length(krho))
  W1 = numeric(length(krho))
  losx = rev(losx)
  losx = losx[1:nkrho]
  estc1 = mop.MOP(osx, krho, p = 0)
  M[, 1] = estc1$EVI
  c12 = cumsum(losx^2)/(1:nkrho)
  c22 = (losx)^2
  c32 = cumsum(losx)/(1:nkrho)
  estc2 = c12[-nkrho] + c22[-1] - 2 * c32[-nkrho] * losx[-1]
  M[, 2] = estc2[krho]
  c13 = cumsum(losx^3)/(1:nkrho)
  c23 = (losx)^3
  estc3 = c13[-nkrho] - c23[-1] - 3 * (losx[-1] * c12[-nkrho] -
                                         c32[-nkrho] * c22[-1])
  M[, 3] = estc3[krho]
  W0 = (log(M[, 1]) - (1/2) * log(M[, 2]/2))/((1/2) * log(M[,
                                                            2]/2) - (1/3) * log(M[, 3]/6))
  W1 = (M[, 1] - (M[, 2]/2)^(1/2))/((M[, 2]/2)^(1/2) - (M[,
                                                          3]/6)^(1/3))
  tau0 = -abs(3 * (W0 - 1)/(W0 - 3))
  tau1 = -abs(3 * (W1 - 1)/(W1 - 3))
  tau0median = median(tau0)
  tau1median = median(tau1)
  sumtau0 = as.numeric((tau0 - tau0median) %*% (tau0 - tau0median))
  sumtau1 = as.numeric((tau1 - tau1median) %*% (tau1 - tau1median))
  if ((sumtau0 < sumtau1) || (sumtau0 == sumtau1)) {
    return(tau0[length(krho)])
  }
  else {
    return(tau1[length(krho)])
  }
}

mop.beta <- function (losx, rhoest)
{
  n = length(losx)
  k1 = floor(n^(0.999))
  v = c(0, rhoest, 2 * rhoest)
  D = numeric(length(v))
  c1 = ((1:k1)/k1)^(-v[1])
  c2 = ((1:k1)/k1)^(-v[2])
  c3 = ((1:k1)/k1)^(-v[3])
  c = (1:k1) * ((losx[n:(n - k1 + 1)]) - (losx[(n - 1):(n -
                                                          k1)]))
  D[1] = sum(c1 * c)/k1
  D[2] = sum(c2 * c)/k1
  D[3] = sum(c3 * c)/k1
  d = sum(c2)/k1
  betaest = (k1/n)^(v[2]) * (d * D[1] - D[2])/(d * D[2] - D[3])
  return(betaest)
}

mop <- function (x, k, p, method = c("MOP", "RBMOP"))
{
  n = length(x)
  if (n < 2) {
    stop("Data vector x with at least two sample points required.")
  }
  if (is.null(p) || any(is.na(p))) {
    stop("p is not specified")
  }
  if (is.null(k) || any(is.na(k))) {
    stop("k is not specified")
  }
  if (any(k < 1) || any(k > n) || any(k == n) || !is.numeric(k) ||
      isTRUE(any(k != as.integer(k)))) {
    stop("Each k must be integer and greater than or equal to 1 and less than sample size.")
  }
  if (!is.numeric(x)) {
    stop("Some of the data points are not real.")
  }
  if (!is.numeric(p)) {
    stop("p must be a real.")
  }
  osx <- sort(x[x > 0])
  method = match.arg(method)
  if (method == "MOP") {
    EVI.est = mop.MOP(osx, k, p)
  }
  else if (method == "RBMOP") {
    EVI.est = mop.RBMOP(osx, k, p)
  }
  colnames(EVI.est$EVI) = p
  rownames(EVI.est$EVI) = k
  return(EVI.est)
}

gammaest <- function(x,k){
  n <- length(x)
  sop <- mop(x, k, 0, "RBMOP")
  sop$lambda <- sqrt(k) * sop$EVI * sop$beta * (n/k)^sop$rho
  return(c(sop$EVI, sop$rho, sop$beta, sop$lambda))
}

# Tail homogeneity test (Daouia, A., S.A. Padoan and G. Stupfler, 2024)
PooledTailIndex <- function(gamHat, rhoHat, betaHat, lambdaHat, m, n, k, type="AMSE",
                            bias=TRUE, poolSO=FALSE, etailCop=NULL, test="DPS",
                            Vmat=NULL, alpha=0.05)
{
  # DEFINE IMPORTANT QUANTITIES FOR THE SEUBSEQUENT COMPUTATIONS
  # compute the total effective sample size
  K <- sum(k)
  # compute the total sample size
  N <- sum(n)
  # compute the scaling matrix
  D <- sqrt(K) * diag(sqrt(1/k))
  # critical value for the hypothesis testing
  critVal <- qchisq(1-alpha, m-1)
  # define the vector of ones
  ones <- array(1, c(m,1))

  # 1) IF SECOND ORDER PARAMETERS ARE GIVEN THEN THEY ARE POLLED TOGETHER
  if(poolSO){
    rhoHat <- rep(sum(n * rhoHat / N), m)
    betaHat <- rep(sum(n * betaHat / N), m)
    lambdaHat <- sqrt(k) * rep(sum(k * gamHat / K), m) * betaHat * (n/k)^rhoHat
  }
  # 2) COMPUTE A DESIGN VERCTOR THAT IS USEFUL FOR THE COMPUTATION OF THE ASYMPTOTIC BIAS TERMS
  B <- D %*% array(lambdaHat / (1 - rhoHat), c(m,1))
  # 3) COMPUTE A DESIGN MATRIX THAT IS USEFUL FOR THE COMPUTATION OF THE ASYMPTOTIC
  #    ASYMPTOTIC VARIANCE
  # define the V matrix (diagonal) in the case of the distributed inference
  VargamHat <- (sum(k * gamHat / K))^2
  V <- diag(VargamHat, nrow=m, ncol=m)
  # off diagonal elements
  if(!is.null(etailCop)){
    # define the V matrix (full) in the case of the non-distributed inference
    VargamHat <- (mean(gamHat))^2
    V <- diag(VargamHat, nrow=m, ncol=m)
    V[lower.tri(V)] <- VargamHat * etailCop
    V <- t(V)
    V[lower.tri(V)] <- VargamHat * etailCop
  }
  if(all(!is.null(Vmat), is.matrix(Vmat))) V <- Vmat
  # define the matrix called in the paper as V_c
  V <- D %*% V %*% D
  # compute inverse of the V_c matrix
  Vinv <- try(solve(V), silent = TRUE)
  if(!is.matrix(Vinv)) Vinv <- diag(NaN, nrow=m, ncol=m)
  # 4) COMPUTE THE OPTIMAL WEIGTHS
  switch(type,
         # ASYMPTOTIC VARIANCE
         AVAR={
           # compute numertator
           VRowSums <- Vinv %*% ones
           # compute denominator
           VTotSum <- c(t(ones) %*% VRowSums)
           # optimal weights
           w <- c(VRowSums) / VTotSum
         },
         # ASYMPTOTIC BIAS
         ABIAS={
           VRawSums <- Vinv %*% ones
           VTotSum <- c(t(ones) %*% VRawSums)
           S <- Vinv %*% B
           ScolSums <- c(t(ones) %*% S)
           Q <- c(t(B) %*% S)
           # compute numertator
           num <- c(Q * VRawSums) - c(ScolSums * S)
           # compute denominator
           den <- c(Q * VTotSum) - ScolSums^2
           # optimal weights
           w <- num/den
         },
         # AYMPTOTIC MEAN-SQUARED-ERROR
         AMSE={
           VRawSums <- Vinv %*% ones
           VTotSum <- c(t(ones) %*% VRawSums)
           S <- Vinv %*% B
           ScolSums <- c(t(ones) %*% S)
           Q <- c(t(B) %*% S)
           # compute numerator
           num <- c((1 + Q) * VRawSums) - c(ScolSums * S)
           # compute denominator
           den <- c((1 + Q) * VTotSum) - ScolSums^2
           # optimal weights
           w <- num/den
         },
         # NAIVE APPROACH
         NOWEI={
           w <- rep(1/m, m)
         }
  )
  # 4) COMPUTE THE POOLED ESTIMATE
  gamHatP <- sum(w * gamHat)
  # 5) COMPUTE AN ESTIMATE OF THE ASYMPTOTIC VARIANCE OF THE POOLED ESTIMATOR
  VarGamHatP <- c(t(w) %*% V %*% w) / K
  # 6) DEFINE THE ASYMPTOTIC BIAS TERM OF THE POOLED ESTIMATOR
  BiasGamHatP <- 0
  if(bias) BiasGamHatP <- sum(w * B) / sqrt(K)
  # 7) DEFINE AN APPROXIMATE (1-alpha)100% CONFIDENCE INTERVAL FOR THE TAIL INDEX
  ME <- sqrt(VarGamHatP) * qnorm(1-alpha/2)
  # defines an approximate (1-alpha)100% confidence interval for the tail index
  CIGamHatP <- c(gamHatP-BiasGamHatP-ME, gamHatP-BiasGamHatP+ME)
  # 8) PERFORM HYPOTHESYS TESTING FOR EQUALITY OF TAIL INDICES
  switch(test,
         KINS={
           gamHDelta <- array(gamHat, c(m,1)) - gamHatP*ones
           # COMPUTE THE TEST STATISTIC
           logLikR <- K * c(t(gamHDelta) %*% Vinv %*% (gamHDelta))
         },
         DPS={
           Vtest <- diag(gamHat^2)
           if(!is.null(etailCop)){
             Vtest <- gamHat%*%t(gamHat)
             Vtest[lower.tri(Vtest)] <- Vtest[lower.tri(Vtest)] * etailCop
             Vtest <- t(Vtest)
             Vtest[lower.tri(Vtest)] <- Vtest[lower.tri(Vtest)] * etailCop
           }
           Vtest <- D %*% Vtest %*% D
           Vtestinv <- try(solve(Vtest), silent = TRUE)
           if(!is.matrix(Vtestinv)) Vtestinv <- diag(NaN, nrow=m, ncol=m)

           gamHatPtest <- c(t(ones) %*% Vtestinv %*% gamHat) / c(t(ones) %*% Vtestinv %*% ones)
           gamHDelta <- array(gamHat, c(m,1)) - gamHatPtest*ones
           # COMPUTE THE TEST STATISTIC
           logLikR <- K * c(t(gamHDelta) %*% Vtestinv %*% (gamHDelta))
         }
  )
  # compute the p-value
  PVal <- 1 - pchisq(q=logLikR, df=m-1)

  return(list(gamHatP=gamHatP-BiasGamHatP,
              VarGamHatP=VarGamHatP,
              CIGamHatP=CIGamHatP,
              BiasGamHatP=BiasGamHatP,
              logLikR=logLikR,
              critVal=critVal,
              PVal=PVal))
}

################################################
######################## NONSTATIONARY SETTING
################################################
# Generate Dirichlet
rdirichlet <- function(n = 1, alpha) {
  Gam <- matrix(0, n, length(alpha))
  for (i in 1:length(alpha)) Gam[, i] <- rgamma(n, shape = alpha[i])
  Gam / rowSums(Gam)
}


# Extreme quantile with asymmetric credibility intervals - continuous GP
QAhat_c <- function(cx, threshold, sig.ps, gam.ps, n, qlev, k, alpha = 0.05) {
  Q <- threshold + sig.ps * (((n * qlev) / (k * cx))^(-gam.ps) - 1) / gam.ps
  return(c(quantile(Q, alpha / 2), mean(Q), quantile(Q, 1 - alpha / 2)))
}
# Extreme quantile with symmetric credibility intervals - continuous GP
QShat_c <- function(cx, threshold, sig.ps, gam.ps, n, qlev, k, alpha = 0.05) {
  Q <- threshold + sig.ps * (((n * qlev) / (k * cx))^(-gam.ps) - 1) / gam.ps
  Qm <- mean(Q)
  Qs <- sd(Q)
  return(c(Qm - qnorm(1 - alpha / 2) * Qs, Qm, Qm + qnorm(1 - alpha / 2) * Qs))
}
# Extreme quantile with asymmetric credibility intervals - discrete GP
QAhat_d <- function(cx, threshold, sig.ps, gam.ps, n, qlev, k, alpha = 0.05) {
  Q <- floor(
    threshold + sig.ps * (((n * qlev) / (k * cx))^(-gam.ps) - 1) / gam.ps - 1
  )
  return(c(
    floor(quantile(Q, alpha / 2)),
    mean(Q),
    floor(quantile(Q, 1 - alpha / 2))
  ))
}
# Extreme quantile with symmetric credibility intervals - discrete GP
QShat_d <- function(cx, threshold, sig.ps, gam.ps, n, qlev, k, alpha = 0.05) {
  Q <- floor(
    threshold + sig.ps * (((n * qlev) / (k * cx))^(-gam.ps) - 1) / gam.ps - 1
  )
  Qm <- mean(Q)
  Qs <- sd(Q)
  return(c(
    floor(Qm - qnorm(1 - alpha / 2) * Qs),
    Qm,
    floor(Qm + qnorm(1 - alpha / 2) * Qs)
  ))
}

# Continuous GP density
h.gpd <- function(x, gam, eps = 1e-6) {
  if (gam > eps) {
    res <- ifelse(x > 0, (1 + gam * x)^(-1 / gam - 1), 0)
  }
  if (gam < -eps) {
    res <- ifelse(
      x > 0,
      ifelse(x < -1 / gam, (1 + gam * x)^(-1 / gam - 1), 0),
      0
    )
  }
  if (abs(gam) <= eps) {
    res <- ifelse(x > 0, exp(-x), 0)
  }
  return(res)
}


# Continuous GP density at the intermediate level
preddens1_c <- function(theta, y, t) {
  loc <- t + theta[1] * (theta[3]^theta[2] - 1) / theta[2]
  scale <- theta[1] * theta[3]^theta[2]
  z <- (y - loc) / scale
  res <- h.gpd(z, theta[2])
  return(res)
}

# Continuous GP density at the extreme level
preddens2_c <- function(theta, p, y, t, k, n) {
  loc <- t + theta[1] * ((theta[3] * 1 / p * k / n)^theta[2] - 1) / theta[2]
  scale <- theta[1] * (theta[3] * 1 / p * k / n)^theta[2]
  z <- (y - loc) / scale
  res <- h.gpd(z, theta[2])
  return(res)
}

# Discrete GP density at the intermediate level
preddens1_d <- function(theta, y, t) {
  loc <- floor(t + theta[1] * (theta[3]^theta[2] - 1) / theta[2])
  scale <- theta[1] * theta[3]^theta[2]
  z1 <- pmax(0, (y - loc) / scale)
  z2 <- pmax(0, (y + 1 - loc) / scale)
  res <- (1 + theta[2] * z1)^(-1 / theta[2]) -
    (1 + theta[2] * z2)^(-1 / theta[2])
  return(res)
}

# Discrete GP density at the extreme level
preddens2_d <- function(theta, p, y, t, k, n) {
  loc <- floor(
    t + theta[1] * ((theta[3] * 1 / p * k / n)^theta[2] - 1) / theta[2]
  )
  scale <- theta[1] * (theta[3] * 1 / p * k / n)^theta[2]
  z1 <- pmax(0, (y - loc) / scale)
  z2 <- pmax(0, (y + 1 - loc) / scale)
  res <- (1 + theta[2] * z1)^(-1 / theta[2]) -
    (1 + theta[2] * z2)^(-1 / theta[2])
  return(res)
}


################################################################################
### START
### Main-routines (VISIBLE FUNCTIONS)
###
################################################################################

###########################################
######################## STATIONARY SETTING
###########################################

#' Maximum likelihood estimation of the parameters of the discrete generalized Pareto distribution
#'
#' Given a sample of exceedances, estimate the parameters via maximum likelihood along with \eqn{1-\alpha} level confidence intervals.
#'
#' @param excess vector of positive exceedances, i.e., \eqn{Y - t \mid Y > t}, with \eqn{t} being the threshold
#' @param alpha level for confidence interval of scale and shape parameters. Default: 0.05, giving 95\% confidence intervals
#' @return a list with elements
#' \itemize{
#' \item \code{mle} vector of dimension 2 containing estimated scale and shape parameters
#' \item \code{CI} matrix of dimension 2 by 2 containing the \eqn{1-\alpha} level confidence intervals for scale and shape
#' }
#' @references
#' Hitz, A.S., G. Samorodnistsky and R.A. Davis (2024). Discrete Extremes, \emph{Journal of Data Science}, 22(\bold{4}), pp. 524-536.
#' @export
#' @examples
#' fitdGPD(rpois(1000,2))
fitdGPD <- function(excess, alpha = 0.05) {
  k <- length(excess)
  ntries <- 100
  lb <- c(log(0.01), 0.001)
  ub <- c(log(3), 1)
  start <- matrix(NA, ntries, 2)
  nllk <- numeric()
  # initialization
  for (t in 1:ntries) {
    d <- runif(1)
    start[t, ] <- lb * (1 - d) + ub * d
    nllk[t] <- nll_dgpd(start[t, ], excess)
  }
  initpar <- start[which(nllk == min(nllk)), ]
  # estimation
  est <- optim(
    initpar,
    fn = nll_dgpd,
    control = list(maxit = 1e6),
    hessian = FALSE,
    excess = excess
  )
  mle <- est$par
  if (est$convergence > 0) {
    warning("Optimization algorithm did not converge.")
  }
  # extract hessian
  est2 <- optim(
    c(exp(mle[1]), mle[2]),
    fn = nll_dgpd,
    control = list(maxit = 1),
    hessian = TRUE,
    excess = excess,
    constraint = FALSE
  )
  # confidence intervals
  vcov <- solve(est2$hessian)
  se <- sqrt(diag(vcov))
  lower <- mle + qnorm(alpha / 2) * se
  upper <- mle + qnorm(1 - alpha / 2) * se
  CI <- rbind(lower, upper)
  names(mle) <- colnames(CI) <- c("scale", "shape")
  list("mle" = mle, "CI" = CI)
}


#' @title Estimation of generalized Pareto distributions
#' @description Bayesian or frequentist estimation of the scale and shape parameters of the continuous or discrete generalized Pareto distribution.
#' @param data numeric vector of length n containing complete data values (under and above threshold)
#' @param k double indicating the effective sample size. Default: 10
#' @param pn numeric vector containing one or more percentile level at which extreme quantiles are estimated, with \eqn{p < k/n}. Default: NULL
#' @param type string indicating distribution types. Default: \code{c('continuous','discrete')}
#' @param method string indicating estimation methods. Default: \code{c('bayesian', 'frequentist')}
#' @param prior string indicating prior distribution (uniform or empirical). Default: \code{'empirical'}
#' @param start list of 2 containing starting values for scale and shape parameters. Default: \code{NULL}
#' @param sig0 double indicating the initial value for the update of the variance in the MCMC algorithm. Default: \code{NULL}
#' @param nsim double indicating the total number of iterations of the MCMC algorithm in the Bayesian estimation case. Default: 5000L
#' @param burn double indicating the number of iterations to exclude in the MCMC algorithm of the Bayesian estimation case. Default: 1000L
#' @param p double indicating the desired overall sampler acceptance probability. Default: 0.234
#' @param optim.method string indicating the optimization method in the frequentist estimation case. Default: \code{'Nelder-Mead'}
#' @param control list containing additional parameters for the minimization function \link[stats]{optim}. Default: \code{NULL}
#' @param ... other arguments passed to the function
#' @return a list with the following elements
#' \itemize{
#' \item Bayesian estimation case
#' \itemize{
#' \item \code{Q.est} matrix with \code{nsim-burn} rows and \code{length(pn)} columns containing the posterior sample of the
#' extreme quantile estimated at level given in pn
#' \item \code{post_sample} matrix with \code{nsim-burn} rows and 2 columns containing the posterior sample of the scale and
#' shape parameters for the continuous or discrete generalized Pareto distribution
#' \item \code{burn} double indicating the number of iterations excluded in the MCMC algorithm
#' \item \code{straight.reject} vector of length \code{nsim-burn+1} indicating the iterations in which the proposed parameters do not respect basic constraints
#' \item \code{sig.vec} vector of length \code{nsim}\eqn{-\lfloor(5 / (p (1 - p)))\rfloor+1} containing the values of the variance updated at each iteration of the MCMC algorithm
#' \item \code{accept.prob} matrix containing the values of acceptance probability (second column) corresponding to specific iterations (first column)
#' \item \code{msg} character string containing an output message on the result of the Bayesian estimation procedure
#' \item \code{mle} vector of length 2 containing the maximum likelihood estimates of the scale and shape parameters of the continuous or discrete generalized Pareto distribution
#' \item \code{t} double indicating the threshold for the generalized Pareto model, corresponding to the \eqn{n-k}th order statistic of the sample
#' }
#' \item Frequentist estimation case
#' \itemize{
#' \item \code{est} vector of length 2 containing the maximum likelihood estimates of the scale and shape parameters of the continuous or discrete generalized Pareto distribution
#' \item \code{t} double indicating the threshold
#' \item \code{Q.est} vector of dimension \code{length(pn)} containing the estimates of the extreme quantile at level given in pn
#' \item \code{VarCov} \eqn{2 \times 2} variance-covariance matrix of (gamma, sigma)
#' \item \code{Q.VC} variance of Q.est
#' \item \code{msg} character string containing an output message on the result of the frequentist estimation procedure
#' }
#' }
#' @seealso
#'  \code{\link[evd]{fpot}}
#' @references
#' Dombry, C., S. Padoan and S. Rizzelli (2025). Asymptotic theory for Bayesian inference and prediction: from the ordinary to a conditional Peaks-Over-Threshold method, arXiv:2310.06720v2.
#' @export
#' @examples
#' \dontrun{
#' # generate data
#' set.seed(1234)
#' n <- 500
#' samp <- evd::rfrechet(n,0,3,4)
#' # set effective sample size and threshold
#' k <- 50
#' threshold <- sort(samp,decreasing = TRUE)[k+1]
#' # preliminary mle estimates of scale and shape parameters
#' mlest <- evd::fpot(samp, threshold)
#' # empirical bayes procedure
#' proc <- estPOT(
#'   samp,
#'   k = k,
#'   pn = c(0.01, 0.005),
#'   type = "continuous",
#'   method = "bayesian",
#'   prior = "empirical",
#'   start = as.list(mlest$estimate),
#'   sig0 = 0.1)
#'  }
#' @importFrom evd fpot
#' @importFrom crch dtt
estPOT <- function(
  data,
  k = 10L,
  pn = NULL,
  type = c("continuous", "discrete"),
  method = c("bayesian", "frequentist"),
  prior = "empirical",
  start = NULL,
  sig0 = NULL,
  nsim = 5000L,
  burn = 1000L,
  p = 0.234,
  optim.method = "Nelder-Mead",
  control = NULL,
  ...
) {
  type <- match.arg(type)
  if (type == "continuous") {
    res <- cPOT(
      data,
      k,
      pn,
      method,
      prior,
      start,
      sig0,
      nsim,
      burn,
      p,
      optim.method,
      control,
      verbose = TRUE
    )
  } else {
    res <- dPOT(
      data,
      k,
      pn,
      method,
      prior,
      start,
      sig0,
      nsim,
      burn,
      p,
      optim.method,
      control,
      verbose = TRUE
    )
  }
}

#' Bayesian extreme quantile
#'
#' Given posterior samples for the parameters of the continuous or discrete generalized Pareto distribution,
#' return the posterior mean and \eqn{1-\alpha} level credibility intervals of the extreme quantile
#'
#' @param threshold threshold for the generalized Pareto model, corresponding to the \eqn{n-k}th order statistic of the sample
#' @param postsamp a \code{m} by \code{2} matrix with columns containing the posterior samples of scale and shape parameters of the generalized Pareto distribution, respectively
#' @param k integer, number of exceedances for the generalized Pareto (only used if \code{extrapolation=TRUE})
#' @param n integer, number of observations in the full sample. Must be greater than \code{k}  (only used if \code{extrapolation=TRUE})
#' @param retp double indicating the value of the return period
#' @param alpha level for credibility interval. Default: 0.05 giving 95\% credibility intervals
#' @param type string indicating distribution types. Default: \code{c('continuous','discrete')}
#' @return a list with components
#' \itemize{
#' \item \code{mQ} posterior mean of the extreme quantile
#' \item \code{ciQ} vector of dimension 2 returning the \eqn{\alpha/2} and \eqn{1-\alpha/2} empirical quantiles of the posterior distribution of the extreme quantile
#' }
#' @export
#' @examples
#' \dontrun{
#' # generate data
#' set.seed(1234)
#' n <- 500
#' samp <- evd::rfrechet(500,0,3,4)
#' # set effective sample size and threshold
#' k <- 50
#' threshold <- sort(samp,decreasing = TRUE)[k+1]
#' # preliminary mle estimates of scale and shape parameters
#' mlest <- evd::fpot(samp, threshold)
#' # empirical bayes procedure
#' proc <- estPOT(
#'   samp,
#'   k = k,
#'   pn = c(0.01, 0.005),
#'   type = "continuous",
#'   method = "bayesian",
#'   prior = "empirical",
#'   start = as.list(mlest$estimate),
#'   sig0 = 0.1)
#' # extreme quantile corresponding to a return period of 100
#' extBQuant(
#'   proc$t,proc$post_sample,
#'   k,
#'   n,
#'   100,
#'   0.05,
#'   type = "continuous")
#' }
extBQuant <- function(
  threshold,
  postsamp,
  k,
  n,
  retp,
  alpha = 0.05,
  type = c("continuous", "discrete")
) {
  if (type == "continuous") {
    out <- threshold +
      postsamp[, 1] * ((k / (n * 1 / retp))^postsamp[, 2] - 1) / postsamp[, 2]
    mout <- mean(out)
    qout <- quantile(out, c(alpha / 2, (1 - alpha / 2)))
  } else {
    out <- threshold +
      postsamp[, 1] * ((k / (n * 1 / retp))^postsamp[, 2] - 1) / postsamp[, 2] -
      1
    mout <- floor(mean(out))
    qout <- floor(quantile(out, c(alpha / 2, (1 - alpha / 2))))
  }

  return(list("mQ" = mout, "ciQ" = qout))
}

#' Predictive posterior density of peak-over-threshold models
#'
#' Given posterior samples for the parameters of the continuous or discrete generalized Pareto distribution, return the predictive posterior density
#' of a peak above an intermediate or extreme threshold using the threshold stability property.
#'
#' @param x vector of length \code{r} containing the points at which to evaluate the density
#' @param postsamp a \code{m} by \code{2} matrix with columns containing the posterior samples of scale and shape parameters of the generalized Pareto distribution, respectively
#' @param threshold threshold for the generalized Pareto model, corresponding to the \eqn{n-k}th order statistic of the sample
#' @param type data type, either \code{"continuous"} or \code{"discrete"} for count data.
#' @param extrapolation logical; if \code{TRUE}, extrapolate using the threshold stability property
#' @param k integer, number of exceedances for the generalized Pareto (only used if \code{extrapolation=TRUE})
#' @param n integer, number of observations in the full sample. Must be greater than \code{k}  (only used if \code{extrapolation=TRUE})
#' @param p scalar tail probability for the extrapolation. Must be smaller than \eqn{k/n} (only used if \code{extrapolation=TRUE})
#' @return a vector of length \code{r} of posterior predictive density values associated to \code{x}
#' @export
#' @examples
#' \dontrun{
#' # generate data
#' set.seed(1234)
#' n <- 500
#' samp <- evd::rfrechet(n,0,3,4)
#' # set effective sample size and threshold
#' k <- 50
#' threshold <- sort(samp,decreasing = TRUE)[k+1]
#' # preliminary mle estimates of scale and shape parameters
#' mlest <- evd::fpot(samp, threshold)
#' # empirical bayes procedure
#' proc <- estPOT(
#'   samp,
#'   k = k,
#'   pn = c(0.01, 0.005),
#'   type = "continuous",
#'   method = "bayesian",
#'   prior = "empirical",
#'   start = as.list(mlest$estimate),
#'   sig0 = 0.1)
#' # predictive density estimation
#' yg <- seq(0, 50, by = 2)
#' nyg <- length(yg)
#' predDens_int <- predDens(
#'   yg,
#'   proc$post_sample,
#'   proc$t,
#'   "continuous",
#'   extrapolation = FALSE)
#' predDens_ext <- predDens(
#'   yg,
#'   proc$post_sample,
#'   proc$t,
#'   "continuous",
#'   extrapolation = TRUE,
#'   p = 0.001,
#'   k = k,
#'   n = n)
#' # plot
#' plot(
#'  x = yg,
#'  y = predDens_int,
#'  type = "l",
#'  lwd = 2,
#'  col = "dodgerblue",
#'  ylab = "",
#'  main = "Predictive posterior density")
#' lines(
#'  x = yg,
#'  y = predDens_ext,
#'  lwd = 2,
#'  col = "violet")
#' legend(
#'   "topright",
#'   legend = c("Intermediate threshold", "Extreme threshold"),
#'   lwd = 2,
#'   col = c("dodgerblue", "violet"))
#'   }
predDens <- function(
  x,
  postsamp,
  threshold,
  type = c("continuous", "discrete"),
  extrapolation = FALSE,
  p,
  k,
  n
) {
  extrapolation <- isTRUE(extrapolation)
  if (extrapolation) {
    stopifnot(
      length(k) == 1L,
      length(n) == 1L,
      length(p) == 1L
    )
    stopifnot(
      k <= n,
      p < 0.5,
      p > 0
    )
  }

  nx <- length(x)
  out <- numeric(nx)
  ns <- nrow(postsamp)

  if (type == "continuous" & extrapolation) {
    locExt <- threshold +
      postsamp[, 1] * ((n / k * p)^(-postsamp[, 2]) - 1) / postsamp[, 2]
    scaleExt <- postsamp[, 1] * (n / k * p)^(-postsamp[, 2])

    for (i in 1:nx) {
      out[i] <- mean(apply(
        cbind(locExt, scaleExt, postsamp[, 2]),
        1,
        Fdgpd_c,
        x = x[i]
      ))
    }
  } else if (type == "continuous" & !extrapolation) {
    for (i in 1:nx) {
      out[i] <- mean(apply(
        cbind(rep(threshold, ns), postsamp),
        1,
        Fdgpd_c,
        x = x[i]
      ))
    }
  } else if (type == "discrete" & extrapolation) {
    locExt <- threshold +
      floor(postsamp[, 1] * ((n / k * p)^(-postsamp[, 2]) - 1) / postsamp[, 2])
    scaleExt <- postsamp[, 1] * (n / k * p)^(-postsamp[, 2])

    for (i in 1:nx) {
      out[i] <- mean(apply(
        cbind(locExt, scaleExt, postsamp[, 2]),
        1,
        Fdgpd_d,
        x = x[i]
      ))
    }
  } else {
    for (i in 1:nx) {
      out[i] <- mean(apply(
        cbind(rep(threshold, ns), postsamp),
        1,
        Fdgpd_d,
        x = x[i]
      ))
    }
  }
  return(out)
}


#' Predictive quantile based on the generalized Pareto model
#'
#' Bayesian Generalize Pareto-based predictive quantile for continuous and discrete predictive distribution conditioned on intermediate and extreme levels.
#'
#' @param qlev double, quantile level
#' @param postsamp a \code{m} by \code{2} matrix with columns containing the posterior samples of scale and shape parameters of the generalized Pareto distribution, respectively
#' @param threshold threshold for the generalized Pareto model, corresponding to the \eqn{n-k}th order statistic of the sample
#' @param lb double, the lower bound of the admissible region for the quantile value
#' @param ub double, the upper bound of the admissible region for the quantile value
#' @param type data type, either \code{"continuous"} or \code{"discrete"} for count data.
#' @param extrapolation logical; if \code{TRUE}, extrapolate using the threshold stability property
#' @param k integer, number of exceedances for the generalized Pareto (only used if \code{extrapolation=TRUE})
#' @param n integer, number of observations in the full sample. Must be greater than \code{k}  (only used if \code{extrapolation=TRUE})
#' @param p scalar tail probability for the extrapolation. Must be smaller than \eqn{k/n} (only used if \code{extrapolation=TRUE})
#' @return a double indicating the value of the quantile
#' @examples
#' \dontrun{
#' # generate data
#' set.seed(1234)
#' n <- 500
#' samp <- evd::rfrechet(n,0,3,4)
#' # set effective sample size and threshold
#' k <- 50
#' threshold <- sort(samp,decreasing = TRUE)[k+1]
#' # preliminary mle estimates of scale and shape parameters
#' mlest <- evd::fpot(samp, threshold)
#' # empirical bayes procedure
#' proc <- estPOT(
#'   samp,
#'   k = k,
#'   pn = c(0.01, 0.005),
#'   type = "continuous",
#'   method = "bayesian",
#'   prior = "empirical",
#'   start = as.list(mlest$estimate),
#'   sig0 = 0.1)
#' # predictive density estimation
#' yg <- seq(0, 50, by = 2)
#' nyg <- length(yg)
#' predDens_int <- predDens(
#'   yg,
#'   proc$post_sample,proc$t,
#'   "continuous",
#'   extrapolation=FALSE)
#' predQuant_int <- predQuant(
#'   0.5,
#'   proc$post_sample,
#'   proc$t,
#'   proc$t + 0.01,
#'   50,
#'   "continuous",
#'   extrapolation = FALSE)
#' predDens_ext <- predDens(
#'   yg,
#'   proc$post_sample,
#'   proc$t,
#'   "continuous",
#'   extrapolation = TRUE,
#'   p = 0.001,
#'   k = k,
#'   n = n)
#' predQuant_ext <- predQuant(
#'   0.5,
#'   proc$post_sample,
#'   proc$t,
#'   proc$t + 0.01,
#'   100,
#'   "continuous",
#'   extrapolation = TRUE,
#'   p = 0.005,
#'   k = k,
#'   n = n)
#'   }
#' @export
predQuant <- function(
  qlev,
  postsamp,
  threshold,
  lb,
  ub,
  type = c("continuous", "discrete"),
  extrapolation = FALSE,
  p,
  k,
  n
) {
  extrapolation <- isTRUE(extrapolation)
  if (extrapolation) {
    stopifnot(
      length(k) == 1L,
      length(n) == 1L,
      length(p) == 1L
    )
    stopifnot(
      k <= n,
      p < 0.5,
      p > 0
    )
  }

  ns <- nrow(postsamp)

  if (type == "continuous" & extrapolation) {
    locExt <- threshold +
      postsamp[, 1] * ((n / k * p)^(-postsamp[, 2]) - 1) / postsamp[, 2]
    scaleExt <- postsamp[, 1] * (n / k * p)^(-postsamp[, 2])

    out <- Bqgpd_c(
      qlev,
      post = cbind(locExt, scaleExt, postsamp[, 2]),
      lower = lb,
      upper = ub
    )
  } else if (type == "continuous" & !extrapolation) {
    out <- Bqgpd_c(
      qlev,
      post = cbind(rep(threshold, ns), postsamp),
      lower = lb,
      upper = ub
    )
  } else if (type == "discrete" & extrapolation) {
    locExt <- threshold +
      floor(postsamp[, 1] * ((n / k * p)^(-postsamp[, 2]) - 1) / postsamp[, 2])
    scaleExt <- postsamp[, 1] * (n / k * p)^(-postsamp[, 2])

    out <- Bqgpd_d(
      qlev,
      post = cbind(locExt, scaleExt, postsamp[, 2]),
      lower = lb,
      upper = ub
    )
  } else {
    out <- Bqgpd_d(
      qlev,
      post = cbind(rep(threshold, ns), postsamp),
      lower = lb,
      upper = ub
    )
  }
  return(out)
}


#' Test on the effect of concomitant covariate on the extremes of the response variable
#'
#' Given observed data, perform a Kolmogorov-Smirnov type test comparing the cumulative distribution function of the concomitant covariate, defined as \eqn{X \mid Y > t}, with \eqn{t} being the threshold,
#' against the cumulative distribution function of the random vector of covariate.
#'
#' @param data design matrix of dimension \code{n} by \code{2} containing the complete data for the dependent variable (first column) and covariate (second column) in [0,1]
#' @param k integer, number of exceedances for the generalized Pareto
#' @param M integer, number of samples to draw from the posterior distrinution of the law of the concomitant covariate. Default: 1000
#' @param ng length of covariate grid
#' @param xg vector of covariate grid of dimension \code{ng} by \code{1} containing a sequence between zero and the last value of the corresponding covariate
#' @param bayes logical indicating the bootstrap method. If \code{FALSE}, a frequentist bootstrap on the empirical cumulative distribution function of the concomitant covariate is performed. Default to \code{TRUE}
#' @param C integer, hypermparameter entering the posterior distributyion of the law of the concomitant covariate. Default: 5
#' @param alpha double, significance level for the critical value of the test, computed as the \eqn{(1-alpha)} level empirical quantile of the sample of distances between the empirical cumulative distribution function of the concomitant and complete covariate. Default: 0.05
#' @return a list with components
#' \itemize{
#' \item \code{Delta} maximum observed distance between the empirical distribution functions of the concomitant and complete covariate
#' \item \code{DeltaM} vector of length M containing the sample of maximum distances between the empirical distribution function of the concomitant complete covariate
#' \item \code{critical} double, critical value for the test statistic, computed as the \eqn{(1-alpha)} level empirical quantile of DeltaM
#' \item \code{pval} double, p-value
#' }
#' @examples
#' \dontrun{
#' # generate data
#' set.seed(1234)
#' n <- 500
#' samp <- evd::rfrechet(n,0,1:n,4)
#' # set effective sample size and threshold
#' k <- 50
#' threshold <- sort(samp,decreasing = TRUE)[k+1]
#' # preliminary mle estimates of scale and shape parameters
#' mlest <- evd::fpot(samp,
#'  threshold,
#'  control=list(maxit = 500))
#' # empirical bayes procedure
#' proc <- estPOT(
#'   samp,
#'   k = k,
#'   pn = c(0.01, 0.005),
#'   type = "continuous",
#'   method = "bayesian",
#'   prior = "empirical",
#'   start = as.list(mlest$estimate),
#'   sig0 = 0.1)
#' # conditional predictive density estimation
#' yg <- seq(0, 50, by = 2)
#' nyg <- length(yg)
#' # estimation of scedasis function
#' # setting
#' M <- 1e3
#' C <- 5
#' alpha <- 0.05
#' bw <- .5
#' nsim <- 5000
#' burn <- 1000
#' # create covariate
#' # in sample obs
#' n_in = n
#' # number of years ahead
#' nY = 1
#' n_out = 365 * nY
#' # total obs
#' n_tot = n_in + n_out
#' # total covariate (in+out sample period)
#' x <- seq(0, 1, length = n_tot)
#' # in sample grid dimension for covariate
#' ng_in <- 150
#' xg <- seq(0, x[n_in], length = ng_in)
#' # in+out of sample grid
#' xg <- c(xg,
#'  seq(x[n_in + 1],
#'      x[(n_tot)],
#'      length = ng_in))
#' # in+out sample grid dimension
#' nxg <- length(xg)
#' xg <- array(xg, c(nxg, 1))
#' # in sample observations
#' samp_in <- samp[1:n_in]
#' ssamp_in <- sort(samp_in, decreasing = TRUE, index = TRUE)
#' x_in <- x[1:n_in] # in sample covariate
#' xs <- x_in[ssamp_in$ix[1:k]] # in sample concomitant covariate
#' # test on covariate effect
#' test <- scedastic.test(
#'   cbind(samp, x[1:n]),
#'   k,
#'   M,
#'   array(xg[1:ng_in], c(ng_in, 1)),
#'   ng_in,
#'   TRUE,
#'   C,
#'   0.05
#' )
#' }
#' @export
#' @references
#' Dombry, C., S. Padoan and S. Rizzelli (2025). Asymptotic theory for Bayesian inference and prediction: from the ordinary to a conditional Peaks-Over-Threshold method, arXiv:2310.06720v2.
#'
scedastic.test <- function(
  data,
  k,
  M = 1000L,
  xg,
  ng,
  bayes = TRUE,
  C = 5L,
  alpha = 0.05
) {
  # SORT DATA
  sdata <- sort(data[, 1], decreasing = TRUE, index.return = TRUE)
  xs <- data[sdata$ix[1:k], 2]
  # COMPUTE THE EMPIRICAL DISTRIBUTION RELATIVE TO P*
  Fsk <- ecdf(xs)
  # COMPUTE THE EMPIRICAL DISTRIBUTION RELATIVE TO P
  Fn <- ecdf(data[, 2])
  # COMPUTE MAX DISTANCE
  Delta <- sqrt(k) * max(abs(Fsk(xg) - Fn(xg)))
  ### BOOTSTRAP STEP:
  fboot <- function(xs, k, xg) {
    # weights generation
    w <- rexp(k)
    w <- w / sum(w) - 1 / k
    myf <- function(data, xg, w) return(sum(w * (data <= xg)))
    return(sqrt(k) * max(abs(apply(xg, 2, myf, data = xs, w = w))))
  }
  ### BAYESIAN BOOTSTRAP:
  bboot <- function(shape1, shape2, k) {
    theta <- rdirichlet(n = 1, shape1 + shape2)
    Fsb <- cumsum(theta)
    Fn <- 1 / k * cumsum(shape2)
    return(sqrt(k) * max(abs(Fsb - Fn)))
  }
  if (isTRUE(bayes)) {
    npoints <- function(I, xs) return(sum((xs >= I[1]) & (xs <= I[2])))
    shape1 <- C * diff(xg)
    shape2 <- apply(cbind(xg[1:(ng - 1)], xg[2:ng]), 1, npoints, xs = xs)
    DeltaM <- replicate(M, bboot(shape1, shape2, k))
  } else {
    DeltaM <- replicate(M, fboot(xs, k, xg))
  }

  return(list(
    Delta = Delta,
    DeltaM = DeltaM,
    critical = quantile(DeltaM, 1 - alpha),
    pval = mean(Delta < DeltaM)
  ))
}

#' Test on the effect of concomitant covariate on the extremes of the response variable
#'
#' Given observed data, perform a Kolmogorov-Smirnov type test comparing the cumulative distribution function of the concomitant covariate, defined as \eqn{X \mid Y > t}, with \eqn{t} being the threshold,
#' against the cumulative distribution function of the random vector of covariate.
#'
#' @param data design matrix of dimension \code{n} by \code{2} containing the complete data for the dependent variable (first column) and covariate (second column) in [0,1]
#' @param k integer, number of exceedances for the generalized Pareto
#' @param M integer, number of samples to draw from the posterior distrinution of the law of the concomitant covariate. Default: 1000
#' @param ng length of covariate grid
#' @param xg vector of covariate grid of dimension \code{ng} by \code{1} containing a sequence between zero and the last value of the corresponding covariate
#' @param bayes logical indicating the bootstrap method. If \code{FALSE}, a frequentist bootstrap on the empirical cumulative distribution function of the concomitant covariate is performed. Default to \code{TRUE}
#' @param C integer, hypermparameter entering the posterior distributyion of the law of the concomitant covariate. Default: 5
#' @param alpha double, significance level for the critical value of the test, computed as the \eqn{(1-alpha)} level empirical quantile of the sample of distances between the empirical cumulative distribution function of the concomitant and complete covariate. Default: 0.05
#' @return a list with components
#' \itemize{
#' \item \code{Delta} maximum observed distance between the empirical distribution functions of the concomitant and complete covariate
#' \item \code{DeltaM} vector of length M containing the sample of maximum distances between the empirical distribution function of the concomitant complete covariate
#' \item \code{critical} double, critical value for the test statistic, computed as the \eqn{(1-alpha)} level empirical quantile of DeltaM
#' \item \code{pval} double, p-value
#' }
#' @keywords internal
#' @export
schedastic.test <- function(
  data,
  k,
  M = 1000L,
  xg,
  ng,
  bayes = TRUE,
  C = 5L,
  alpha = 0.05
) {
  .Deprecated(new = "scedastic.test")
  scedastic.test(
    data = data,
    k = k,
    M = M,
    xg = xg,
    ng = ng,
    bayes = bayes,
    C = C,
    alpha = alpha
  )
}

#' Estimation of the scedasis function
#'
#' Kernel-based method for the estimation of the scedasis function. Given the values of the complete and concomitant covariate, defined as \eqn{X \mid Y > t}, with \eqn{t} being the threshold, it returns
#' a matrix containing a posterior sample of the scedasis function for each covariate value.
#'
#' @param N integer, number of samples to draw from the distribution of the concomitant covariate
#' @param x one-dimensional vector of in-sample covariate in [0,1]
#' @param xs one-dimensional vector of concomitant covariate
#' @param xg one-dimensional vector of length m containing the grid of in-sample and possibly out-sample covariate in [0,1]
#' @param bw double, bandwidth for the computation of the kernel
#' @param k integer, number of exceedances for the generalized Pareto
#' @param C integer, hyperparameter entering the posterior distribution of the law of the concomitant covariate. Default: 5
#' @return an \code{N} by \code{m} matrix containing the values of the posterior samples of the scedasis function (rows) for each value of \code{xg} (columns)
#' @export
#' @examples
#' \dontrun{
#' # generate data
#' set.seed(1234)
#' n <- 500
#' samp <- evd::rfrechet(n, 0, 1:n, 4)
#' # set effective sample size and threshold
#' k <- 50
#' threshold <- sort(samp, decreasing = TRUE)[k+1]
#' # preliminary mle estimates of scale and shape parameters
#' mlest <- evd::fpot(
#'   samp,
#'   threshold,
#'   control = list(maxit = 500))
#' # empirical bayes procedure
#' proc <- estPOT(
#'   samp,
#'   k = k,
#'   pn = c(0.01, 0.005),
#'   type = "continuous",
#'   method = "bayesian",
#'   prior = "empirical",
#'   start = as.list(mlest$estimate),
#'   sig0 = 0.1)
#' # conditional predictive density estimation
#' yg <- seq(0, 50, by = 2)
#' nyg <- length(yg)
#' # estimation of scedasis function
#' # setting
#' M <- 1e3
#' C <- 5
#' alpha <- 0.05
#' bw <- .5
#' nsim <- 5000
#' burn <- 1000
#' # create covariate
#' # in sample obs
#' n_in = n
#' # number of years ahead
#' nY = 1
#' n_out = 365 * nY
#' # total obs
#' n_tot = n_in + n_out
#' # total covariate (in+out sample period)
#' x <- seq(0, 1, length = n_tot)
#' # in sample grid dimension for covariate
#' ng_in <- 150
#' xg <- seq(0, x[n_in], length = ng_in)
#' # in+out of sample grid
#' xg <- c(xg, seq(x[n_in + 1], x[(n_tot)], length = ng_in))
#' # in+out sample grid dimension
#' nxg <- length(xg)
#' xg <- array(xg, c(nxg, 1))
#' # in sample observations
#' samp_in <- samp[1:n_in]
#' ssamp_in <- sort(samp_in, decreasing = TRUE, index = TRUE)
#' x_in <- x[1:n_in] # in sample covariate
#' xs <- x_in[ssamp_in$ix[1:k]]
#' # in sample concomitant covariate
#' # estimate scedasis function over the in and out of sample period
#' res_stat <- apply(
#'   xg,
#'   1,
#'   cpost_stat,
#'   N = nsim - burn,
#'   x = x_in,
#'   xs = xs,
#'   bw = bw,
#'   k = k,
#'   C = C
#' )
#' }
cpost_stat <- function(N, x, xs, xg, bw, k, C = 5L) {
  sdist <- sum(abs(xs - xg) <= bw)
  a <- C * bw + sdist
  b <- C * (1 - bw) + k - sdist
  den <- mean(abs(x - xg) <= bw)
  return(rbeta(N, a, b) / den)
}


#' Conditional Bayesian extreme quantile
#'
#' Given posterior samples for the parameters of the continuous or discrete generalized Pareto distribution and scedasis function for a set of covariates,
#' return the posterior mean and \eqn{1-\alpha} level credibility intervals of the extreme quantile for each value of the scedasis function
#'
#' @param cx an \code{m} by \code{p} matrix, obtained by evaluating the scedasis function for every of the \code{p} covariate values and every \code{m} posterior draw
#' @param postsamp a \code{m} by \code{2} matrix with columns containing the posterior samples of scale and shape parameters of the generalized Pareto distribution, respectively
#' @param threshold threshold for the generalized Pareto model, corresponding to the \eqn{n-k}th order statistic of the sample
#' @param k integer, number of exceedances for the generalized Pareto (only used if \code{extrapolation=TRUE})
#' @param n integer, number of observations in the full sample. Must be greater than \code{k}
#' @param qlev double indicating the percentile level at which the extreme quantile is estimated. Must be smaller than \eqn{k/n}
#' @param type string indicating distribution types. Default: \code{c('continuous','discrete')}
#' @param confint type of confidence interval. Default: \code{c('asymmetric', 'symmetric')}
#' @param alpha level for credibility interval. Default 0.05, giving 95\% credibility intervals
#' @param ... additional arguments, for back-compatilibity
#' @return a list with components
#' \itemize{
#' \item \code{mQ} posterior mean of the extreme quantile
#' \item \code{ciQ} vector of dimension 2 returning the \eqn{\alpha/2} and \eqn{1-\alpha/2} empirical quantiles of the posterior distribution of the extreme quantile
#' }
#' @export
#' @examples
#' \dontrun{
#' # generate data
#' set.seed(1234)
#' n <- 500
#' samp <- evd::rfrechet(n,0,1:n,4)
#' # set effective sample size and threshold
#' k <- 50
#' threshold <- sort(samp,decreasing = TRUE)[k+1]
#' # preliminary mle estimates of scale and shape parameters
#' mlest <- evd::fpot(
#'   samp,
#'   threshold,
#'   control = list(maxit = 500))
#' # empirical bayes procedure
#' proc <- estPOT(
#'   samp,
#'   k = k,
#'   pn = c(0.01, 0.005),
#'   type = "continuous",
#'   method = "bayesian",
#'   prior = "empirical",
#'   start = as.list(mlest$estimate),
#'   sig0 = 0.1)
#' # conditional predictive density estimation
#' yg <- seq(0, 50, by = 2)
#' nyg <- length(yg)
#' # estimation of scedasis function
#' # setting
#' M <- 1e3
#' C <- 5
#' alpha <- 0.05
#' bw <- .5
#' nsim <- 5000
#' burn <- 1000
#' # create covariate
#' # in sample obs
#' n_in = n
#' # number of years ahead
#' nY = 1
#' n_out = 365 * nY
#' # total obs
#' n_tot = n_in + n_out
#' # total covariate (in+out sample period)
#' x <- seq(0, 1, length = n_tot)
#' # in sample grid dimension for covariate
#' ng_in <- 150
#' xg <- seq(0, x[n_in], length = ng_in)
#' # in+out of sample grid
#' xg <- c(xg, seq(x[n_in + 1], x[(n_tot)], length = ng_in))
#' # in+out sample grid dimension
#' nxg <- length(xg)
#' xg <- array(xg, c(nxg, 1))
#' # in sample observations
#' samp_in <- samp[1:n_in]
#' ssamp_in <- sort(samp_in, decreasing = TRUE, index = TRUE)
#' x_in <- x[1:n_in] # in sample covariate
#' xs <- x_in[ssamp_in$ix[1:k]] # in sample concomitant covariate
#' # estimate scedasis function over the in and out of sample period
#' res_stat <- apply(
#'   xg,
#'   1,
#'   cpost_stat,
#'   N = nsim - burn,
#'   x = x_in,
#'   xs = xs,
#'   bw = bw,
#'   k = k,
#'   C = C
#' )
#' # conditional predictive posterior density
#' probq = 1 - 0.99
#' res_AQ <- extBQuantx(
#'  cx = res_stat,
#'  postsamp = proc$post_sample,
#'  threshold = proc$t,
#'  n = n,
#'  qlev = probq,
#'  k = k,
#'  type = "continuous",
#'  confint = "asymmetric")
#'  }
extBQuantx <- function(
    cx,
    postsamp,
    threshold,
    n,
    qlev,
    k,
    type = c("continuous", "discrete"),
    confint = c("asymmetric", "symmetric"),
    alpha = 0.05,
    ...
) {
  args <- list(...)
  sig.ps <- postsamp[, 1]
  gam.ps <- postsamp[, 2]
  if (!is.null(args$confinty)) {
    confinty <- args$confinty
  } else {
    confinty <- confint
  }
  if (type == "continuous") {
    if (confinty == "asymmetric") {
      out <- apply(
        X = cx,
        MARGIN = 2,
        FUN = QAhat_c,
        threshold = threshold,
        sig.ps = sig.ps,
        gam.ps = gam.ps,
        n = n,
        qlev = qlev,
        k = k,
        alpha = alpha
      )
    } else {
      out <- apply(
        X = cx,
        MARGIN = 2,
        FUN = QShat_c,
        threshold = threshold,
        sig.ps = sig.ps,
        gam.ps = gam.ps,
        n = n,
        qlev = qlev,
        k = k,
        alpha = alpha
      )
    }
  } else {
    if (confinty == "asymmetric") {
      out <- apply(
        X = cx,
        MARGIN = 2,
        FUN = QAhat_d,
        threshold = threshold,
        sig.ps = sig.ps,
        gam.ps = gam.ps,
        n = n,
        qlev = qlev,
        k = k,
        alpha = alpha
      )
    } else {
      out <- apply(
        X = cx,
        MARGIN = 2,
        FUN = QShat_d,
        threshold = threshold,
        sig.ps = sig.ps,
        gam.ps = gam.ps,
        n = n,
        qlev = qlev,
        k = k,
        alpha = alpha
      )
    }
  }

  return(out)
}


#' Conditional predictive posterior density of peaks-over-threshold models
#'
#' Given posterior samples for the parameters of the continuous or discrete generalized Pareto distribution and scedasis function for a set of covariates, evaluated at every draw of the latter,
#' return the predictive posterior density of a peak above an intermediate or extreme threshold using the threshold stability property.
#'
#' @param x vector of length \code{r} containing the points at which to evaluate the density
#' @param postsamp a \code{m} by \code{2} matrix with columns containing the posterior samples of scale and shape parameters of the generalized Pareto distribution, respectively
#' @param scedasis an \code{m} by \code{p} matrix, obtained by evaluating the scedasis function for every of the \code{p} covariate values and every \code{m} posterior draw
#' @param threshold threshold for the generalized Pareto model, corresponding to the \eqn{n-k}th order statistic of the sample
#' @param type data type, either \code{"continuous"} or \code{"discrete"} for count data.
#' @param extrapolation logical; if \code{TRUE}, extrapolate using the threshold stability property
#' @param k integer, number of exceedances for the generalized Pareto (only used if \code{extrapolation=TRUE})
#' @param n integer, number of observations in the full sample. Must be greater than \code{k}  (only used if \code{extrapolation=TRUE})
#' @param p scalar tail probability for the extrapolation. Must be smaller than \eqn{k/n} (only used if \code{extrapolation=TRUE})
#' @return a list with components
#' \itemize{
#' \item \code{x} the vector at which the posterior density is evaluated
#' \item \code{preddens} an \code{r} by \code{p} matrix of predictive density corresponding to each combination of \code{x} and scedasis value
#' }
#' @export
#' @examples
#' \dontrun{
#' # generate data
#' set.seed(1234)
#' n <- 500
#' samp <- evd::rfrechet(n,0,1:n,4)
#' # set effective sample size and threshold
#' k <- 50
#' threshold <- sort(samp,decreasing = TRUE)[k+1]
#' # preliminary mle estimates of scale and shape parameters
#' mlest <- evd::fpot(samp, threshold, control=list(maxit = 500))
#' # empirical bayes procedure
#' proc <- estPOT(
#'   samp,
#'   k = k,
#'   pn = c(0.01, 0.005),
#'   type = "continuous",
#'   method = "bayesian",
#'   prior = "empirical",
#'   start = as.list(mlest$estimate),
#'   sig0 = 0.1)
#' # conditional predictive density estimation
#' yg <- seq(0, 50, by = 2)
#' nyg <- length(yg)
#' # estimation of scedasis function
#' # setting
#' M <- 1e3
#' C <- 5
#' alpha <- 0.05
#' bw <- .5
#' nsim <- 5000
#' burn <- 1000
#' # create covariate
#' # in sample obs
#' n_in = n
#' # number of years ahead
#' nY = 1
#' n_out = 365 * nY
#' # total obs
#' n_tot = n_in + n_out
#' # total covariate (in+out sample period)
#' x <- seq(0, 1, length = n_tot)
#' # in sample grid dimension for covariate
#' ng_in <- 150
#' xg <- seq(0, x[n_in], length = ng_in)
#' # in+out of sample grid
#' xg <- c(xg, seq(x[n_in + 1], x[(n_tot)], length = ng_in))
#' # in+out sample grid dimension
#' nxg <- length(xg)
#' xg <- array(xg, c(nxg, 1))
#' # in sample observations
#' samp_in <- samp[1:n_in]
#' ssamp_in <- sort(samp_in, decreasing = TRUE, index = TRUE)
#' x_in <- x[1:n_in] # in sample covariate
#' xs <- x_in[ssamp_in$ix[1:k]] # in sample concomitant covariate
#' # estimate scedasis function over the in and out of sample period
#' res_stat <- apply(
#'   xg,
#'   1,
#'   cpost_stat,
#'   N = nsim - burn,
#'   x = x_in,
#'   xs = xs,
#'   bw = bw,
#'   k = k,
#'   C = C
#' )
#' # conditional predictive posterior density
#' yg <- seq(500, 5000, by = 100)
#' nyg = length(yg)
#' # intermediate threshold
#' predDens_intx <- predDensx(
#'   x = yg,
#'   postsamp = proc$post_sample,
#'   scedasis = res_stat,
#'   threshold = proc$t,
#'   "continuous",
#'   extrapolation = FALSE)
#' # extreme threshold
#' predDens_extx <- predDensx(
#'   x = yg,
#'   postsamp = proc$post_sample,
#'   scedasis = res_stat,
#'   threshold = proc$t,
#'   "continuous",
#'   extrapolation = TRUE,
#'   p = 0.001,
#'   k = k,
#'   n = n)
#' # plot intermediate and extreme density conditional on a specific value of scedasis function
#' # disclaimer: to speed up the procedure, we specify a coarse grid
#' plot(
#'  x = predDens_intx$x,
#'  y = predDens_intx$preddens[,20],
#'  type = "l",
#'  lwd = 2,
#'  col="dodgerblue",
#'  ylab = "",
#'  xlab = "yg",
#'  main = "Conditional predictive posterior density")
#' lines(
#'   x = predDens_extx$x,
#'   y = predDens_extx$preddens[,20],
#'   lwd = 2,
#'   col = "violet")
#' legend("topright",
#'   legend = c("Intermediate threshold","Extreme threshold"),
#'   lwd = 2,
#'   col = c("dodgerblue", "violet"))
#' }
predDensx <- function(
  x,
  postsamp,
  scedasis,
  threshold,
  type = c("continuous", "discrete"),
  extrapolation = FALSE,
  p,
  k,
  n
) {
  extrapolation <- isTRUE(extrapolation)
  if (missing(k) || missing(n) || missing(p)) {
    extrapolation <- FALSE
  }
  if (extrapolation) {
    stopifnot(
      length(k) == 1L,
      length(n) == 1L,
      length(p) == 1L
    )
    stopifnot(
      k <= n,
      p < 0.5,
      p > 0
    )
  }

  m <- nrow(postsamp)
  ps <- ncol(scedasis)
  r <- length(x)

  stopifnot(
    nrow(scedasis) == m,
    ncol(postsamp) == 2L,
    length(threshold) == 1L
  )

  out <- matrix(nrow = r, ncol = ps)
  type <- match.arg(type)

  if (type == "continuous" && extrapolation) {
    for (i in seq_len(ps)) {
      tmp <- apply(
        X = cbind(postsamp, scedasis[, i]),
        MARGIN = 1,
        FUN = function(pars) {
          preddens2_c(
            theta = pars,
            p = p,
            y = x,
            t = threshold,
            k = k,
            n = n
          )
        }
      )
      out[, i] <- rowMeans(tmp)
    }
  } else if (type == "continuous" && !extrapolation) {
    for (i in seq_len(ps)) {
      tmp <- apply(
        X = cbind(postsamp, scedasis[, i]),
        MARGIN = 1,
        FUN = function(pars) {
          preddens1_c(
            theta = pars,
            y = x,
            t = threshold
          )
        }
      )
      out[, i] <- rowMeans(tmp)
    }
  } else if (type == "discrete" && extrapolation) {
    for (i in seq_len(ps)) {
      tmp <- apply(
        X = cbind(postsamp, scedasis[, i]),
        MARGIN = 1,
        FUN = function(pars) {
          preddens2_d(
            theta = pars,
            p = p,
            y = x,
            t = threshold,
            k = k,
            n = n
          )
        }
      )
      out[, i] <- rowMeans(tmp)
    }
  } else {
    # discrete & !extrapolation
    for (i in seq_len(ps)) {
      tmp <- apply(
        X = cbind(postsamp, scedasis[, i]),
        MARGIN = 1,
        FUN = function(pars) {
          preddens1_d(
            theta = pars,
            y = x,
            t = threshold
          )
        }
      )
      out[, i] <- rowMeans(tmp)
    }
  }

  list(x = x, preddens = out)
}


#' Predictive quantile of peaks-over-threshold models
#'
#' Given an estimated predictive density evaluated at a grid of points, return the smallest value of the quantile of the predictive distribution function
#' below which a specific probability level corresponds. Namely, \eqn{Q(p) = \inf\{ x \in \mathbb{R}: p \leq F(x)\}}, where \eqn{F(x)} is the
#' predictive distribution function of a peak-over-threshold object.
#' WARNING: this function will not be exported in a future version of the package.
#'
#' @param x vector of grid of points at which the predictive density is evaluated
#' @param y vector containing estimates of the predictive density, e.g., the posterior predictive density
#' @param p double indicating the probability level corresponding to the quantile
#' @return double containing the value of the quantile
#' @export
#' @keywords internal
#' @examples
#' \dontrun{
#' # generate data
#' set.seed(1234)
#' n <- 500
#' samp <- evd::rfrechet(n,0,1:n,4)
#' # set effective sample size and threshold
#' k <- 50
#' threshold <- sort(samp,decreasing = TRUE)[k+1]
#' # preliminary mle estimates of scale and shape parameters
#' mlest <- evd::fpot(samp, threshold, control=list(maxit = 500))
#' # empirical bayes procedure
#' proc <- estPOT(
#'   samp,
#'   k = k,
#'   pn = c(0.01, 0.005),
#'   type = "continuous",
#'   method = "bayesian",
#'   prior = "empirical",
#'   start = as.list(mlest$estimate),
#'   sig0 = 0.1)
#' # conditional predictive density estimation
#' yg <- seq(0, 50, by = 2)
#' nyg <- length(yg)
#' # estimation of scedasis function
#' # setting
#' M <- 1e3
#' C <- 5
#' alpha <- 0.05
#' bw <- .5
#' nsim <- 5000
#' burn <- 1000
#' # create covariate
#' # in sample obs
#' n_in = n
#' # number of years ahead
#' nY = 1
#' n_out = 365 * nY
#' # total obs
#' n_tot = n_in + n_out
#' # total covariate (in+out sample period)
#' x <- seq(0, 1, length = n_tot)
#' # in sample grid dimension for covariate
#' ng_in <- 150
#' xg <- seq(0, x[n_in], length = ng_in)
#' # in+out of sample grid
#' xg <- c(xg, seq(x[n_in + 1], x[(n_tot)], length = ng_in))
#' # in+out sample grid dimension
#' nxg <- length(xg)
#' xg <- array(xg, c(nxg, 1))
#' # in sample observations
#' samp_in <- samp[1:n_in]
#' ssamp_in <- sort(samp_in, decreasing = TRUE, index = TRUE)
#' x_in <- x[1:n_in] # in sample covariate
#' xs <- x_in[ssamp_in$ix[1:k]] # in sample concomitant covariate
#' # estimate scedasis function over the in and out of sample period
#' res_stat <- apply(
#'   xg,
#'   1,
#'   cpost_stat,
#'   N = nsim - burn,
#'   x = x_in,
#'   xs = xs,
#'   bw = bw,
#'   k = k,
#'   C = C
#' )
#' # conditional predictive posterior density
#' yg <- seq(500, 5000, by = 100)
#' nyg = length(yg)
#' # intermediate threshold
#' predDens_intx <- predDensx(
#'   x = yg,
#'   postsamp = proc$post_sample,
#'   scedasis = res_stat,
#'   threshold = proc$t,
#'   "continuous",
#'   extrapolation = FALSE)
#' # extreme threshold
#' predDens_extx <- predDensx(
#'   x = yg,
#'   postsamp = proc$post_sample,
#'   scedasis = res_stat,
#'   threshold = proc$t,
#'   "continuous",
#'   extrapolation = TRUE,
#'   p = 0.001,
#'   k = k,
#'   n = n)
#'   # predictive conditional quantiles
#' predQuant_intxL <- predQuant_intxU <- predQuant_extxL <- predQuant_extxU <- numeric()
#' for (i in 1:nxg){
#'   predQuant_intxU[i] <- apply(
#'     array(predDens_intx$preddens[,i], c(1, nyg)),
#'     1,
#'     quantF,
#'     x = yg,
#'     p = c(0.975)
#'   )
#'   predQuant_intxL[i] <- apply(
#'     array(predDens_intx$preddens[,i], c(1, nyg)),
#'     1,
#'     quantF,
#'     x = yg,
#'     p = c(0.025)
#'   )
#'   predQuant_extxU[i] <- apply(
#'     array(predDens_extx$preddens[,i], c(1, nyg)),
#'     1,
#'     quantF,
#'     x = yg,
#'     p = c(0.975)
#'   )
#'   predQuant_extxL[i] <- apply(
#'     array(predDens_extx$preddens[,i], c(1, nyg)),
#'     1,
#'     quantF,
#'     x = yg,
#'     p = c(0.025)
#'   )
#' }
#' }
quantF <- function(x, y, p) {
  csy <- cumsum(y)
  cdfy <- csy / csy[length(y)]
  indx <- which(cdfy > p)[1]
  return(x[indx])
}

#' Test on tail homogeneity
#'
#' Given observed samples and effective sample size, return the results for a likelihood ratio-type test on tail homogeneity.
#'
#' @param y list, containing the samples on which the test is to be performed
#' @param k integer, number of exceedances for the generalized Pareto
#' @param alpha double indicating the confidence level for the test. Default: 0.05
#' @return list of 7 containing
#' \itemize{
#' \item \code{gamHatP} the pooled tail index
#' \item \code{VarGamHatP} the variance of \code{gamHatP}
#' \item \code{CIGamHatP} \eqn{(1-\alpha)} level confidence interval for \code{gamHatP}
#' \item \code{BiasGamHatP} bias term of \code{gamHatP}
#' \item \code{logLikR} value of the likelihood ratio-type of test statistic
#' \item \code{PVal} p-value of the test
#' }
#' @references
#' Daouia, A., S.A. Padoan and G. Stupfler (2024). Optimal weighted pooling for inference about
#' the tail index and extreme quantiles, \emph{Bernoulli}, 30(\bold{2}), pp. 1287–1312.
#' @export
#' @examples
#' \dontrun{
#' # generate two samples of data
#' set.seed(1234)
#' y1 <- evd::rgpd(500, 0, 1, 0.2)
#' y2 <- evd::rgpd(700, 0, 2, 0.7)
#' y <- list(y1 = y1, y2 = y2)
#' # set effective sample size
#' k <- 50
#' # perform test
#' test <- testTailHomo(y,k)
#' }
testTailHomo <- function(y, k, alpha=0.05)
{
  m <- length(y)
  n <- numeric(m)
  # estimate gamma, rho, beta, lambda
  gamma <- array(NA,dim=c(m,4))
  for (i in 1:m){
    n[i] <- length(y[[i]])
    gamma[i,] <- gammaest(y[[i]],k)
  }
  # perform test on tail homogeneity
  testout <- PooledTailIndex(
    gamma[,1],
    gamma[,2],
    gamma[,3],
    gamma[,4],
    m=m,
    n=n,
    k=rep(k,m)
    )
}



#' Plot empirical Bayes inference results for continuous and discrete generalized Pareto distribution
#'
#' Given a sample of posterior draws of the scale or shape parameter, return a histogram on the density scale for the posterior distribution along with a theoretical prior and posterior comparison curve based on the MLE, pointwise Wald-based normal 95\% confidence intervals for the mean of the sample, and pointwise credible intervals (asymmetric by design).
#'
#' @param x a vector of posterior samples
#' @param mle vector of length 2 containing the maximum likelihood estimator for the scale and shape parameters, respectively (only used if \code{param="scale"})
#' @param alpha level for intervals. Default to 0.05 giving 95\% confidence intervals
#' @param param character string indicating the parameter. Default: \code{c("scale", "shape")}
#' @param cols vector of length three containing colors for posterior mean, confidence intervals, and credible intervals. Default to \code{c("mediumseagreen", "goldenrod", "gold4")}
#' @param ... additional arguments for plotting function; only \code{xlab} is allowed
#' @return \code{NULL}; used to create a plot
#' @export
#' @examples
#' \dontrun{
#' # generate data
#' set.seed(1234)
#' n <- 500
#' samp <- evd::rfrechet(n, 0, 3, 4)
#' # set effective sample size and threshold
#' k <- 50
#' threshold <- sort(samp, decreasing = TRUE)[k+1]
#' # preliminary mle estimates of scale and shape parameters
#' mlest <- evd::fpot(samp, threshold)
#' # empirical bayes procedure
#' proc <- estPOT(
#'  samp,
#'  k = k,
#'  pn = c(0.01, 0.005),
#'  type = "continuous",
#'  method = "bayesian",
#'  prior = "empirical",
#'  start = as.list(mlest$estimate),
#'  sig0 = 0.1)
#' plotBayes(
#'   proc$post_sample[,1],
#'   mlest$estimate,
#'   param = "scale")
#' plotBayes(
#'   proc$post_sample[,2],
#'   param = "shape")
#' }
plotBayes <- function(
  x,
  mle,
  alpha = 0.05,
  param = c("scale", "shape"),
  cols = c("mediumseagreen", "goldenrod", "gold4"),
  ...
) {
  cols <- rep(cols, length.out = 3L)
  postsample <- x
  args <- list(...)
  xlab <- args$xlab
  if (is.null(xlab)) {
    xlab <- ""
  }
  param <- match.arg(param)
  oldpar <- par(no.readonly = TRUE)
  par(mar = c(5, 5, 2, 1))
  on.exit(par(oldpar))
  cax <- 1.8
  caxlab <- 2.2
  alphalev = 1 - alpha / 2
  # Prior densities
  priordens_fn <- switch(
    param,
    "scale" = function(x) {
      dgamma(x, 1, scale = mle[1])
    },
    "shape" = function(x) {
      dtt(x, df = 1, left = -1)
    }
  )
  # Extract label for main
  if (!is.null(args$main)) {
    label <- args$main
  } else {
    label <- paste(switch(param, scale = "Scale", shape = "Shape"), "parameter")
  }
  ran_y <- range(
    dnorm(postsample, mean(postsample), sd(postsample)),
    priordens_fn(postsample),
    hist(postsample, plot = FALSE)$density
  )
  hist(
    postsample,
    col = "lightgrey",
    border = "white",
    freq = FALSE,
    ylim = ran_y,
    xlab = xlab,
    main = label,
    cex.axis = cax,
    cex.lab = caxlab,
    cex.main = caxlab,
  )
  curve(dnorm(x, mean(postsample), sd(postsample)), lwd = 4, add = TRUE)
  curve(priordens_fn, lty = 2, lwd = 4, add = TRUE)

  CIl = mean(postsample) -
    qt(alphalev, df = length(postsample) - 1) * sd(postsample)
  CIu = mean(postsample) +
    qt(alphalev, df = length(postsample) - 1) * sd(postsample)
  points(mean(postsample), 0, col = cols[1], pch = 15, cex = 3)
  points(CIu, 0, col = cols[2], pch = 1, cex = 3, lwd = 3) # symmetric CI
  points(CIl, 0, col = cols[2], pch = 1, cex = 3, lwd = 3)
  points(
    quantile(postsample, alphalev),
    0,
    pch = 2,
    cex = 3,
    col = cols[3],
    lwd = 3
  ) # asymmetric CI
  points(
    quantile(postsample, 1 - alphalev),
    0,
    pch = 2,
    cex = 3,
    col = cols[3],
    lwd = 3
  )
  legend(
    "topright",
    c(
      "Posterior",
      "Prior",
      "PM",
      paste0("S-", round(100 * (1 - alpha), 0), "%-CI"),
      paste0("A-", round(100 * (1 - alpha), 0), "%-CI")
    ),
    lty = c(1, 2, 0, 0, 0, 0),
    bty = "n",
    lwd = c(4, 4, 1, 3, 3, 3),
    pch = c(-1, -1, 15, 1, 2, 19),
    pt.cex = 2,
    col = c(1, 1, cols),
    cex = 1.8,
    x.intersp = 0.5
  )
  return(invisible(NULL))
}

#########################################################
### Authors: Simone Padoan and Gilles Stuplfer        ###
### Emails: simone.padoan@unibocconi.it               ###
### gilles.stupfler@ensai.fr                          ###
### Institution: Department of Decision Sciences,     ###
### University Bocconi of Milan, Italy,               ###
### Ecole Nationale de la Statistique et de           ###
### l'Analyse de l'Information, France                ###
### File name: Simulation.r                           ###
### Description:                                      ###
### This file contains a set of procedures            ###
### for simulating i.i.d. data from univariate        ###
### and multivariate heavy-tailed parametric          ###
### families of distributions and serial dependent    ###
### data from univariate and multivariate families    ###
### of time series models with heavy-tailed           ###
### marginal disribution                              ###
### Last change: 2020-07-14                           ###
#########################################################


################################################################################
### START
### Sub-routines (HIDDEN FUNCTIONS)
###
################################################################################

rdoublefrechet <- function(ndata, scale, shape){
  coin <- rbinom(ndata, 1, .5)
  data <- (2*coin-1)*rfrechet(ndata, scale=scale, shape=shape)
  return(data)
}

rdoublepareto <- function(ndata, scale, shape){
  coin <- rbinom(ndata, 1, .5)
  data <- (2*coin-1)*scale*(rgpd(ndata, loc=1, scale=1, shape=1))^(1/shape)
  return(data)
}

rmdoublepareto <- function(ndata, scale, shape, sigma){
  coin <- rbinom(ndata, 1, .5)
  u <- pnorm(rmvnorm(ndata, sigma=sigma))
  data <- (2*coin-1)*scale*(1-u)^(-1/shape)
  return(data)
}

rmcopdoublepareto <- function(ndata, dim, copmod, scale, shape){
  coin <- rbinom(ndata, 1, .5)
  data <- u <-  rCopula(ndata, copmod)
  for(i in 1:dim) data[,i] <- (2*coin-1)*scale[i]*(1-u[,i])^(-1/shape[i])
  return(data)
}

rmcopfrechet <- function(ndata, dim, copmod, scale, shape){
  data <- u <-  rCopula(ndata, copmod)
  for(i in 1:dim) data[,i] <- scale[i]*(-log(u[,i]))^(-1/shape[i])
  return(data)
}

rmcopdoublefrechet <- function(ndata, dim, copmod, scale, shape){
  coin <- rbinom(ndata, 1, .5)
  data <- u <-  rCopula(ndata, copmod)
  for(i in 1:dim) data[,i] <- (2*coin-1)*scale[i]*(-log(u[,i]))^(-1/shape[i])
  return(data)
}


################################################################################


################################################################################
### START
### Main-routines (VISIBLE FUNCTIONS)
###
################################################################################

rtimeseries <- function(ndata, dist="studentT", type="AR", par, burnin=1e+03)
{
  # main body
  # Check whether the distribution in input is available
  if(!(dist %in% c("double-Frechet", "double-Pareto", "Frechet", "Gaussian",
                  "Pareto", "studentT"))) stop("insert the correct DISTRIBUTION!")

  # Set output
  res <- NULL
  # consider the class of stationary sequence with Independent and Identical Distributions
  if(type=="IID"){
    # IID realizations from a Frechet distribution
    if(dist=="Frechet"){
      scale <- par[1]
      shape <- par[2]
      data <- rfrechet(ndata, scale=scale, shape=shape)
    }
    # IID realizations from a Pareto distribution
    if(dist=="Pareto"){
      scale <- par[1]
      shape <- par[2]
      data <- scale*(rgpd(ndata, loc=1, scale=1, shape=1))^(1/shape)
    }
    # IID realizations from a Student-t distribution
    if(dist=="studentT") data <- rt(ndata, par)
  }
  # consider the class of Auto-Regressive time series models
  if(type=="AR"){
   if(dist=="studentT"){
     # Stationary auto-regressive time series with Student-t iid innovations
     # Set parameters
     corr <- par[1]
     df <- par[2]
     # Set total number of data
     nsim <- ndata+burnin
     data <- rep(0, nsim)
     z <- sqrt(1-corr^2) * rt(nsim, df)
     for(i in 1:(nsim-1)) data[i+1] <- corr*data[i] + z[i]
     data <- data[(burnin+1):nsim]
   }
   if(dist=="double-Frechet"){
     # Stationary auto-regressive time series with double Frechet iid innovations
     # Set parameters
     corr <- par[1]
     scale <- par[2]
     shape <- par[3]
     # Set total number of data
     nsim <- ndata+burnin
     data <- rep(0, nsim)
     z <- sqrt(1-corr^2) * rdoublefrechet(nsim, scale=scale, shape=shape)
     for(i in 1:(nsim-1)) data[i+1] <- corr*data[i] + z[i]
     data <- data[(burnin+1):nsim]
   }
   if(dist=="double-Pareto"){
     # Stationary auto-regressive time series with double Pareto iid innovations
     # Set parameters
     corr <- par[1]
     scale <- par[2]
     shape <- par[3]
     # Set total number of data
     nsim <- ndata+burnin
     data <- rep(0, nsim)
     z <- sqrt(1-corr^2) * rdoublepareto(nsim, scale=scale, shape=1/shape)
     for(i in 1:(nsim-1)) data[i+1] <- corr*data[i] + z[i]
     data <- data[(burnin+1):nsim]
   }
 }
 # compute the design matrix:
 if(type=="ARMA"){
  if(dist=="studentT"){
    # Stationary auto-regressive time series with Student-t iid innovations
    # Set parameters
    corr <- par[1]
    df <- par[2]
    smooth <- par[3]
    # Set total number of data
    nsim <- ndata+burnin
    data <- rep(0, nsim)
    z <- sqrt(1-corr^2) * rt(nsim, df)
    for(i in 1:(nsim-1)) data[i+1] <- corr*data[i] + smooth * z[i] + z[i+1]
    data <- data[(burnin+1):nsim]
  }
  if(dist=="double-Frechet"){
    # Stationary auto-regressive time series with double Frechet iid innovations
    # Set parameters
    corr <- par[1]
    scale <- par[2]
    shape <- par[3]
    smooth <- par[4]
    # Set total number of data
    nsim <- ndata+burnin
    data <- rep(0, nsim)
    z <- sqrt(1-corr^2) * rdoublefrechet(nsim, scale=scale, shape=shape)
    for(i in 1:(nsim-1)) data[i+1] <- corr*data[i] + smooth * z[i] + z[i+1]
    data <- data[(burnin+1):nsim]
  }
  if(dist=="double-Pareto"){
    # Stationary auto-regressive time series with double Pareto iid innovations
    # Set parameters
    corr <- par[1]
    scale <- par[2]
    shape <- par[3]
    smooth <- par[4]
    # Set total number of data
    nsim <- ndata+burnin
    data <- rep(0, nsim)
    z <- sqrt(1-corr^2) * rdoublepareto(nsim, scale=scale, shape=1/shape)
    for(i in 1:(nsim-1)) data[i+1] <- corr*data[i] + smooth * z[i] + z[i+1]
    data <- data[(burnin+1):nsim]
  }
}
   if(type=="GARCH"){
     if(dist=="Gaussian"){
       # Stationary GARCH time series with Gaussian iid innovations
       # Set parameters
       alpha <- par[1:2]
       beta <- par[3]
       # Set total number of data
       nsim <- ndata+burnin
       data <- rep(0, nsim)
       sigma <- rep(0, nsim)
       z <- rnorm(nsim)
       for(i in 1:(nsim-1)){
         sigma[i+1] <- sqrt(alpha[1] + alpha[2] * data[i]^2 + beta * sigma[i]^2)
         data[i+1] <- sigma[i+1] * z[i]
       }
       data <- data[(burnin+1):nsim]
     }
   }
   if(type=="ARMAX"){
     if(dist=="Frechet"){
       # Set parameters
       corr <- par[1]
       scale <- par[2]
       shape <- par[3]
       # Set total number of data
       nsim <- ndata+burnin
       data <- rep(0, nsim)
       z <- (1-corr)^(1/shape) * rfrechet(nsim, scale=scale, shape=shape)
       for(i in 1:(nsim-1)) data[i+1] <- max((corr)^(1/shape) * data[i],  z[i])
       data <- data[(burnin+1):nsim]
     }
   }
 return(data)
 }


 rbtimeseries <- function(ndata, dist="studentT", type="AR", copula="Gumbel", par, burnin=1e+03)
 {
   # main body
   # Check whether the distribution in input is available
   if(!(dist %in% c("studentT", "AStudentT", "TStudentT", "double-Pareto", "ADouble-Pareto", "Gaussian", "AGaussian"))) stop("insert the correct DISTRIBUTION!")
   # Set output
   res <- NULL
   # compute the design matrix:
   if(type=="IID"){
     if(dist=="studentT"){
       dep <- par$dep
       sigma <- matrix(c(1, dep, dep, 1), ncol=2)
       df <- par$df
       # Stationary time series with symmetric Student-t iid innovations
       data <- rmvt(ndata, sigma, df)
     }
   }
    # compute the design matrix:
    if(type=="AR"){
      if(dist=="studentT" & copula=="studentT"){
        # Stationary auto-regressive time series with symmetric Student-t iid innovations
        # Set parameters
        corr <- par$corr
        df <- par$df
        dep <- par$dep
        sigma <- matrix(c(1, dep, dep, 1), ncol=2)
        # Set total number of data
        nsim <- ndata+burnin
        data <- array(0, c(nsim, 2))
        z <- sqrt(1-corr^2) * rmvt(nsim, sigma, df)
        for(i in 1:(nsim-1)) data[i+1,] <- corr*data[i,] + z[i,]
        data <- data[(burnin+1):nsim,]
      }
      if(dist=="AStudentT" & copula=="studentT"){
        # Stationary auto-regressive time series with asymmetric Student-t iid innovations
        # Set parameters
        corr <- par$corr
        df <- par$df
        dep <- par$dep
        sigma <- matrix(c(1, dep, dep, 1), ncol=2)
        # Set total number of data
        nsim <- ndata+burnin
        data <- array(0, c(nsim, 2))
        eps <- rmvt(nsim, sigma, df)
        I <- eps[,1]<0
        eps[I,1] <- -sqrt(-eps[I,1])
        z <- sqrt(1-corr^2) * eps
        for(i in 1:(nsim-1)) data[i+1,] <- corr*data[i,] + z[i,]
        data <- data[(burnin+1):nsim,]
      }
    }
    if(type=="ARMA"){
     if(dist=="double-Pareto" & any(copula=="Gaussian", copula=="Gumbel")){
       # Stationary auto-regressive time series with symmetric double Pareto iid innovations
       # Set parameters
       corr <- par$corr
       scale <- par$scale
       shape <- par$shape
       smooth <- par$smooth
       dep <- par$dep
       if(copula=="Gumbel") depCopula <- gumbelCopula(dep)
       if(copula=="Gaussian") depCopula <- normalCopula(dep)
       # Set total number of data
       nsim <- ndata+burnin
       data <- array(0, c(nsim, 2))
       z <- rmcopdoublepareto(nsim, 2, depCopula, scale, shape)
       z <- sqrt(1-corr^2)*z
       for(i in 1:(nsim-1)) data[i+1,] <- corr*data[i,] + smooth * z[i,] + z[i+1,]
       data <- data[(burnin+1):nsim,]
     }
     if(dist=="ADouble-Pareto" & any(copula=="Gaussian", copula=="Gumbel")){
       # Stationary auto-regressive time series with asymmetric double Pareto iid innovations
       # Set parameters
       corr <- par$corr
       scale <- par$scale
       shape <- par$shape
       smooth <- par$smooth
       dep <- par$dep
       if(copula=="Gumbel") depCopula <- gumbelCopula(dep)
       if(copula=="Gaussian") depCopula <- normalCopula(dep)
       # Set total number of data
       nsim <- ndata+burnin
       data <- array(0, c(nsim, 2))
       eps <- rmcopdoublepareto(nsim, 2, depCopula, scale, shape)
       I <- eps[,1]<0
       eps[I,1] <- -sqrt(-eps[I,1])
       z <- sqrt(1-corr^2)*eps
       for(i in 1:(nsim-1)) data[i+1,] <- corr*data[i,] + smooth * z[i,] + z[i+1,]
       data <- data[(burnin+1):nsim,]
     }
   }
   if(type=="GARCH"){
     if(dist=="Gaussian" & copula=="Gaussian"){
       # Stationary GARCH time series with Gaussian iid innovations
       alpha <- par$alpha[1:2]
       beta <- par$beta
       dep <- par$dep
       varcov <- matrix(c(1, dep, dep, 1), ncol=2)
       # Set total number of data
       nsim <- ndata+burnin
       data <- array(0, c(nsim,2))
       sigma <- array(0, c(nsim, 2))
       z <- rmvnorm(nsim, sigma=varcov)
       for(i in 1:(nsim-1)){
         sigma[i+1,] <- sqrt(alpha[1] + alpha[2] * data[i,]^2 + beta * sigma[i,]^2)
         data[i+1,] <- sigma[i+1,] * z[i,]
       }
       data <- data[(burnin+1):nsim,]
     }
     if(dist=="Gaussian" & copula=="StudentT"){
       # Stationary GARCH time series with Gaussian iid innovations
       alpha <- par$alpha[1:2]
       beta <- par$beta
       dep <- par$dep
       varcov <- matrix(c(1, dep, dep, 1), ncol=2)
       df <- par$df
       # Set total number of data
       nsim <- ndata+burnin
       data <- array(0, c(nsim,2))
       sigma <- array(0, c(nsim, 2))
       z <- rmvt(nsim, sigma, df)
       for(i in 1:(nsim-1)){
         sigma[i+1,] <- sqrt(alpha[1] + alpha[2] * data[i,]^2 + beta * sigma[i,]^2)
         data[i+1,] <- sigma[i+1,] * z[i,]
       }
       data <- data[(burnin+1):nsim,]
     }
     if(dist=="AGaussian" & copula=="StudentT"){
       # Stationary GARCH time series with Gaussian iid innovations
       alpha <- par$alpha[1:2]
       beta <- par$beta
       dep <- par$dep
       varcov <- matrix(c(1, dep, dep, 1), ncol=2)
       df <- par$df
       # Set total number of data
       nsim <- ndata+burnin
       data <- array(0, c(nsim,2))
       sigma <- array(0, c(nsim, 2))
       u <- pt(rmvt(nsim, varcov, df), df)
       I <- u[,1] <= 0.5
       z <- cbind(-log(2*(1-u[,1])), qnorm(u[,2]))
       z[I,1] <- 2*u[I,1]-1
       for(i in 1:(nsim-1)){
         sigma[i+1,] <- sqrt(alpha[1] + alpha[2] * data[i,]^2 + beta * sigma[i,]^2)
         data[i+1,] <- sigma[i+1,] * z[i,]
       }
       data <- data[(burnin+1):nsim,]
     }
   if(dist=="AGaussian" & copula=="Gumbel"){
     # Stationary GARCH time series with Gaussian iid innovations
     alpha <- par$alpha[1:2]
     beta <- par$beta
     dep <- par$dep
     # Set total number of data
     nsim <- ndata+burnin
     data <- array(0, c(nsim,2))
     sigma <- array(0, c(nsim, 2))
     u <- rCopula(nsim, gumbelCopula(dep))
     I <- u[,1] <= 0.5
     z <- cbind(-log(2*(1-u[,1])), qnorm(u[,2]))
     z[I,1] <- 2*u[I,1]-1
     for(i in 1:(nsim-1)){
       sigma[i+1,] <- sqrt(alpha[1] + alpha[2] * data[i,]^2 + beta * sigma[i,]^2)
       data[i+1,] <- sigma[i+1,] * z[i,]
     }
     data <- data[(burnin+1):nsim,]
   }
 }
  return(data)
 }

 rmdata <- function(ndata, dist="studentT", copula="studentT", par)
 {
   # main body
   # Check whether the distribution in input is available
   if(!(dist %in% c("studentT", "AstudentT", "TStudentT", "ATStudentT", "double-Pareto", "double-Frechet", "Frechet", "Gaussian"))) stop("insert the correct DISTRIBUTION!")
   # initialization output
   res <- NULL
   # list of possible models
     if(dist=="studentT" & copula=="studentT"){
       # Stationary series of Student-t iid observations on R^d
       df <- par$df
       sigma <- par$sigma
       data <- rmvt(ndata, sigma, df)
     }
     if(dist=="studentT" & copula=="None"){
       # Stationary series of Student-t iid observations on R^d
       df <- par$df
       dimData <- par$dim
       data <- replicate(dimData, rt(ndata, df))
     }
     if(dist=="studentT" & copula=="Gaussian"){
       # Stationary series of Student-t iid observations on R^d
       df <- par$df
       sigma <- par$sigma
       data <- qt(pnorm(rmvnorm(ndata, sigma=sigma)), df)
     }
     if(dist=="AstudentT" & copula=="studentT"){
       # Stationary series of Student-t iid observations on R^d
       df <- par$df
       shape <- par$shape
       sigma <- par$sigma
       data <- rmvt(ndata, sigma, df)
       I <- data[,1] > 0
       data[I,1] <- data[I,1]^(df/shape)
       data[!I,1] <- -(-data[!I,1])^(df/shape)
     }
     if(dist=="TStudentT" & copula=="studentT"){
       # Stationary series of Student-t iid observations on R^d_+
       df <- par$df
       sigma <- par$sigma
       dimData <- nrow(sigma)
       lower <- rep(0, dimData)
       upper <- rep(Inf, dimData)
       data <- rtmvt(ndata, sigma=sigma, df=df, lower=lower, upper=upper)
     }
     if(dist=="AstudentT" & copula=="Gaussian"){
       # Stationary series of Student-t iid observations on R^d
       df <- par$df
       shape <- par$shape
       sigma <- par$sigma
       data <- qt(pnorm(rmvnorm(ndata, sigma=sigma)), df)
       I <- data[,1] > 0
       data[I,1] <- data[I,1]^(df/shape)
       data[!I,1] <- -(-data[!I,1])^(df/shape)
     }
     if(dist=="Frechet" & copula=="None"){
       # Stationary series of Frechet iid observations on R^d_+
       dimData <- par$dim
       scale <- par$scale
       shape <- par$shape
       data <- replicate(dimData, rfrechet(ndata, scale=scale, shape=shape))
     }
     if(dist=="Frechet" & copula=="Clayton"){
       # Stationary series of Frechet iid observations on R^d_+
       dep <- par$dep
       dimData <- par$dim
       scale <- par$scale
       shape <- par$shape
       copmod <- claytonCopula(dep, dimData)
       data <- rmcopfrechet(ndata, dimData, copmod, scale, shape)
     }
     if(dist=="Frechet" & copula=="Gumbel"){
       # Stationary series of Frechet iid observations on R^d_+
       dep <- par$dep
       dimData <- par$dim
       scale <- par$scale
       shape <- par$shape
       copmod <- gumbelCopula(dep, dimData)
       data <- rmcopfrechet(ndata, dimData, copmod, scale, shape)
     }
     if(dist=="Frechet" & copula=="Frank"){
       # Stationary series of Frechet iid observations on R^d_+
       dep <- par$dep
       dimData <- par$dim
       scale <- par$scale
       shape <- par$shape
       copmod <- frankCopula(dep, dimData)
       data <- rmcopfrechet(ndata, dimData, copmod, scale, shape)
     }
     if(dist=="double-Pareto" & copula=="Gumbel"){
       dep <- par$dep
       dimData <- par$dim
       scale <- par$scale
       shape <- par$shape
       copmod <- gumbelCopula(dep, dimData)
       data <- rmcopdoublepareto(ndata, dimData, copula, scale, shape)
     }
     if(dist=="double-Pareto" & copula=="Clayton"){
       dep <- par$dep
       dimData <- par$dim
       scale <- par$scale
       shape <- par$shape
       copmod <- claytonCopula(dep, dimData)
       data <- rmcopdoublepareto(ndata, dimData, copula, scale, shape)
     }
     if(dist=="double-Pareto" & copula=="Frank"){
       dep <- par$dep
       dimData <- par$dim
       scale <- par$scale
       shape <- par$shape
       copmod <- frankCopula(dep, dimData)
       data <- rmcopdoublepareto(ndata, dimData, copula, scale, shape)
     }
     if(dist=="double-Frechet" & copula=="Gumbel"){
       dep <- par$dep
       dimData <- par$dim
       scale <- par$scale
       shape <- par$shape
       copmod <- gumbelCopula(dep, dimData)
       data <- rmcopdoublefrechet(ndata, dimData, copmod, scale, shape)
     }
     if(dist=="double-Frechet" & copula=="Clayton"){
       dep <- par$dep
       dimData <- par$dim
       scale <- par$scale
       shape <- par$shape
       copmod <- claytonCopula(dep, dimData)
       data <- rmcopdoublefrechet(ndata, dimData, copmod, scale, shape)
     }
     if(dist=="double-Frechet" & copula=="Frank"){
       dep <- par$dep
       dimData <- par$dim
       scale <- par$scale
       shape <- par$shape
       copmod <- frankCopula(dep, dimData)
       data <- rmcopdoublefrechet(ndata, dimData, copmod, scale, shape)
     }
  return(data)
}


 ################################################################################
 ### END
 ### Main-routines (VISIBLE FUNCTIONS)
 ###
 ################################################################################

#########################################################
### Authors: Simone Padoan and Gilles Stuplfer        ###
### Emails: simone.padoan@unibocconi.it               ###
### gilles.stupfler@ensai.fr                          ###
### Institution: Department of Decision Sciences,     ###
### University Bocconi of Milan, Italy,               ###
### Ecole Nationale de la Statistique et de           ###
### l'Analyse de l'Information, France                ###
### File name: Estimation.r                           ###
### Description:                                      ###
### This file contains a set of procedures            ###
### for estimating (point and interval estimation)    ###
### risk measures as Value-At-Risk and Expectile      ###
### for i.i.d. and dependent data                     ###
### Last change: 2020-04-05                           ###
#########################################################


#########################################################
### START
### Sub-routines (HIDDEN FUNCTIONS)
###
#########################################################

# Defines the so-called expectile check function
etafun <- function(x, tau, u) return(abs(tau-(x<=u))*(x-u)^2)

# Defines the asymmetric quadratic loss function
costfun <- function(data, tau, u) return(mean(etafun(data,tau,u)))

# Computse the deviance term of the empirical version of the asymptotic variance
devcontrib <- function(i, data, est, meanexc, p, sn){
  sdata <- data[i:(sn+(i-1))]
  res <- sum(sdata[sdata>est] - meanexc) / (est*sqrt(sn*p))
  return(res)
}

# Computes the number of high exceedances
numexceed <- function(i, data, t, rn){
  sdata <- data[i:(rn+(i-1))]
  res <- sum(sdata>t)
  return(res)
}

# compute the variance as in formula 32 of Drees(2003) Bernoulli
DHVar <- function(data, tau, tau1, j, k, ndata){
  den <- 0
  num <- 0

  # Computes an estimate of the extreme quantile
  extQ <- extQuantile(data, tau, tau1, k=k)$ExtQHat
  # apply formula 32
  for(i in j:k){
    den <- den + (i^(-1/2) - log(k/(ndata*(1-tau1)))/log(i/(ndata*(1-tau1))) * 1/sqrt(k))^2
    iextQ <- extQuantile(data, 1-i/ndata, tau1, k=i)$ExtQHat
    num <- num + (log(iextQ/extQ)/log(i/(ndata*(1-tau1))))^2
  }
  return(num/den)
}

# Defines the log-likelihood for the generalised Pareto model
pllik <- function(par, data, k){
  sigma <- par[1]
  gamma <- par[2]
  llik <- -1e+10

  if(sigma<=0 || gamma<= -0.5) return(llik)

  z <- 1+gamma*data/sigma
  if(any(z<=0)) return(llik)

  llik <- -k*log(sigma) - (1/gamma+1)*sum(log(z))
  if(is.infinite(llik)) llik <- -1e+10
  return(llik)
}

Mka <- function(data, k, a){
  dataord <- sort(data, decreasing=TRUE)
  res <- mean((log(dataord[1:k]) - log(dataord[k+1]))^a)
  return(res)
}

Rka <- function(data, k, a){
  mka <- Mka(data, k, a)
  mk1 <- Mka(data, k, 1)
  mk2 <- Mka(data, k, 2)

  res <- (mka - gamma(a+1) * mk1^a) / (mk2 - 2 * mk1^2)
  return(res)
}

Ska <- function(data, k, a){
  rk2a <- Rka(data, k, 2*a)
  rkap1 <- Rka(data, k, a+1)

  res <- (a * (a+1)^2 * (gamma(a))^2 * rk2a) / (4 * gamma(2 * a) * (rkap1)^2)
  return(res)
}

# compute the bias term
biasTerm <- function(data, k, gammaHat){
  # Subroutines:
  Sk2 <- function(data, k){
    dataord <- sort(data, decreasing=TRUE)
    z <- log(dataord[1:k]) - log(dataord[k+1])
    mk1 <- mean(z)
    mk2 <- mean(z^2)
    mk3 <- mean(z^3)
    mk4 <- mean(z^4)

    res <- (3/4) * ((mk4 - 24 * mk1^4) * (mk2 - 2 * mk1^2)) / (mk3 - 6 * mk1^3)^2
    return(res)
  }
  RhoHat <- function(data, k){
    sk2 <- Sk2(data, k)
    if(sk2 >= 2/3 & sk2 <= 3/4) res <- (6 * sk2 + sqrt(3 * sk2 - 2) - 4) / (4 * sk2 - 3)
    else res <- -Inf
    return(res)
  }
  # Main body
  m <- sum(data > 0)
  K <- round(min(m-1, (2*m)/log(log(m))))

  I <- array(1:K, c(K,1))
  rho <- apply(I, 1, RhoHat, data=data)

  kr <- max(I[rho != -Inf])
  rhoHat <- rho[kr]

  mk2 <- Mka(data, k, 2)
  # compute important terms
  twoGammaHat <- 2 * gammaHat
  omRhoHat <- 1 - rhoHat
  numerator <- (mk2 - 2 * gammaHat^2)
  # compute the bias term
  bias <- numerator / (twoGammaHat * rhoHat * omRhoHat^(-1))
  # compute the adjustment for the computation of the quantile estimator at the extreme level
  adjust <- numerator * omRhoHat^2 / (twoGammaHat * rhoHat^2)
  return(list(bias=bias, adjust=adjust))
}

# compute an estimate of the variance of the expectile estimator at the intermediate level
# for time series and iid observations
estExpectileVar <- function(data, ndata, tau, est, tailest, varType, bigBlock,
                            smallBlock, k){
  # main body:
  # initialization the return value
  res <- NULL
  # 1) Compute the asymptotic variance for the expectile estimator at the intermediate level tau_n
  # At the moment the asymptotic variance is only available for the LAWS expectile estimator

  # compute an estimate of the tail index
  if(tailest=="Hill") gammaHat <- HTailIndex(data, k)$gammaHat
  if(tailest=="ExpBased") gammaHat <- EBTailIndex(data, tau, est)

  # A) ASSUME A STATIONARY TIME-SERIES WITH IID OBSERVATIONS
  if(varType=="asym-Ind"){
    # expectile estimator variance using the formula for iid data
    res <- 2 * gammaHat^3 / (1 - 2 * gammaHat)
    if(res<0 || res==Inf) res <- 0
    return(res)
  }
  # B) ASSUME A STATIONARY TIME-SERIES WITH DEPENDENT OBSERVATIONS
  # First, compute the sigma^2(gamma, R), i.e. incremental factor of the asymptotic variance due to the dependence
  # See formula in Lemma 5.3 of our article
  # compute the block size and number of blocks
  if(is.null(smallBlock)){
    # this method implements the fact that the estimator in Lemma 5.3 is
    # consistent estimator of the expectile variance (it doesn not use small- and big-block approach)
    # define block size
    sizeBlock <- bigBlock
    # define the number of blocks
    nBlocks <- ndata-sizeBlock+1
    indx <- array(1:nBlocks,c(nBlocks,1))
  }
  else{
    # this method uses the small- and big-block approach
    sizeBlock <- bigBlock+smallBlock
    indx <- seq(1, ndata, by=sizeBlock)[1:floor(ndata/sizeBlock)]
    indx <- array(indx, c(length(indx),1))
  }
  # compute the mean of exceedances
  meanexc <- mean(data[data>est])
  # compute  the total amount of the normalised expectile exceedances, over the big-blocks of observations
  # See the formula of Lemma 5.3 of our article
  dev <- apply(indx, 1, devcontrib, data=data, est=est, meanexc=meanexc,
               p=(1-tau), sn=sizeBlock)
  # a) COMPUTE THE ASYMPTOTIC VARIANCE ACCORDING TO THE FORMULA OF THEOREM 4.1
  # THIS IS THE STANDARD RESULT
  if(varType=="asym-Dep"){
    adjust <- gammaHat^2
    res <- adjust * var(dev)
    if(res<0 || res==Inf) res <- 0
    return(res)
  }
  # b) COMPUTE THE ASYMPTOTIC VARIANCE ACCORDING TO THE FORMULA OF THEOREM 4.1
  # BUT CONSIDEYING THE ADJUSTED VERSION OF THE STANDARD FORMULA
  # THE ADJUSTMENT TAKES INTO ACCOUNT THE HEAVYNESS OF THE DISTRIBUTION TAIL
  if(varType=="asym-Dep-Adj"){
    seriesDep <-as.numeric(acf(data, plot=FALSE)$acf)
    isStrong <- all(seriesDep[2:25]>=.1)
    if(isStrong){
      if(gammaHat > (1/5)) adjust <- gammaHat^(1/2)
      if((gammaHat > (1/8)) & (gammaHat <= (1/5))) adjust <- gammaHat
      if(gammaHat <= (1/8)) adjust <- gammaHat^2
    }
    else{
      if(gammaHat > (1/3)) adjust <- gammaHat^(1/2)
      if((gammaHat > (1/5)) & (gammaHat <= (1/3))) adjust <- gammaHat
      if(gammaHat <= (1/5)) adjust <- gammaHat^2
    }
    res <- adjust * var(dev)
    if(res<0 || res==Inf) res <- 0
    return(res)
  }
}

# compute an estimate of the asymptotic variance of the Hill estimator
# for time series and iid observations
estHillVar <- function(data, ndata, gammaHat, varType, bigBlock, smallBlock, k){
# Main body:
# computes the asymptotic variance for iid data
if(varType=="asym-Ind") {
  AdjVar <- 1
  mu <- 2
}
# computes the asymptotic variance for serial dependence data
if(varType=="asym-Dep"){
  # computes the dependence component
  t <- as.numeric(quantile(data, 1-k/ndata))
  sizeBlock <- bigBlock+smallBlock
  indx <- seq(1, ndata, by=sizeBlock)[1:floor(ndata/sizeBlock)]
  indx <- array(indx, c(length(indx),1))
  sexc <- apply(indx, 1, numexceed, data=data, t=t, rn=bigBlock)
  AdjVar <- ndata/(bigBlock*k) * var(sexc)
  # set the estimated gamma index power to the stadard value 2
  mu <- 2
}
# computes the asymptotic variance for serial dependence data with adjustment
if(varType=="asym-Dep-Adj"){
  # computes the dependence component
  t <- as.numeric(quantile(data, 1-k/ndata))
  sizeBlock <- bigBlock+smallBlock
  indx <- seq(1, ndata, by=sizeBlock)[1:floor(ndata/sizeBlock)]
  indx <- array(indx, c(length(indx),1))
  sexc <- apply(indx, 1, numexceed, data=data, t=t, rn=bigBlock)
  AdjVar <- ndata/(bigBlock*k) * var(sexc)
  # set the estimated gamma index power to the ajusted value
  seriesDep <-as.numeric(acf(data, plot=FALSE)$acf)
  isStrong <- all(seriesDep[2:25]>=.1)
  if(isStrong){
    if(gammaHat > (1/5)) mu <- 1/2
    if((gammaHat > (1/8)) & (gammaHat <= (1/5))) mu <- 1
    if(gammaHat <= (1/8)) mu <- 2
  }
  else{
    if(gammaHat > (1/3)) mu <- 1/2
    if((gammaHat > (1/5)) & (gammaHat <= (1/3))) mu <- 1
    if(gammaHat <= (1/5)) mu <- 2
  }
}
# computes the asymptotic variance
res <- gammaHat^mu * AdjVar
return(res)
}


# computes the Quantile-Based marginal Expected Shortfall
# @ the intermediate level
QBmarExpShortF <- function(data, tau, n, q){
  res <- sum(data[,1]* (data[,1]>0 & data[,2]>q))
  return(res / (n*(1-tau)))
}

# computes the Expectile-Based marginal Expected Shortfall
# @ the intermediate level
EBmarExpShortF <- function(data, tau, n, xi){
  res <- sum(data[,1]* (data[,1]>0 & data[,2]>xi))
  return(res / sum(data[,2]>xi))
}


#########################################################
### END
### Sub-routines (HIDDEN FUNCTIONS)
###
#########################################################


#########################################################
### START
### Main-routines (VISIBLE FUNCTIONS)
###
#########################################################

#########################################################
### START
###
################## UNIVARIATE CASE ######################


# Estimation of the tail index using the Hill estimator
HTailIndex <- function(data, k, var=FALSE, varType="asym-Dep", bias=FALSE,
                       bigBlock=NULL, smallBlock=NULL, alpha=0.05){
  # initialization
  VarGamHat <- NULL
  CIgamHat <- NULL
  BiasGamHat <- 0
  AdjExtQHat <- 0

  # main body:
  if(is.null(k)) stop("Enter a value for the parameter 'k' ")
  if(!any(varType=="asym-Dep", varType=="asym-Dep-Adj", varType=="asym-Ind"))
  stop("The parameter 'varType' can only be 'asym-Dep', 'asym-Dep-Adj' or 'asym-Ind' ")
  # compute the sample size
  ndata <- length(data)
  dataord <- sort(data, decreasing=TRUE)
  gammaHat <- mean(log(dataord[1:k])) - log(dataord[k+1])

  # computes the bias term of the Hill esitmator and the adjustment term
  # for the Weissman quantile estimator
  if(bias) {
    bt <- biasTerm(data, k, gammaHat)
    BiasGamHat <- bt$bias
    AdjExtQHat <- bt$adjust
  }
  # computes the asymptotic variance of the Hill estimator
  if(var) {
    VarGamHat <- estHillVar(data, ndata, gammaHat, varType, bigBlock, smallBlock, k)
    # Computes the margin error
    ME <- sqrt(VarGamHat)*qnorm(1-alpha/2) / sqrt(k)
    # Defines an approximate (1-alpha)100% confidence interval for the tail index
    CIgamHat <- c((gammaHat-BiasGamHat)-ME, (gammaHat-BiasGamHat)+ME)
  }
  return(list(gammaHat=gammaHat,
              VarGamHat=VarGamHat,
              CIgamHat=CIgamHat,
              BiasGamHat=BiasGamHat,
              AdjExtQHat=AdjExtQHat))
}

# Estimation of the tail index using the ML estimator
MLTailIndex <- function(data, k, var=FALSE, varType="asym-Dep", bigBlock=NULL, smallBlock=NULL, alpha=0.05){
  #initialization
  VarGamHat <- NULL
  CIgamHat <- NULL

  # main body:
  if(is.null(k)) stop("Enter a value for the parameter 'k' ")
  if(!any(varType=="asym-Dep", varType=="asym-Ind")) stop("The parameter 'varType' can only be 'asym-Dep' or 'asym-Ind' ")
  # compute the sample size
  ndata <- length(data)
  # derive the ordered data
  dataord <- sort(data, decreasing=TRUE)
  # compute the peaks above a threshold
  z <- dataord[1:k] - dataord[k+1]
  # initialize parameters
  par <- c(5,.5)
  gammaHat <- optim(par, pllik, data=z, k=k, method="Nelder-Mead", control=list(fnscale=-1,
               factr=1, pgtol=1e-14, maxit=1e8))$par[2]

  if(var){
    AdjVar <- 1
    if(varType=="asym-Dep"){
      t <- as.numeric(quantile(data, 1-k/ndata))
      sizeBlock <- bigBlock+smallBlock
      indx <- seq(1, ndata, by=sizeBlock)[1:floor(ndata/sizeBlock)]
      indx <- array(indx, c(length(indx),1))
      sexc <- apply(indx, 1, numexceed, data=data, t=t, rn=bigBlock)
      AdjVar <- ndata/(bigBlock*k) * var(sexc)
    }
    # Computes the asymptotic variance
    VarGamHat <- (1+gammaHat)^2 * AdjVar
    # Computes the margin error
    ME <- sqrt(VarGamHat)*qnorm(1-alpha/2) / sqrt(k)
    # Defines an approximate (1-alpha)100% confidence interval for the tail index
    CIgamHat <- c(gammaHat-ME, gammaHat+ME)
    }
  return(list(gammaHat=gammaHat, VarGamHat=VarGamHat, CIgamHat=CIgamHat))
}

# Estimation of the tail index using the moment estimator
MomTailIndex <- function(data, k){
  # main body:
  if(is.null(k)) stop("Enter a value for the parameter 'k' ")
  # derive the ordered data
  dataord <- sort(data, decreasing=TRUE)
  # compute the peaks above a threshold
  z <- log(dataord[1:k]) - log(dataord[k+1])

  M1 <- mean(z)
  M2 <- mean(z^2)

  res <- M1 + 1 - 0.5/(1 - M1^2 / M2)
  return(res)
}

# Estimation of the tail index using the estimator based on the asymptotic
# behaviour of the survival function compute at the intermediate level expectile
EBTailIndex <- function(data, tau, est=NULL){
  # main body:
  if(is.null(tau)) stop("Enter a value for the parameter 'tau' ")
  # compute the exceeding probability
  p <- 1-tau
  # compute an estimate of the expectile if it is not provided
  if(is.null(est)) est <- estExpectiles(data, tau)$ExpctHat
  # compute the empirical distribution function
  decdf <- ecdf(data)
  # evaluate the ecdf at the expectile value
  eecdf <- decdf(est)
  # compute the empirical surival function at the expectile value
  eesf <- 1-eecdf
  # compute an estimate of tje tail index
  res <- (1+eesf/p)^(-1)
  return(res)
}

# Estimation of the quantile using the Weissman estimator
extQuantile <- function(data, tau, tau1, var=FALSE, varType="asym-Dep", bias=FALSE,
                        bigBlock=NULL, smallBlock=NULL, k=NULL, alpha=0.05){
  # initialization
  gammaHat <- NULL
  VarExQHat <- NULL
  CIExtQ <- NULL

  # main body:
  isNk <- is.null(k)
  isNtau <- is.null(tau)
  if(isNk & isNtau) stop("Enter a value for at least one of the two parameters 'tau' and 'k' ")
  # compute the sample size
  ndata <- length(data)
  if(isNk) k <- floor(ndata*(1-tau))
  if(isNtau) tau <- 1-k/ndata

  # Estimation of the quantile at the intermediate level based on the empirical quantile
  QHat <- as.numeric(quantile(data, tau))
  # Estimation of the tail index and related quantities using the Hill estimator
  if(varType=="asym-Alt-Dep") estHill <- HTailIndex(data, k)
  else estHill <- HTailIndex(data, k, var, varType, bias, bigBlock, smallBlock, alpha)

  # Computes a an estimate of the tail index with a bias-correction
  gammaHat <- estHill$gammaHat - estHill$BiasGamHat
  # Estimation of the quantile at the extreme level tau1 using the Weissman estimator
  # with a bias-correction
  extQ <- QHat * ((1-tau1)/(1-tau))^(-gammaHat) * (1 - estHill$AdjExtQHat)

  # Estimation of the asymptotic variance of the Hill estimator
  if(var){
    # The default approach for computing the asymptotic variance with serial dependent observations
    # is the method proposed by Drees (2003). In this case it is assumed that there is no bias term
    # If it is assumed that the Hill estimator requires a bias correction then the the asymptotic
    # variance is estimated after correcting the tail index estimate by the bias correction
    VarExQHat <- estHill$VarGamHat

    if((varType=="asym-Alt-Dep") & (!bias)){
      # Drees (2003) approach
      j <- max(2, ceiling(ndata*(1-tau1)))
      VarExQHat <- DHVar(data, tau, tau1, j, k, ndata)
    }
    # compute lower and upper bounds of the (1-alpha)% CI
    K <- sqrt(VarExQHat)*log((1-tau)/(1-tau1))/sqrt(k)
    lbound <- extQ * exp(qnorm(alpha/2)*K)
    ubound <- extQ * exp(qnorm(1-alpha/2)*K)
    CIExtQ <- c(lbound, ubound)
  }

  return(list(ExtQHat=extQ, VarExQHat=VarExQHat, CIExtQ=CIExtQ))
}

# Estimation of expectiles using the mean square based estimator
# and the 2nd order aproximation estimator
estExpectiles <- function(data, tau, method="LAWS", tailest="Hill", var=FALSE,
                          varType="asym-Dep-Adj", bigBlock=NULL, smallBlock=NULL,
                          k=NULL, alpha=0.05){
  # initialization
  ExpctHat <- NULL
  VarExpHat <- NULL
  CIExpct <- NULL

  # main body:
  isNk <- is.null(k)
  isNtau <- is.null(tau)
  if(isNk & isNtau) stop("Enter a value for at least one of the two parameters 'tau' and 'k' ")
  # compute the sample size
  ndata <- length(data)
  if(isNk) k <- floor(ndata*(1-tau))
  if(isNtau) tau <- 1-k/ndata

  # Estimation of the expectile at the intermediate level based on the direct LAWS estimator
  if(method=="LAWS"){
    u <- as.numeric(quantile(data, tau))
    ExpctHat <- optim(u, costfun, method="BFGS", tau=tau, data=data)$par
    if(var){
      if(varType!="asym-Ind" & any(is.null(bigBlock), is.null(smallBlock))) stop("Insert the value of the big-blocks and small-blocks")
      VarExpHat <- estExpectileVar(data, ndata, tau, ExpctHat, tailest, varType,
                                  bigBlock, smallBlock, k)

      # computes the "margin error"
      K <- sqrt(VarExpHat)/sqrt(ndata*(1-tau))
      # computes lower and upper bounds of the (1-alpha)100% CI
      lbound <- ExpctHat * exp(qnorm(alpha/2)*K)
      ubound <- ExpctHat * exp(qnorm(1-alpha/2)*K)
      # defines the (1-alpha)% CI
      CIExpct <- c(lbound, ubound)
    }
  }
  # Estimation of the expectile at the intermediate level based on the
  # indirect quantile based estimator
  if(method=="QB"){
    # check whether the asymptotic variance is requested
    if(var) stop("The asymptotic variance concerning the indirect Quantile-Based estimator has not been implemented yet!")
    # estimate gamma index
    if(tailest=="Hill") gammaHat <- HTailIndex(data, k)$gammaHat
    if(tailest=="ExpBased") gammaHat <- EBTailIndex(data, tau)
    QHat <- as.numeric(quantile(data,tau))
    ExpctHat <- QHat*(1/gammaHat - 1)^(-gammaHat)
  }
  return(list(ExpctHat=ExpctHat, VarExpHat=VarExpHat, CIExpct=CIExpct))
}

# Prediction of expectiles at the extreme level tau1 beyond the observed data
predExpectiles <- function(data, tau, tau1, method="LAWS", tailest="Hill", var=FALSE,
                          varType="asym-Dep", bias=FALSE, bigBlock=NULL, smallBlock=NULL,
                          k=NULL, alpha_n=NULL, alpha=0.05){
  # initialization
  EExpcHat <- NULL
  VarExtHat <- NULL
  CIExpct <- NULL
  AdjExtQHat <- 0

  # main body:
  isNk <- is.null(k)
  isNtau <- is.null(tau)
  if(isNk & isNtau) stop("Enter a value for at least one of the two parameters 'tau' and 'k' ")
  # compute the sample size
  ndata <- length(data)
  if(isNk) k <- floor(ndata*(1-tau))
  if(isNtau) tau <- 1-k/ndata

  # Compute an estimate of the tail index using the Hill estimator or
  # the expectile based estimator
  if(tailest=="Hill") {
    # Estimation of the tail index and related quantities using the Hill estimator
    estHill <- HTailIndex(data, k, var, varType, bias, bigBlock, smallBlock, alpha)
    gammaHat <- estHill$gammaHat - estHill$BiasGamHat
    AdjExtQHat <- estHill$AdjExtQHat
  }
  if(tailest=="ExpBased") gammaHat <- EBTailIndex(data, tau)
  # Estimation of the extreme level if requested
  if(is.null(tau1)) tau1 <- estExtLevel(alpha_n, gammaHat=gammaHat)$tauHat

  # Estimation of the expectile at the extreme level based on the LAWS estimator
  if(method=="LAWS"){
    # compute an estimate of the expectile at the intermediate level tau:
    ExpcHat <- estExpectiles(data, tau)$ExpctHat
    # compute an estimate of the expectile at the extreme level tau1:
    EExpcHat <- ExpcHat * ((1-tau1)/(1-tau))^(-gammaHat)
  }
  # Estimation of the expectile at the extreme level based on the QB estimator
  if(method=="QB"){
    # Estimation of the quantile at the intermediate level based on the empirical quantile
    QHat <- as.numeric(quantile(data, tau))
    # Estimation of the quantile at the extreme level tau1 using the Weissman estimator
    # If requested the bias-correction is implemented
    extQ <- QHat * ((1-tau1)/(1-tau))^(-gammaHat) * (1 - AdjExtQHat)
    # Compute an estimate of the expectile at the extreme level tau1 based on the the Weissman estimator
    EExpcHat <- extQ * (1/gammaHat - 1)^(-gammaHat)
  }
  # Compute an estimate of the asymptotic variance of the expecile estimator at the extreme level
  if(var){
    if(tailest=="ExpBased") stop("The asymptotic variance has not been implemented yet! when estimating the tail index with the expectile based estimator")
    # Computes the asymptotic variance for independent and serial dependent data
    VarExtHat <- estHill$VarGamHat
    # Computes the "margin error"
    K <- sqrt(VarExtHat)*log((1-tau)/(1-tau1))/sqrt(k)
    # compute lower and upper bounds of the (1-alpha)% CI
    lbound <- EExpcHat * exp(qnorm(alpha/2)*K)
    ubound <- EExpcHat * exp(qnorm(1-alpha/2)*K)
    # defines the (1-alpha)% CI
    CIExpct <- c(lbound, ubound)
  }
  return(list(EExpcHat=EExpcHat, VarExtHat=VarExtHat, CIExpct=CIExpct))
}

# Given the confidence level (1-alpha_n) of an extreme quantile
# computes an point and interval estimate of the extreme level tau'_n given
estExtLevel <- function(alpha_n, data=NULL, gammaHat=NULL, VarGamHat=NULL,
                        tailest="Hill", k=NULL, var=FALSE, varType="asym-Dep",
                        bigBlock=NULL, smallBlock=NULL, alpha=0.05){
  # initialize output
  tauHat <- NULL
  tauVar <- NULL
  tauCI <- NULL
  if(is.null(gammaHat)){
    if(is.null(data)) stop("Insert a data sample")
    if(is.null(k)) stop("insert a value for the intermediate sequence")
    if(tailest=="Hill") {
      res <- HTailIndex(data, k, var, varType, bigBlock=bigBlock,
                        smallBlock=smallBlock)
      gammaHat <- res$gammaHat
      VarGamHat <- res$VarGamHat
    }
    if(tailest=="ML") {
      res <- MLTailIndex(data, k, var, varType, bigBlock=bigBlock,
                        smallBlock=smallBlock)
      gammaHat <- res$gammaHat
      VarGamHat <- res$VarGamHat
    }
    if(tailest=="Moment") gammaHat <- MomTailIndex(data, k)
    if(tailest=="ExpBased") gammaHat <- EBTailIndex(data, 1-k/length(data))
  }
  # computes a point estimates of the extreme level
  tauHat <- 1 - (1 - alpha_n) * gammaHat / (1 - gammaHat)
  if(!is.null(VarGamHat)){
    # computes the variance of the tau estimator using the delta method
    tauVar <- VarGamHat * (alpha_n-1)^2 / (1-gammaHat)^4
    # compute an symptotic confidence interval for the extreme level
    if(!is.null(k)){
      ME <- qnorm(1-alpha/2) * sqrt(tauVar / k)
      tauCI <- c(tauHat - ME, tauHat + ME)
    }
  }
  return(list(tauHat=tauHat, tauVar=tauVar, tauCI=tauCI))
}

#########################################################
###
### Estimation of the Quantile-Based marginal
### Expected Shortfall
### @ the extreme level
###
#########################################################

QuantMES <- function(data, tau, tau1, var=FALSE, varType="asym-Dep", bias=FALSE,
                     bigBlock=NULL, smallBlock=NULL, k=NULL, alpha=0.05){
  VarHatQMES <- NULL
  CIHatQMES <- NULL
  CIHatQMES <- NULL

  # main body:
  isNk <- is.null(k)
  isNtau <- is.null(tau)
  if(isNk & isNtau) stop("Enter a value for at least one of the two parameters 'tau' and 'k' ")
  # compute the sample size
  ndata <- nrow(data)
  if(isNk) k <- floor(ndata*(1-tau))
  if(isNtau) tau <- 1-k/ndata

  # Estimation of the quantile at the intermediate level based on the empirical quantile
  QHat <- as.numeric(quantile(data[,2], tau))
  # Estimation of the tail index and related quantities using the Hill estimator
  if(varType=="asym-Alt-Dep") estHill <- HTailIndex(data[,1], k)
  else estHill <- HTailIndex(data[,1], k, var, varType, bias, bigBlock, smallBlock, alpha)
  # Computes a an estimate of the tail index with a bias-correction
  gammaHat <- estHill$gammaHat - estHill$BiasGamHat
  # Estimation of the QB marginal Expected Shortfall @ the intermediate level
  QMESt <- QBmarExpShortF(data, tau, ndata, QHat)
  # Estimation of the QB marginal Expected Shortfall @ the intermediate level
  QMESt1 <- ((1-tau1)/(1-tau))^(-gammaHat) * QMESt
  # Estimation of the asymptotic variance of the Hill estimator
  if(var){
    # The default approach for computing the asymptotic variance with serial dependent observations
    # is the method proposed by Drees (2003). In this case it is assumed that there is no bias term
    # If it is assumed that the Hill estimator requires a bias correction then the the asymptotic
    # variance is estimated after correcting the tail index estimate by the bias correction
    VarHatQMES <- estHill$VarGamHat

    if((varType=="asym-Alt-Dep") & (!bias)){
      # Drees (2003) approach
      j <- max(2, ceiling(ndata*(1-tau1)))
      VarHatQMES <- DHVar(data, tau, tau1, j, k, ndata)
    }
    # compute lower and upper bounds of the (1-alpha)% CI
    K <- sqrt(VarHatQMES)*log((1-tau)/(1-tau1))/sqrt(k)
    lbound <- QMESt1 * exp(qnorm(alpha/2)*K)
    ubound <- QMESt1 * exp(qnorm(1-alpha/2)*K)
    CIHatQMES <- c(lbound, ubound)
  }
  return(list(HatQMES=QMESt1, VarHatQMES=VarHatQMES, CIHatQMES=CIHatQMES))
}

ExpectMES <- function(data, tau, tau1, method="LAWS", var=FALSE, varType="asym-Dep",
                      bias=FALSE, bigBlock=NULL, smallBlock=NULL, k=NULL,
                      alpha_n=NULL, alpha=0.05){
  VarHatXMES <- NULL
  CIHatXMES <- NULL
  CIHatXMES <- NULL
  # main body:
  isNk <- is.null(k)
  isNtau <- is.null(tau)
  if(isNk & isNtau) stop("Enter a value for at least one of the two parameters 'tau' and 'k' ")
  # compute the sample size
  ndata <- nrow(data)
  if(isNk) k <- floor(ndata*(1-tau))
  if(isNtau) tau <- 1-k/ndata

  # Estimation of the tail index for X using the Hill estimator
  estHill <- HTailIndex(data[,1], k, var, varType, bias, bigBlock, smallBlock, alpha)
  gammaHatX <- estHill$gammaHat - estHill$BiasGamHat
  # Estimation of the tail index for Y using the Hill estimator
  gammaHatY <- HTailIndex(data[,2], k)$gammaHat
  # Estimation of the extreme level if requested
  if(is.null(tau1)) tau1 <- estExtLevel(alpha_n, gammaHat=gammaHatY)$tauHat
  # Expectile-Based Esitmation
  if(method=="LAWS"){
    ExpctHat <- estExpectiles(data[,2], tau)$ExpctHat
    # Estimation of the EB marginal Expected Shortfall @ the intermediate level
    XMESt <- EBmarExpShortF(data, tau, ndata, ExpctHat)
    # Estimation of the Expectile-EB marginal Expected Shortfall @ the extreme level
    XMESt1 <- ((1-tau1)/(1-tau))^(-gammaHatX) * XMESt
  }
  # Quantile-Based Estimation
  if(method=="QB"){
    # Estimation of the QB marginal Expected Shortfall @ the intermediate level
    QMESt <- QuantMES(data, NULL, tau1, k=k)$HatQMES
    # Estimation of the Expectile-QB marginal Expected Shortfall @ the extreme level
    XMESt1 <- (1/gammaHatY - 1)^(-gammaHatX) * QMESt
  }
  # Compute an estimate of the asymptotic variance of the expecile estimator at the extreme level
  if(var){
    # Computes the asymptotic variance for independent and serial dependent data
    VarHatXMES <- estHill$VarGamHat
    # Computes the "margin error"
    K <- sqrt(VarHatXMES)*log((1-tau)/(1-tau1))/sqrt(k)
    # compute lower and upper bounds of the (1-alpha)% CI
    lbound <- XMESt1 * exp(qnorm(alpha/2)*K)
    ubound <- XMESt1 * exp(qnorm(1-alpha/2)*K)
    # defines the (1-alpha)% CI
    CIHatXMES <- c(lbound, ubound)
  }
  return(list(HatXMES=XMESt1, VarHatXMES=VarHatXMES, CIHatXMES=CIHatXMES))
}


#########################################################
### END
###
################## UNIVARIATE CASE ######################


#########################################################
### END
### Main-routines (VISIBLE FUNCTIONS)
###
#########################################################

\name{MultiHTailIndex}
\alias{MultiHTailIndex}

\title{Multidimensional Hill Tail Index Estimation}
\description{Computes point estimates and \eqn{(1-\alpha)100\%} confidence regions estimate of \eqn{d}-dimensional tail indices based on the Hill's estimator.}
\usage{
MultiHTailIndex(data, k, var=FALSE, varType="asym-Dep", bias=FALSE,
                alpha=0.05, plot=FALSE)
}

\arguments{
	\item{data}{A matrix of \eqn{(n \times d)}{(n x d)} observations.}
	\item{k}{An integer specifying the value of the intermediate sequence \eqn{k_n}. See \bold{Details}.}
    \item{var}{If \code{var=TRUE} then an estimate of the variance-covariance matrix of the tail indices estimators is computed.}
    \item{varType}{A string specifying the asymptotic variance to compute. By default \code{varType="asym-Dep"} specifies the variance estimator for \eqn{d} dependent marginal variables. See \bold{Details}.}
    \item{bias}{A logical value. By default \code{biast=FALSE} specifies that no bias correction is computed. See \bold{Details}.}
	\item{alpha}{A real in \eqn{(0,1)} specifying the confidence level \eqn{(1-\alpha)100\%} of the approximate confidence interval for the tail index.}
    \item{plot}{A logical value. By default \code{plot=FALSE} specifies that no graphical representation of the estimates is provided. See \bold{Details}.}
}

\details{
	For a dataset \code{data} of \eqn{(n \times d)}{(n x d)} observations, where \eqn{d} is the number of variables and \eqn{n} is the sample size, the tail index \eqn{\gamma} of the \eqn{d} marginal distributions is estimated by applying the Hill estimator. Together with a point estimate a \eqn{(1-\alpha)100\%} confidence region is computed. The data are regarded as \code{d}-dimensional temporal independent observations coming from dependent variables.
	\itemize{
        \item \code{k} or \eqn{k_n} is the value of the so-called intermediate sequence \eqn{k_n}, \eqn{n=1,2,\ldots}{n=1,2,...}. Its represents a sequence of positive integers such that \eqn{k_n \to \infty}{k_n -> \infty} and \eqn{k_n/n \to 0}{k_n/n -> 0} as \eqn{n \to \infty}{n -> \infty}. Practically, the value \eqn{k_n} specifies the number of \code{k}\eqn{+1} larger order statistics to be used to estimate each marginal tail index \eqn{\gamma_j} for \eqn{j=1,\ldots,d}{j=1,...,d}.
		\item If \code{var=TRUE} then an estimate of the asymptotic variance-covariance matrix of the multivariate Hill estimator is computed. With independent observations the asymptotic variance-covariance matrix is estimated by the matrix \eqn{\hat{\Sigma}^{LAWS}_{j,\ell}(\gamma,R)(1,1)}, see bottom formula in page 14 of Padoan and Stupfler (2020). This is achieved through \code{varType="asym-Dep"} which means \eqn{d} dependent marginal variables. When \code{varType="asym-Ind"} \eqn{d} marginal variables are regarded as independent and the returned variance-covariance matrix \eqn{\hat{\Sigma}^{LAWS}_{j,\ell}(\gamma,R)(1,1)} is a diagonal matrix with only variance terms.
        \item If \code{bias=TRUE} then an estimate of the bias term of the Hill estimator is computed implementing using formula (4.2) in de Haan et al. (2016). In this case the asymptotic variance is not estimated using the formula in Haan et al. (2016) Theorem 4.1 but instead for simplicity the formula at the bottom of page 14 in Padoan and Stupfler (2020) is still used.
        \item Given a small value \eqn{\alpha\in (0,1)} then an estimate of an asymptotic confidence region for \eqn{\gamma_j}, for \eqn{j=1,\ldots,d}{j=1,...,d}, with approximate nominal confidence level \eqn{(1-\alpha)100\%}, is computed. The confidence intervals are computed exploiting the asymptotic normality of multivariate Hill estimator appropriately normalized (the logarithmic scale is not used), see Padoan and Stupfler (2020) for details.
        \item If \code{plot=TRUE} then a graphical representation of the estimates is not provided.
	}
}

\value{
	A list with elements:
	\itemize{
		\item \code{gammaHat}: an estimate of the \eqn{d} tail indices \eqn{\gamma_j}, for \eqn{j=1,\ldots,d}{j=1,...,d};
		\item \code{VarCovGHat}: an estimate of the asymptotic variance-covariance matrix of the multivariate Hill estimator;
		\item \code{biasTerm}: an estimate of bias term of the multivariate Hill estimator;
        \item \code{EstConReg}: an estimate of the \eqn{(1-\alpha)100\%} confidence region.
	}
}

\references{
  Simone A. Padoan and Gilles Stupfler (2022). Joint inference on extreme expectiles for multivariate heavy-tailed distributions, \emph{Bernoulli} \bold{28}(2), 1021-1048.

  de Haan, L., Mercadier, C. and Zhou, C. (2016). Adapting extreme value statistics
  to financial time series: dealing with bias and serial dependence. \emph{Finance and Stochastics},
  \bold{20}, 321-354.

  de Haan, L. and Ferreira, A. (2006). Extreme Value Theory: An Introduction. \emph{Springer}-\emph{Verlag}, New York.

}

\author{
	Simone Padoan, \email{simone.padoan@unibocconi.it},
	\url{https://www.unibocconi.it/en/faculty/simone-padoan};
	Gilles Stupfler, \email{gilles.stupfler@univ-angers.fr},
	\url{https://math.univ-angers.fr/~stupfler/}
}

\seealso{
	\link{HTailIndex}, \link{rmdata}
}

\examples{
# Tail index estimation based on the multivariate Hill estimator obtained with
# n observations simulated from a d-dimensional random vector with a multivariate
# distribution with equal Frechet margins and a Clayton copula.
library(plot3D)
library(copula)
library(evd)

# distributional setting
copula <- "Clayton"
dist <- "Frechet"

# parameter setting
dep <- 3
dim <- 3
scale <- rep(1, dim)
shape <- rep(3, dim)
par <- list(dep=dep, scale=scale, shape=shape, dim=dim)

# Number of larger order statistics
k <- 150

# sample size
ndata <- 1000

# Simulates a sample from a multivariate distribution with equal Frechet
# marginals distributions and a Clayton copula
data <- rmdata(ndata, dist, copula, par)
scatter3D(data[,1], data[,2], data[,3])

# tail indices estimation
est <- MultiHTailIndex(data, k, TRUE)
est$gammaHat
est$VarCovGHat
# run the following command to see the graphical representation
\dontrun{
 est <- MultiHTailIndex(data, k, TRUE, plot=TRUE)
}
}

\name{Bqgpd_c}
\alias{Bqgpd_c}
\title{Bayesian predictive quantile for generalized Pareto distribution}
\usage{
Bqgpd_c(p, post, lower, upper)
}
\arguments{
\item{p}{probability levels}

\item{post}{matrix of posterior samples}

\item{lower}{lower bound for the search for the quantile}

\item{upper}{upper bound for the region over which to search for the quantile}
}
\value{
a vector of quantiles of the same length as \code{p}
}
\description{
WARNING: this function will not be exported in a future version of the package.
}
\keyword{internal}

\name{EBTailIndex}
\alias{EBTailIndex}

\title{Expectile Based Tail Index Estimation}
\description{Computes a point estimate of the tail index based on the Expectile Based (EB) estimator.}
\usage{
EBTailIndex(data, tau, est=NULL)
}

\arguments{
	\item{data}{A vector of \eqn{(1 \times n)}{(1 x n)} observations.}
	\item{tau}{A real in \eqn{(0,1)} specifying the intermediate level \eqn{\tau_n}. See \bold{Details}\.}
	\item{est}{A real specifying the estimate of the expectile at the intermediate level \code{tau}.}
}

\details{
	For a dataset \code{data} of sample size \eqn{n}, the tail index \eqn{\gamma} of its (marginal) distribution is estimated using the EB estimator:

    \eqn{
    \hat{\gamma}_n^E =\left(1+\frac{\hat{\bar{F}}_n(\tilde{\xi}_{\tau_n})}{1-\tau_n}\right)^{-1}
	}{\gamma_n^E=(1+\frac{hat{bar{F}}_n(tilde{xi}_{tau_n})}{1-tau_n})^{-1}},

	where \eqn{\hat{\bar{F}}_n}{\hat{\bar{F}}_n} is the empirical survival function of the observations, \eqn{\tilde{\xi}_{\tau_n}}{tilde{xi}_{tau_n}} is an estimate of the \eqn{\tau_n}-\emph{th} expectile.
	The observations can be either independent or temporal dependent. See Padoan and Stupfler (2020) and Daouia et al. (2018) for details.
	\itemize{
	    \item The so-called intermediate level \code{tau} or \eqn{\tau_n}{tau_n} is a sequence of positive reals such that \eqn{\tau_n \to 1}{\tau_n -> 1} as \eqn{n \to \infty}{n -> \infty}. Practically, \eqn{\tau_n \in (0,1)}{\tau_n in (0,1)} is the ratio between the empirical mean distance of the \eqn{\tau_n}-\emph{th} expectile from the smaller observations and the empirical mean distance of of the \eqn{\tau_n}-\emph{th} expectile from all the observations. An estimate of \eqn{\tau_n}-\emph{th} expectile is computed and used in turn to estimate \eqn{\gamma}.
		\item The value \code{est}, if provided, is meant to be an esitmate of the \eqn{\tau_n}-\emph{th} expectile which is used to estimate \eqn{\gamma}. On the contrary, if \code{est=NULL}, then the routine \code{EBTailIndex} estimate first the \eqn{\tau_n}-\emph{th} expectile expectile and then use it to estimate \eqn{\gamma}.
	}
}

\value{
	An estimate of the tain index \eqn{\gamma}.
}

\references{
Anthony C. Davison, Simone A. Padoan and Gilles Stupfler (2023). Tail Risk Inference via Expectiles in Heavy-Tailed Time Series, \emph{Journal of Business & Economic Statistics}, \bold{41}(3) 876-889.

Daouia, A., Girard, S. and Stupfler, G. (2018). Estimation of tail risk based on extreme expectiles. \emph{Journal of the Royal Statistical Society: Series B}, \bold{80}, 263-292.
}

\author{
	Simone Padoan, \email{simone.padoan@unibocconi.it},
	\url{https://www.unibocconi.it/en/faculty/simone-padoan};
	Gilles Stupfler, \email{gilles.stupfler@univ-angers.fr},
	\url{https://math.univ-angers.fr/~stupfler/}
}

\seealso{
	\link{HTailIndex}, \link{MomTailIndex}, \link{MLTailIndex},
}

\examples{
# Tail index estimation based on the Expectile based estimator obtained with data
# simulated from an AR(1) with 1-dimensional Student-t distributed innovations

tsDist <- "studentT"
tsType <- "AR"

# parameter setting
corr <- 0.8
df <- 3
par <- c(corr, df)

# Big- small-blocks setting
bigBlock <- 65
smallblock <- 15

# Intermediate level (or sample tail probability 1-tau)
tau <- 0.97

# sample size
ndata <- 2500

# Simulates a sample from an AR(1) model with Student-t innovations
data <- rtimeseries(ndata, tsDist, tsType, par)

# tail index estimation
gammaHat <- EBTailIndex(data, tau)
gammaHat
}

\name{QuantMES}
\alias{QuantMES}

\title{Marginal Expected Shortfall Quantile Based Estimation}
\description{Computes a point and interval estimate of the Marginal Expected Shortfall (MES) using a quantile based approach.}
\usage{
QuantMES(data, tau, tau1, var=FALSE, varType="asym-Dep", bias=FALSE, bigBlock=NULL,
         smallBlock=NULL, k=NULL, alpha=0.05)
}

\arguments{
	\item{data}{A vector of \eqn{(1 \times n)}{(1 x n)} observations.}
	\item{tau}{A real in \eqn{(0,1)} specifying the intermediate level \eqn{\tau_n}. See \bold{Details}.}
	\item{tau1}{A real in \eqn{(0,1)} specifying the extreme level \eqn{\tau'_n}. See \bold{Details}.}
	\item{var}{If \code{var=TRUE} then an estimate of the asymptotic variance of the MES estimator is computed.}
	\item{varType}{A string specifying the type of asymptotic variance to compute. By default \code{varType="asym-Dep"} specifies the variance estimator for serial dependent observations. See \bold{Details}.}
	\item{bias}{A logical value. By default \code{bias=FALSE} specifies that no bias correction is computed. See \bold{Details}.}
	\item{bigBlock}{An interger specifying the size of the big-block used to estimaste the asymptotic variance. See \bold{Details}.}
	\item{smallBlock}{An interger specifying the size of the small-block used to estimaste the asymptotic variance. See \bold{Details}.}
	\item{k}{An integer specifying the value of the intermediate sequence \eqn{k_n}. See \bold{Details}.}
	\item{alpha}{A real in \eqn{(0,1)} specifying the confidence level \eqn{(1-\alpha)100\%} of the approximate confidence interval for the expecile at the intermedite level.}
}

\details{
	For a dataset \code{data} of sample size \eqn{n}, an estimate of the \eqn{\tau'_n}-\emph{th} MES is computed. The estimation of the MES at the extreme level \code{tau1} (\eqn{\tau'_n}) is indeed meant to be a prediction. Estimates are obtained through the quantile based estimator defined in page 12 of Padoan and Stupfler (2020). Such an estimator depends on the estimation of the tail index \eqn{\gamma}. Here, \eqn{\gamma} is estimated using the Hill estimation (see \link{HTailIndex} for details).
	The observations can be either independent or temporal dependent. See Section 4 in Padoan and Stupfler (2020) for details.
	\itemize{
		\item The so-called intermediate level \code{tau} or \eqn{\tau_n} is a sequence of positive reals such that \eqn{\tau_n \to 1}{\tau_n -> 1} as \eqn{n \to \infty}{n -> \infty}. See \link{predExpectiles} for details.
		\item The so-called extreme level \code{tau1} or \eqn{\tau'_n} is a sequence of positive reals such that \eqn{\tau'_n \to 1}{\tau'_n -> 1} as \eqn{n \to \infty}{n -> \infty}. See \link{predExpectiles} for details.
		\item If \code{var=TRUE} then an esitmate of the asymptotic variance of the \eqn{tau'_n}-\emph{th} MES is computed. Notice that the estimation of the asymptotic variance \bold{is only available} when \eqn{\gamma} is estimated using the Hill estimator (see \link{HTailIndex}). With independent observations the asymptotic variance is estimated by \eqn{\hat{\gamma}^2}, see Corollary 4.3 in Padoan and Stupfler (2020). This is achieved through \code{varType="asym-Ind"}.  With serial dependent observations the asymptotic variance is estimated by the formula in Corollary 4.2 of Padoan and Stupfler (2020). This is achieved through \code{varType="asym-Dep"}. See Section 4 and 5 in Padoan and Stupfler (2020) for details. In this latter case the computation of the serial dependence is based on the "big blocks seperated by small blocks" techinque which is a standard tools in time series, see e.g. Leadbetter et al. (1986).
		The size of the big and small blocks are specified by the parameters \code{bigBlock} and \code{smallBlock}, respectively.
		\item If \code{bias=TRUE} then \eqn{\gamma} is estimated using formula (4.2) of Haan et al. (2016). This is used by the LAWS and QB estimators. Furthermore, the \eqn{\tau'_n}--\emph{th} quantile is estimated using the formula in page 330 of de Haan et al. (2016). This provides a bias corrected version of the Weissman estimator. This is used by the QB estimator.
		However, in this case the asymptotic variance is not estimated using the formula in Haan et al. (2016) Theorem 4.2. Instead, for simplicity the asymptotic variance is estimated by the formula in Corollary 3.8, with serial dependent observations, and \eqn{\hat{\gamma}^2} with independent observation (see e.g. de Drees 2000, for the details).
		\item \code{k} or \eqn{k_n} is the value of the so-called intermediate sequence \eqn{k_n}, \eqn{n=1,2,\ldots}{n=1,2,...}. Its represents a sequence of positive integers such that \eqn{k_n \to \infty}{k_n -> \infty} and \eqn{k_n/n \to 0}{k_n/n -> 0} as \eqn{n \to \infty}{n -> \infty}. \eqn{k_n} specifies the number of \code{k}\eqn{+1} larger order statistics used in the definition of the Hill estimator (see \link{HTailIndex} for details).
        \item If the quantile's extreme level is provided by \code{alpha_n}, then expectile's extreme level \eqn{tau'_n} is replaced by \eqn{tau'_n(\alpha_n)} which is estimated by the method described in Section 6 of Padoan and Stupfler (2020). See \link{estExtLevel} for details.
		\item Given a small value \eqn{\alpha\in (0,1)} then an estimate of an asymptotic confidence interval for \eqn{tau'_n}-\emph{th} expectile, with approximate nominal confidence level \eqn{(1-\alpha)100\%}, is computed. The confidence intervals are computed exploiting formula in Corollary 4.2, Theorem 6.2 of Padoan and Stupfler (2020) and (46) in Drees (2003). See Sections 4-6 in Padoan and Stupfler (2020) for details. When \code{biast=TRUE} confidence intervals are computed in the same way but after correcting the tail index estimate by an estimate of the bias term, see formula (4.2) in de Haan et al. (2016) for details.
	}
}

\value{
  A list with elements:
    \itemize{
	  \item \code{HatQMES}: an estimate of the \eqn{\tau'_n}-\emph{th} quantile based MES;
	  \item \code{VarHatQMES}: an estimate of the asymptotic variance of the quantile based MES estimator;
	  \item \code{CIHatQMES}: an estimate of the approximate \eqn{(1-\alpha)100\%} confidence interval for \eqn{\tau'_n}-\emph{th} MES.
	}
}

\references{
Anthony C. Davison, Simone A. Padoan and Gilles Stupfler (2023). Tail Risk Inference via Expectiles in Heavy-Tailed Time Series, \emph{Journal of Business & Economic Statistics}, \bold{41}(3) 876-889.

Daouia, A., Girard, S. and Stupfler, G. (2018). Estimation of tail risk based on extreme expectiles. \emph{Journal of the Royal Statistical Society: Series B}, \bold{80}, 263-292.

de Haan, L., Mercadier, C. and Zhou, C. (2016). Adapting extreme value statistics to financial time series: dealing with bias and serial dependence. \emph{Finance and Stochastics},
\bold{20}, 321-354.

Drees, H. (2003). Extreme quantile estimation for dependent data, with applications to finance.
\emph{Bernoulli}, \bold{9}, 617-657.

Drees, H. (2000). Weighted approximations of tail processes for \eqn{\beta}-mixing random variables.
\emph{Annals of Applied Probability}, \bold{10}, 1274-1301.

Leadbetter, M.R., Lindgren, G. and Rootzen, H. (1989). Extremes and related
properties of random sequences and processes. \emph{Springer}.
}

\author{
	Simone Padoan, \email{simone.padoan@unibocconi.it},
	\url{https://www.unibocconi.it/en/faculty/simone-padoan};
	Gilles Stupfler, \email{gilles.stupfler@univ-angers.fr},
	\url{https://math.univ-angers.fr/~stupfler/}
}


\seealso{
	\link{ExpectMES}, \link{HTailIndex},  \link{predExpectiles}, \link{extQuantile}
}

\examples{
# Marginl Expected Shortfall quantile based estimation at the extreme level
# obtained with 2-dimensional data simulated from an AR(1) with bivariate
# Student-t distributed innovations


tsDist <- "AStudentT"
tsType <- "AR"
tsCopula <- "studentT"

# parameter setting
corr <- 0.8
dep <- 0.8
df <- 3
par <- list(corr=corr, dep=dep, df=df)

# Big- small-blocks setting
bigBlock <- 65
smallBlock <- 15

# quantile's extreme level
tau1 <- 0.9995

# sample size
ndata <- 2500

# Simulates a sample from an AR(1) model with Student-t innovations
data <- rbtimeseries(ndata, tsDist, tsType, tsCopula, par)


# Extreme MES expectile based estimation
MESHat <- QuantMES(data, NULL, tau1, var=TRUE, k=150, bigBlock=bigBlock,
                   smallBlock=smallBlock)
MESHat

}

\name{estExtLevel}
\alias{estExtLevel}

\title{Extreme Level Estimation}
\description{Estimates the expectile's extreme level corresponding to a quantile's extreme level.}
\usage{
estExtLevel(alpha_n, data=NULL, gammaHat=NULL, VarGamHat=NULL, tailest="Hill", k=NULL,
            var=FALSE, varType="asym-Dep", bigBlock=NULL, smallBlock=NULL, alpha=0.05)
}

\arguments{
	\item{alpha_n}{A real in \eqn{(0,1)} specifying the extreme level \eqn{\alpha_n} for the quantile. See \bold{Details}.}
    \item{data}{A vector of \eqn{(1 \times n)}{(1 x n)} observations to be used to estimate the tail index in the case it is not provided. By default \code{data=NULL} specifies that no data are given.}
    \item{gammaHat}{A real specifying an estimate of the tail index. By default \code{gammaHat=NULL} specifies that no estimate is given. See \bold{Details}.}
    \item{VarGamHat}{A real specifying an estimate of the variance of the tail index estimate. By default \code{VarGamHat=NULL} specifies that no estimate is given. See \bold{Details}.}
	\item{tailest}{A string specifying the type of tail index estimator to be used. By default \code{tailest="Hill"} specifies the use of Hill estimator. See \bold{Details}.}
    \item{k}{An integer specifying the value of the intermediate sequence \eqn{k_n}. See \bold{Details}.}
	\item{var}{If \code{var=TRUE} then an estimate of the variance of the extreme level estimator is computed.}
	\item{varType}{A string specifying the asymptotic variance to compute. By default \code{varType="asym-Dep"} specifies the variance estimator for serial dependent observations. See \bold{Details}.}
	\item{bigBlock}{An interger specifying the size of the big-block used to estimaste the asymptotic variance. See \bold{Details}.}
	\item{smallBlock}{An interger specifying the size of the small-block used to estimaste the asymptotic variance. See \bold{Details}.}
	\item{alpha}{A real in \eqn{(0,1)} specifying the confidence level \eqn{(1-\alpha)100\%} of the approximate confidence interval for the expecile at the intermedite level.}
}

\details{
	For a given extreme level \eqn{\alpha_n} for the \eqn{\alpha_n}-\emph{th} quantile, an estimate of the extreme level \eqn{\tau_n'(\alpha_n)} is computed such that \eqn{\xi_{\tau_n'(\alpha_n)}=q_{\alpha_n}}. The estimator is defined by

    \eqn{\hat{\tau}_n'(\alpha_n) = 1 - (1 - \alpha_n)\frac{\hat{\gamma}_n}{1-\hat{\gamma}_n}}

    where \eqn{\hat{\gamma}_n} is a consistent estimator of the tail index \eqn{\gamma}. If a value for the parameter \code{gammaHat} is given, then such a value is used to compute \eqn{\hat{\tau}_n'}. If \code{gammaHat} is \code{NULL} and a dataset is provided through the parameter \code{data}, then the tail index \eqn{\gamma} is estimated by a suitable estimator \eqn{\hat{\gamma}_n}. See Section 6 in Padoan and Stupfler (2020) for more details.
	\itemize{
		\item If \code{VarGamHat} is specified, i.e. the variance of the tail index estimator, then the variance of the extreme level estimator \eqn{\hat{\tau}_n'} is computed by using such value.
		\item When estimating the tail index, if \code{tailest='Hill'} then \eqn{\gamma} is estimated using the Hill estimator (see also \link{HTailIndex}). If \code{tailest='ML'} then \eqn{\gamma} is estimated using the Maximum Likelihood estimator (see \link{MLTailIndex}).
        If \code{tailest='ExpBased'} then \eqn{\gamma} is estimated using the expectile based estimator (see \link{EBTailIndex}).
        If \code{tailest='Moment'} then \eqn{\gamma} is estimated using the moment based estimator (see \link{MomTailIndex}). See Padoan and Stupfler (2020) for details.
		\item \code{k} or \eqn{k_n} is the value of the so-called intermediate sequence \eqn{k_n}, \eqn{n=1,2,\ldots}{n=1,2,...}. Its represents a sequence of positive integers such that \eqn{k_n \to \infty}{k_n -> \infty} and \eqn{k_n/n \to 0}{k_n/n -> 0} as \eqn{n \to \infty}{n -> \infty}. Practically, when \code{tailest="Hill"} then the value \eqn{k_n} specifies the number of \code{k}\eqn{+1} larger order statistics to be used to estimate \eqn{\gamma} by the Hill estimator. See \link{MLTailIndex}, \link{EBTailIndex} and \link{MomTailIndex} for the other estimators.
		\item If \code{var=TRUE} then the asymptotic variance of the extreme level estimator is computed by applying the delta method, i.e.

        \eqn{Var(\tau_n') = Var(\hat{\gamma}_n) * (\alpha_n-1)^2 / (1-\hat{\gamma}_n)^4}

        where \eqn{Var(\hat{\gamma}_n} is provided by \code{VarGamHat} or is estimated when esitmating the tail index through \code{tailest='Hill'} and \code{tailest='ML'}. See  \link{HTailIndex} and \link{MLTailIndex} for details on how the variance is computed.
		\item Given a small value \eqn{\alpha\in (0,1)} then an asymptotic confidence interval for the extreme level, \eqn{\tau_n'(\alpha_n)}, with approximate nominal confidence level \eqn{(1-\alpha)100\%} is computed.
	}
}

\value{
  A list with elements:
    \itemize{
	  \item \code{tauHat}: an estimate of the extreme level \eqn{\tau_n'};
	  \item \code{tauVar}: an estimate of the asymptotic variance of the extreme level estimator \eqn{\hat{\tau}_n'(\alpha_n)};
	  \item \code{tauCI}: an estimate of the approximate \eqn{(1-\alpha)100\%} confidence interval for the extreme level \eqn{\tau_n'(\alpha_n)}.
	}
}

\references{
Anthony C. Davison, Simone A. Padoan and Gilles Stupfler (2023). Tail Risk Inference via Expectiles in Heavy-Tailed Time Series, \emph{Journal of Business & Economic Statistics}, \bold{41}(3) 876-889.

Daouia, A., Girard, S. and Stupfler, G. (2018). Estimation of tail risk based on extreme expectiles. \emph{Journal of the Royal Statistical Society: Series B}, \bold{80}, 263-292.

}

\author{
	Simone Padoan, \email{simone.padoan@unibocconi.it},
	\url{https://www.unibocconi.it/en/faculty/simone-padoan};
	Gilles Stupfler, \email{gilles.stupfler@univ-angers.fr},
	\url{https://math.univ-angers.fr/~stupfler/}
}

\seealso{
	\link{estExpectiles}, \link{predExpectiles}, \link{extQuantile}
}

\examples{
# Extreme level estimation for a given quantile's extreme level alpha_n
# obtained with 1-dimensional data simulated from an AR(1) with Student-t innovations

tsDist <- "studentT"
tsType <- "AR"

# parameter setting
corr <- 0.8
df <- 3
par <- c(corr, df)

# Big- small-blocks setting
bigBlock <- 65
smallBlock <- 15

# quantile's extreme level
alpha_n <- 0.999

# sample size
ndata <- 2500

# Simulates a sample from an AR(1) model with Student-t innovations
data <- rtimeseries(ndata, tsDist, tsType, par)

# expectile's extreme level estimation
tau1Hat <- estExtLevel(alpha_n, data, var=TRUE, k=150, bigBlock=bigBlock,
                       smallBlock=smallBlock)
tau1Hat
}

\name{extMultiQuantile}
\alias{extMultiQuantile}

\title{Multidimensional Value-at-Risk (VaR) or Extreme Quantile (EQ) Estimation}
\description{Computes point estimates and \eqn{(1-\alpha)100\%} confidence regions for \code{d}-dimensional VaR based on the Weissman estimator.}
\usage{
extMultiQuantile(data, tau, tau1, var=FALSE, varType="asym-Ind-Log", bias=FALSE,
                 k=NULL, alpha=0.05, plot=FALSE)
}

\arguments{
	\item{data}{A matrix of \eqn{(n \times d)}{(n x d)} observations.}
	\item{tau}{A real in \eqn{(0,1)} specifying the intermediate level \eqn{\tau_n}. See \bold{Details}.}
	\item{tau1}{A real in \eqn{(0,1)} specifying the extreme level \eqn{\tau'_n}. See \bold{Details}.}
	\item{var}{If \code{var=TRUE} then an estimate of the asymptotic variance-covariance matrix of the \code{d}-dimensional VaR estimator is computed.}
	\item{varType}{A string specifying the type of asymptotic variance-covariance matrix to compute. By default \code{varType="asym-Ind-Log"} specifies that the variance-covariance matrix is obtained assuming dependent variables and exploiting the logarithmic scale. See \bold{Details}.}
	\item{bias}{A logical value. By default \code{biast=FALSE} specifies that no bias correction is computed. See \bold{Details}.}
	\item{k}{An integer specifying the value of the intermediate sequence \eqn{k_n}. See \bold{Details}.}
	\item{alpha}{A real in \eqn{(0,1)} specifying the confidence level \eqn{(1-\alpha)100\%} of the approximate confidence region for the \code{d}-dimensional VaR.}
    \item{plot}{A logical value. By default \code{plot=FALSE} specifies that no graphical representation of the estimates is not provided. See \bold{Details}.}
}

\details{
	For a dataset \code{data} of \code{d}-dimensional observations and sample size \eqn{n}, the VaR or EQ, correspoding to the extreme level \code{tau1}, is computed by applying the \code{d}-dimensional Weissman estimator. The definition of the Weissman estimator depends on the estimation of the \code{d}-dimensional tail index \eqn{\gamma}. Here, \eqn{\gamma} is estimated using the Hill estimation (see \link{MultiHTailIndex}). The data are regarded as \code{d}-dimensional temporal independent observations coming from dependent variables. See Padoan and Stupfler (2020) for details.
	\itemize{
		\item The so-called intermediate level \code{tau} or \eqn{\tau_n} is a sequence of positive reals such that \eqn{\tau_n \to 1}{\tau_n -> 1} as \eqn{n \to \infty}{n -> \infty}. Practically, for each variable, \eqn{(1-\tau_n)\in (0,1)}{(1-\tau_n) in (0,1)} is a small proportion of observations in the observed data sample that exceed the \eqn{tau_n}-\emph{th} empirical quantile. Such proportion of observations is used to estimate the individual \eqn{tau_n}-\emph{th} quantile and tail index \eqn{\gamma}.
		\item The so-called extreme level \code{tau1} or \eqn{\tau'_n} is a sequence of positive reals such that \eqn{\tau'_n \to 1}{\tau'_n -> 1} as \eqn{n \to \infty}{n -> \infty}.
		For each variable, the value \eqn{(1-tau'_n) \in (0,1)}{(1-\tau'_n) in (0,1)} is meant to be a small tail probability such that \eqn{(1-\tau'_n)=1/n} or \eqn{(1-\tau'_n) < 1/n}. It is also assumed that \eqn{n(1-\tau'_n) \to C}{n(1-tau'_n) -> C} as \eqn{n \to \infty}{n -> \infty}, where \eqn{C} is a positive finite constant. The value \eqn{C} is the expected number of exceedances of the individual \eqn{\tau'_n}-\emph{th} quantile. Typically, \eqn{C \in (0,1)}{C in (0,1)} which means that it is expected that there are no observations in a data sample exceeding the individual quantile of level \eqn{(1-\tau_n')}.
		\item If \code{var=TRUE} then an estimate of the asymptotic variance-covariance matrix of the \eqn{tau'_n}-\emph{th} \code{d}-dimensional quantile is computed. The data are regarded as temporal independent observations coming from dependent variables. The asymptotic variance-covariance matrix is estimated exploiting the formula in Section 5 of Padoan and Stupfler (2020). In particular, the variance-covariance matrix is computed exploiting the asymptotic behaviour of the normalized quantile estimator which is expressed in logarithmic scale. This is achieved through \code{varType="asym-Ind-Log"}. If \code{varType="asym-Ind"} then the variance-covariance matrix is computed exploiting the asymptotic behaviour of the \code{d}-dimensional relative quantile estimator appropriately normalized (see formula in Section 5 of Padoan and Stupfler (2020)).
		\item If \code{bias=TRUE} then an estimate of each individual \eqn{\tau'_n}--\emph{th} quantile is estimated using the formula in page 330 of de Haan et al. (2016), which provides a bias corrected version of the Weissman estimator. However, in this case the asymptotic variance is not estimated using the formula in Haan et al. (2016) Theorem 4.2. For simplicity standard the variance-covariance matrix is still computed using formula in Section 5 of Padoan and Stupfler (2020).
		\item \code{k} or \eqn{k_n} is the value of the so-called intermediate sequence \eqn{k_n}, \eqn{n=1,2,\ldots}{n=1,2,...}. Its represents a sequence of positive integers such that \eqn{k_n \to \infty}{k_n -> \infty} and \eqn{k_n/n \to 0}{k_n/n -> 0} as \eqn{n \to \infty}{n -> \infty}. Practically, for each marginal distribution, the value \eqn{k_n} specifies the number of \code{k}\eqn{+1} larger order statistics to be used to estimate the individual \eqn{\tau_n}-\emph{th} empirical quantile and individual tail index \eqn{\gamma_j} for \eqn{j=1,\ldots,d}{j=1,...,d}. The intermediate level \eqn{\tau_n} can be seen defined as \eqn{\tau_n=1-k_n/n}.
        \item Given a small value \eqn{\alpha\in (0,1)} then an estimate of an asymptotic confidence region for \eqn{tau'_n}-\emph{th} \code{d}-dimensional quantile, with approximate nominal confidence level \eqn{(1-\alpha)100\%}, is computed. The confidence regions are computed exploiting the asymptotic behaviour of the normalized quantile estimator in logarithmic scale. This is an "asymmetric" region and it is achieved through \code{varType="asym-Ind-Log"}. A "symmetric" region is obtained exploiting the asymptotic behaviour of the relative quantile estimator appropriately normalized, see formula in Section 5 of Padoan and Stupfler (2020). This is achieved through \code{varType="asym-Ind"}.
        \item If \code{plot=TRUE} then a graphical representation of the estimates is not provided.}
}

\value{
	A list with elements:
	\itemize{
		\item \code{ExtQHat}: an estimate of the \code{d}-dimensional VaR or \eqn{\tau'_n}-\emph{th} \code{d}-dimensional quantile;
		\item \code{VarCovExQHat}: an estimate of the asymptotic variance-covariance of the \code{d}-dimensional VaR estimator;
		\item \code{EstConReg}: an estimate of the approximate \eqn{(1-\alpha)100\%} confidence regions for the \code{d}-dimensional VaR.
	}
}

\references{
  Simone A. Padoan and Gilles Stupfler (2022). Joint inference on extreme expectiles for multivariate heavy-tailed distributions, \emph{Bernoulli} \bold{28}(2), 1021-1048.

  de Haan, L., Mercadier, C. and Zhou, C. (2016). Adapting extreme value statistics
  to financial time series: dealing with bias and serial dependence. \emph{Finance and Stochastics},
  \bold{20}, 321-354.

  de Haan, L. and Ferreira, A. (2006). Extreme Value Theory: An Introduction. \emph{Springer}-\emph{Verlag}, New York.
 }

\author{
 	Simone Padoan, \email{simone.padoan@unibocconi.it},
 	\url{https://www.unibocconi.it/en/faculty/simone-padoan};
 	Gilles Stupfler, \email{gilles.stupfler@univ-angers.fr},
 	\url{https://math.univ-angers.fr/~stupfler/}
 }

\seealso{
	\link{MultiHTailIndex},  \link{estMultiExpectiles}, \link{predMultiExpectiles}
}

\examples{
# Extreme quantile estimation at the extreme level tau1 obtained with
# d-dimensional observations simulated from a joint distribution with
# a Gumbel copula and equal Frechet marginal distributions.
library(plot3D)
library(copula)
library(evd)

# distributional setting
copula <- "Gumbel"
dist <- "Frechet"

# parameter setting
dep <- 3
dim <- 3
scale <- rep(1, dim)
shape <- rep(3, dim)
par <- list(dep=dep, scale=scale, shape=shape, dim=dim)

# Intermediate level (or sample tail probability 1-tau)
tau <- 0.95
# Extreme level (or tail probability 1-tau1 of unobserved quantile)
tau1 <- 0.9995

# sample size
ndata <- 1000

# Simulates a sample from a multivariate distribution with equal Frechet
# marginals distributions and a Gumbel copula
data <- rmdata(ndata, dist, copula, par)
scatter3D(data[,1], data[,2], data[,3])

# High d-dimensional expectile (intermediate level) estimation
extQHat <- extMultiQuantile(data, tau, tau1, TRUE)

extQHat$ExtQHat
extQHat$VarCovExQHat
# run the following command to see the graphical representation
\dontrun{
 extQHat <- extMultiQuantile(data, tau, tau1, TRUE, plot=TRUE)
}
}

\name{predDensx}
\alias{predDensx}
\title{Conditional predictive posterior density of peaks-over-threshold models}
\usage{
predDensx(
  x,
  postsamp,
  scedasis,
  threshold,
  type = c("continuous", "discrete"),
  extrapolation = FALSE,
  p,
  k,
  n
)
}
\arguments{
\item{x}{vector of length \code{r} containing the points at which to evaluate the density}

\item{postsamp}{a \code{m} by \code{2} matrix with columns containing the posterior samples of scale and shape parameters of the generalized Pareto distribution, respectively}

\item{scedasis}{an \code{m} by \code{p} matrix, obtained by evaluating the scedasis function for every of the \code{p} covariate values and every \code{m} posterior draw}

\item{threshold}{threshold for the generalized Pareto model, corresponding to the \eqn{n-k}th order statistic of the sample}

\item{type}{data type, either \code{"continuous"} or \code{"discrete"} for count data.}

\item{extrapolation}{logical; if \code{TRUE}, extrapolate using the threshold stability property}

\item{p}{scalar tail probability for the extrapolation. Must be smaller than \eqn{k/n} (only used if \code{extrapolation=TRUE})}

\item{k}{integer, number of exceedances for the generalized Pareto (only used if \code{extrapolation=TRUE})}

\item{n}{integer, number of observations in the full sample. Must be greater than \code{k}  (only used if \code{extrapolation=TRUE})}
}
\value{
a list with components
\itemize{
\item \code{x} the vector at which the posterior density is evaluated
\item \code{preddens} an \code{r} by \code{p} matrix of predictive density corresponding to each combination of \code{x} and scedasis value
}
}
\description{
Given posterior samples for the parameters of the continuous or discrete generalized Pareto distribution and scedasis function for a set of covariates, evaluated at every draw of the latter,
return the predictive posterior density of a peak above an intermediate or extreme threshold using the threshold stability property.
}
\examples{
\dontrun{
# generate data
set.seed(1234)
n <- 500
samp <- evd::rfrechet(n,0,1:n,4)
# set effective sample size and threshold
k <- 50
threshold <- sort(samp,decreasing = TRUE)[k+1]
# preliminary mle estimates of scale and shape parameters
mlest <- evd::fpot(samp, threshold, control=list(maxit = 500))
# empirical bayes procedure
proc <- estPOT(
  samp,
  k = k,
  pn = c(0.01, 0.005),
  type = "continuous",
  method = "bayesian",
  prior = "empirical",
  start = as.list(mlest$estimate),
  sig0 = 0.1)
# conditional predictive density estimation
yg <- seq(0, 50, by = 2)
nyg <- length(yg)
# estimation of scedasis function
# setting
M <- 1e3
C <- 5
alpha <- 0.05
bw <- .5
nsim <- 5000
burn <- 1000
# create covariate
# in sample obs
n_in = n
# number of years ahead
nY = 1
n_out = 365 * nY
# total obs
n_tot = n_in + n_out
# total covariate (in+out sample period)
x <- seq(0, 1, length = n_tot)
# in sample grid dimension for covariate
ng_in <- 150
xg <- seq(0, x[n_in], length = ng_in)
# in+out of sample grid
xg <- c(xg, seq(x[n_in + 1], x[(n_tot)], length = ng_in))
# in+out sample grid dimension
nxg <- length(xg)
xg <- array(xg, c(nxg, 1))
# in sample observations
samp_in <- samp[1:n_in]
ssamp_in <- sort(samp_in, decreasing = TRUE, index = TRUE)
x_in <- x[1:n_in] # in sample covariate
xs <- x_in[ssamp_in$ix[1:k]] # in sample concomitant covariate
# estimate scedasis function over the in and out of sample period
res_stat <- apply(
  xg,
  1,
  cpost_stat,
  N = nsim - burn,
  x = x_in,
  xs = xs,
  bw = bw,
  k = k,
  C = C
)
# conditional predictive posterior density
yg <- seq(500, 5000, by = 100)
nyg = length(yg)
# intermediate threshold
predDens_intx <- predDensx(
  x = yg,
  postsamp = proc$post_sample,
  scedasis = res_stat,
  threshold = proc$t,
  "continuous",
  extrapolation = FALSE)
# extreme threshold
predDens_extx <- predDensx(
  x = yg,
  postsamp = proc$post_sample,
  scedasis = res_stat,
  threshold = proc$t,
  "continuous",
  extrapolation = TRUE,
  p = 0.001,
  k = k,
  n = n)
# plot intermediate and extreme density conditional on a specific value of scedasis function
# disclaimer: to speed up the procedure, we specify a coarse grid
plot(
 x = predDens_intx$x,
 y = predDens_intx$preddens[,20],
 type = "l",
 lwd = 2,
 col="dodgerblue",
 ylab = "",
 xlab = "yg",
 main = "Conditional predictive posterior density")
lines(
  x = predDens_extx$x,
  y = predDens_extx$preddens[,20],
  lwd = 2,
  col = "violet")
legend("topright",
  legend = c("Intermediate threshold","Extreme threshold"),
  lwd = 2,
  col = c("dodgerblue", "violet"))
}
}

\name{extQuantile}
\alias{extQuantile}

\title{Value-at-Risk (VaR) or Extreme Quantile (EQ) Estimation}
\description{Computes a point and interval estimate of the VaR based on the Weissman estimator.}
\usage{
extQuantile(data, tau, tau1, var=FALSE, varType="asym-Dep", bias=FALSE, bigBlock=NULL,
            smallBlock=NULL, k=NULL, alpha=0.05)
}

\arguments{
	\item{data}{A vector of \eqn{(1 \times n)}{(1 x n)} observations.}
	\item{tau}{A real in \eqn{(0,1)} specifying the intermediate level \eqn{\tau_n}. See \bold{Details}.}
	\item{tau1}{A real in \eqn{(0,1)} specifying the extreme level \eqn{\tau'_n}. See \bold{Details}.}
	\item{var}{If \code{var=TRUE} then an estimate of the asymptotic variance of the VaR estimator is computed.}
	\item{varType}{A string specifying the type of asymptotic variance to compute. By default \code{varType="asym-Dep"} specifies the variance estimator for serial dependent observations. See \bold{Details}.}
	\item{bias}{A logical value. By default \code{biast=FALSE} specifies that no bias correction is computed. See \bold{Details}.}
	\item{bigBlock}{An interger specifying the size of the big-block used to estimaste the asymptotic variance. See \bold{Details}.}
	\item{smallBlock}{An interger specifying the size of the small-block used to estimaste the asymptotic variance. See \bold{Details}.}
	\item{k}{An integer specifying the value of the intermediate sequence \eqn{k_n}. See \bold{Details}.}
	\item{alpha}{A real in \eqn{(0,1)} specifying the confidence level \eqn{(1-\alpha)100\%} of the approximate confidence interval for the VaR.}
}

\details{
	For a dataset \code{data} of sample size \eqn{n}, the VaR or EQ, correspoding to the extreme level \code{tau1}, is computed by applying the Weissman estimator. The definition of the Weissman estimator depends on the estimation of the tail index \eqn{\gamma}. Here, \eqn{\gamma} is estimated using the Hill estimation (see \link{HTailIndex}). The observations can be either independent or temporal dependent (see e.g. de Haan and Ferreira 2006; Drees 2003; de Haan et al. 2016 for details).
	\itemize{
		\item The so-called intermediate level \code{tau} or \eqn{\tau_n} is a sequence of positive reals such that \eqn{\tau_n \to 1}{\tau_n -> 1} as \eqn{n \to \infty}{n -> \infty}. Practically, \eqn{(1-\tau_n)\in (0,1)}{(1-\tau_n) in (0,1)} is a small proportion of observations in the observed data sample that exceed the \eqn{tau_n}-\emph{th} empirical quantile. Such proportion of observations is used to estimate the \eqn{tau_n}-\emph{th} quantile and \eqn{\gamma}.
		\item The so-called extreme level \code{tau1} or \eqn{\tau'_n} is a sequence of positive reals such that \eqn{\tau'_n \to 1}{\tau'_n -> 1} as \eqn{n \to \infty}{n -> \infty}.
		The value \eqn{(1-tau'_n) \in (0,1)}{(1-\tau'_n) in (0,1)} is meant to be a small tail probability such that \eqn{(1-\tau'_n)=1/n} or \eqn{(1-\tau'_n) < 1/n}. It is also assumed that \eqn{n(1-\tau'_n) \to C}{n(1-tau'_n) -> C} as \eqn{n \to \infty}{n -> \infty}, where \eqn{C} is a positive finite constant. The value \eqn{C} is the expected number of exceedances of the \eqn{\tau'_n}-\emph{th} quantile. Typically, \eqn{C \in (0,1)}{C in (0,1)} which means that it is expected that there are no observations in a data sample exceeding the quantile of level \eqn{(1-\tau_n')}.
		\item If \code{var=TRUE} then an estimate of the asymptotic variance of the \eqn{tau'_n}-\emph{th} quantile is computed. With independent observations the asymptotic variance is estimated by the formula \eqn{\hat{\gamma}^2} (see e.g. de Drees 2000, 2003, for details). This is achieved through \code{varType="asym-Ind"}. With serial dependent data the asymptotic variance is estimated by the formula in 1288 in Drees (2000). This is achieved through  \code{varType="asym-Dep"}. In this latter case the computation of the serial dependence is based on the "big blocks seperated by small blocks" techinque which is a standard tools in time series, see e.g. Leadbetter et al. (1986). The size of the big and small blocks are specified by the parameters \code{bigBlock} and \code{smallBlock}, respectively. With serial dependent data the asymptotic variance can also be estimated by formula (32) of Drees (2003). This is achieved through \code{varType="asym-Alt-Dep"}.
		\item If \code{bias=TRUE} then an estimate of the \eqn{\tau'_n}--\emph{th} quantile is computed using the formula in page 330 of de Haan et al. (2016), which provides a bias corrected version of the Weissman estimator. However, in this case the asymptotic variance is not estimated using the formula in Haan et al. (2016) Theorem 4.2. Instead, for simplicity standard formula in Drees (2000) page 1288 is used.
		\item \code{k} or \eqn{k_n} is the value of the so-called intermediate sequence \eqn{k_n}, \eqn{n=1,2,\ldots}{n=1,2,...}. Its represents a sequence of positive integers such that \eqn{k_n \to \infty}{k_n -> \infty} and \eqn{k_n/n \to 0}{k_n/n -> 0} as \eqn{n \to \infty}{n -> \infty}. Practically, the value \eqn{k_n} specifies the number of \code{k}\eqn{+1} larger order statistics to be used to estimate the \eqn{\tau_n}-\emph{th} empirical quantile and \eqn{\gamma}. The intermediate level \eqn{\tau_n} can be seen defined as \eqn{\tau_n=1-k_n/n}.
        \item Given a small value \eqn{\alpha\in (0,1)} then an estimate of an asymptotic confidence interval for \eqn{tau'_n}-\emph{th} quantile, with approximate nominal confidence level \eqn{(1-\alpha)100\%}, is computed. The confidence intervals are computed exploiting the formulas (33) and (46) of Drees (2003). When \code{biast=TRUE} confidence intervals are computed in the same way but after correcting the tail index estimate by an estimate of the bias term, see formula (4.2) in de Haan et al. (2016) for details. Furthermore, in this case with serial dependent data the asymptotic variance is estimated using the formula in Drees (2000) page 1288.
	}
}

\value{
	A list with elements:
	\itemize{
		\item \code{ExtQHat}: an estimate of the VaR or \eqn{\tau'_n}-\emph{th} quantile;
		\item \code{VarExQHat}: an estimate of the asymptotic variance of the VaR estimator;
		\item \code{CIExtQ}: an estimate of the approximate \eqn{(1-\alpha)100\%} confidence interval for the VaR.
	}
}

\references{
  Anthony C. Davison, Simone A. Padoan and Gilles Stupfler (2023). Tail Risk Inference via Expectiles in Heavy-Tailed Time Series, \emph{Journal of Business & Economic Statistics}, \bold{41}(3) 876-889.

  de Haan, L., Mercadier, C. and Zhou, C. (2016). Adapting extreme value statistics to financial time series: dealing with bias and serial dependence. \emph{Finance and Stochastics},
  \bold{20}, 321-354.

  de Haan, L. and Ferreira, A. (2006). Extreme Value Theory: An Introduction. \emph{Springer}-\emph{Verlag}, New York.

  Drees, H. (2000). Weighted approximations of tail processes for \eqn{\beta}-mixing random variables.
  \emph{Annals of Applied Probability}, \bold{10}, 1274-1301.

  Drees, H. (2003). Extreme quantile estimation for dependent data, with applications to finance.
  \emph{Bernoulli}, \bold{9}, 617-657.

  Leadbetter, M.R., Lindgren, G. and Rootzen, H. (1989). Extremes and related
  properties of random sequences and processes. \emph{Springer}.
}

\author{
	Simone Padoan, \email{simone.padoan@unibocconi.it},
	\url{https://www.unibocconi.it/en/faculty/simone-padoan};
	Gilles Stupfler, \email{gilles.stupfler@univ-angers.fr},
	\url{https://math.univ-angers.fr/~stupfler/}
}


\seealso{
	\link{HTailIndex}, \link{EBTailIndex}, \link{estExpectiles}
}

\examples{
# Extreme quantile estimation at the level tau1 obtained with 1-dimensional data
# simulated from an AR(1) with univariate Student-t distributed innovations

tsDist <- "studentT"
tsType <- "AR"

# parameter setting
corr <- 0.8
df <- 3
par <- c(corr, df)

# Big- small-blocks setting
bigBlock <- 65
smallBlock <- 15

# Intermediate level (or sample tail probability 1-tau)
tau <- 0.97
# Extreme level (or tail probability 1-tau1 of unobserved quantile)
tau1 <- 0.9995

# sample size
ndata <- 2500

# Simulates a sample from an AR(1) model with Student-t innovations
data <- rtimeseries(ndata, tsDist, tsType, par)

# VaR (extreme quantile) estimation
extQHat1 <- extQuantile(data, tau, tau1, TRUE, bigBlock=bigBlock, smallBlock=smallBlock)
extQHat1$ExtQHat
extQHat1$CIExtQ

# VaR (extreme quantile) estimation with bias correction
extQHat2 <- extQuantile(data, tau, tau1, TRUE, bias=TRUE, bigBlock=bigBlock, smallBlock=smallBlock)
extQHat2$ExtQHat
extQHat2$CIExtQ
}

\name{fitdGPD}
\alias{fitdGPD}
\title{Maximum likelihood estimation of the parameters of the discrete generalized Pareto distribution}
\usage{
fitdGPD(excess, alpha = 0.05)
}
\arguments{
\item{excess}{vector of positive exceedances, i.e., \eqn{Y - t \mid Y > t}, with \eqn{t} being the threshold}

\item{alpha}{level for confidence interval of scale and shape parameters. Default: 0.05, giving 95\% confidence intervals}
}
\value{
a list with elements
\itemize{
\item \code{mle} vector of dimension 2 containing estimated scale and shape parameters
\item \code{CI} matrix of dimension 2 by 2 containing the \eqn{1-\alpha} level confidence intervals for scale and shape
}
}
\description{
Given a sample of exceedances, estimate the parameters via maximum likelihood along with \eqn{1-\alpha} level confidence intervals.
}
\examples{
fitdGPD(rpois(1000,2))
}
\references{
Hitz, A.S., G. Samorodnistsky and R.A. Davis (2024). Discrete Extremes, \emph{Journal of Data Science}, 22(\bold{4}), pp. 524-536.
}

\name{HTailIndex}
\alias{HTailIndex}

\title{Hill Tail Index Estimation}
\description{Computes a point and interval estimate of the tail index based on the Hill's estimator.}
\usage{
HTailIndex(data, k, var=FALSE, varType="asym-Dep", bias=FALSE, bigBlock=NULL,
           smallBlock=NULL, alpha=0.05)
}

\arguments{
	\item{data}{A vector of \eqn{(1 \times n)}{(1 x n)} observations.}
	\item{k}{An integer specifying the value of the intermediate sequence \eqn{k_n}. See \bold{Details}.}
    \item{var}{If \code{var=TRUE} then an estimate of the variance of the tail index estimator is computed.}
    \item{varType}{A string specifying the asymptotic variance to compute. By default \code{varType="asym-Dep"} specifies the variance estimator for serial dependent observations. See \bold{Details}.}
    \item{bias}{A logical value. By default \code{biast=FALSE} specifies that no bias correction is computed. See \bold{Details}.}
    \item{bigBlock}{An interger specifying the size of the big-block used to estimaste the asymptotic variance. See \bold{Details}.}
	\item{smallBlock}{An interger specifying the size of the small-block used to estimaste the asymptotic variance. See \bold{Details}.}
	\item{alpha}{A real in \eqn{(0,1)} specifying the confidence level \eqn{(1-\alpha)100\%} of the approximate confidence interval for the tail index.}

}

\details{
	For a dataset \code{data} of sample size \eqn{n}, the tail index \eqn{\gamma} of its (marginal) distribution is computed by applying the Hill estimator. The observations can be either independent or temporal dependent.
	\itemize{
        \item \code{k} or \eqn{k_n} is the value of the so-called intermediate sequence \eqn{k_n}, \eqn{n=1,2,\ldots}{n=1,2,...}. Its represents a sequence of positive integers such that \eqn{k_n \to \infty}{k_n -> \infty} and \eqn{k_n/n \to 0}{k_n/n -> 0} as \eqn{n \to \infty}{n -> \infty}. Practically, the value \eqn{k_n} specifies the number of \code{k}\eqn{+1} larger order statistics to be used to estimate \eqn{\gamma}.
		\item If \code{var=TRUE} then an estimate of the asymptotic variance of the Hill estimator is computed. With independent observations the asymptotic variance is estimated by the formula \eqn{\hat{\gamma}^2}, see Theorem 3.2.5 of de Haan and Ferreira (2006). This is achieved through \code{varType="asym-Ind"}. With serial dependent observations the asymptotic variance is estimated by the formula in 1288 in Drees (2000). This is achieved through \code{varType="asym-Dep"}. In this latter case the serial dependence is estimated by exploiting the "big blocks seperated by small blocks" techinque which is a standard tools in time series, see Leadbetter et al. (1986). See also formula (11) in Drees (2003). The size of the big and small blocks are specified by the parameters \code{bigBlock} and \code{smallBlock}, respectively.
        \item If \code{bias=TRUE} then an estimate of the bias term of the Hill estimator is computed implementing using formula (4.2) in de Haan et al. (2016). However, in this case the asymptotic variance is not estimated using the formula in Haan et al. (2016) Theorem 4.1. Instead for simplicity standard formulas have been used (see de Haan and Ferreira 2006 Theorem 3.2.5 and Drees 2000 page 1288).
        \item Given a small value \eqn{\alpha\in (0,1)} then an estimate of an asymptotic confidence interval for \eqn{\gamma}, with approximate nominal confidence level \eqn{(1-\alpha)100\%}, is computed. The confidence intervals are computed exploiting the formulas in de Haan and Ferreira (2006) Theorem 3.2.5 and Drees (2000) page 1288. When \code{biast=TRUE} the confidence intervals are computed in the same way but after correcting the tail index estimate by an estimate of the bias term, see formula (4.2) in de Haan et al. (2016) for details.
	}
}

\value{
	A list with elements:
	\itemize{
		\item \code{gammaHat}: an estimate of tail index \eqn{\gamma};
		\item \code{VarGamHat}: an estimate of the asymptotic variance of the Hill estimator;
		\item \code{BiasGamHat}: an estimate of bias term of the Hill estimator;
        \item \code{AdjExtQHat}: the adjustment to correct the Weissman estimator of an extreme quantile.
	}
}

\references{
  Anthony C. Davison, Simone A. Padoan and Gilles Stupfler (2023). Tail Risk Inference via Expectiles in Heavy-Tailed Time Series, \emph{Journal of Business & Economic Statistics}, \bold{41}(3) 876-889.

  de Haan, L., Mercadier, C. and Zhou, C. (2016). Adapting extreme value statistics
  to financial time series: dealing with bias and serial dependence. \emph{Finance and Stochastics},
  \bold{20}, 321-354.

  de Haan, L. and Ferreira, A. (2006). Extreme Value Theory: An Introduction. \emph{Springer}-\emph{Verlag}, New York.

  Drees, H. (2000). Weighted approximations of tail processes for \eqn{\beta}-mixing random variables.
  \emph{Annals of Applied Probability}, \bold{10}, 1274-1301.

  Leadbetter, M.R., Lindgren, G. and Rootzen, H. (1989). Extremes and related
  properties of random sequences and processes. \emph{Springer}.
}

\author{
	Simone Padoan, \email{simone.padoan@unibocconi.it},
	\url{https://www.unibocconi.it/en/faculty/simone-padoan};
	Gilles Stupfler, \email{gilles.stupfler@univ-angers.fr},
	\url{https://math.univ-angers.fr/~stupfler/}
}

\seealso{
	\link{MLTailIndex}, \link{MomTailIndex}, \link{EBTailIndex}
}

\examples{
# Tail index estimation based on the Hill estimator obtained with
# 1-dimensional data simulated from an AR(1) with univariate Student-t
# distributed innovations

tsDist <- "studentT"
tsType <- "AR"

# parameter setting
corr <- 0.8
df <- 3
par <- c(corr, df)

# Big- small-blocks setting
bigBlock <- 65
smallBlock <- 15

# Number of larger order statistics
k <- 150

# sample size
ndata <- 2500

# Simulates a sample from an AR(1) model with Student-t innovations
data <- rtimeseries(ndata, tsDist, tsType, par)

# tail index estimation
gammaHat1 <- HTailIndex(data, k, TRUE, bigBlock=bigBlock, smallBlock=smallBlock)
gammaHat1$gammaHat
gammaHat1$CIgamHat

# tail index estimation with bias correction
gammaHat2 <- HTailIndex(data, 2*k, TRUE, bias=TRUE, bigBlock=bigBlock, smallBlock=smallBlock)
gammaHat2$gammaHat-gammaHat2$BiasGamHat
gammaHat2$CIgamHat
}

\name{scedastic.test}
\alias{scedastic.test}
\title{Test on the effect of concomitant covariate on the extremes of the response variable}
\usage{
scedastic.test(data, k, M = 1000L, xg, ng, bayes = TRUE, C = 5L, alpha = 0.05)
}
\arguments{
\item{data}{design matrix of dimension \code{n} by \code{2} containing the complete data for the dependent variable (first column) and covariate (second column) in [0,1]}

\item{k}{integer, number of exceedances for the generalized Pareto}

\item{M}{integer, number of samples to draw from the posterior distrinution of the law of the concomitant covariate. Default: 1000}

\item{xg}{vector of covariate grid of dimension \code{ng} by \code{1} containing a sequence between zero and the last value of the corresponding covariate}

\item{ng}{length of covariate grid}

\item{bayes}{logical indicating the bootstrap method. If \code{FALSE}, a frequentist bootstrap on the empirical cumulative distribution function of the concomitant covariate is performed. Default to \code{TRUE}}

\item{C}{integer, hypermparameter entering the posterior distributyion of the law of the concomitant covariate. Default: 5}

\item{alpha}{double, significance level for the critical value of the test, computed as the \eqn{(1-alpha)} level empirical quantile of the sample of distances between the empirical cumulative distribution function of the concomitant and complete covariate. Default: 0.05}
}
\value{
a list with components
\itemize{
\item \code{Delta} maximum observed distance between the empirical distribution functions of the concomitant and complete covariate
\item \code{DeltaM} vector of length M containing the sample of maximum distances between the empirical distribution function of the concomitant complete covariate
\item \code{critical} double, critical value for the test statistic, computed as the \eqn{(1-alpha)} level empirical quantile of DeltaM
\item \code{pval} double, p-value
}
}
\description{
Given observed data, perform a Kolmogorov-Smirnov type test comparing the cumulative distribution function of the concomitant covariate, defined as \eqn{X \mid Y > t}, with \eqn{t} being the threshold,
against the cumulative distribution function of the random vector of covariate.
}
\examples{
\dontrun{
# generate data
set.seed(1234)
n <- 500
samp <- evd::rfrechet(n,0,1:n,4)
# set effective sample size and threshold
k <- 50
threshold <- sort(samp,decreasing = TRUE)[k+1]
# preliminary mle estimates of scale and shape parameters
mlest <- evd::fpot(samp,
 threshold,
 control=list(maxit = 500))
# empirical bayes procedure
proc <- estPOT(
  samp,
  k = k,
  pn = c(0.01, 0.005),
  type = "continuous",
  method = "bayesian",
  prior = "empirical",
  start = as.list(mlest$estimate),
  sig0 = 0.1)
# conditional predictive density estimation
yg <- seq(0, 50, by = 2)
nyg <- length(yg)
# estimation of scedasis function
# setting
M <- 1e3
C <- 5
alpha <- 0.05
bw <- .5
nsim <- 5000
burn <- 1000
# create covariate
# in sample obs
n_in = n
# number of years ahead
nY = 1
n_out = 365 * nY
# total obs
n_tot = n_in + n_out
# total covariate (in+out sample period)
x <- seq(0, 1, length = n_tot)
# in sample grid dimension for covariate
ng_in <- 150
xg <- seq(0, x[n_in], length = ng_in)
# in+out of sample grid
xg <- c(xg,
 seq(x[n_in + 1],
     x[(n_tot)],
     length = ng_in))
# in+out sample grid dimension
nxg <- length(xg)
xg <- array(xg, c(nxg, 1))
# in sample observations
samp_in <- samp[1:n_in]
ssamp_in <- sort(samp_in, decreasing = TRUE, index = TRUE)
x_in <- x[1:n_in] # in sample covariate
xs <- x_in[ssamp_in$ix[1:k]] # in sample concomitant covariate
# test on covariate effect
test <- scedastic.test(
  cbind(samp, x[1:n]),
  k,
  M,
  array(xg[1:ng_in], c(ng_in, 1)),
  ng_in,
  TRUE,
  C,
  0.05
)
}
}
\references{
Dombry, C., S. Padoan and S. Rizzelli (2025). Asymptotic theory for Bayesian inference and prediction: from the ordinary to a conditional Peaks-Over-Threshold method, arXiv:2310.06720v2.
}

\name{dowjones}
\alias{dowjones}
\alias{dowjones}

\title{Negative log-returns of DOW JONES.}

\description{Series of negative log-returns of the U.S. stock market index Dow Jones.}

\details{From the series of n = 8785 closing prices \eqn{S_{t}}, \eqn{t=1,2,...}, for the Dow Jones stock market index, recorded from January 29, 1985 to December 12, 2019, the series of negative log-returns.

\eqn{
X_{t+1} = - \log(S_{t+1}/S_t), \quad 1\leq t\leq n-1}{X_{t+1} = - log(S_{t+1}/S_t),   1<= t <= n-1}


is available. Hence the dataset (negative log-returns) contains \eqn{8784} observations.
}

\format{A \eqn{8784*2}{8784*2} data frame.}

\keyword{datasets}

\name{rtimeseries}
\alias{rtimeseries}

\title{Simulation of One-Dimensional Temporally Dependent Observations}
\description{Simulates samples from parametric families of time series models.}
\usage{
rtimeseries(ndata, dist="studentT", type="AR", par, burnin=1e+03)

}

\arguments{
	\item{ndata}{A positive interger specifying the number of observations to simulate.}
	\item{dist}{A string specifying the parametric family of the innovations distribution. By default \code{dist="studentT"} specifies a
    Student-\emph{t} family of distributions. See \bold{Details}.}
	\item{type}{A string specifying the type of time series. By default \code{type="AR"} specifies a linear Auto-Regressive time series. See \bold{Details}.}
	\item{par}{A vector of \eqn{(1 \times p)}{(1 x p)} parameters to be specified for the univariate time series parametric family. See \bold{Details}.}
	\item{burnin}{A positive interger specifying the number of initial observations to discard from the simulated sample.}

}

\details{
	For a time series class (\code{type}) with a parametric family (\code{dist}) for the innovations, a sample of size \code{ndata} is simulated. See for example Brockwell and Davis (2016).
	\itemize{
		\item The available categories of time series models are: Auto-Regressive (\code{type="AR"}), Auto-Regressive and Moving-Average (\code{type="ARMA"}), Generalized-Autoregressive-Conditional-Heteroskedasticity (\code{type="GARCH"}) and Auto-Regressive and Moving-Maxima (\code{type="ARMAX"}).
		\item With AR(1) and ARMA(1,1) times series the available families of distributions for the innovations are:
        \itemize{
        \item Student-\emph{t} (\code{dist="studentT"}) with parameters: \eqn{\phi\in(-1,1)}{phi in(-1,1)} (autoregressive coefficient), \eqn{\nu>0} (degrees of freedom) specified by \code{par=c(corr, df)};
        \item symmetric Frechet (\code{dist="double-Frechet"}) with parameters \eqn{\phi\in(-1,1)}{phi in(-1,1)} (autoregressive coefficient), \eqn{\sigma>0} (scale), \eqn{\alpha>0} (shape), \eqn{\theta} (movingaverage coefficient), specified by \code{par=c(corr, scale, shape, smooth)};
        \item symmetric Pareto (\code{dist="double-Pareto"}) with parameters \eqn{\phi\in(-1,1)}{phi in(-1,1)} (autoregressive coefficient), \eqn{\sigma>0} (scale), \eqn{\alpha>0} (shape), \eqn{\theta} (movingaverage coefficient), specified by \code{par=c(corr, scale, shape, smooth)}.
        }
        With ARCH(1)/GARCH(1,1) time series the Gaussian family of distributions is available for the innovations (\code{dist="Gaussian"}) with parameters, \eqn{\alpha_0}, \eqn{\alpha_1}, \eqn{\beta} specified by \code{par=c(alpha0, alpha1, beta)}. Finally, with ARMAX(1) times series the Frechet families of distributions is available for the innovations (\code{dist="Frechet"}) with parameters, \eqn{\phi\in(-1,1)}{phi in(-1,1)} (autoregressive coefficient), \eqn{\sigma>0} (scale), \eqn{\alpha>0} (shape) specified by \code{par=c(corr, scale, shape)}.
	}
}

\value{
  A vector of \eqn{(1 \times n)}{(1 x n)} observations simulated from a specified time series model.

}

\references{
Brockwell, Peter J., and Richard A. Davis. (2016). Introduction to time series and forecasting. \emph{Springer}.

Anthony C. Davison, Simone A. Padoan and Gilles Stupfler (2023). Tail Risk Inference via Expectiles in Heavy-Tailed Time Series, \emph{Journal of Business & Economic Statistics}, \bold{41}(3) 876-889.
}

\author{
	Simone Padoan, \email{simone.padoan@unibocconi.it},
	\url{https://www.unibocconi.it/en/faculty/simone-padoan};
	Gilles Stupfler, \email{gilles.stupfler@univ-angers.fr},
	\url{https://math.univ-angers.fr/~stupfler/}
}

\seealso{
	\link{expectiles}
}

\examples{
# Data simulation from a 1-dimensional AR(1) with univariate Student-t
# distributed innovations

tsDist <- "studentT"
tsType <- "AR"

# parameter setting
corr <- 0.8
df <- 3
par <- c(corr, df)

# sample size
ndata <- 2500

# Simulates a sample from an AR(1) model with Student-t innovations
data <- rtimeseries(ndata, tsDist, tsType, par)

# Graphic representation
plot(data, type="l")
acf(data)

}

\name{extBQuant}
\alias{extBQuant}
\title{Bayesian extreme quantile}
\usage{
extBQuant(
  threshold,
  postsamp,
  k,
  n,
  retp,
  alpha = 0.05,
  type = c("continuous", "discrete")
)
}
\arguments{
\item{threshold}{threshold for the generalized Pareto model, corresponding to the \eqn{n-k}th order statistic of the sample}

\item{postsamp}{a \code{m} by \code{2} matrix with columns containing the posterior samples of scale and shape parameters of the generalized Pareto distribution, respectively}

\item{k}{integer, number of exceedances for the generalized Pareto (only used if \code{extrapolation=TRUE})}

\item{n}{integer, number of observations in the full sample. Must be greater than \code{k}  (only used if \code{extrapolation=TRUE})}

\item{retp}{double indicating the value of the return period}

\item{alpha}{level for credibility interval. Default: 0.05 giving 95\% credibility intervals}

\item{type}{string indicating distribution types. Default: \code{c('continuous','discrete')}}
}
\value{
a list with components
\itemize{
\item \code{mQ} posterior mean of the extreme quantile
\item \code{ciQ} vector of dimension 2 returning the \eqn{\alpha/2} and \eqn{1-\alpha/2} empirical quantiles of the posterior distribution of the extreme quantile
}
}
\description{
Given posterior samples for the parameters of the continuous or discrete generalized Pareto distribution,
return the posterior mean and \eqn{1-\alpha} level credibility intervals of the extreme quantile
}
\examples{
\dontrun{
# generate data
set.seed(1234)
n <- 500
samp <- evd::rfrechet(500,0,3,4)
# set effective sample size and threshold
k <- 50
threshold <- sort(samp,decreasing = TRUE)[k+1]
# preliminary mle estimates of scale and shape parameters
mlest <- evd::fpot(samp, threshold)
# empirical bayes procedure
proc <- estPOT(
  samp,
  k = k,
  pn = c(0.01, 0.005),
  type = "continuous",
  method = "bayesian",
  prior = "empirical",
  start = as.list(mlest$estimate),
  sig0 = 0.1)
# extreme quantile corresponding to a return period of 100
extBQuant(
  proc$t,proc$post_sample,
  k,
  n,
  100,
  0.05,
  type = "continuous")
}
}

\name{quantF}
\alias{quantF}
\title{Predictive quantile of peaks-over-threshold models}
\usage{
quantF(x, y, p)
}
\arguments{
\item{x}{vector of grid of points at which the predictive density is evaluated}

\item{y}{vector containing estimates of the predictive density, e.g., the posterior predictive density}

\item{p}{double indicating the probability level corresponding to the quantile}
}
\value{
double containing the value of the quantile
}
\description{
Given an estimated predictive density evaluated at a grid of points, return the smallest value of the quantile of the predictive distribution function
below which a specific probability level corresponds. Namely, \eqn{Q(p) = \inf\{ x \in \mathbb{R}: p \leq F(x)\}}, where \eqn{F(x)} is the
predictive distribution function of a peak-over-threshold object.
WARNING: this function will not be exported in a future version of the package.
}
\examples{
\dontrun{
# generate data
set.seed(1234)
n <- 500
samp <- evd::rfrechet(n,0,1:n,4)
# set effective sample size and threshold
k <- 50
threshold <- sort(samp,decreasing = TRUE)[k+1]
# preliminary mle estimates of scale and shape parameters
mlest <- evd::fpot(samp, threshold, control=list(maxit = 500))
# empirical bayes procedure
proc <- estPOT(
  samp,
  k = k,
  pn = c(0.01, 0.005),
  type = "continuous",
  method = "bayesian",
  prior = "empirical",
  start = as.list(mlest$estimate),
  sig0 = 0.1)
# conditional predictive density estimation
yg <- seq(0, 50, by = 2)
nyg <- length(yg)
# estimation of scedasis function
# setting
M <- 1e3
C <- 5
alpha <- 0.05
bw <- .5
nsim <- 5000
burn <- 1000
# create covariate
# in sample obs
n_in = n
# number of years ahead
nY = 1
n_out = 365 * nY
# total obs
n_tot = n_in + n_out
# total covariate (in+out sample period)
x <- seq(0, 1, length = n_tot)
# in sample grid dimension for covariate
ng_in <- 150
xg <- seq(0, x[n_in], length = ng_in)
# in+out of sample grid
xg <- c(xg, seq(x[n_in + 1], x[(n_tot)], length = ng_in))
# in+out sample grid dimension
nxg <- length(xg)
xg <- array(xg, c(nxg, 1))
# in sample observations
samp_in <- samp[1:n_in]
ssamp_in <- sort(samp_in, decreasing = TRUE, index = TRUE)
x_in <- x[1:n_in] # in sample covariate
xs <- x_in[ssamp_in$ix[1:k]] # in sample concomitant covariate
# estimate scedasis function over the in and out of sample period
res_stat <- apply(
  xg,
  1,
  cpost_stat,
  N = nsim - burn,
  x = x_in,
  xs = xs,
  bw = bw,
  k = k,
  C = C
)
# conditional predictive posterior density
yg <- seq(500, 5000, by = 100)
nyg = length(yg)
# intermediate threshold
predDens_intx <- predDensx(
  x = yg,
  postsamp = proc$post_sample,
  scedasis = res_stat,
  threshold = proc$t,
  "continuous",
  extrapolation = FALSE)
# extreme threshold
predDens_extx <- predDensx(
  x = yg,
  postsamp = proc$post_sample,
  scedasis = res_stat,
  threshold = proc$t,
  "continuous",
  extrapolation = TRUE,
  p = 0.001,
  k = k,
  n = n)
  # predictive conditional quantiles
predQuant_intxL <- predQuant_intxU <- predQuant_extxL <- predQuant_extxU <- numeric()
for (i in 1:nxg){
  predQuant_intxU[i] <- apply(
    array(predDens_intx$preddens[,i], c(1, nyg)),
    1,
    quantF,
    x = yg,
    p = c(0.975)
  )
  predQuant_intxL[i] <- apply(
    array(predDens_intx$preddens[,i], c(1, nyg)),
    1,
    quantF,
    x = yg,
    p = c(0.025)
  )
  predQuant_extxU[i] <- apply(
    array(predDens_extx$preddens[,i], c(1, nyg)),
    1,
    quantF,
    x = yg,
    p = c(0.975)
  )
  predQuant_extxL[i] <- apply(
    array(predDens_extx$preddens[,i], c(1, nyg)),
    1,
    quantF,
    x = yg,
    p = c(0.025)
  )
}
}
}
\keyword{internal}

\name{estPOT}
\alias{estPOT}
\title{Estimation of generalized Pareto distributions}
\usage{
estPOT(
  data,
  k = 10L,
  pn = NULL,
  type = c("continuous", "discrete"),
  method = c("bayesian", "frequentist"),
  prior = "empirical",
  start = NULL,
  sig0 = NULL,
  nsim = 5000L,
  burn = 1000L,
  p = 0.234,
  optim.method = "Nelder-Mead",
  control = NULL,
  ...
)
}
\arguments{
\item{data}{numeric vector of length n containing complete data values (under and above threshold)}

\item{k}{double indicating the effective sample size. Default: 10}

\item{pn}{numeric vector containing one or more percentile level at which extreme quantiles are estimated, with \eqn{p < k/n}. Default: NULL}

\item{type}{string indicating distribution types. Default: \code{c('continuous','discrete')}}

\item{method}{string indicating estimation methods. Default: \code{c('bayesian', 'frequentist')}}

\item{prior}{string indicating prior distribution (uniform or empirical). Default: \code{'empirical'}}

\item{start}{list of 2 containing starting values for scale and shape parameters. Default: \code{NULL}}

\item{sig0}{double indicating the initial value for the update of the variance in the MCMC algorithm. Default: \code{NULL}}

\item{nsim}{double indicating the total number of iterations of the MCMC algorithm in the Bayesian estimation case. Default: 5000L}

\item{burn}{double indicating the number of iterations to exclude in the MCMC algorithm of the Bayesian estimation case. Default: 1000L}

\item{p}{double indicating the desired overall sampler acceptance probability. Default: 0.234}

\item{optim.method}{string indicating the optimization method in the frequentist estimation case. Default: \code{'Nelder-Mead'}}

\item{control}{list containing additional parameters for the minimization function \link[stats]{optim}. Default: \code{NULL}}

\item{...}{other arguments passed to the function}
}
\value{
a list with the following elements
\itemize{
\item Bayesian estimation case
\itemize{
\item \code{Q.est} matrix with \code{nsim-burn} rows and \code{length(pn)} columns containing the posterior sample of the
extreme quantile estimated at level given in pn
\item \code{post_sample} matrix with \code{nsim-burn} rows and 2 columns containing the posterior sample of the scale and
shape parameters for the continuous or discrete generalized Pareto distribution
\item \code{burn} double indicating the number of iterations excluded in the MCMC algorithm
\item \code{straight.reject} vector of length \code{nsim-burn+1} indicating the iterations in which the proposed parameters do not respect basic constraints
\item \code{sig.vec} vector of length \code{nsim}\eqn{-\lfloor(5 / (p (1 - p)))\rfloor+1} containing the values of the variance updated at each iteration of the MCMC algorithm
\item \code{accept.prob} matrix containing the values of acceptance probability (second column) corresponding to specific iterations (first column)
\item \code{msg} character string containing an output message on the result of the Bayesian estimation procedure
\item \code{mle} vector of length 2 containing the maximum likelihood estimates of the scale and shape parameters of the continuous or discrete generalized Pareto distribution
\item \code{t} double indicating the threshold for the generalized Pareto model, corresponding to the \eqn{n-k}th order statistic of the sample
}
\item Frequentist estimation case
\itemize{
\item \code{est} vector of length 2 containing the maximum likelihood estimates of the scale and shape parameters of the continuous or discrete generalized Pareto distribution
\item \code{t} double indicating the threshold
\item \code{Q.est} vector of dimension \code{length(pn)} containing the estimates of the extreme quantile at level given in pn
\item \code{VarCov} \eqn{2 \times 2} variance-covariance matrix of (gamma, sigma)
\item \code{Q.VC} variance of Q.est
\item \code{msg} character string containing an output message on the result of the frequentist estimation procedure
}
}
}
\description{
Bayesian or frequentist estimation of the scale and shape parameters of the continuous or discrete generalized Pareto distribution.
}
\examples{
\dontrun{
# generate data
set.seed(1234)
n <- 500
samp <- evd::rfrechet(n,0,3,4)
# set effective sample size and threshold
k <- 50
threshold <- sort(samp,decreasing = TRUE)[k+1]
# preliminary mle estimates of scale and shape parameters
mlest <- evd::fpot(samp, threshold)
# empirical bayes procedure
proc <- estPOT(
  samp,
  k = k,
  pn = c(0.01, 0.005),
  type = "continuous",
  method = "bayesian",
  prior = "empirical",
  start = as.list(mlest$estimate),
  sig0 = 0.1)
 }
}
\references{
Dombry, C., S. Padoan and S. Rizzelli (2025). Asymptotic theory for Bayesian inference and prediction: from the ordinary to a conditional Peaks-Over-Threshold method, arXiv:2310.06720v2.
}
\seealso{
\code{\link[evd]{fpot}}
}

\name{rmdata}
\alias{rmdata}

\title{Simulation of \eqn{d}-Dimensional Temporally Independent Observations}
\description{Simulates samples of independent \eqn{d}-dimensional observations from parametric families of joint distributions with a given copula and equal marginal distributions.}
\usage{
rmdata (ndata, dist="studentT", copula="studentT", par)
}

\arguments{
	\item{ndata}{A positive interger specifying the number of observations to simulate.}
	\item{dist}{A string specifying the parametric family of equal marginal distributions. By default \code{dist="studentT"} specifies a
    Student-\emph{t} family of distributions. See \bold{Details}.}
	\item{copula}{A string specifying the type copula to be used. By default \code{copula="studentT"} specifies the Student-\emph{t} copula. See \bold{Details}.}
	\item{par}{A list of \eqn{p} parameters to be specified for the multivariate parametric family of distributions. See \bold{Details}.}
}

\details{
	For a joint multivariate distribution with a given parametric copula class (\code{copula}) and a given parametric family of equal marginal distributions (\code{dist}), a sample of size \code{ndata} is simulated.
	\itemize{
		\item The available copula classes are: Student-\emph{t} (\code{copula="studentT"}) with \eqn{\nu>0} degrees of freedom (\code{df}) and scale parameters \eqn{\rho_{i,j}\in (-1,1)} for \eqn{i \neq j=1,\ldots,d}{i != j=1,...,d} (\code{sigma}), Gaussian (\code{copula="Gaussian"}) with correlation parameters \eqn{\rho_{i,j}\in (-1,1)} for \eqn{i \neq j=1,\ldots,d}{i != j=1,...,d} (\code{sigma}), Clayton (\code{copula="Clayton"}) with dependence parameter \eqn{\theta>0} (\code{dep}), Gumbel (\code{copula="Gumbel"}) with dependence parameter \eqn{\theta\geq 1} (\code{dep}) and Frank (\code{copula="Frank"}) with dependence parameter \eqn{\theta>0} (\code{dep}).
        \item The available families of marginal distributions are:
        \itemize{
        \item Student-\emph{t} (\code{dist="studentT"}) with \eqn{\nu>0} degrees of freedom (\code{df});
		\item Asymmetric Student-\emph{t} (\code{dist="AStudentT"}) with \eqn{\nu>0} degrees of freedom (\code{df}). In this case all the observations are only positive;
		\item Frechet (\code{dist="Frechet"}) with scale \eqn{\sigma>0} (\code{scale}) and shape \eqn{\alpha>0} (\code{shape}) parameters.
		\item Frechet (\code{dist="double-Frechet"}) with scale \eqn{\sigma>0} (\code{scale}) and shape \eqn{\alpha>0} (\code{shape}) parameters. In this case positive and negative observations are allowed;
		\item symmetric Pareto (\code{dist="double-Pareto"}) with scale \eqn{\sigma>0} (\code{scale}) and shape \eqn{\alpha>0} (\code{shape}) parameters. In this case positive and negative observations are allowed.
        }
		\item The available classes of multivariate joint distributions are:
        \itemize{
        \item studentT-studentT (\code{dist="studentT"} and \code{copula="studentT"}) with parameters \code{par <- list(df, sigma)};
		\item studentT (\code{dist="studentT"} and \code{copula="None"} with parameters \code{par <- list(df, dim)}. In this case the \code{d} variables are regarded as independent;
		\item studentT-AstudentT (\code{dist="AstudentT"} and \code{copula="studentT"}) with parameters \code{par <- list(df, sigma, shape)};
		\item Gaussian-studentT (\code{dist="studentT"} and \code{copula="Gaussian"}) with parameters \code{par <- list(df, sigma)};
		\item Gaussian-AstudentT (\code{dist="AstudentT"} and \code{copula="Gaussian"}) with parameters \code{par <- list(df, sigma, shape)};
		\item Frechet (\code{dist="Frechet"} and \code{copula="None"}) with parameters \code{par <- list(shape, dim)}. In this case the \code{d} variables are regarded as independent;
		\item Clayton-Frechet (\code{dist="Frechet"} and \code{copula="Clayton"}) with parameters \code{par <- list(dep, dim, scale, shape)};
		\item Gumbel-Frechet (\code{dist="Frechet"} and \code{copula="Gumbel"}) with parameters \code{par <- list(dep, dim, scale, shape)};
		\item Frank-Frechet (\code{dist="Frechet"} and \code{copula="Frank"}) with parameters \code{par <- list(dep, dim, scale, shape)};
		\item Clayton-double-Frechet (\code{dist="double-Frechet"} and \code{copula="Clayton"}) with parameters \code{par <- list(dep, dim, scale, shape)};
		\item Gumbel-double-Frechet (\code{dist="double-Frechet"} and \code{copula="Gumbel"}) with parameters \code{par <- list(dep, dim, scale, shape)};
		\item Frank-double-Frechet (\code{dist="double-Frechet"} and \code{copula="Frank"}) with parameters \code{par <- list(dep, dim, scale, shape)};
		\item Clayton-double-Pareto (\code{dist="double-Pareto"} and \code{copula="Clayton"}) with parameters \code{par <- list(dep, dim, scale, shape)};
		\item Gumbel-double-Pareto (\code{dist="double-Pareto"} and \code{copula="Gumbel"}) with parameters \code{par <- list(dep, dim, scale, shape)};
		\item Frank-double-Pareto (\code{dist="double-Pareto"} and \code{copula="Frank"}) with parameters \code{par <- list(dep, dim, scale, shape)}.
        }
		Note that above \code{dim} indicates the number of \code{d} marginal variables.
	}
}

\value{
A matrix of \eqn{(n \times d)}{(n x d)} observations simulated from a specified multivariate parametric joint distribution.
}


\references{
Joe, H. (2014). Dependence Modeling with Copulas. Chapman & Hall/CRC Press, Boca
Raton, USA.

Simone A. Padoan and Gilles Stupfler (2022). Joint inference on extreme expectiles for multivariate heavy-tailed distributions, \emph{Bernoulli} \bold{28}(2), 1021-1048.
}

\author{
	Simone Padoan, \email{simone.padoan@unibocconi.it},
	\url{https://www.unibocconi.it/en/faculty/simone-padoan};
	Gilles Stupfler, \email{gilles.stupfler@univ-angers.fr},
	\url{https://math.univ-angers.fr/~stupfler/}
}

\seealso{
	\link{rtimeseries}, \link{rbtimeseries}
}

\examples{
library(plot3D)
library(copula)
library(evd)

# Data simulation from a 3-dimensional random vector a with multivariate distribution
# given by a Gumbel copula and three equal Frechet marginal distributions

# distributional setting
copula <- "Gumbel"
dist <- "Frechet"

# parameter setting
dep <- 3
dim <- 3
scale <- rep(1, dim)
shape <- rep(3, dim)
par <- list(dep=dep, scale=scale, shape=shape, dim=dim)

# sample size
ndata <- 1000

# Simulates a sample from a multivariate distribution with equal Frechet
# marginal distributions and a Gumbel copula
data <- rmdata(ndata, dist, copula, par)
scatter3D(data[,1], data[,2], data[,3])


# Data simulation from a 3-dimensional random vector a with multivariate distribution
# given by a Gaussian copula and three equal Student-t marginal distributions

# distributional setting
dist <- "studentT"
copula <- "Gaussian"

# parameter setting
rho <- c(0.9, 0.8, 0.7)
sigma <- c(1, 1, 1)
Sigma <- sigma^2 * diag(dim)
Sigma[lower.tri(Sigma)] <- rho
Sigma <- t(Sigma)
Sigma[lower.tri(Sigma)] <- rho
df <- 3
par <- list(sigma=Sigma, df=df)

# sample size
ndata <- 1000

# Simulates a sample from a multivariate distribution with equal Student-t
# marginal distributions and a Gaussian copula
data <- rmdata(ndata, dist, copula, par)
scatter3D(data[,1], data[,2], data[,3])
}

\name{cpost_stat}
\alias{cpost_stat}
\title{Estimation of the scedasis function}
\usage{
cpost_stat(N, x, xs, xg, bw, k, C = 5L)
}
\arguments{
\item{N}{integer, number of samples to draw from the distribution of the concomitant covariate}

\item{x}{one-dimensional vector of in-sample covariate in [0,1]}

\item{xs}{one-dimensional vector of concomitant covariate}

\item{xg}{one-dimensional vector of length m containing the grid of in-sample and possibly out-sample covariate in [0,1]}

\item{bw}{double, bandwidth for the computation of the kernel}

\item{k}{integer, number of exceedances for the generalized Pareto}

\item{C}{integer, hyperparameter entering the posterior distribution of the law of the concomitant covariate. Default: 5}
}
\value{
an \code{N} by \code{m} matrix containing the values of the posterior samples of the scedasis function (rows) for each value of \code{xg} (columns)
}
\description{
Kernel-based method for the estimation of the scedasis function. Given the values of the complete and concomitant covariate, defined as \eqn{X \mid Y > t}, with \eqn{t} being the threshold, it returns
a matrix containing a posterior sample of the scedasis function for each covariate value.
}
\examples{
\dontrun{
# generate data
set.seed(1234)
n <- 500
samp <- evd::rfrechet(n, 0, 1:n, 4)
# set effective sample size and threshold
k <- 50
threshold <- sort(samp, decreasing = TRUE)[k+1]
# preliminary mle estimates of scale and shape parameters
mlest <- evd::fpot(
  samp,
  threshold,
  control = list(maxit = 500))
# empirical bayes procedure
proc <- estPOT(
  samp,
  k = k,
  pn = c(0.01, 0.005),
  type = "continuous",
  method = "bayesian",
  prior = "empirical",
  start = as.list(mlest$estimate),
  sig0 = 0.1)
# conditional predictive density estimation
yg <- seq(0, 50, by = 2)
nyg <- length(yg)
# estimation of scedasis function
# setting
M <- 1e3
C <- 5
alpha <- 0.05
bw <- .5
nsim <- 5000
burn <- 1000
# create covariate
# in sample obs
n_in = n
# number of years ahead
nY = 1
n_out = 365 * nY
# total obs
n_tot = n_in + n_out
# total covariate (in+out sample period)
x <- seq(0, 1, length = n_tot)
# in sample grid dimension for covariate
ng_in <- 150
xg <- seq(0, x[n_in], length = ng_in)
# in+out of sample grid
xg <- c(xg, seq(x[n_in + 1], x[(n_tot)], length = ng_in))
# in+out sample grid dimension
nxg <- length(xg)
xg <- array(xg, c(nxg, 1))
# in sample observations
samp_in <- samp[1:n_in]
ssamp_in <- sort(samp_in, decreasing = TRUE, index = TRUE)
x_in <- x[1:n_in] # in sample covariate
xs <- x_in[ssamp_in$ix[1:k]]
# in sample concomitant covariate
# estimate scedasis function over the in and out of sample period
res_stat <- apply(
  xg,
  1,
  cpost_stat,
  N = nsim - burn,
  x = x_in,
  xs = xs,
  bw = bw,
  k = k,
  C = C
)
}
}

\name{rbtimeseries}
\alias{rbtimeseries}

\title{Simulation of Two-Dimensional Temporally Dependent Observations}
\description{Simulates samples from parametric families of bivariate time series models.}
\usage{
rbtimeseries(ndata, dist="studentT", type="AR", copula="Gumbel", par, burnin=1e+03)
}

\arguments{
	\item{ndata}{A positive interger specifying the number of observations to simulate.}
	\item{dist}{A string specifying the parametric family of the innovations distribution. By default \code{dist="studentT"} specifies a
    Student-\emph{t} family of distributions. See \bold{Details}.}
	\item{type}{A string specifying the type of time series. By default \code{type="AR"} specifies a linear Auto-Regressive time series. See \bold{Details}.}
	\item{copula}{A string specifying the type copula to be used. By default \code{copula="Gumbel"} specifies the Gumbel copula. See \bold{Details}.}
	\item{par}{A list of \eqn{p} parameters to be specified for the bivariate time series parametric family. See \bold{Details}.}
	\item{burnin}{A positive interger specifying the number of initial observations to discard from the simulated sample.}
}

\details{
	For a time series class (\code{type}), with a parametric family (\code{dist}) for the innovations, a sample of size \code{ndata} is simulated. See for example Brockwell and Davis (2016).
	\itemize{
		\item The available categories of bivariate time series models are: Auto-Regressive (\code{type="AR"}), Auto-Regressive and Moving-Average (\code{type="ARMA"}), Generalized-Autoregressive-Conditional-Heteroskedasticity (\code{type="GARCH"}) and Auto-Regressive.
        \item With AR(1) times series the available families of distributions for the innovations and the dependence structure (copula) are:
        \itemize{
        \item Student-\emph{t} (\code{dist="studentT"} and \code{copula="studentT"}) with marginal parameters (equal for both distributions): \eqn{\phi\in(-1,1)}{phi in(-1,1)} (autoregressive coefficient), \eqn{\nu>0} (degrees of freedom) and dependence parameter \eqn{dep\in(-1,1)}{dep in (-1,1)}. The parameters are specified as \code{par <- list(corr, df, dep)};
        \item Asymmetric Student-\emph{t} (\code{dist="AStudentT"} and \code{copula="studentT"}) with marginal parameters (equal for both distributions): \eqn{\phi\in(-1,1)}{phi in(-1,1)} (autoregressive coefficient), \eqn{\nu>0} (degrees of freedom) and dependence parameter \eqn{dep\in(-1,1)}{dep in (-1,1)}. The paraters are specified as \code{par <- list(corr, df, dep)}. Note that in this case the tail index of the lower and upper tail of the first marginal are different, see Padoan and Stupfler (2020) for details;
        }
		\item With ARMA(1,1) times series the available families of distributions for the innovations and the dependence structure (copula) are:
        \itemize{
        \item symmetric Pareto (\code{dist="double-Pareto"} and \code{copula="Gumbel"} or \code{copula="Gaussian"}) with marginal parameters (equal for both distributions): \eqn{\phi\in(-1,1)}{phi in(-1,1)} (autoregressive coefficient), \eqn{\sigma>0} (scale), \eqn{\alpha>0} (shape), \eqn{\theta} (movingaverage coefficient), and dependence parameter \eqn{dep} (\eqn{dep>0} if \code{copula="Gumbel"} or \eqn{dep\in(-1,1)}{dep in (-1,1)} if \code{copula="Gaussian"}). The parameters are specified as \code{par <- list(corr, scale, shape, smooth, dep)}.
        \item symmetric Pareto (\code{dist="double-Pareto"} and \code{copula="Gumbel"} or \code{copula="Gaussian"}) with marginal parameters (equal for both distributions): \eqn{\phi\in(-1,1)}{phi in(-1,1)} (autoregressive coefficient), \eqn{\sigma>0} (scale), \eqn{\alpha>0} (shape), \eqn{\theta} (movingaverage coefficient), and dependence parameter \eqn{dep} (\eqn{dep>0} if \code{copula="Gumbel"} or \eqn{dep\in(-1,1)}{dep in (-1,1)} if \code{copula="Gaussian"}). The parameters are specified as \code{par <- list(corr, scale, shape, smooth, dep)}. Note that in this case the tail index of the lower and upper tail of the first marginal are different, see Padoan and Stupfler (2020) for details;
        }
        \item With ARCH(1)/GARCH(1,1) time series the distribution of the innovations are symmetric Gaussian (\code{dist="Gaussian"}) or asymmetric Gaussian \code{dist="AGaussian"}. In both cases the marginal parameters (equal for both distributions) are: \eqn{\alpha_0}, \eqn{\alpha_1}, \eqn{\beta}. In the asymmetric Gaussian case the tail index of the lower and upper tail of the first marginal are different, see Padoan and Stupfler (2020) for details. The available copulas are: Gaussian (\code{copula="Gaussian"}) with dependence parameter \eqn{dep\in(-1,1)}, Student-\emph{t} (\code{copula="studentT"}) with dependence parameters \eqn{dep\in(-1,1)}{dep in (-1,1)} and \eqn{\nu>0} (degrees of freedom), Gumbel (\code{copula="Gumbel"}) with dependence parameter \eqn{dep>0}. The parameters are specified as \code{par <- list(alpha0, alpha1, beta, dep)} or \code{par <- list(alpha0, alpha1, beta, dep, df)}.
	}
}

\value{
  A vector of \eqn{(2 \times n)}{(2 x n)} observations simulated from a specified bivariate time series model.
}

\references{
Brockwell, Peter J., and Richard A. Davis. (2016). Introduction to time series and forecasting. \emph{Springer}.

Anthony C. Davison, Simone A. Padoan and Gilles Stupfler (2023). Tail Risk Inference via Expectiles in Heavy-Tailed Time Series, \emph{Journal of Business & Economic Statistics}, \bold{41}(3) 876-889.
}

\author{
	Simone Padoan, \email{simone.padoan@unibocconi.it},
	\url{https://www.unibocconi.it/en/faculty/simone-padoan};
	Gilles Stupfler, \email{gilles.stupfler@univ-angers.fr},
	\url{https://math.univ-angers.fr/~stupfler/}
}

\seealso{
	\link{rtimeseries}, \link{expectiles}
}

\examples{
# Data simulation from a 2-dimensional AR(1) with bivariate Student-t distributed
# innovations, with one marginal distribution whose lower and upper tail indices
# that are different

tsDist <- "AStudentT"
tsType <- "AR"
tsCopula <- "studentT"

# parameter setting
corr <- 0.8
dep <- 0.8
df <- 3
par <- list(corr=corr, dep=dep, df=df)

# sample size
ndata <- 2500

# Simulates a sample from an AR(1) model with Student-t innovations
data <- rbtimeseries(ndata, tsDist, tsType, tsCopula, par)

# Extreme expectile estimation
plot(data, pch=21)
plot(data[,1], type="l")
plot(data[,2], type="l")
}

\name{MLTailIndex}
\alias{MLTailIndex}

\title{Maximum Likelihood Tail Index Estimation}
\description{Computes a point and interval estimate of the tail index based on the Maximum Likelihood (ML) estimator.}
\usage{
MLTailIndex(data, k, var=FALSE, varType="asym-Dep", bigBlock=NULL,
            smallBlock=NULL, alpha=0.05)
}

\arguments{
	\item{data}{A vector of \eqn{(1 \times n)}{(1 x n)} observations.}
	\item{k}{An integer specifying the value of the intermediate sequence \eqn{k_n}. See \bold{Details}.}
	\item{var}{If \code{var=TRUE} then an estimate of the asymptotic variance of the tail index estimator is computed.}
	\item{varType}{A string specifying the asymptotic variance to compute. By default \code{varType="asym-Dep"} specifies the variance estimator for serial dependent observations. See \bold{Details}.}
	\item{bigBlock}{An interger specifying the size of the big-block used to estimaste the asymptotic variance. See \bold{Details}.}
	\item{smallBlock}{An interger specifying the size of the small-block used to estimaste the asymptotic variance. See \bold{Details}.}
	\item{alpha}{A real in \eqn{(0,1)} specifying the confidence level \eqn{(1-\alpha)100\%} of the approximate confidence interval for the tail index.}

}

\details{
	For a dataset \code{data} of sample size \eqn{n}, the tail index \eqn{\gamma} of its (marginal) distribution is computed by applying the ML estimator. The observations can be either independent or temporal dependent.
	\itemize{
	    \item \code{k} or \eqn{k_n} is the value of the so-called intermediate sequence \eqn{k_n}, \eqn{n=1,2,\ldots}{n=1,2,...}. Its represents a sequence of positive integers such that \eqn{k_n \to \infty}{k_n -> \infty} and \eqn{k_n/n \to 0}{k_n/n -> 0} as \eqn{n \to \infty}{n -> \infty}. Practically, the value \eqn{k_n} specifies the numer of \code{k}\eqn{+1} larger order statistics to be used to estimate \eqn{\gamma}.
		\item If \code{var=TRUE} then the asymptotic variance of the Hill estimator is computed. With independent observations the asymptotic variance is estimated by the formula in Theorem 3.4.2 of de Haan and Ferreira (2006). This is achieved through \code{varType="asym-Ind"}. With serial dependent observations the asymptotic variance is estimated by the formula in 1288 in Drees (2000). This is achieved through \code{varType="asym-Dep"}. In this latter case the serial dependence is estimated by exploiting the "big blocks seperated by small blocks" techinque which is a standard tools in time series, see Leadbetter et al. (1986). See also formula (11) in Drees (2003). The size of the big and small blocks are specified by the parameters \code{bigBlock} and \code{smallBlock}, respectively.
		\item Given a small value \eqn{\alpha\in (0,1)} then an asymptotic confidence interval for the tail index, with approximate nominal confidence level \eqn{(1-\alpha)100\%} is computed.
	}
}

\value{
	A list with elements:
	\itemize{
		\item \code{gammaHat}: an estimate of tail index \eqn{\gamma};
		\item \code{VarGamHat}: an estimate of the variance of the ML estimator;
		\item \code{CIgamHat}: an estimate of the approximate \eqn{(1-\alpha)100\%} confidence interval for \eqn{\gamma}.
	}
}

\references{
  Anthony C. Davison, Simone A. Padoan and Gilles Stupfler (2023). Tail Risk Inference via Expectiles in Heavy-Tailed Time Series, \emph{Journal of Business & Economic Statistics}, \bold{41}(3) 876-889.

  Drees, H. (2000). Weighted approximations of tail processes for \eqn{\beta}{beta}-mixing random variables.
  \emph{Annals of Applied Probability}, \bold{10}, 1274-1301.

  de Haan, L. and Ferreira, A. (2006). Extreme Value Theory: An Introduction. \emph{Springer}-\emph{Verlag}, New York.

  Leadbetter, M.R., Lindgren, G. and Rootzen, H. (1989). Extremes and related
  properties of random sequences and processes. \emph{Springer}.
}

\author{
	Simone Padoan, \email{simone.padoan@unibocconi.it},
	\url{https://www.unibocconi.it/en/faculty/simone-padoan};
	Gilles Stupfler, \email{gilles.stupfler@univ-angers.fr},
	\url{https://math.univ-angers.fr/~stupfler/}
}

\seealso{
	\link{HTailIndex}, \link{MomTailIndex}, \link{EBTailIndex}
}

\examples{
# Tail index estimation based on the Maximum Likelihood estimator obtained with
# 1-dimensional data simulated from an AR(1) with univariate Student-t
# distributed innovations

tsDist <- "studentT"
tsType <- "AR"

# parameter setting
corr <- 0.8
df <- 3
par <- c(corr, df)

# Big- small-blocks setting
bigBlock <- 65
smallBlock <- 15

# Number of larger order statistics
k <- 150

# sample size
ndata <- 2500

# Simulates a sample from an AR(1) model with Student-t innovations
data <- rtimeseries(ndata, tsDist, tsType, par)

# tail index estimation
gammaHat <- MLTailIndex(data, k, TRUE, bigBlock=bigBlock, smallBlock=smallBlock)
gammaHat$gammaHat
gammaHat$CIgamHat
}

\name{MomTailIndex}
\alias{MomTailIndex}

\title{Moment based Tail Index Estimation}
\description{Computes a point estimate of the tail index based on the Moment Based (MB) estimator.}
\usage{
MomTailIndex(data, k)
}

\arguments{
	\item{data}{A vector of \eqn{(1 \times n)}{(1 x n)} observations.}
	\item{k}{An integer specifying the value of the intermediate sequence \eqn{k_n}. See \bold{Details}.}
}

\details{
	For a dataset \code{data} of sample size \eqn{n}, the tail index \eqn{\gamma} of its (marginal) distribution is computed by applying the MB estimator. The observations can be either independent or temporal dependent. For details see de Haan and Ferreira (2006).
	\itemize{
        \item \code{k} or \eqn{k_n} is the value of the so-called intermediate sequence \eqn{k_n}, \eqn{n=1,2,\ldots}{n=1,2,...}. Its represents a sequence of positive integers such that \eqn{k_n \to \infty}{k_n -> \infty} and \eqn{k_n/n \to 0}{k_n/n -> 0} as \eqn{n \to \infty}{n -> \infty}. Practically, the value \eqn{k_n} specifies the number of \code{k}\eqn{+1} larger order statistics to be used to estimate \eqn{\gamma}.
	}
}

\value{
An estimate of the tail index \eqn{\gamma}.
}

\references{
de Haan, L. and Ferreira, A. (2006). Extreme Value Theory: An Introduction. \emph{Springer}-\emph{Verlag}, New York.
}

\author{
	Simone Padoan, \email{simone.padoan@unibocconi.it},
	\url{https://www.unibocconi.it/en/faculty/simone-padoan};
	Gilles Stupfler, \email{gilles.stupfler@univ-angers.fr},
	\url{https://math.univ-angers.fr/~stupfler/}
}

\seealso{
	\link{HTailIndex}, \link{MLTailIndex}, \link{EBTailIndex}
}

\examples{
# Tail index estimation based on the Moment estimator obtained with
# 1-dimensional data simulated from an AR(1) with univariate Student-t
# distributed innovations

tsDist <- "studentT"
tsType <- "AR"

# parameter setting
corr <- 0.8
df <- 3
par <- c(corr, df)

# Big- small-blocks setting
bigBlock <- 65
smallblock <- 15

# Number of larger order statistics
k <- 150

# sample size
ndata <- 2500

# Simulates a sample from an AR(1) model with Student-t innovations
data <- rtimeseries(ndata, tsDist, tsType, par)

# tail index estimation
gammaHat <- MomTailIndex(data, k)
gammaHat
}

\name{predMultiExpectiles}
\alias{predMultiExpectiles}

\title{Multidimensional Extreme Expectile Estimation}
\description{Computes point estimates and \eqn{(1-\alpha)100\%} confidence regions for \code{d}-dimensional expectile at the extreme level (Expectile Prediction).}
\usage{
predMultiExpectiles(data, tau, tau1, method="LAWS", tailest="Hill", var=FALSE,
                    varType="asym-Ind-Adj-Log", bias=FALSE, k=NULL, alpha=0.05,
                    plot=FALSE)
}
\arguments{
	\item{data}{A matrix of \eqn{(n \times d)}{(n x d)} observations.}
	\item{tau}{A real in \eqn{(0,1)} specifying the intermediate level \eqn{\tau_n}. See \bold{Details}.}
	\item{tau1}{A real in \eqn{(0,1)} specifying the extreme level \eqn{\tau'_n}. See \bold{Details}.}
	\item{method}{A string specifying the method used to estimate the expecile. By default \code{est="LAWS"} specifies the use of the LAWS based estimator. See \bold{Details}.}
	\item{tailest}{A string specifying the tail index estimator. By default \code{tailest="Hill"} specifies the use of Hill estimator. See \bold{Details}.}
	\item{var}{If \code{var=TRUE} then an estimate of the asymptotic variance of the expectile estimator is computed.}
	\item{varType}{A string specifying the type of asymptotic variance-covariance matrix to compute. By default \code{varType="asym-Ind-Adj-Log"} specifies that the variance-covariance matrix is computed assuming dependent variables and exploiting the log scale and a suitable adjustment. See \bold{Details}.}
	\item{bias}{A logical value. By default \code{bias=FALSE} specifies that no bias correction is computed. See \bold{Details}.}
	\item{k}{An integer specifying the value of the intermediate sequence \eqn{k_n}. See \bold{Details}.}
	\item{alpha}{A real in \eqn{(0,1)} specifying the confidence level \eqn{(1-\alpha)100\%} of the approximate confidence region for the \code{d}-dimensional expecile at the extreme level.}
    \item{plot}{A logical value. By default \code{plot=FALSE} specifies that no graphical representation of the estimates is provided. See \bold{Details}.}
}

\details{
	For a dataset \code{data} of \code{d}-dimensional observations and sample size \eqn{n}, an estimate of the \eqn{\tau'_n}-\emph{th} \code{d}-dimensional expectile is computed. The estimation of the \code{d}-dimensional expectile at the extreme level \code{tau1} (\eqn{\tau'_n}) is meant to be a prediction beyond the observed sample.  Two estimators are available: the so-called Least Asymmetrically Weighted Squares (LAWS) based estimator and the Quantile-Based (QB) estimator. The definition of both estimators depends on the estimation of the \code{d}-dimensional tail index \eqn{\gamma}. Here, \eqn{\gamma} is estimated using the Hill estimation (see \link{MultiHTailIndex} for details).
	The data are regarded as \code{d}-dimensional temporal independent observations coming from dependent variables. See Padoan and Stupfler (2020) for details.
	\itemize{
		\item The so-called intermediate level \code{tau} or \eqn{\tau_n} is a sequence of positive reals such that \eqn{\tau_n \to 1}{\tau_n -> 1} as \eqn{n \to \infty}{n -> \infty}. Practically, for each marginal distribution, \eqn{\tau_n \in (0,1)}{\tau_n in (0,1)} is the ratio between \emph{N} (Numerator) and \emph{D} (Denominator). Where \emph{N} is the empirical mean distance of the \eqn{\tau_n}-\emph{th} expectile from the observations smaller than it, and \emph{D} is the empirical mean distance of \eqn{\tau_n}-\emph{th} expectile from all the observations.
		\item The so-called extreme level \code{tau1} or \eqn{\tau'_n} is a sequence of positive reals such that \eqn{\tau'_n \to 1}{\tau'_n -> 1} as \eqn{n \to \infty}{n -> \infty}.
		For each marginal distribution, the value \eqn{(1-tau'_n) \in (0,1)}{(1-tau'_n) in (0,1)} is meant to be a small tail probability such that \eqn{(1-\tau'_n)=1/n} or \eqn{(1-\tau'_n) < 1/n}. It is also assumed that \eqn{n(1-\tau'_n) \to C}{n(1-\tau'_n) -> C} as \eqn{n \to \infty}{n -> \infty}, where \eqn{C} is a positive finite constant. Typically, \eqn{C \in (0,1)}{C in (0,1)} so it is expected that there are no observations in a data sample that are greater than the expectile at the extreme level \eqn{\tau_n'}.
		\item When \code{method='LAWS'}, then the \eqn{\tau'_n}-\emph{th} \code{d}-dimensional expectile is estimated using the LAWS based estimator. When \code{method='QB'}, the expectile is instead estimated using the QB esimtator. The definition of both estimators depend on the estimation of the \code{d}-dimensional tail index \eqn{\gamma}. The \code{d}-dimensional tail index \eqn{\gamma} is estimated using the \code{d}-dimensional Hill estimator (\code{tailest='Hill'}), see \link{MultiHTailIndex}). This is the only available option so far (soon more results will be available). See Section 2.2 in Padoan and Stupfler (2020) for details.
		\item If \code{var=TRUE} then an estimate of the asymptotic variance-covariance matrix of the \eqn{tau'_n}-\emph{th} \code{d}-dimensional expectile is computed. Notice that the estimation of the asymptotic variance-covariance matrix \bold{is only available} when \eqn{\gamma} is estimated using the Hill estimator (see \link{MultiHTailIndex}). The data are regarded as temporal independent observations coming from dependent variables. The asymptotic variance-covariance matrix is estimated exploiting the formulas in Section 3.2 of Padoan and Stupfler (2020). The variance-covariance matrix is computed exploiting the asymptotic behaviour of the normalized expectile estimator which is  expressed in logarithmic scale. In addition, a suitable adjustment is considered. This is achieved through \code{varType="asym-Ind-Adj-Log"}.  The data can also be regarded as \code{d}-dimensional temporal independent observations coming from independent variables. In this case the asymptotic variance-covariance matrix is diagonal and is also computed exploiting the formulas in Section 3.2 of Padoan and Stupfler (2020). This is achieved through \code{varType="asym-Ind-Log"}. If \code{varType="asym-Ind-Adj"}, then the variance-covariance matrix is computed exploiting the asymptotic behaviour of the relative expectile estimator appropriately normalized and exploiting a suitable adjustment. This concerns the case of dependent variables. The case of independent variables is achieved through \code{varType="asym-Ind"}.
		\item If \code{bias=TRUE} then \code{d}-dimensional \eqn{\gamma} is estimated using formula (4.2) of Haan et al. (2016). This is used by the LAWS and QB estimators. Furthermore, the \eqn{\tau'_n}--\emph{th} quantile is estimated using the formula in page 330 of de Haan et al. (2016). This provides a bias corrected version of the Weissman estimator. This is used by the QB estimator.
		However, in this case the asymptotic variance is not estimated using the formula in Haan et al. (2016) Theorem 4.2. Instead, for simplicity the asymptotic variance-covariance matrix is estimated by the formulas Section 3.2 of Padoan and Stupfler (2020).
		\item \code{k} or \eqn{k_n} is the value of the so-called intermediate sequence \eqn{k_n}, \eqn{n=1,2,\ldots}{n=1,2,...}. Its represents a sequence of positive integers such that \eqn{k_n \to \infty}{k_n -> \infty} and \eqn{k_n/n \to 0}{k_n/n -> 0} as \eqn{n \to \infty}{n -> \infty}. Practically, for each marginal distribution when \code{tau=NULL} and \code{method='LAWS'} or \code{method='QB'}, then \eqn{\tau_n=1-k_n/n} is the intermediate level of the expectile to be stimated. When \code{tailest='Hill'}, for each marginal distributions, then \eqn{k_n} specifies the number of \code{k}\eqn{+1} larger order statistics used in the definition of the Hill estimator.
		\item Given a small value \eqn{\alpha\in (0,1)} then an estimate of an asymptotic confidence region for \eqn{tau'_n}-\emph{th} \code{d}-dimensional expectile, with approximate nominal confidence level \eqn{(1-\alpha)100\%}, is computed. The confidence regions are computed exploiting the formulas in Section 3.2 of Padoan and Stupfler (2020). If \code{varType="asym-Ind-Adj-Log"}, then an "asymmetric" confidence regions is computed exploiting the asymptotic behaviour of the normalized expectile estimator in logarithmic scale and using a suitable adjustment. This choice is recommended. If \code{varType="asym-Ind-Adj"}, then the a "symmetric" confidence regions is computed exploiting the asymptotic behaviour of the relative explectile estimator appropriately normalized.
        \item If \code{plot=TRUE} then a graphical representation of the estimates is not provided.}
}

\value{
  A list with elements:
    \itemize{
	  \item \code{ExpctHat}: an estimate of the \eqn{\tau'_n}-\emph{th} \code{d}-dimensional expecile;
      \item \code{biasTerm}: an estimate of the bias term of yje \eqn{\tau'_n}-\emph{th} \code{d}-dimensional expecile;
	  \item \code{VarCovEHat}: an estimate of the asymptotic variance-covariance of the \code{d}-dimensional expectile estimator;
	  \item \code{EstConReg}: an estimate of the approximate \eqn{(1-\alpha)100\%} confidence regions for \eqn{\tau'_n}-\emph{th} \code{d}-dimensional expecile.}
}

\references{
Simone A. Padoan and Gilles Stupfler (2022). Joint inference on extreme expectiles for multivariate heavy-tailed distributions, \emph{Bernoulli} \bold{28}(2), 1021-1048.
}

\author{
	Simone Padoan, \email{simone.padoan@unibocconi.it},
	\url{https://www.unibocconi.it/en/faculty/simone-padoan};
	Gilles Stupfler, \email{gilles.stupfler@univ-angers.fr},
	\url{https://math.univ-angers.fr/~stupfler/}
}

\seealso{
	\link{MultiHTailIndex},  \link{estMultiExpectiles}, \link{extMultiQuantile}
}

\examples{
# Extreme expectile estimation at the extreme level tau1 obtained with
# d-dimensional observations simulated from a joint distribution with
# a Gumbel copula and equal Frechet marginal distributions.
library(plot3D)
library(copula)
library(evd)

# distributional setting
copula <- "Gumbel"
dist <- "Frechet"

# parameter setting
dep <- 3
dim <- 3
scale <- rep(1, dim)
shape <- rep(3, dim)
par <- list(dep=dep, scale=scale, shape=shape, dim=dim)

# Intermediate level (or sample tail probability 1-tau)
tau <- 0.95
# Extreme level (or tail probability 1-tau1 of unobserved expectile)
tau1 <- 0.9995

# sample size
ndata <- 1000

# Simulates a sample from a multivariate distribution with equal Frechet
# marginals distributions and a Gumbel copula
data <- rmdata(ndata, dist, copula, par)
scatter3D(data[,1], data[,2], data[,3])

# High d-dimensional expectile (intermediate level) estimation
expectHat <- predMultiExpectiles(data, tau, tau1, var=TRUE)

expectHat$ExpctHat
expectHat$VarCovEHat
# run the following command to see the graphical representation
\dontrun{
 expectHat <- predMultiExpectiles(data, tau, tau1, var=TRUE, plot=TRUE)
}
}

\name{schedastic.test}
\alias{schedastic.test}
\title{Test on the effect of concomitant covariate on the extremes of the response variable}
\usage{
schedastic.test(data, k, M = 1000L, xg, ng, bayes = TRUE, C = 5L, alpha = 0.05)
}
\arguments{
\item{data}{design matrix of dimension \code{n} by \code{2} containing the complete data for the dependent variable (first column) and covariate (second column) in [0,1]}

\item{k}{integer, number of exceedances for the generalized Pareto}

\item{M}{integer, number of samples to draw from the posterior distrinution of the law of the concomitant covariate. Default: 1000}

\item{xg}{vector of covariate grid of dimension \code{ng} by \code{1} containing a sequence between zero and the last value of the corresponding covariate}

\item{ng}{length of covariate grid}

\item{bayes}{logical indicating the bootstrap method. If \code{FALSE}, a frequentist bootstrap on the empirical cumulative distribution function of the concomitant covariate is performed. Default to \code{TRUE}}

\item{C}{integer, hypermparameter entering the posterior distributyion of the law of the concomitant covariate. Default: 5}

\item{alpha}{double, significance level for the critical value of the test, computed as the \eqn{(1-alpha)} level empirical quantile of the sample of distances between the empirical cumulative distribution function of the concomitant and complete covariate. Default: 0.05}
}
\value{
a list with components
\itemize{
\item \code{Delta} maximum observed distance between the empirical distribution functions of the concomitant and complete covariate
\item \code{DeltaM} vector of length M containing the sample of maximum distances between the empirical distribution function of the concomitant complete covariate
\item \code{critical} double, critical value for the test statistic, computed as the \eqn{(1-alpha)} level empirical quantile of DeltaM
\item \code{pval} double, p-value
}
}
\description{
Given observed data, perform a Kolmogorov-Smirnov type test comparing the cumulative distribution function of the concomitant covariate, defined as \eqn{X \mid Y > t}, with \eqn{t} being the threshold,
against the cumulative distribution function of the random vector of covariate.
}
\keyword{internal}

\name{expectiles}
\alias{expectiles}

\title{Expectile Computation}
\description{Computes the true expectile for some families of parametric models.}
\usage{
expectiles(par, tau, tsDist="gPareto", tsType="IID", trueMethod="true",
           estMethod="LAWS", nrep=1e+05, ndata=1e+06, burnin=1e+03)
}

\arguments{
	\item{par}{A vector of \eqn{(1 \times p)}{(1 x p)} parameters of the time series parametric family. See \bold{Details}.}
	\item{tau}{A real in \eqn{(0,1)} specifying the level \eqn{\tau} of the expectile to be computed. See \bold{Details}.}
	\item{tsDist}{A string specifying the parametric family of the innovations distribution. By default \code{tsDist="gPareto"} specifies a
    Pareto family of distributions. See \bold{Details}.}
    \item{tsType}{A string specifying the type of time series. By default \code{tsType="IID"} specifies a sequence of independent and indentically distributed random variables. See \bold{Details}.}
	\item{trueMethod}{A string specifying the method used to computed the expecile. By default \code{trueMethod="true"} specifies that the true analytical expression to computed the expectile is used. See \bold{Details}.}
	\item{estMethod}{A string specifying the method used to estimate the expecile. By default \code{est="LAWS"} specifies the use of the direct LAWS estimator. See \bold{Details}.}
	\item{nrep}{A positive interger specifying the number of simulations to use for computing an approximation of the expectile. See \bold{Details}.}
	\item{ndata}{A positive interger specifying the number of observations to genreated for each simulation. See \bold{Details}.}
	\item{burnin}{A positive interger specifying the number of initial observations to discard from the simulated sample.}

}

\details{
	For a parametric family of time series models or a parametric family of distributions (for the case of independent observations) the \eqn{\tau}-\emph{th} expectile (or expectile of level \code{tau}) is computed.
	\itemize{
		\item There are two methods to compute the \eqn{\tau}-\emph{th} expectile. For the Generalised Pareto and Student-\emph{t} parametric families of distributions, the analytical epxression of the expectile is available. This is used to compute the \eqn{\tau}-\emph{th} expectile if the parameter \code{trueMethod="true"} is specified. For most of parametric family of distributions or parametric families of time series models the analytical epxression of the expectile is not available. In this case an approximate value of the \eqn{\tau}-\emph{th} expectile is computed via a Monte Carlo method if the parameter \code{trueMethod=="approx"} is specified. In particular, \code{ndata} observations from a family of time series models (e.g. \code{tsType="AR"} and \code{tsDist="studentT"}) or a sequence of independent and indentically distributed random variables with common family of distributions (e.g. \code{tsType="IID"} and \code{tsDist="gPareto"}) are simulated \code{nrep} times. For each simulation the \eqn{\tau}-\emph{th} expectile is estimate by the estimation method specified by \code{estMethod}. The mean of such estimate provides an approximate value of the \eqn{\tau}-\emph{th} expectile. The available estimator to esitmate the expecile are the direct LAWS (\code{estMethod="LAWS"}) and the indirect QB (\code{estMethod="QB"}), see \link{estExpectiles} for details. The available families of distributions are: Generalised Pareto (\code{tsDist="gPareto"}), Student-\emph{t} (\code{tsDist="studentT"}) and Frechet (\code{tsDist="Frechet"}). The available classes of time series with parametric innovations families of distributions are specified in \link{rtimeseries}.
	}
}

\value{
  The \eqn{\tau}-\emph{th} expectile.
}

\references{
Anthony C. Davison, Simone A. Padoan and Gilles Stupfler (2023). Tail Risk Inference via Expectiles in Heavy-Tailed Time Series, \emph{Journal of Business & Economic Statistics}, \bold{41}(3) 876-889.
}


\author{
	Simone Padoan, \email{simone.padoan@unibocconi.it},
	\url{https://www.unibocconi.it/en/faculty/simone-padoan};
	Gilles Stupfler, \email{gilles.stupfler@univ-angers.fr},
	\url{https://math.univ-angers.fr/~stupfler/}
}

\seealso{
	\link{rtimeseries}
}

\examples{
# Derivation of the true tau-th expectile for the Pareto distribution
# via accurate simulation

# parameter value
par <- c(1, 0.3)

# Intermediate level (or sample tail probability 1-tau)
tau <- 0.99

trueExp <- expectiles(par, tau)
trueExp

\dontrun{
# tau-th expectile of the AR(1) with Student-t innovations
tsDist <- "studentT"
tsType <- "AR"

# Approximation via Monte Carlo methods
trueMethod <- "approx"

# parameter setting
corr <- 0.8
df <- 3
par <- c(corr, df)

# Intermediate level (or sample tail probability 1-tau)
tau <- 0.99

trueExp <- expectiles(par, tau, tsDist, tsType, trueMethod)
trueExp
}
}

\name{extBQuantx}
\alias{extBQuantx}
\title{Conditional Bayesian extreme quantile}
\usage{
extBQuantx(
  cx,
  postsamp,
  threshold,
  n,
  qlev,
  k,
  type = c("continuous", "discrete"),
  confint = c("asymmetric", "symmetric"),
  alpha = 0.05,
  ...
)
}
\arguments{
\item{cx}{an \code{m} by \code{p} matrix, obtained by evaluating the scedasis function for every of the \code{p} covariate values and every \code{m} posterior draw}

\item{postsamp}{a \code{m} by \code{2} matrix with columns containing the posterior samples of scale and shape parameters of the generalized Pareto distribution, respectively}

\item{threshold}{threshold for the generalized Pareto model, corresponding to the \eqn{n-k}th order statistic of the sample}

\item{n}{integer, number of observations in the full sample. Must be greater than \code{k}}

\item{qlev}{double indicating the percentile level at which the extreme quantile is estimated. Must be smaller than \eqn{k/n}}

\item{k}{integer, number of exceedances for the generalized Pareto (only used if \code{extrapolation=TRUE})}

\item{type}{string indicating distribution types. Default: \code{c('continuous','discrete')}}

\item{confint}{type of confidence interval. Default: \code{c('asymmetric', 'symmetric')}}

\item{alpha}{level for credibility interval. Default 0.05, giving 95\% credibility intervals}

\item{...}{additional arguments, for back-compatilibity}
}
\value{
a list with components
\itemize{
\item \code{mQ} posterior mean of the extreme quantile
\item \code{ciQ} vector of dimension 2 returning the \eqn{\alpha/2} and \eqn{1-\alpha/2} empirical quantiles of the posterior distribution of the extreme quantile
}
}
\description{
Given posterior samples for the parameters of the continuous or discrete generalized Pareto distribution and scedasis function for a set of covariates,
return the posterior mean and \eqn{1-\alpha} level credibility intervals of the extreme quantile for each value of the scedasis function
}
\examples{
\dontrun{
# generate data
set.seed(1234)
n <- 500
samp <- evd::rfrechet(n,0,1:n,4)
# set effective sample size and threshold
k <- 50
threshold <- sort(samp,decreasing = TRUE)[k+1]
# preliminary mle estimates of scale and shape parameters
mlest <- evd::fpot(
  samp,
  threshold,
  control = list(maxit = 500))
# empirical bayes procedure
proc <- estPOT(
  samp,
  k = k,
  pn = c(0.01, 0.005),
  type = "continuous",
  method = "bayesian",
  prior = "empirical",
  start = as.list(mlest$estimate),
  sig0 = 0.1)
# conditional predictive density estimation
yg <- seq(0, 50, by = 2)
nyg <- length(yg)
# estimation of scedasis function
# setting
M <- 1e3
C <- 5
alpha <- 0.05
bw <- .5
nsim <- 5000
burn <- 1000
# create covariate
# in sample obs
n_in = n
# number of years ahead
nY = 1
n_out = 365 * nY
# total obs
n_tot = n_in + n_out
# total covariate (in+out sample period)
x <- seq(0, 1, length = n_tot)
# in sample grid dimension for covariate
ng_in <- 150
xg <- seq(0, x[n_in], length = ng_in)
# in+out of sample grid
xg <- c(xg, seq(x[n_in + 1], x[(n_tot)], length = ng_in))
# in+out sample grid dimension
nxg <- length(xg)
xg <- array(xg, c(nxg, 1))
# in sample observations
samp_in <- samp[1:n_in]
ssamp_in <- sort(samp_in, decreasing = TRUE, index = TRUE)
x_in <- x[1:n_in] # in sample covariate
xs <- x_in[ssamp_in$ix[1:k]] # in sample concomitant covariate
# estimate scedasis function over the in and out of sample period
res_stat <- apply(
  xg,
  1,
  cpost_stat,
  N = nsim - burn,
  x = x_in,
  xs = xs,
  bw = bw,
  k = k,
  C = C
)
# conditional predictive posterior density
probq = 1 - 0.99
res_AQ <- extBQuantx(
 cx = res_stat,
 postsamp = proc$post_sample,
 threshold = proc$t,
 n = n,
 qlev = probq,
 k = k,
 type = "continuous",
 confint = "asymmetric")
 }
}

\name{testTailHomo}
\alias{testTailHomo}
\title{Test on tail homogeneity}
\usage{
testTailHomo(y, k, alpha = 0.05)
}
\arguments{
\item{y}{list, containing the samples on which the test is to be performed}

\item{k}{integer, number of exceedances for the generalized Pareto}

\item{alpha}{double indicating the confidence level for the test. Default: 0.05}
}
\value{
list of 7 containing
\itemize{
\item \code{gamHatP} the pooled tail index
\item \code{VarGamHatP} the variance of \code{gamHatP}
\item \code{CIGamHatP} \eqn{(1-\alpha)} level confidence interval for \code{gamHatP}
\item \code{BiasGamHatP} bias term of \code{gamHatP}
\item \code{logLikR} value of the likelihood ratio-type of test statistic
\item \code{PVal} p-value of the test
}
}
\description{
Given observed samples and effective sample size, return the results for a likelihood ratio-type test on tail homogeneity.
}
\examples{
\dontrun{
# generate two samples of data
set.seed(1234)
y1 <- evd::rgpd(500, 0, 1, 0.2)
y2 <- evd::rgpd(700, 0, 2, 0.7)
y <- list(y1 = y1, y2 = y2)
# set effective sample size
k <- 50
# perform test
test <- testTailHomo(y,k)
}
}
\references{
Daouia, A., S.A. Padoan and G. Stupfler (2024). Optimal weighted pooling for inference about
the tail index and extreme quantiles, \emph{Bernoulli}, 30(\bold{2}), pp. 1287–1312.
}

\name{estMultiExpectiles}
\alias{estMultiExpectiles}

\title{Multidimensional High Expectile Estimation}
\description{Computes point estimates and \eqn{(1-\alpha)100\%} confidence regions for \code{d}-dimensional expectiles at the intermediate level.}
\usage{
estMultiExpectiles(data, tau, method="LAWS", tailest="Hill", var=FALSE,
                   varType="asym-Ind-Adj", k=NULL, alpha=0.05, plot=FALSE)
}

\arguments{
	\item{data}{A matrix of \eqn{(n \times d)}{(n x d)} observations.}
	\item{tau}{A real in \eqn{(0,1)} specifying the intermediate level \eqn{\tau_n}. See \bold{Details}.}
	\item{method}{A string specifying the method used to estimate the expecile. By default \code{est="LAWS"} specifies the use of the direct LAWS estimator. See \bold{Details}.}
	\item{tailest}{A string specifying the type of tail index estimator. By default \code{tailest="Hill"} specifies the use of Hill estimator. See \bold{Details}.}
	\item{var}{If \code{var=TRUE} then an estimate of the variance of the expectile estimator is computed.}
	\item{varType}{A string specifying the asymptotic variance-covariance matrix to compute. By default \code{varType="asym-Ind-Adj"} specifies that the variance-covariance matrix is computed assuming dependent variables and exploiting a suitable adjustment. See \bold{Details}.}
	\item{k}{An integer specifying the value of the intermediate sequence \eqn{k_n}. See \bold{Details}.}
	\item{alpha}{A real in \eqn{(0,1)} specifying the confidence level \eqn{(1-\alpha)100\%} of the approximate confidence region for the \code{d}-dimensional expecile at the intermedite level.}
    \item{plot}{A logical value. By default \code{plot=FALSE} specifies that no graphical representation of the estimates is not provided. See \bold{Details}.}
}

\details{
	For a dataset \code{data} of \code{d}-dimensional observations and sample size \eqn{n}, an estimate of the \eqn{\tau_n}-\emph{th} \code{d}-dimensional is computed. Two estimators are available: the so-called direct Least Asymmetrically Weighted Squares (LAWS) and indirect Quantile-Based (QB). The QB estimator depends on the estimation of the \code{d}-dimensional tail index \eqn{\gamma}. Here, \eqn{\gamma} is estimated using the Hill estimator (see \link{MultiHTailIndex}). The data are regarded as \code{d}-dimensional temporal independent observations coming from dependent variables. See Padoan and Stupfler (2020) for details.
	\itemize{
		\item The so-called intermediate level \code{tau} or \eqn{\tau_n} is a sequence of positive reals such that \eqn{\tau_n \to 1}{\tau_n -> 1} as \eqn{n \to \infty}{n -> \infty}. Practically, for each individual marginal distribution \eqn{\tau_n \in (0,1)}{\tau_n in (0,1)} is the ratio between \emph{N} (Numerator) and \emph{D} (Denominator). Where \emph{N} is the empirical mean distance of the \eqn{\tau_n}-\emph{th} expectile from the observations smaller than it, and \emph{D} is the empirical mean distance of \eqn{\tau_n}-\emph{th} expectile from all the observations.
		\item If \code{method='LAWS'}, then the expectile at the intermediate level \eqn{\tau_n} is estimated applying the direct LAWS estimator. Instead, If \code{method='QB'} the indirect QB esimtator is used to estimate the expectile. See Section 2.1 in Padoan and Stupfler (2020) for details.
		\item When the expectile is estimated by the indirect QB esimtator (\code{method='QB'}), an estimate of the \code{d}-dimensional tail index \eqn{\gamma} is needed. Here the \code{d}-dimensional tail index \eqn{\gamma} is estimated using the \code{d}-dimensional Hill estimator (\code{tailest='Hill'}, see \link{MultiHTailIndex}). This is the only available option so far (soon more results will be available).
		\item \code{k} or \eqn{k_n} is the value of the so-called intermediate sequence \eqn{k_n}, \eqn{n=1,2,\ldots}{n=1,2,...}. Its represents a sequence of positive integers such that \eqn{k_n \to \infty}{k_n -> \infty} and \eqn{k_n/n \to 0}{k_n/n -> 0} as \eqn{n \to \infty}{n -> \infty}. Practically, for each marginal distribution, when \code{method='LAWS'} and \code{tau=NULL}, \eqn{k_n} specifies by \eqn{\tau_n=1-k_n/n}{tau_n = 1 - k_n / n} the intermediate level of the expectile. Instead, for each marginal distribution, when \code{method='QB'}, then the value \eqn{k_n} specifies the number of \code{k}\eqn{+1} larger order statistics to be used to estimate \eqn{\gamma} by the Hill estimator and if \code{tau=NULL} then it also specifies by \eqn{\tau_n=1-k_n/n} the confidence level \eqn{\tau_n} of the quantile to estimate.
		\item If \code{var=TRUE} then an estimate of the asymptotic variance-covariance matrix of the \code{d}-dimensional expecile estimator is computed. If the data are regarded as \code{d}-dimensional temporal independent observations coming from dependent variables. Then, the asymptotic variance-covariance matrix is estimated by the formulas in section 3.1 of Padoan and Stupfler (2020). In particular, the variance-covariance matrix is computed exploiting the asymptotic behaviour of the relative explectile estimator appropriately normalized and using a suitable adjustment. This is achieved through \code{varType="asym-Ind-Adj"}. The data can also be regarded as \code{d}-dimensional temporal independent observations coming from independent variables. In this case the asymptotic variance-covariance matrix is diagonal and is also computed exploiting the formulas in section 3.1 of Padoan and Stupfler (2020). This is achieved through \code{varType="asym-Ind"}.
		\item Given a small value \eqn{\alpha\in (0,1)} then an asymptotic confidence region for the \eqn{\tau_n}-\emph{th} expectile, with approximate nominal confidence level \eqn{(1-\alpha)100\%} is computed. In particular, a "symmetric" confidence regions is computed exploiting the asymptotic behaviour of the relative explectile estimator appropriately normalized. See Sections 3.1 of Padoan and Stupfler (2020) for detailed.
        \item If \code{plot=TRUE} then a graphical representation of the estimates is not provided.}
}

\value{
  A list with elements:
    \itemize{
	  \item \code{ExpctHat}: an point estimate of the \eqn{\tau_n}-\emph{th} \code{d}-dimensional expecile;
      \item \code{biasTerm}: an point estimate of the bias term of the estimated expecile;
	  \item \code{VarCovEHat}: an estimate of the asymptotic variance of the expectile estimator;
	  \item \code{EstConReg}: an estimate of the approximate \eqn{(1-\alpha)100\%} confidence region for \eqn{\tau_n}-\emph{th} \code{d}-dimensional expecile.}
}

\references{
Simone A. Padoan and Gilles Stupfler (2022). Joint inference on extreme expectiles for multivariate heavy-tailed distributions, \emph{Bernoulli} \bold{28}(2), 1021-1048.
}

\author{
	Simone Padoan, \email{simone.padoan@unibocconi.it},
	\url{https://www.unibocconi.it/en/faculty/simone-padoan};
	Gilles Stupfler, \email{gilles.stupfler@univ-angers.fr},
	\url{https://math.univ-angers.fr/~stupfler/}
}

\seealso{
	\link{MultiHTailIndex},  \link{predMultiExpectiles}, \link{extMultiQuantile}
}

\examples{
# Extreme expectile estimation at the intermediate level tau obtained with
# d-dimensional observations simulated from a joint distribution with
# a Gumbel copula and equal Frechet marginal distributions.
library(plot3D)
library(copula)
library(evd)

# distributional setting
copula <- "Gumbel"
dist <- "Frechet"

# parameter setting
dep <- 3
dim <- 3
scale <- rep(1, dim)
shape <- rep(3, dim)
par <- list(dep=dep, scale=scale, shape=shape, dim=dim)

# Intermediate level (or sample tail probability 1-tau)
tau <- .95

# sample size
ndata <- 1000

# Simulates a sample from a multivariate distribution with equal Frechet
# marginals distributions and a Gumbel copula
data <- rmdata(ndata, dist, copula, par)
scatter3D(data[,1], data[,2], data[,3])

# High d-dimensional expectile (intermediate level) estimation
expectHat <- estMultiExpectiles(data, tau, var=TRUE)
expectHat$ExpctHat
expectHat$VarCovEHat
# run the following command to see the graphical representation
\dontrun{
 expectHat <- estMultiExpectiles(data, tau, var=TRUE, plot=TRUE)
}
}

\name{HypoTesting}
\alias{HypoTesting}

\title{Wald-Type Hypothesis Testing}
\description{Wald-type hypothesis tes for testing equality of high or extreme expectiles and quantiles}
\usage{
HypoTesting(data, tau, tau1=NULL, type="ExpectRisks", level="extreme",
            method="LAWS", bias=FALSE, k=NULL, alpha=0.05)
}
\arguments{
	\item{data}{A matrix of \eqn{(n \times d)}{(n x d)} observations.}
	\item{tau}{A real in \eqn{(0,1)} specifying the intermediate level \eqn{\tau_n}. See \bold{Details}.}
	\item{tau1}{A real in \eqn{(0,1)} specifying the extreme level \eqn{\tau'_n}. See \bold{Details}.}
    \item{type}{A string specifying the type of test. By default \code{type="ExpectRisks"} specifies the test for testing the equality of expectiles. See \bold{Details}.}
    \item{level}{A string specifying the level of the expectile. This make sense when \code{type="ExpectRisks"}. By default \code{level="extreme"} specifies that the test concerns expectiles at the extreme level. See \bold{Details}.}
	\item{method}{A string specifying the method used to estimate the expecile. By default \code{est="LAWS"} specifies the use of the LAWS based estimator. See \bold{Details}.}
	\item{bias}{A logical value. By default \code{bias=FALSE} specifies that no bias correction is computed. See \bold{Details}.}
	\item{k}{An integer specifying the value of the intermediate sequence \eqn{k_n}. See \bold{Details}.}
	\item{alpha}{A real in \eqn{(0,1)} specifying the significance level of the test.}
}

\details{
	 With a dataset \code{data} of \code{d}-dimensional observations and sample size \eqn{n}, a Wald-type hypothesis testing is performed in order to check whether the is empirical evidence against the null hypothesis. The null hypothesis concerns the equality among the expectiles or quantiles or tail indices of the marginal distributions. The three tests depend on the depends on the estimation of the \code{d}-dimensional tail index \eqn{\gamma}. Here, \eqn{\gamma} is estimated using the Hill estimation (see \link{MultiHTailIndex} for details).
	 The data are regarded as \code{d}-dimensional temporal independent observations coming from dependent variables. See Padoan and Stupfler (2020) for details.
	\itemize{
		\item The so-called intermediate level \code{tau} or \eqn{\tau_n} is a sequence of positive reals such that \eqn{\tau_n \to 1}{\tau_n -> 1} as \eqn{n \to \infty}{n -> \infty}. Practically, for each marginal distribution, \eqn{\tau_n \in (0,1)}{\tau_n in (0,1)} is the ratio between \emph{N} (Numerator) and \emph{D} (Denominator). Where \emph{N} is the empirical mean distance of the \eqn{\tau_n}-\emph{th} expectile from the observations smaller than it, and \emph{D} is the empirical mean distance of \eqn{\tau_n}-\emph{th} expectile from all the observations.
		\item The so-called extreme level \code{tau1} or \eqn{\tau'_n} is a sequence of positive reals such that \eqn{\tau'_n \to 1}{\tau'_n -> 1} as \eqn{n \to \infty}{n -> \infty}.
		For each marginal distribution, the value \eqn{(1-tau'_n) \in (0,1)}{(1-tau'_n) in (0,1)} is meant to be a small tail probability such that \eqn{(1-\tau'_n)=1/n} or \eqn{(1-\tau'_n) < 1/n}. It is also assumed that \eqn{n(1-\tau'_n) \to C}{n(1-\tau'_n) -> C} as \eqn{n \to \infty}{n -> \infty}, where \eqn{C} is a positive finite constant. Typically, \eqn{C \in (0,1)}{C in (0,1)} so it is expected that there are no observations in a data sample that are greater than the expectile at the extreme level \eqn{\tau_n'}.
        \item When \code{type="ExpectRisks"}, the null hypothesis of the hypothesis testing concerns the equality among the expectiles of the marginal distributions. See Section 3.3 of Padoan and Stupfler (2020) for details. When \code{type="QuantRisks"}, the null hypothesis of the hypothesis testing concerns the equality among the quantiles of the marginal distributions. See Section 5 of Padoan and Stupfler (2020) for details. Note that in this case the test is based on the asymptotic distribution of normalized quantile estimator in the logarithmic scale. When \code{type="tails"}, the null hypothesis of the hypothesis testing concerns the equality among the tail indices of the marginal distributions. See Sections 3.2 and 3.3 of Padoan and Stupfler (2020) for details.
        \item When \code{type="ExpectRisks"}, the null hypothesis concerns the equality among the expectiles of the marginal distributions at the intermediate level and this is achieved through \code{level=="inter"}. In this case the test is obtained exploiting the asymptotic distribution of relative expectile appropriately normalised. See Section 2.1, 3.1 and 3.3 of Padoan and Stupfler (2020) for details. Instead, if \code{level=="extreme"} the null hypothesis concerns the equality among the expectiles of the marginal distributions at the extreme level.
		\item When \code{method='LAWS'}, then the \eqn{\tau'_n}-\emph{th} \code{d}-dimensional expectile is estimated using the LAWS based estimator. When \code{method='QB'}, the expectile is instead estimated using the QB esimtator. The definition of both estimators depend on the estimation of the \code{d}-dimensional tail index \eqn{\gamma}. The \code{d}-dimensional tail index \eqn{\gamma} is estimated using the \code{d}-dimensional Hill estimator (\code{tailest='Hill'}), see \link{MultiHTailIndex}). See Section 2.2 in Padoan and Stupfler (2020) for details.
		\item If \code{bias=TRUE} then \code{d}-dimensional \eqn{\gamma} is estimated using formula (4.2) of Haan et al. (2016). This is used by the LAWS and QB estimators. Furthermore, the \eqn{\tau'_n}--\emph{th} quantile is estimated using the formula in page 330 of de Haan et al. (2016). This provides a bias corrected version of the Weissman estimator. This is used by the QB estimator.
		However, in this case the asymptotic variance is not estimated using the formula in Haan et al. (2016) Theorem 4.2. Instead, for simplicity the asymptotic variance-covariance matrix is estimated by the formulas Section 3.2 of Padoan and Stupfler (2020).
		\item \code{k} or \eqn{k_n} is the value of the so-called intermediate sequence \eqn{k_n}, \eqn{n=1,2,\ldots}{n=1,2,...}. Its represents a sequence of positive integers such that \eqn{k_n \to \infty}{k_n -> \infty} and \eqn{k_n/n \to 0}{k_n/n -> 0} as \eqn{n \to \infty}{n -> \infty}. Practically, for each marginal distribution when \code{tau=NULL} and \code{method='LAWS'} or \code{method='QB'}, then \eqn{\tau_n=1-k_n/n} is the intermediate level of the expectile to be stimated. When \code{tailest='Hill'}, for each marginal distributions, then \eqn{k_n} specifies the number of \code{k}\eqn{+1} larger order statistics used in the definition of the Hill estimator.
		\item A small value \eqn{\alpha\in (0,1)} specifies the significance level of Wald-type hypothesis testing.}
}

\value{
  A list with elements:
    \itemize{
	  \item \code{logLikR}: the observed value of log-likelihood ratio statistic test;
      \item \code{critVal}: the quantile (critical level) of a chi-square distribution with \code{d} degrees of freedom and confidence level \eqn{\alpha}.}
}

\references{
Simone A. Padoan and Gilles Stupfler (2022). Joint inference on extreme expectiles for multivariate heavy-tailed distributions, \emph{Bernoulli} \bold{28}(2), 1021-1048.
}

\author{
	Simone Padoan, \email{simone.padoan@unibocconi.it},
	\url{https://www.unibocconi.it/en/faculty/simone-padoan};
	Gilles Stupfler, \email{gilles.stupfler@univ-angers.fr},
	\url{https://math.univ-angers.fr/~stupfler/}
}

\seealso{
	\link{MultiHTailIndex},  \link{predMultiExpectiles}, \link{extMultiQuantile}
}

\examples{
# Hypothesis testing on the equality extreme expectiles based on a sample of
# d-dimensional observations simulated from a joint distribution with
# a Gumbel copula and equal Frechet marginal distributions.
library(plot3D)
library(copula)
library(evd)

# distributional setting
copula <- "Gumbel"
dist <- "Frechet"

# parameter setting
dep <- 3
dim <- 3
scale <- rep(1, dim)
shape <- rep(3, dim)
par <- list(dep=dep, scale=scale, shape=shape, dim=dim)

# Intermediate level (or sample tail probability 1-tau)
tau <- 0.95
# Extreme level (or tail probability 1-tau1 of unobserved expectile)
tau1 <- 0.9995

# sample size
ndata <- 1000

# Simulates a sample from a multivariate distribution with equal Frechet
# marginals distributions and a Gumbel copula
data <- rmdata(ndata, dist, copula, par)
scatter3D(data[,1], data[,2], data[,3])

# Performs Wald-type hypothesis testing
HypoTesting(data, tau, tau1)

# Hypothesis testing on the equality extreme expectiles based on a sample of
# d-dimensional observations simulated from a joint distribution with
# a Clayton copula and different Frechet marginal distributions.

# distributional setting
copula <- "Clayton"
dist <- "Frechet"

# parameter setting
dim <- 3
dep <- 2
scale <- rep(1, dim)
shape <- c(2.1, 3, 4.5)
par <- list(dep=dep, scale=scale, shape=shape, dim=dim)

# Intermediate level (or sample tail probability 1-tau)
tau <- 0.95
# Extreme level (or tail probability 1-tau1 of unobserved expectile)
tau1 <- 0.9995

# sample size
ndata <- 1000

# Simulates a sample from a multivariate distribution with equal Frechet
# marginals distributions and a Gumbel copula
data <- rmdata(ndata, dist, copula, par)
scatter3D(data[,1], data[,2], data[,3])

# Performs Wald-type hypothesis testing
HypoTesting(data, tau, tau1)
}

\name{plotBayes}
\alias{plotBayes}
\title{Plot empirical Bayes inference results for continuous and discrete generalized Pareto distribution}
\usage{
plotBayes(
  x,
  mle,
  alpha = 0.05,
  param = c("scale", "shape"),
  cols = c("mediumseagreen", "goldenrod", "gold4"),
  ...
)
}
\arguments{
\item{x}{a vector of posterior samples}

\item{mle}{vector of length 2 containing the maximum likelihood estimator for the scale and shape parameters, respectively (only used if \code{param="scale"})}

\item{alpha}{level for intervals. Default to 0.05 giving 95\% confidence intervals}

\item{param}{character string indicating the parameter. Default: \code{c("scale", "shape")}}

\item{cols}{vector of length three containing colors for posterior mean, confidence intervals, and credible intervals. Default to \code{c("mediumseagreen", "goldenrod", "gold4")}}

\item{...}{additional arguments for plotting function; only \code{xlab} is allowed}
}
\value{
\code{NULL}; used to create a plot
}
\description{
Given a sample of posterior draws of the scale or shape parameter, return a histogram on the density scale for the posterior distribution along with a theoretical prior and posterior comparison curve based on the MLE, pointwise Wald-based normal 95\% confidence intervals for the mean of the sample, and pointwise credible intervals (asymmetric by design).
}
\examples{
\dontrun{
# generate data
set.seed(1234)
n <- 500
samp <- evd::rfrechet(n, 0, 3, 4)
# set effective sample size and threshold
k <- 50
threshold <- sort(samp, decreasing = TRUE)[k+1]
# preliminary mle estimates of scale and shape parameters
mlest <- evd::fpot(samp, threshold)
# empirical bayes procedure
proc <- estPOT(
 samp,
 k = k,
 pn = c(0.01, 0.005),
 type = "continuous",
 method = "bayesian",
 prior = "empirical",
 start = as.list(mlest$estimate),
 sig0 = 0.1)
plotBayes(
  proc$post_sample[,1],
  mlest$estimate,
  param = "scale")
plotBayes(
  proc$post_sample[,2],
  param = "shape")
}
}

\name{predQuant}
\alias{predQuant}
\title{Predictive quantile based on the generalized Pareto model}
\usage{
predQuant(
  qlev,
  postsamp,
  threshold,
  lb,
  ub,
  type = c("continuous", "discrete"),
  extrapolation = FALSE,
  p,
  k,
  n
)
}
\arguments{
\item{qlev}{double, quantile level}

\item{postsamp}{a \code{m} by \code{2} matrix with columns containing the posterior samples of scale and shape parameters of the generalized Pareto distribution, respectively}

\item{threshold}{threshold for the generalized Pareto model, corresponding to the \eqn{n-k}th order statistic of the sample}

\item{lb}{double, the lower bound of the admissible region for the quantile value}

\item{ub}{double, the upper bound of the admissible region for the quantile value}

\item{type}{data type, either \code{"continuous"} or \code{"discrete"} for count data.}

\item{extrapolation}{logical; if \code{TRUE}, extrapolate using the threshold stability property}

\item{p}{scalar tail probability for the extrapolation. Must be smaller than \eqn{k/n} (only used if \code{extrapolation=TRUE})}

\item{k}{integer, number of exceedances for the generalized Pareto (only used if \code{extrapolation=TRUE})}

\item{n}{integer, number of observations in the full sample. Must be greater than \code{k}  (only used if \code{extrapolation=TRUE})}
}
\value{
a double indicating the value of the quantile
}
\description{
Bayesian Generalize Pareto-based predictive quantile for continuous and discrete predictive distribution conditioned on intermediate and extreme levels.
}
\examples{
\dontrun{
# generate data
set.seed(1234)
n <- 500
samp <- evd::rfrechet(n,0,3,4)
# set effective sample size and threshold
k <- 50
threshold <- sort(samp,decreasing = TRUE)[k+1]
# preliminary mle estimates of scale and shape parameters
mlest <- evd::fpot(samp, threshold)
# empirical bayes procedure
proc <- estPOT(
  samp,
  k = k,
  pn = c(0.01, 0.005),
  type = "continuous",
  method = "bayesian",
  prior = "empirical",
  start = as.list(mlest$estimate),
  sig0 = 0.1)
# predictive density estimation
yg <- seq(0, 50, by = 2)
nyg <- length(yg)
predDens_int <- predDens(
  yg,
  proc$post_sample,proc$t,
  "continuous",
  extrapolation=FALSE)
predQuant_int <- predQuant(
  0.5,
  proc$post_sample,
  proc$t,
  proc$t + 0.01,
  50,
  "continuous",
  extrapolation = FALSE)
predDens_ext <- predDens(
  yg,
  proc$post_sample,
  proc$t,
  "continuous",
  extrapolation = TRUE,
  p = 0.001,
  k = k,
  n = n)
predQuant_ext <- predQuant(
  0.5,
  proc$post_sample,
  proc$t,
  proc$t + 0.01,
  100,
  "continuous",
  extrapolation = TRUE,
  p = 0.005,
  k = k,
  n = n)
  }
}

\name{predExpectiles}
\alias{predExpectiles}

\title{Extreme Expectile Estimation}
\description{Computes a point and interval estimate of the expectile at the extreme level (Expectile Prediction).}
\usage{
predExpectiles(data, tau, tau1, method="LAWS", tailest="Hill", var=FALSE,
               varType="asym-Dep", bias=FALSE, bigBlock=NULL, smallBlock=NULL,
               k=NULL, alpha_n=NULL, alpha=0.05)
}

\arguments{
	\item{data}{A vector of \eqn{(1 \times n)}{(1 x n)} observations.}
	\item{tau}{A real in \eqn{(0,1)} specifying the intermediate level \eqn{\tau_n}. See \bold{Details}.}
	\item{tau1}{A real in \eqn{(0,1)} specifying the extreme level \eqn{\tau'_n}. See \bold{Details}.}
	\item{method}{A string specifying the method used to estimate the expecile. By default \code{est="LAWS"} specifies the use of the LAWS based estimator. See \bold{Details}.}
	\item{tailest}{A string specifying the tail index estimator. By default \code{tailest="Hill"} specifies the use of Hill estimator. See \bold{Details}.}
	\item{var}{If \code{var=TRUE} then an estimate of the asymptotic variance of the expectile estimator is computed.}
	\item{varType}{A string specifying the type of asymptotic variance to compute. By default \code{varType="asym-Dep"} specifies the variance estimator for serial dependent observations. See \bold{Details}.}
	\item{bias}{A logical value. By default \code{bias=FALSE} specifies that no bias correction is computed. See \bold{Details}.}
	\item{bigBlock}{An interger specifying the size of the big-block used to estimaste the asymptotic variance. See \bold{Details}.}
	\item{smallBlock}{An interger specifying the size of the small-block used to estimaste the asymptotic variance. See \bold{Details}.}
	\item{k}{An integer specifying the value of the intermediate sequence \eqn{k_n}. See \bold{Details}.}
    \item{alpha_n}{A real in \eqn{(0,1)} specifying the quantile's extreme level to be use in order to estimate the expectile's extreme level.}
	\item{alpha}{A real in \eqn{(0,1)} specifying the confidence level \eqn{(1-\alpha)100\%} of the approximate confidence interval for the expecile at the intermedite level.}
}

\details{
	For a dataset \code{data} of sample size \eqn{n}, an estimate of the \eqn{\tau'_n}-\emph{th} expectile is computed. The estimation of the expectile at the extreme level \code{tau1} (\eqn{\tau'_n}) is meant to be a prediction beyond the observed sample.  Two estimators are available: the so-called Least Asymmetrically Weighted Squares (LAWS) based estimator and the Quantile-Based (QB) estimator. The definition of both estimators depends on the estimation of the tail index \eqn{\gamma}. Here, \eqn{\gamma} is estimated using the Hill estimation (see \link{HTailIndex} for details) or in alternative using the the expectile based estimator (see \link{EBTailIndex}).
	The observations can be either independent or temporal dependent. See Section 3.2 in Padoan and Stupfler (2020) for details.
	\itemize{
		\item The so-called intermediate level \code{tau} or \eqn{\tau_n} is a sequence of positive reals such that \eqn{\tau_n \to 1}{\tau_n -> 1} as \eqn{n \to \infty}{n -> \infty}. Practically, \eqn{\tau_n \in (0,1)}{\tau_n in (0,1)} is the ratio between \emph{N} (Numerator) and \emph{D} (Denominator). Where \emph{N} is the empirical mean distance of the \eqn{\tau_n}-\emph{th} expectile from the observations smaller than it, and \emph{D} is the empirical mean distance of \eqn{\tau_n}-\emph{th} expectile from all the observations.
		\item The so-called extreme level \code{tau1} or \eqn{\tau'_n} is a sequence of positive reals such that \eqn{\tau'_n \to 1}{\tau'_n -> 1} as \eqn{n \to \infty}{n -> \infty}.
		The value \eqn{(1-tau'_n) \in (0,1)}{(1-tau'_n) in (0,1)} is meant to be a small tail probability such that \eqn{(1-\tau'_n)=1/n} or \eqn{(1-\tau'_n) < 1/n}. It is also assumed that \eqn{n(1-\tau'_n) \to C}{n(1-\tau'_n) -> C} as \eqn{n \to \infty}{n -> \infty}, where \eqn{C} is a positive finite constant. Typically, \eqn{C \in (0,1)}{C in (0,1)} so it is expected that there are no observations in a data sample that are greater than the expectile at the extreme level \eqn{\tau_n'}.
		\item When \code{method='LAWS'}, then the \eqn{\tau'_n}-\emph{th} expectile is estimated using the LAWS based estimator. When \code{method='QB'}, the expectile is instead estimated using the QB esimtator. The definition of both estimators depend on the estimation of the tail index \eqn{\gamma}. When \code{tailest='Hill'} then \eqn{\gamma} is estimated using the Hill estimator (see \link{HTailIndex}). When \code{tailest='ExpBased'}, then \eqn{\gamma} is estimated using the expectile based estimator (see \link{EBTailIndex}). See Section 3.2 in Padoan and Stupfler (2020) for details.
		\item If \code{var=TRUE} then an esitmate of the asymptotic variance of the \eqn{tau'_n}-\emph{th} expectile is computed. Notice that the estimation of the asymptotic variance \bold{is only available} when \eqn{\gamma} is estimated using the Hill estimator (see \link{HTailIndex}). With independent observations the asymptotic variance is estimated by \eqn{\hat{\gamma}^2}, see the remark below Theorem 3.5 in Padoan and Stupfler (2020). This is achieved through \code{varType="asym-Ind"}.  With serial dependent observations the asymptotic variance is estimated by the formula in Throrem 3.5 of Padoan and Stupfler (2020). This is achieved through \code{varType="asym-Dep"}. See Section 3.2 in Padoan and Stupfler (2020) for details. In this latter case the computation of the serial dependence is based on the "big blocks seperated by small blocks" techinque which is a standard tools in time series, see e.g. Leadbetter et al. (1986).
		The size of the big and small blocks are specified by the parameters \code{bigBlock} and \code{smallBlock}, respectively.
		\item If \code{bias=TRUE} then \eqn{\gamma} is estimated using formula (4.2) of Haan et al. (2016). This is used by the LAWS and QB estimators. Furthermore, the \eqn{\tau'_n}--\emph{th} quantile is estimated using the formula in page 330 of de Haan et al. (2016). This provides a bias corrected version of the Weissman estimator. This is used by the QB estimator.
		However, in this case the asymptotic variance is not estimated using the formula in Haan et al. (2016) Theorem 4.2. Instead, for simplicity the asymptotic variance is estimated by the formula in Corollary 3.8, with serial dependent observations, and \eqn{\hat{\gamma}^2} with independent observation (see e.g. de Drees 2000, for the details).
		\item \code{k} or \eqn{k_n} is the value of the so-called intermediate sequence \eqn{k_n}, \eqn{n=1,2,\ldots}{n=1,2,...}. Its represents a sequence of positive integers such that \eqn{k_n \to \infty}{k_n -> \infty} and \eqn{k_n/n \to 0}{k_n/n -> 0} as \eqn{n \to \infty}{n -> \infty}. Practically, when \code{tau=NULL} and \code{method='LAWS'}, then \eqn{\tau_n=1-k_n/n} is the intermediate level of the expectile to be stimated. The latter is also used to estimate the tail index when \code{tailest='ExpBased'}. Instead, if \code{tailest='Hill'}, then \eqn{k_n} specifies the number of \code{k}\eqn{+1} larger order statistics used in the definition of the Hill estimator. Differently, When \code{tau=NULL} and \code{method='QB'}, then \eqn{\tau_n=1-k_n/n} is the intermediate level of the quantile to be stimated and of the expectile to be stimated when \code{tailest='ExpBased'}. Instead, when \code{tailest='Hill'} it is the numer of \code{k}\eqn{+1} larger order statistics used in the definition of the Hill estimator.
        \item If quantile's extreme level is provided by \code{alpha_n}, then expectile's extreme level \eqn{tau'_n(\alpha_n)} is replaced by \eqn{tau'_n(\alpha_n)} which is esitmated using the method described in Section 6 of Padoan and Stupfler (2020). See \link{estExtLevel} for details.
		\item Given a small value \eqn{\alpha\in (0,1)} then an estimate of an asymptotic confidence interval for \eqn{tau'_n}-\emph{th} expectile, with approximate nominal confidence level \eqn{(1-\alpha)100\%}, is computed. The confidence intervals are computed exploiting formula (10) and (11) in Padoan and Stupfler (2020) and (46) in Drees (2003). See Section 5 in Padoan and Stupfler (2020) for details. When \code{biast=TRUE} confidence intervals are computed in the same way but after correcting the tail index estimate by an estimate of the bias term, see formula (4.2) in de Haan et al. (2016) for details.
	}
}

\value{
  A list with elements:
    \itemize{
	  \item \code{EExpcHat}: an estimate of the \eqn{\tau'_n}-\emph{th} expecile;
	  \item \code{VarExtHat}: an estimate of the asymptotic variance of the expectile estimator;
	  \item \code{CIExpct}: an estimate of the approximate \eqn{(1-\alpha)100\%} confidence interval for \eqn{\tau'_n}-\emph{th} expecile.
	}
}

\references{
Anthony C. Davison, Simone A. Padoan and Gilles Stupfler (2023). Tail Risk Inference via Expectiles in Heavy-Tailed Time Series, \emph{Journal of Business & Economic Statistics}, \bold{41}(3) 876-889.

Daouia, A., Girard, S. and Stupfler, G. (2018). Estimation of tail risk based on extreme expectiles. \emph{Journal of the Royal Statistical Society: Series B}, \bold{80}, 263-292.

de Haan, L., Mercadier, C. and Zhou, C. (2016). Adapting extreme value statistics to financial time series: dealing with bias and serial dependence. \emph{Finance and Stochastics},
\bold{20}, 321-354.

Drees, H. (2003). Extreme quantile estimation for dependent data, with applications to finance.
\emph{Bernoulli}, \bold{9}, 617-657.

Drees, H. (2000). Weighted approximations of tail processes for \eqn{\beta}-mixing random variables.
\emph{Annals of Applied Probability}, \bold{10}, 1274-1301.

Leadbetter, M.R., Lindgren, G. and Rootzen, H. (1989). Extremes and related
properties of random sequences and processes. \emph{Springer}.
}

\author{
	Simone Padoan, \email{simone.padoan@unibocconi.it},
	\url{https://www.unibocconi.it/en/faculty/simone-padoan};
	Gilles Stupfler, \email{gilles.stupfler@univ-angers.fr},
	\url{https://math.univ-angers.fr/~stupfler/}
}

\seealso{
	\link{HTailIndex},  \link{EBTailIndex}, \link{estExpectiles}, \link{extQuantile}
}

\examples{
# Extreme expectile estimation at the extreme level tau1 obtained with
# 1-dimensional data simulated from an AR(1) with univariate
# Student-t distributed innovations

tsDist <- "studentT"
tsType <- "AR"

# parameter setting
corr <- 0.8
df <- 3
par <- c(corr, df)

# Big- small-blocks setting
bigBlock <- 65
smallBlock <- 15

# Intermediate level (or sample tail probability 1-tau)
tau <- 0.95
# Extreme level (or tail probability 1-tau1 of unobserved expectile)
tau1 <- 0.9995

# sample size
ndata <- 2500

# Simulates a sample from an AR(1) model with Student-t innovations
data <- rtimeseries(ndata, tsDist, tsType, par)

# Extreme expectile estimation
expectHat1 <- predExpectiles(data, tau, tau1, var=TRUE, bigBlock=bigBlock,
	                         smallBlock=smallBlock)
expectHat1$EExpcHat
expectHat1$CIExpct
# Extreme expectile estimation with bias correction
tau <- 0.80
expectHat2 <- predExpectiles(data, tau, tau1, "QB", var=TRUE, bias=TRUE, bigBlock=bigBlock,
							 smallBlock=smallBlock)
expectHat2$EExpcHat
expectHat2$CIExpct
}

\name{estExpectiles}
\alias{estExpectiles}

\title{High Expectile Estimation}
\description{Computes a point and interval estimate of the expectile at the intermediate level.}
\usage{
estExpectiles(data, tau, method="LAWS", tailest="Hill", var=FALSE, varType="asym-Dep-Adj",
              bigBlock=NULL, smallBlock=NULL, k=NULL, alpha=0.05)
}

\arguments{
	\item{data}{A vector of \eqn{(1 \times n)}{(1 x n)} observations.}
	\item{tau}{A real in \eqn{(0,1)} specifying the intermediate level \eqn{\tau_n}. See \bold{Details}.}
	\item{method}{A string specifying the method used to estimate the expecile. By default \code{est="LAWS"} specifies the use of the direct LAWS estimator. See \bold{Details}.}
	\item{tailest}{A string specifying the type of tail index estimator. By default \code{tailest="Hill"} specifies the use of Hill estimator. See \bold{Details}.}
	\item{var}{If \code{var=TRUE} then an estimate of the variance of the expectile estimator is computed.}
	\item{varType}{A string specifying the asymptotic variance to compute. By default \code{varType="asym-Dep-Adj"} specifies the variance estimator for serial dependent observations implemented with a suitable adjustment. See \bold{Details}.}
	\item{bigBlock}{An interger specifying the size of the big-block used to estimaste the asymptotic variance. See \bold{Details}.}
	\item{smallBlock}{An interger specifying the size of the small-block used to estimaste the asymptotic variance. See \bold{Details}.}
	\item{k}{An integer specifying the value of the intermediate sequence \eqn{k_n}. See \bold{Details}.}
	\item{alpha}{A real in \eqn{(0,1)} specifying the confidence level \eqn{(1-\alpha)100\%} of the approximate confidence interval for the expecile at the intermedite level.}
}

\details{
	For a dataset \code{data} of sample size \eqn{n}, an estimate of the \eqn{\tau_n}-\emph{th} expectile is computed. Two estimators are available: the so-called direct Least Asymmetrically Weighted Squares (LAWS) and indirect Quantile-Based (QB). The definition of the QB estimator depends on the estimation of the tail index \eqn{\gamma}. Here, \eqn{\gamma} is estimated using the Hill estimation (see \link{HTailIndex}) or in alternative using the the expectile based estimator (see \link{EBTailIndex}). The observations can be either independent or temporal dependent. See Section 3.1 in Padoan and Stupfler (2020) for details.
	\itemize{
		\item The so-called intermediate level \code{tau} or \eqn{\tau_n} is a sequence of positive reals such that \eqn{\tau_n \to 1}{\tau_n -> 1} as \eqn{n \to \infty}{n -> \infty}. Practically, \eqn{\tau_n \in (0,1)}{\tau_n in (0,1)} is the ratio between \emph{N} (Numerator) and \emph{D} (Denominator). Where \emph{N} is the empirical mean distance of the \eqn{\tau_n}-\emph{th} expectile from the observations smaller than it, and \emph{D} is the empirical mean distance of \eqn{\tau_n}-\emph{th} expectile from all the observations.
		\item If \code{method='LAWS'}, then the expectile at the intermediate level \eqn{\tau_n} is estimated applying the direct LAWS estimator. Instead, If \code{method='QB'} the indirect QB esimtator is used to estimate the expectile. See Section 3.1 in Padoan and Stupfler (2020) for details.
		\item When the expectile is estimated by the indirect QB esimtator (\code{method='QB'}), an estimate of the tail index \eqn{\gamma} is needed. If \code{tailest='Hill'} then \eqn{\gamma} is estimated using the Hill estimator (see also \link{HTailIndex}). If \code{tailest='ExpBased'} then \eqn{\gamma} is estimated using the expectile based estimator (see \link{EBTailIndex}). See Section 3.1 in Padoan and Stupfler (2020) for details.
		\item \code{k} or \eqn{k_n} is the value of the so-called intermediate sequence \eqn{k_n}, \eqn{n=1,2,\ldots}{n=1,2,...}. Its represents a sequence of positive integers such that \eqn{k_n \to \infty}{k_n -> \infty} and \eqn{k_n/n \to 0}{k_n/n -> 0} as \eqn{n \to \infty}{n -> \infty}. Practically, when \code{method='LAWS'} and \code{tau=NULL}, \eqn{k_n} specifies by \eqn{\tau_n=1-k_n/n}{tau_n = 1 - k_n / n} the intermediate level of the expectile. Instead, when \code{method='QB'}, if \code{tailest="Hill"} then the value \eqn{k_n} specifies the number of \code{k}\eqn{+1} larger order statistics to be used to estimate \eqn{\gamma} by the Hill estimator and if \code{tau=NULL} then it also specifies by \eqn{\tau_n=1-k_n/n} the confidence level \eqn{\tau_n} of the quantile to estimate. Finally, if \code{tailest="ExpBased"} and \code{tau=NULL} then it also specifies by \eqn{\tau_n=1-k_n/n} the intermediate level expectile based esitmator of \eqn{\gamma} (see \link{EBTailIndex}).
		\item If \code{var=TRUE} then the asymptotic variance of the expecile estimator is computed. With independent observations the asymptotic variance is computed by the formula Theorem 3.1 of Padoan and Stupfler (2020). This is achieved through \code{varType="asym-Ind"}. With serial dependent observations the asymptotic variance is estimated by the formula in Theorem 3.1 of Padoan and Stupfler (2020). This is achieved through \code{varType="asym-Dep"}. In this latter case the computation of the asymptotic variance is based on the "big blocks seperated by small blocks" techinque which is a standard tools in time series, see Leadbetter et al. (1986). See also Section C.1 in Appendix of Padoan and Stupfler (2020). The size of the big and small blocks are specified by the parameters \code{bigblock} and \code{smallblock}, respectively. Still with serial dependent observations, If \code{varType="asym-Dep-Adj"}, then the asymptotic variance is estimated using formula (C.79) in Padoan and Stupfler (2020), see Section C.1 of the Appendix for details.
		\item Given a small value \eqn{\alpha\in (0,1)} then an asymptotic confidence interval for the \eqn{\tau_n}-\emph{th} expectile, with approximate nominal confidence level \eqn{(1-\alpha)100\%} is computed. See Sections 3.1 and C.1 in the Appendix of Padoan and Stupfler (2020).
	}
}

\value{
  A list with elements:
    \itemize{
	  \item \code{ExpctHat}: a point estimate of the \eqn{\tau_n}-\emph{th} expecile;
	  \item \code{VarExpHat}: an estimate of the asymptotic variance of the expectile estimator;
	  \item \code{CIExpct}: an estimate of the approximate \eqn{(1-\alpha)100\%} confidence interval for \eqn{\tau_n}-\emph{th} expecile.
	}
}

\references{
Anthony C. Davison, Simone A. Padoan and Gilles Stupfler (2023). Tail Risk Inference via Expectiles in Heavy-Tailed Time Series, \emph{Journal of Business & Economic Statistics}, \bold{41}(3) 876-889.

Daouia, A., Girard, S. and Stupfler, G. (2018). Estimation of tail risk based on extreme expectiles. \emph{Journal of the Royal Statistical Society: Series B}, \bold{80}, 263-292.

Leadbetter, M.R., Lindgren, G. and Rootzen, H. (1989). Extremes and related
properties of random sequences and processes. \emph{Springer}.
}

\author{
	Simone Padoan, \email{simone.padoan@unibocconi.it},
	\url{https://www.unibocconi.it/en/faculty/simone-padoan};
	Gilles Stupfler, \email{gilles.stupfler@univ-angers.fr},
	\url{https://math.univ-angers.fr/~stupfler/}
}

\seealso{
	\link{HTailIndex},  \link{EBTailIndex}, \link{predExpectiles}, \link{extQuantile}
}

\examples{
# Extreme expectile estimation at the intermediate level tau obtained with
# 1-dimensional data simulated from an AR(1) with Student-t innovations

tsDist <- "studentT"
tsType <- "AR"

# parameter setting
corr <- 0.8
df <- 3
par <- c(corr, df)

# Big- small-blocks setting
bigBlock <- 65
smallBlock <- 15

# Intermediate level (or sample tail probability 1-tau)
tau <- 0.99

# sample size
ndata <- 2500

# Simulates a sample from an AR(1) model with Student-t innovations
data <- rtimeseries(ndata, tsDist, tsType, par)

# High expectile (intermediate level) estimation
expectHat <- estExpectiles(data, tau, var=TRUE, bigBlock=bigBlock, smallBlock=smallBlock)
expectHat$ExpctHat
expectHat$CIExpct
}
